Package: lumi
Type: Package
Title: BeadArray Specific Methods for Illumina Methylation and
        Expression Microarrays
Version: 2.62.0
Date: 2020-10-02
Depends: R (>= 2.10), Biobase (>= 2.5.5)
Imports: affy (>= 1.23.4), methylumi (>= 2.3.2), GenomicFeatures,
        GenomicRanges, annotate, lattice, mgcv (>= 1.4-0), nleqslv,
        KernSmooth, preprocessCore, RSQLite, DBI, AnnotationDbi, MASS,
        graphics, stats, stats4, methods
Suggests: beadarray, limma, vsn, lumiBarnes, lumiHumanAll.db,
        lumiHumanIDMapping, genefilter, RColorBrewer
Author: Pan Du, Richard Bourgon, Gang Feng, Simon Lin
Maintainer: Lei Huang <lhuang1998@gmail.com>
Description: The lumi package provides an integrated solution for the
    Illumina microarray data analysis. It includes functions of
    Illumina BeadStudio (GenomeStudio) data input, quality control,
    BeadArray-specific variance stabilization, normalization and gene
    annotation at the probe level. It also includes the functions of
    processing Illumina methylation microarrays, especially Illumina
    Infinium methylation microarrays.
License: LGPL (>= 2)
LazyLoad: yes
biocViews: Microarray, OneChannel, Preprocessing, DNAMethylation,
        QualityControl, TwoChannel
NeedsCompilation: no
Packaged: 2025-10-30 04:45:22 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/lumi
git_branch: RELEASE_3_22
git_last_commit: 1123e59
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29

import(methods)

importFrom(grDevices, colors)
importFrom(utils, head, packageDescription, read.table, tail,
           write.table)
importFrom(stats4, plot, summary)

importClassesFrom(
    Biobase, AnnotatedDataFrame, AssayData, eSet,
    ExpressionSet, Versioned, VersionedBiobase,
    Versions)
importMethodsFrom(
    Biobase, annotation, "annotation<-", assayData,
    "assayData<-", experimentData, "experimentData<-",
    featureData, "featureData<-", featureNames,
    "featureNames<-", pData, "pData<-", phenoData,
    "phenoData<-", exprs, "exprs<-", se.exprs,
    "se.exprs<-", sampleNames, "sampleNames<-",
    varLabels, varMetadata, coerce, combine, dim,
    initialize, notes, storageMode, show, "[", "[[",
    "$")
importFrom(
    Biobase, AnnotatedDataFrame, assayDataElement,
    "assayDataElement<-", assayDataElementNames,
    assayDataElementReplace, assayDataValidMembers,
    description, fData, "fData<-", "notes<-",
    protocolData, "protocolData<-", rowMin, "storageMode<-")
importClassesFrom(methylumi, MethyLumi, MethyLumiSet, MethyLumiQC, MethyLumiM)
importMethodsFrom(methylumi, methylated, unmethylated, "methylated<-", "unmethylated<-",
                 "betas", "betas<-", QCdata, combine, "[", 
 "controlData", "controlData<-", detection, "detection<-", "dataType", "dataType<-")
importFrom(methylumi, controlTypes, lumIDAT, methylumiR)

importFrom(mgcv, gam, smoothCon, mono.con, pcls, s, Predict.matrix)
importFrom(KernSmooth, locpoly)
importFrom(lattice, bwplot, panel.violin)
importFrom(nleqslv, nleqslv)
importFrom(annotate, lookUp, getSYMBOL)
importClassesFrom(RSQLite, SQLiteDriver, SQLiteConnection)
importMethodsFrom(RSQLite, dbReadTable, dbListTables, dbConnect, dbDisconnect, dbListFields)
importFrom(DBI, dbReadTable, dbListTables, dbConnect, dbDisconnect, dbListFields, dbDriver, 
           dbDataType, dbCommit, dbClearResult, SQLKeywords)

importFrom(AnnotationDbi, as.list, keys)
importClassesFrom(AnnotationDbi, AnnDbBimap)
importMethodsFrom(AnnotationDbi, "[", mappedkeys)
importFrom(graphics, pairs, boxplot, hist, image, lines, matplot, points, abline, axis, par,
           text, Axis, arrows, box, layout,  legend, rect, title)
importFrom(stats, density, heatmap, approx, as.dist, cmdscale, cor,
           density.default, dgamma, dist, ecdf, hclust, lm, median,
           pgamma, predict, quantile, sd, start, supsmu, var)
importFrom(MASS, rlm, psi.bisquare)
importFrom(affy, normalize.loess, mva.pairs, bg.adjust, plotDensity)
importFrom(preprocessCore, normalize.quantiles, normalize.quantiles.robust)

importFrom(GenomicFeatures, features)
importFrom(GenomicRanges, seqnames)


exportClasses(LumiBatch)
exportMethods(beadNum, "beadNum<-", boxplot, combine, density, detection, "detection<-", getHistory, hist, MAplot, pairs, plot, "sampleNames<-", exprs, "exprs<-", se.exprs, "se.exprs<-", show, summary, "[", coerce, "controlData", "controlData<-", asBigMatrix)

export(IlluminaID2nuID, addControlData2lumi, addNuID2lumi, addNuId2lumi,                                                            
bgAdjust, detectOutlier, detectionCall, estimateLumiCV, getChipInfo, getControlData, getControlProbe,
getControlType, getNuIDMappingInfo, id2seq, inverseVST, is.nuID, lumiB, lumiExpresso, lumiN, lumiQ,                       
lumiR, lumiR.batch, lumiT, nuID2EntrezID, nuID2IlluminaID, nuID2RefSeqID, nuID2probeID, nuID2targetID,
plotCDF, plotControlData, plotDensity, plotHousekeepingGene, plotSampleRelation, plotStringencyGene, plotVST, probeID2nuID,
produceGEOPlatformFile, produceGEOSampleInfoTemplate, produceGEOSubmissionFile, rsn, seq2id, ssn, targetID2nuID, vst, rankinvariant,
addAnnotationInfo, addControlData2methyLumiM, adjColorBias.quantile, adjColorBias.ssn, beta2m, bgAdjustMethylation, boxplotColorBias, colorBiasSummary, estimateBeta, estimateM,
estimateIntensity, estimateMethylationBG, importMethyIDAT, lumiMethyB, lumiMethyC, lumiMethyN, lumiMethyR, m2beta, normalizeMethylation.quantile, 
normalizeMethylation.ssn, plotColorBias2D, plotColorBias1D, produceMethylationGEOSubmissionFile, smoothQuantileNormalization, 
lumiMethyStatus, gammaFitEM, methylationCall, plotGammaFit, getChrInfo
)

# Lumi 💧 <a href="https://hits.seeyoufarm.com"><img src="https://hits.seeyoufarm.com/api/count/incr/badge.svg?url=https%3A%2F%2Fgithub.com%2FTanmoy741127%2Flumi&count_bg=%2379C83D&title_bg=%23555555&icon=&icon_color=%23E7E7E7&title=hits&edge_flat=false"/></a>

<img align="right" src="https://raw.githubusercontent.com/Tanmoy741127/cdn/main/lumi/lumi-logo.png" height="100px"/>

Lumi is an nano framework to convert your python functions into a REST API without any extra headache.

* This library is created by taking the concept of **RPC** and blended with **REST API** specs. 
* We need to just register the function and it will be available as a REST API. 
* Web-server written with **Gunicorn**
* Local development server provided for rapid development and prototyping.

## Installation

```bash
pip install lumi
```

## Function <--> API mapping
![function - API maaping](https://raw.githubusercontent.com/Tanmoy741127/cdn/main/lumi/function-api-map.png)


## How to use 🤔

Let's create a simple function to add two numbers.

```python
def add(a, b):
    return a + b

def substract(a, b):
    return a - b
```

Now, we want to expose this function as a REST API. We can do this by registering the function with Lumi.

```python
# app.py

from lumi import Lumi

app = Lumi()

app.register(add) # Registering the function
app.register(substract)

app.runServer(host="127.0.0.1", port=8080)
```

Noice 🎉🎉  API has been generated

Run the sever by
```
python app.py
```
You are going to see this in your terminal 
```
[2022-11-24 17:32:08 +0530] [10490] [INFO] Starting gunicorn 20.1.0
[2022-11-24 17:32:08 +0530] [10490] [INFO] Listening at: http://127.0.0.1:8080 (10490)
[2022-11-24 17:32:08 +0530] [10490] [INFO] Using worker: sync
[2022-11-24 17:32:08 +0530] [10492] [INFO] Booting worker with pid: 10492
...
...
[2022-11-24 17:32:08 +0530] [10500] [INFO] Booting worker with pid: 10500
```

Congratulations 👏. Our Server is online. 


The above code will generate a REST API with the following details.

- Endpoint : `127.0.0.1:8080`
- Route : `/add`
- Method : `POST`
- Sample Request Body : `{"a": 1, "b": 2}`

Let's run the API and test it.

```curl
curl -X POST -H "Content-Type: application/json" -d '{"a": 1, "b": 2}' http://127.0.0.1:8080/add
```

Output

```json
{
    "exit_code": 0, 
    "status_code": 200, 
    "result": 3, 
    "error": ""
}
```

## Custom Routing
Now you may think, the function name will be always same as the route. But, you can change the route by passing the route parameter.

```python
app.register(add, route="/addition")
```
## Custom Request Method
By default, the request method is `POST`. But, you can change it by passing the method parameter. Currenetly , it supports `POST`, `PUT` and `PATCH` methods.

```python
from lumi import Lumi, RequestMethod

app = Lumi()

def add(a, b):
    return a+b


app.register(add, request_method=RequestMethod.PUT) # Register function for PUT method
app.register(add, request_method=RequestMethod.PATCH) # Register function for PATCH method

app.runServer()
```

## Status Codes

| Status Code | Description |
| --- | --- |
| 200 | Request successfully executed and No Error happened during function execution |
| 500 | Request was received but there was an error during function execution |
| 400 | Bad Request (Possible Reason - The required parameters for the function has not provided) |
| 405 | Method Not Allowed (Lumi only supports **POST** request) |
| 404 | The route has no function associated with that |


## Exit Codes
| Exit Code | Description |
| --- | --- |
| 0 | No Error |
| 1 | Error |

> Note : If the function has some error , you can expect the exit code to be 1 and the error message in the response.

## Task Lists
- [x] Base System
- [x] Add support for default parameters that is provided in the function
- [ ] Make available GET request for the function
- [x] Provide option to override POST with PUT if the user wants
- [ ] Add authentication middleware support
- [ ] Support nested routing of urls
- [ ] For local development, create an file observer that can automatically reload the server when the file is changed.
- [ ] Add support for object serialization and deserialization based on argument types of function

## Contributing

Contributions are always welcome!

`seq2id` <-
function(seq) {

	if (length(seq) > 1) {
		return(sapply(seq, seq2id))
	} else {
		## the coding is based on Base64 Content-Transfer-Encoding format (http://www.freesoft.org/CIE/RFC/1521/7.htm)
		##code <- c(LETTERS, letters, as.character(0:9), '+', '/')
		code <- c(LETTERS, letters, as.character(0:9), '_', '.')
		seq <- toupper(seq)

		if(length(grep('[^ACGTU]', seq)) > 0)  stop('Input sequence is not a nucleotide sequnce!')

		## translate A, C, G, T, U, as 0, 1, 2, 3, 3
		num <- chartr("ACGTU", "01233", seq)

		## break the sequence as individual characters
		num <- substring(num, 1:nchar(num), 1:nchar(num))
		num <- as.numeric(num)
		if (length(num) %% 3 != 0) {
			appLen <- 3 - length(num) %% 3
			num <- c(num, rep(0, appLen))
		} else {
			appLen <- 0
		}

		codon <- matrix(num, nrow=3)
		code64 <- codon[1,] * 4^2 + codon[2,] * 4 + codon[3,]

		## check sum error checking
		checkSum <- sum(code64)
		res <- checkSum %% 21

		checkCode <- code[res*3 + appLen + 1]
		id <- paste(checkCode, paste(code[code64 + 1], collapse=''), sep='')
		return(id)
	}
}

`id2seq` <-
function(id) {
	if (length(id) > 1) {
		return(sapply(id, id2seq))
	} else {
		## the coding is based on Base64 Content-Transfer-Encoding format (http://www.freesoft.org/CIE/RFC/1521/7.htm)
		#code <- c(LETTERS, letters, as.character(0:9), '+', '/')
		code <- c(LETTERS, letters, as.character(0:9), '_', '.')
		ind <- 1:length(code)
		names(ind) <- code
		if(length(grep('[^a-zA-Z0-9_.]', id)) > 0)  stop('Input id is not a nuID!')

		id <- substring(id, 1:nchar(id), 1:nchar(id))
		num <- as.numeric(ind[id]) - 1
		checkCode <- num[1]
		num <- num[-1]
		if (checkCode == 63)  warning('Coding error or not a nuID!\n Check code should not include "."!')
		cutLen <- checkCode %% 3
		res <- floor(checkCode/3)

		codon <- rbind(floor(num / 4^2), floor((num %% 4^2) / 4), num %% 4)

		## Check whether the checkCode matches the sequence
		checkSum <- sum(num)
		if (res != checkSum %% 21) {
			warning('Coding error or not a nuID!')
		}

		## nucleotide
		nucleotide <- c('A', 'C', 'G', 'T')
		codon <- nucleotide[codon + 1]
		len <- length(codon)
		seq <- paste(codon[1:(len-cutLen)], collapse='')
		return(seq)
	}
}

`estimateLumiCV` <-
function(x.lumi, type=c('measurement', 'probe'), ifPlot=FALSE, ...) {

	type <- match.arg(type)
	if (type == 'measurement') {
		if (!is(x.lumi, 'LumiBatch')) stop('The object should be class "LumiBatch"!')
		std <- se.exprs(x.lumi)
		if (is.null(std)) stop('The object does not include "se.exprs" matrix in assayData slot!')
		cv <- std / exprs(x.lumi)
		rownames(cv) <- rownames(std)
		colnames(cv) <- colnames(std)
		if (ifPlot) {
			plotDensity(cv, xlab='coefficient of variance', ...)
			return(invisible(cv))	
		}
	} else {
		if (is(x.lumi, 'ExpressionSet')) x.lumi <- exprs(x.lumi)
		cv <- apply(x.lumi, 1, function(x) sd(x)/mean(x))
		names(cv) <- rownames(x.lumi)
	}
	return(cv)
}

`nuID2RefSeqID` <-
function(nuID=NULL, lib.mapping, filterTh=c(Strength1=95, Uniqueness=95), returnAllInfo=FALSE) {
	if (missing(lib.mapping)) stop('Please specify lib.mapping library!')
	# 
	mappingInfo <- getNuIDMappingInfo(nuID, lib.mapping=lib.mapping)

	if (!('Refseq_old' %in% colnames(mappingInfo)) && !is.null(filterTh)) {
		if (!all(names(filterTh) %in% names(mappingInfo))) {
			cat('The provided names of filterTh does not match the field names of nuID_MappingInfo table.\n No filtering will be performed.\n')
		} else {
			filterNames <- names(filterTh)
			selInd <- TRUE
			for (i in seq(filterTh)) {
				selInd <- selInd & as.numeric(mappingInfo[, filterNames[i]]) > filterTh[i]
			}
			mappingInfo <- mappingInfo[selInd,]
		}
	}
	if (!returnAllInfo) {
		nuID <- rownames(mappingInfo)
		if ('Accession' %in% colnames(mappingInfo)) {
			mappingInfo <- mappingInfo[,'Accession']			
		} else if ('Refseq' %in% colnames(mappingInfo)) {
			mappingInfo <- mappingInfo[,'Refseq']			
		}
		names(mappingInfo) <- nuID
	}
	return(mappingInfo)	
}

`monoSmu` <-
function(x, y, newX=NULL, 
	nSupport=min(200, length(x)),   	# downsampled data points
	nKnots=6, rotate=FALSE, ifPlot=FALSE, xlab='x', ylab='y', ...
	) {
	
	x.old <- x
	y.old <- y
	
	if (rotate) {
		y <- y.old - x.old		## Equivalent to M
		x <- y.old + x.old 		## Equivalent to 2*A
	}
	
	# the order of (x,y) pairs does not matter
	# order of newX does matter
	ord <- order(x)
	x.ord <- x[ord]; y.ord <- y[ord]
	
	xy <- data.frame(x=x.ord, y=y.ord)
	result.sm <- supsmu(x=xy$x, y=xy$y, ...)
	
	if (rotate) {
		## Rotate back to the original direction
		x.temp <- (result.sm$x - result.sm$y)/2
		y.temp <- (result.sm$x + result.sm$y)/2
		xy1 <- data.frame(x=x.temp, y=y.temp)
	} else {
	    xy1 <- data.frame(x=result.sm$x, y=result.sm$y)
	}
	
	## down-sampling
	rangeX <- range(xy1$x)
	selInd <- round(seq(from=1, to = nrow(xy1), length =nSupport))
	## make sure there are enough samples at high end
	selX <- seq(rangeX[1] + diff(rangeX)/2, rangeX[2], length=10)
	selIndHigh <- sapply(selX, function(x) which.min(abs(xy1$x - x)))
	selInd <- sort(c(selInd, selIndHigh))
	xy1 <- xy1[selInd, ]
	
	if (is.null(newX))  newX <- x.old 
	
	newY <- monoSpline(x=xy1$x, y=xy1$y, newX=newX, nKnots=nKnots, ifPlot=FALSE)

	if (ifPlot) {
		plot(x.old, y.old, pch='.', xlab=xlab, ylab=ylab)
		points(xy1$x, xy1$y, col=2)
		ii <- order(newX)
		lines(newX[ii], newY[ii], col=3, type='l')
	}
	
	return(newY) 
}

##=================================================
## Define LumiBatch object:

setClass('LumiBatch', 
	representation(history='data.frame', controlData='data.frame', QC='list'), 
	prototype=list(history=data.frame(
		submitted   = I(vector()),
		finished    = I(vector()),
		command     = I(vector()),
		lumiVersion = I(vector())
	), controlData = data.frame(), QC = list()),
	contains='ExpressionSet')

#
setMethod('initialize', 'LumiBatch', function(.Object, 
	exprs = new('matrix'),
	se.exprs = new('matrix'),		# standard deviation of the bead measurements of each probe
	detection = new('matrix'),
	beadNum = new('matrix'),
    ...,
    assayData)
{
	if (missing(assayData)) {
		cmd <- 'assayData <- assayDataNew(exprs=exprs, se.exprs=se.exprs'
		nSample <- ncol(exprs)
		if (ncol(detection) == nSample) cmd <- paste(cmd, ', detection=detection')
		if (ncol(beadNum) == nSample) cmd <- paste(cmd, ', beadNum=beadNum')
		cmd <- paste(cmd, ')')
		eval(parse(text=cmd))
	}
	else if (!missing(exprs) || !missing(se.exprs))
		stop("only one of 'assayData' or ('exprs' and 'se.exprs') allowed")
	callNextMethod(.Object, assayData=assayData, ...)
})


setValidity("LumiBatch", function(object) 
{
    msg <- Biobase:::validMsg(NULL, Biobase:::isValidVersion(object, "ExpressionSet"))
    msg <- Biobase:::validMsg(msg, assayDataValidMembers(assayData(object), c("exprs")))
    msg <- Biobase:::validMsg(msg, assayDataValidMembers(assayData(object), c("se.exprs")))
		## validate controlData slot, added by Mark Cowley
		msg <- Biobase:::validMsg(msg, valid_controlData(object))
    if (is.null(msg)) TRUE else msg
})


## validate controlData slot, added based on the code provided by Mark Cowley
valid_controlData <- function(object) {
	is(object, "LumiBatch") || return("object must be a LumiBatch")
	if ( nrow(controlData(object)) == 0 ) {
		return(TRUE)
	} else if ( ncol(controlData(object)) == (ncol(object) + 2) ) {
	 	if ( !identical(colnames(controlData(object))[1:2], c("controlType", "ProbeID")) ) {
		  cat("The first 2 extra columns in the controlData should be 'controlType' and 'ProbeID'.\n")
			return(FALSE)
		} 
		sampleName.ctrl <- colnames(controlData(object))[-c(1,2)]
	} else {
		sampleName.ctrl <- colnames(controlData(object))
	}

	if (!identical(sampleName.ctrl, sampleNames(object))) {
		cat("The sample names in the controlData don't match sampleNames(object).\n")
		return(FALSE)
	}
	
	return(TRUE)
}


##=================================================
## methods

setMethod("se.exprs", signature(object="ExpressionSet"), function(object) {
	if ('se.exprs' %in% assayDataElementNames(object)) {
		return(assayDataElement(object,"se.exprs"))
	} else {
		return(NULL)
	}
})

setReplaceMethod("se.exprs", signature(object="ExpressionSet"), function(object, value) {
		if (is.null(value)) {
			assay <- assayData(object)
			if (exists('se.exprs', envir=assay)) {
				oldMode <- storageMode(assay)
				storageMode(assay) <- 'environment'
				rm(se.exprs, envir=assay)
				storageMode(assay) <- oldMode
				assayData(object) <- assay
			}
			return(object)
		} else {
			assayDataElementReplace(object, "se.exprs", value)
		}
	})


setMethod("beadNum", signature(object="ExpressionSet"), function(object) {
	if ('beadNum' %in% assayDataElementNames(object)) {
		return(assayDataElement(object,"beadNum"))
	} else {
		return(NULL)
	}
})

setReplaceMethod("beadNum", signature(object="ExpressionSet"), function(object, value) {
		if (is.null(value)) {
			assay <- assayData(object)
			if (exists('beadNum', envir=assay)) {
				oldMode <- storageMode(assay)
				storageMode(assay) <- 'environment'
				rm(beadNum, envir=assay)
				storageMode(assay) <- oldMode
				assayData(object) <- assay
			}
			return(object)
		} else {
			assayDataElementReplace(object, "beadNum", value)
		}
	})

setMethod("detection", signature(object="ExpressionSet"), function(object) {
	if ('detection' %in% assayDataElementNames(object)) {
		return(assayDataElement(object,"detection"))
	} else {
		return(NULL)
	}
})

setReplaceMethod("detection", signature(object="ExpressionSet"), function(object, value) {
	if (is.null(value)) {
		assay <- assayData(object)
		if (exists('detection', envir=assay)) {
			oldMode <- storageMode(assay)
			storageMode(assay) <- 'environment'
			rm(detection, envir=assay)
			storageMode(assay) <- oldMode
			assayData(object) <- assay
		}
		return(object)
	} else {
		assayDataElementReplace(object, "detection", value)
	}
})	

setMethod("controlData", signature(object="LumiBatch"), function(object) {
	object@controlData
})


## added based on the code provided by Mark Cowley
setReplaceMethod("controlData", signature(object="LumiBatch"), function(object, value) {
	
	if (is.null(value)) value <- data.frame()
	
	if (is.character(value) && length(value) == 1) {
		if ( file.exists(value) ) {
			object <- addControlData2lumi(value, object)
			validObject(object)
		}
	} else if (is(value, 'data.frame')) {
		object@controlData <- value
		validObject(object)
	} else {
		stop("value must be one of: NULL, data.frame, or character(1) as the path to a control data file")
	}
	
	return(object)
})	


setMethod("getHistory",signature(object="LumiBatch"), function(object) object@history)


setReplaceMethod("sampleNames", signature(object="LumiBatch",value="ANY"),
	function(object, value) {

	 	object <- callNextMethod()
		ddim <- dim(object)
        
	 	## subsetting the QC information
	 	if (!is.null(object@QC)) {
	 		QC <- object@QC
	 		if (!is.null(QC$sampleSummary))
	 			if (ncol(QC$sampleSummary) == ddim[2])
	 				colnames(QC$sampleSummary) <- value
	 		if (!is.null(QC$BeadStudioSummary))
	 			if (nrow(QC$BeadStudioSummary) == ddim[2])
	 				rownames(QC$BeadStudioSummary) <- value
	 		object@QC <- QC
	 	}
	 	if (!is.null(attr(object, 'vstParameter'))) {
	 		vstParameter <- attr(object, 'vstParameter')
	 		if (!is.null(nrow(vstParameter))) {
	 			if (nrow(vstParameter) == ddim[2]) {
					rownames(vstParameter) <- value
					transformFun <- attr(object, 'transformFun')
					names(transformFun) <- value
	 				attr(object, 'vstParameter') <- vstParameter
	 				attr(object, 'transformFun') <- transformFun
	 			}
	 		}
	 	}
        
	 	## controlData information
	 	if (nrow(object@controlData) > 0) {
			if (ncol(object@controlData) == ddim[2]) {
				colnames(object@controlData) <- value
			} else if (ncol(object@controlData) == ddim[2] + 2) {
				colnames(controlData(object)) <- c("controlType", "ProbeID", value)
			} 				
	 	}
	 	    
	 	validObject(object)
	  return(object)
	})


setMethod("summary",signature(object="LumiBatch"), function(object, type=c('data', 'QC')) 
{
	type <- match.arg(type)
	if (type == 'data') {
		show(object)		
	} else {
		if (is.null(object@QC$sampleSummary)) {
			cat('Run Quality Control estimation ...\n')
			object <- lumiQ(object)
		}
		if (!is.null(object@QC$sampleSummary)) {
			QC <- object@QC
			dimen <- dim(exprs(object))
			cat(paste("Data dimension: ", paste(dimen[1], 'genes', 'x', dimen[2], 'samples', collapse=" "), '\n'))
			sampleSummary <- QC$sampleSummary
			cat(paste("\nSummary of Samples:\n", sep=''))
			print(sampleSummary, quote=FALSE)

			cat('\nMajor Operation History:\n')
			print(QC$history, quote=FALSE) 
		}
	}
})


setMethod("show",signature(object="LumiBatch"), function(object) 
{
	cat('Summary of data information:\n')
	note <- notes(object)
	n.names <- names(note)
	for (i in seq(note)) {
		if (!is.null(n.names)) cat('\t', paste(n.names[i], ':\n\t\t', sep=''))
		cat(note[[i]], sep='\n\t\t')		
	}
	cat('\nMajor Operation History:\n')
	hh <- getHistory(object)
	if (nrow(hh) > 5) {
		print(head(hh, 2))
		cat("...\n")
		print(tail(hh, 2))
	} else {
		print(hh)
	}	
	cat('\nObject Information:\n')
	callNextMethod()
	if (nrow(controlData(object))>0) {
		cat('Control Data: Available\n')
	} else {
		cat('Control Data: N/A\n')
	}
	cat("QC information: Please run summary(x, 'QC') for details!\n")
})


setMethod("[", "LumiBatch", function(x, i, j, ..., drop = FALSE) 
{
	if (missing(drop)) drop <- FALSE
   	history.submitted <- as.character(Sys.time())
	ddim <- dim(x)
	
	sampleName <- sampleNames(x)
	## convert the names as index to avoid some potential problems of name inconsistency
	if (!missing(j)) {
		if (is.character(j)) {
			ind <- seq(sampleName)
			names(ind) <- sampleName
			j <- ind[j]
		}	
	}
	## do default processing of 'ExpressionSet'
	x <- callNextMethod()

	if (!missing(i) && !missing(j)) {
		history.command <- paste('Subsetting', ddim[1], 'features and', ddim[2], 'samples.')		
	} else if (!missing(i)) {
		history.command <- paste('Subsetting', ddim[1], 'features.')
	} else if (!missing(j)) {
		history.command <- paste('Subsetting', ddim[2], 'samples.')
	} else {
		return(x)
	}

	## subsetting the QC information
	if (!missing(j)) {
		if (!is.null(x@QC)) {
			QC <- x@QC
			if (!is.null(QC$sampleSummary)) {
				sampleSummary <- QC$sampleSummary
				if (ncol(sampleSummary) == ddim[2]) {
					QC$sampleSummary <- sampleSummary[,j,drop=FALSE]
				} else if (ncol(sampleSummary) > 0) {
					if (all(sampleName %in% colnames(sampleSummary))) {
						QC$sampleSummary <- sampleSummary[,sampleName[j],drop=FALSE]
					} else {
						warning('The sampleSummary in QC slot does not match the sampleNames.\nThe subsetting did not execute on sampleSummary.\n')
					}
				}
			}
			if (!is.null(QC$BeadStudioSummary)) {
				BeadStudioSummary <- QC$BeadStudioSummary
				if (nrow(BeadStudioSummary) == ddim[2]) {
					QC$BeadStudioSummary <- BeadStudioSummary[j,,drop=FALSE]
				} else if (nrow(BeadStudioSummary) > 0) {
					if (all(sampleName %in% rownames(BeadStudioSummary))) {
						QC$BeadStudioSummary <- BeadStudioSummary[sampleName[j],,drop=FALSE]
					} else {
						warning('The BeadStudioSummary in QC slot does not match the sampleNames.\nThe subsetting did not execute on BeadStudioSummary.\n')
					}
				}
			}
			x@QC <- QC
		}
		if (!is.null(attr(x, 'vstParameter'))) {
			vstParameter <- attr(x, 'vstParameter')
			if (!is.null(nrow(vstParameter))) {
				if (nrow(vstParameter) == ddim[2]) {
					attr(x, 'vstParameter') <- attr(x, 'vstParameter')[j,,drop=FALSE]
					attr(x, 'transformFun') <- attr(x, 'transformFun')[j]
				}
			}
		}

		## controlData information
		if (nrow(controlData(x)) > 0) {
			if (is.logical(j) || is.numeric(j)) j <- sampleName[j]
			if (all(j %in% colnames(controlData(x)))) {
				if (all(c("controlType", "ProbeID") %in% colnames(controlData(x)))) {
				  j <- c("controlType", "ProbeID", j)
				}
				controlData(x) <- controlData(x)[,j, drop=FALSE]
			} else {
				warning('The controlData slot does not match the sampleNames.\nThe subsetting did not execute on controlData.\n')
			}
		}
	}

  # history tracking
  history.finished <- as.character(Sys.time())
	if (is.null(x@history$lumiVersion)) x@history$lumiVersion <- rep(NA, nrow(x@history))
	lumiVersion <- packageDescription('lumi')$Version
	x@history<- rbind(x@history, data.frame(submitted=history.submitted, finished=history.finished, 
			command=history.command, lumiVersion=lumiVersion))

  validObject(x)
	return(x)
})


setMethod("combine", signature=c(x="LumiBatch", y="ExpressionSet"), function(x, y, ...) 
{
	if (missing(y)) return(x)
	warning('The Lumibatch object was forced as ExpressionSet.')
	x <- as(x, 'ExpressionSet')
	if (length(list(...)) > 0) 
		return(combine(x, combine(y, ...)))
	return(combine(x, y))
})

setMethod("combine", signature=c(x="ExpressionSet", y="LumiBatch"), function(x, y, ...) 
{
	if (missing(y)) return(x)
	warning('The LumiBatch object was forced as ExpressionSet.')
	y <- as(y, 'ExpressionSet')
	if (length(list(...)) > 0) 
		return(combine(x, combine(y, ...)))
	return(combine(x, y))
})

setMethod("combine", signature=c(x="LumiBatch", y="LumiBatch"), function(x, y, ...) 
{
	if (missing(y)) return(x)
	if (length(list(...)) > 0) 
		return(combine(x, combine(y, ...)))
	if (class(x) != class(y))
		stop(paste("objects must be the same class, but are ",
                 class(x), ", ", class(y), sep=""))
	
	if (any(sort(featureNames(x)) != sort(featureNames(y)))) stop('Two data sets have different row names.')
	## determine whether there are duplicated sample names
	sampleName.x <- sampleNames(x)
	sampleName.y <- sampleNames(y)
	if (any(sampleName.x %in% sampleName.y)) {
		warning('Two data sets have some duplicated sample names.\n "_2" were attached to the duplicated sample names.')
		sampleName.x <- sampleNames(x)   # paste(sampleNames(x), '_1', sep='')
		sampleName.y <- paste(sampleNames(y), '_2', sep='')
		sampleNames(x) <- sampleName.x
		sampleNames(y) <- sampleName.y
	}
	featureName.x <- featureNames(x)
	featureName.y <- featureNames(y)
	featureName.com <- featureName.x[featureName.x %in% featureName.y]
	if (length(featureName.com) < length(featureName.x) | length(featureName.com) != length(featureName.y)) {
		if (length(featureName.com) > 0) {
			warning('Two data sets have different featureNames, only the common ones were used.')			
		} else {
			stop('Two data sets have totally different featureNames.')
		}
	}
	## make sure two data sets have the sample order of features
	x <- x[featureName.com,]
	y <- y[featureName.com,]
	history.submitted <- as.character(Sys.time())
	dimm.x <- dim(x); dimm.y <- dim(y)
	assayData(x) <- combine(assayData(x), assayData(y))
	experimentData(x) <- combine(experimentData(x),experimentData(y))

	## combine pheno data
	phenoData(x) <- combine(phenoData(x),phenoData(y))
	## combine protocolData 
	protocolData(x) <- combine(protocolData(x),protocolData(y))
	
	## combining the QC information
	if (length(x@QC) > 0 && length(y@QC) > 0) {
		if (all(!is.null(x@QC$BeadStudioSummary) & !is.null(y@QC$BeadStudioSummary))) {
			if (all(ncol(x@QC$BeadStudioSummary) == ncol(y@QC$BeadStudioSummary) & ncol(x@QC$BeadStudioSummary) > 0)) {
				BeadStudioSummary <- rbind(x@QC$BeadStudioSummary, y@QC$BeadStudioSummary)
				x@QC$BeadStudioSummary <- BeadStudioSummary
			} else {
				x@QC <- list()
			}
		} else {
			x@QC <- list()
		}
		if (all(!is.null(x@QC$sampleSummary) & !is.null(y@QC$sampleSummary))) {
			if (nrow(x@QC$sampleSummary) == nrow(y@QC$sampleSummary)) {
				sampleSummary <- cbind(x@QC$sampleSummary, y@QC$sampleSummary)
				x@QC$sampleSummary <- sampleSummary
			} else {
				x@QC <- list()
			}
		} else {
			x@QC <- list()
		}
		if (!is.null(x@QC)) {
			history.x <- x@QC$history
			if (is.null(history.x)) history.x <- data.frame(submitted=NA, finished=NA, command=NA, lumiVersion=NA)
			if (is.null(history.x$lumiVersion)) history.x$lumiVersion <- rep(NA, nrow(history.x))
			history.y <- y@QC$history
			if (is.null(history.y)) history.y <- data.frame(submitted=NA, finished=NA, command=NA, lumiVersion=NA)
			if (is.null(history.y$lumiVersion)) history.y$lumiVersion <- rep(NA, nrow(history.y))
			x@QC$history <- rbind(history.x, history.y)
		} 
	} else {
		x@QC <- list()
	}
	
	## VST transformation parameters
	if (all(!is.null(attr(x, 'vstParameter')) & !is.null(attr(y, 'vstParameter')))) {
		vstParameter.x <- attr(x, 'vstParameter')
		vstParameter.y <- attr(y, 'vstParameter')
		if (is.null(nrow(vstParameter.x))) {
			vstParameter.x <- matrix(vstParameter.x, nrow=1)
		}
		if (is.null(nrow(vstParameter.y))) {
			vstParameter.y <- matrix(vstParameter.y, nrow=1)
		}
		if (any(nrow(vstParameter.x) != dimm.x[2] | nrow(vstParameter.y) != dimm.y[2])) {
			attr(x, 'vstParameter') <- attr(x, 'transformFun') <- NULL
		} else {
			attr(x, 'vstParameter') <- rbind(attr(x, 'vstParameter'), attr(y, 'vstParameter'))
			attr(x, 'transformFun') <- c(attr(x, 'transformFun'), attr(y, 'transformFun'))
		}
	}
	
	## controlData information
	if (nrow(controlData(x)) > 0) {
		if (nrow(controlData(x)) == nrow(controlData(y))) {
			if (all(c('controlType', 'ProbeID') %in% colnames(controlData(x))) &
				  all(c('controlType', 'ProbeID') %in% colnames(controlData(y)))) {
					if (all(controlData(x)$controlType == controlData(y)$controlType) &
				      all(controlData(x)$ProbeID == controlData(y)$ProbeID)) {
								controlData <- cbind(controlData(x), controlData(y)[,-c(1,2)])
								controlData(x) <- as.data.frame(controlData)
					} else {
						warning("controlData slot not combined: controlType and ProbeID do not match.")
					}
			} else {
				warning("controlData slot not combined: controlType and ProbeID are required.")
			}
		} else {
    	warning("controlData slot not combined: different numbers of rows found.") 
    }
	}

	# history tracking
	history.finished <- as.character(Sys.time())
	# history.command <- capture.output(print(match.call(combine)))  
	history.command <- paste(deparse(match.call(combine)), collapse='') 

	x@history<- rbind(x@history, y@history)
	if (is.null(x@history$lumiVersion)) x@history$lumiVersion <- rep(NA, nrow(x@history))
	lumiVersion <- packageDescription('lumi')$Version
    x@history<- rbind(x@history, 
	       data.frame(submitted=history.submitted, finished=history.finished, command=history.command, lumiVersion=lumiVersion))
	return(x)
})


##some special handling of main is needed
setMethod("boxplot",signature(x="ExpressionSet"),
	function(x, range=0, main, logMode=TRUE, subset=NULL, xlab='', ylab='Amplitude', ...) 
{
  	tmp <- description(x)
  	if (missing(main) && (is(tmp, "MIAME")))
     	main <- tmp@title
	
	if (class(x) == 'MethyLumiM') logMode <- FALSE
	expr <- exprs(x)
  	if (logMode && max(exprs(x), na.rm=TRUE) > 50) {
		## force the expression value as positive in the logMode
		# if (min(expr, na.rm=TRUE) < 0) expr <- expr - min(expr, na.rm=TRUE) + 1
		# remove the negative values
		if (min(expr, na.rm=TRUE) < 0) {
			rMin <- rowMin(expr)
			expr <- expr[rMin > 0, , drop=FALSE]
		}
		expr <- log2(expr)
	} 
	if (!is.null(subset)) {
		if (!is.numeric(subset)) stop('subset should be numeric.')
		if (length(subset) == 1) {
			index <- sample(1:nrow(expr), min(subset, nrow(expr)))
		} else {
			index <- subset
			index <- index[index > 0 & index <= nrow(expr)]
		}
	} else {
		index <- 1:nrow(expr)
	}

	dataMatrix <- expr[index,]
	labels <- colnames(dataMatrix)
	if (is.null(labels)) labels <- as.character(1:ncol(dataMatrix))
	## set the margin of the plot
	mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
	old.mar <- par('mar')
	old.xaxt <- par('xaxt')
	par(xaxt='n')
	par(mar=mar)
	boxplot(dataMatrix ~ col(dataMatrix), main=main, range=range, xlab=xlab, ylab=ylab, ...)
	par(xaxt='s')
	axis(1, at=1:ncol(dataMatrix), labels=labels, tick=TRUE, las=2)
	par(mar=old.mar)
	par(xaxt=old.xaxt)
})


setMethod('hist', signature(x='ExpressionSet'), 
	function(x, ...) 
{
	density(x, ...)
})


setMethod('density', signature(x='ExpressionSet'), 
	function(x, logMode=TRUE, xlab = NULL, ylab = "density", type = "l", col=1:dim(x)[2], lty=1:dim(x)[2], 
	lwd=1, xlim=NULL, index.highlight=NULL, color.highlight=2, symmetry=NULL, addLegend=TRUE, legendPos="topright", subset=NULL, main='',...) 
{
	if (is(x, 'ExpressionSet')) {
	    expr <- exprs(x)
	} else {
		stop('Un-supported class of x.')
	}

	if (class(x) == 'MethyLumiM') logMode <- FALSE
		
    if (logMode && (max(expr, na.rm=TRUE) > 50)) {
		## force the expression value as positive in the logMode
		# if (min(expr, na.rm=TRUE) < 0) expr <- expr - min(expr, na.rm=TRUE) + 1
		# remove the negative values
		if (min(expr, na.rm=TRUE) < 0) {
			rMin <- rowMin(expr)
			expr <- expr[rMin > 0, , drop=FALSE]
		}
		expr <- log2(expr)
		if (is.null(xlab)) 
			xlab <- "log2 intensity"
    } else if (is.null(xlab)) 
        xlab <- "intensity"

	if (!is.null(subset)) {
		if (!is.numeric(subset)) stop('subset should be numeric.')
		if (length(subset) == 1) {
			index <- sample(1:nrow(expr), min(subset, nrow(expr)))
		} else {
			index <- subset
			index <- index[index > 0 & index <= nrow(expr)]
		}
		expr <- expr[index,,drop=FALSE]
	} 

	if (!is.null(symmetry)) {
		x.range <- range(expr)
		if (symmetry > x.range[1] && symmetry < x.range[2]) {
			warning('symmetry point should not be within the range of x.')
			symmetry <- NULL
		} else {
			expr <- rbind(expr, 2*symmetry - expr)
		}
	}
	x.density <- apply(expr, 2, density, ...)
    all.x <- do.call(cbind, lapply(x.density, function(x) x$x))
    all.y <- do.call(cbind, lapply(x.density, function(x) x$y))

	if (!is.null(symmetry)) {
		nr <- nrow(all.x)
		if (all.x[1,1] >= x.range[1] && all.x[1,1] <= x.range[2]) {
			all.x <- all.x[1:round(nr/2),]
			all.y <- all.y[1:round(nr/2),]
		} else {
			all.x <- all.x[round(nr/2):nr,]
			all.y <- all.y[round(nr/2):nr,]
		}
	}
	# matplot(all.x, all.y, ylab=ylab, xlab=xlab, type=type, col=col, lty=lty, lwd=lwd, ...)
	matplot(all.x, all.y, ylab=ylab, xlab=xlab, type=type, col=col, lty=lty, lwd=lwd, xlim=xlim, main=main)
	if (!is.null(index.highlight)) {
		if (index.highlight > ncol(all.x) || index.highlight < 1) {
			warning('Highlight index out of range.')
			index.highlight <- 1
		}
		lines(all.x[,index.highlight], all.y[,index.highlight], col=color.highlight, lwd=2, lty=1)
	}
	## add legend
	if (addLegend) {
		labels <- colnames(expr)
		if (is.null(labels)) labels <- as.character(1:ncol(expr))

		col <- col
		lwd <- lwd
		lty <- lty
		if (!is.null(index.highlight)) {
			col[index.highlight] <- color.highlight
			lwd[index.highlight] <- 2
			lty[index.highlight] <- 1
		}
		if (length(legendPos) > 1) {
			x.pos <- legendPos[1]
			y.pos <- legendPos[2]
		} else {
			x.pos <- legendPos[1]
			y.pos <- NULL
		}
		legend(x.pos, y.pos, legend=labels, col=col, lwd=lwd, lty=lty, box.col = par("bg"))
	}
	
    invisible(list(all.x = all.x, all.y = all.y))
})


setMethod("pairs", signature(x="ExpressionSet"), 
	function(x, ..., smoothScatter=FALSE, logMode=TRUE, subset=5000, fold=2, dotColor=1, highlight=NULL, highlightColor=2, main=NULL, checkTransform=TRUE) 
{
	upperPanel <- function(x, y) {
		if (smoothScatter) {
			par(new=TRUE)
			smoothScatter(x[subset], y[subset], main='')
		} else {
			if (length(dotColor) == length(x)) {
				points(x[subset], y[subset], col=dotColor[subset], pch='.', cex=3)
			} else {
				points(x[subset], y[subset], col=dotColor, pch='.', cex=3)
			}
			if (!is.null(highlight)) {
				highlight <- intersect(subset, highlight)
				points(x[highlight], y[highlight], col=highlightColor, pch='.', cex=3)
			}
		}
		abline(0, 1, col="red", lty=1)
		if (!is.null(fold) && !is.na(fold)) {
  		if (logMode) {
  			if (checkTransform) {
  				abline(log2(fold), 1, col="blue", lty=2)
  				abline(log2(1/fold), 1, col="blue", lty=2)
  			} else {
  				abline(fold, 1, col="blue", lty=2)
  				abline(-fold, 1, col="blue", lty=2)
  			}
  		} else {
  			abline(fold, 1, col="blue", lty=2)
  			abline(-fold, 1, col="blue", lty=2)
  		}
		}
	}

	lowerPanel <- function(x, y, cex=1.44) {
		txt <- paste("Cor =", as.character(round(cor(x,y),3)),"\n")
		ex <- par("fin")[1]*0.9
	  if (!is.null(fold) && !is.na(fold)) {
  		if (logMode) {
  			if (checkTransform) {
  				up <- length(which((x-y) > log2(fold)))
  				down <- length(which((y-x) > log2(fold)))
  			} else {
  				up <- length(which((x-y) > fold))
  				down <- length(which((y-x) > fold))
  			}
  		} else {
  			up <- length(which((x/y) > fold))
  			down <- length(which((y/x) > fold))
  		}
  		txt <- paste(txt, up, " (> ", fold, ", up)\n", sep="")
  		txt <- paste(txt, down, " (> ", fold, ", down)\n", sep="")
	  }
		text(mean(range(x)), mean(range(y)), labels=txt, cex=ex)
	}

	## put histograms on the diagonal
	diagPanel <- function(x, ...) {
	    usr <- par("usr"); on.exit(par(usr))
	    par(usr = c(usr[1:2], 0, 1.5) )
	    h <- hist(x, plot = FALSE)
	    breaks <- h$breaks; nB <- length(breaks)
	    y <- h$counts; y <- y/max(y)
	    rect(breaks[-nB], 0, breaks[-1], y, col="cyan")
	}

	if (smoothScatter) subset <- NULL
	if (is.character(highlight)) {
		# convert it as numeric index
		index <- 1:nrow(x)
		names(index) <- featureNames(x)
		highlight <- index[highlight]
	} 

	expr <- exprs(x)
	if (class(x) == 'MethyLumiM') logMode <- FALSE
	if (checkTransform) {
		if(logMode) {
			if (max(expr, na.rm=TRUE) > 50) {
				## force the expression value as positive in the logMode
				# if (min(expr, na.rm=TRUE) < 0) expr <- expr - min(expr, na.rm=TRUE) + 1
				# remove the negative values
				if (min(expr, na.rm=TRUE) < 0) {
					rMin <- rowMin(expr)
					expr <- expr[rMin > 0, , drop=FALSE]
				}
				expr <- log2(expr)
			}
		} else if (class(x) != 'MethyLumiM') {
			if (max(expr, na.rm=TRUE) < 50) {
				expr <- 2^expr
			}
		}
	}


	if (!is.null(subset)) {
		if (!is.numeric(subset)) stop('subset should be numeric.')
		if (length(subset) == 1) {
			subset <- sample(1:nrow(expr), min(subset, nrow(expr)))
		} else {
			subset <- subset[subset > 0 & subset <= nrow(expr)]
		}
		if (is.null(main)) 
			main <- paste('Pair plot based on', length(subset), 'random sampling.')
	} else {
		subset <- 1:nrow(expr)
	}
	
	
	pairs(expr,upper.panel=upperPanel, diag.panel=diagPanel, 
			lower.panel=lowerPanel, main=main, ...)
})
  	

setMethod("MAplot", signature(object="ExpressionSet"), 
	function(object, ..., smoothScatter=FALSE, logMode=TRUE, subset=5000, main=NULL) 
{
	if (smoothScatter) subset <- NULL
	expr <- exprs(object)
	if (class(object) == 'MethyLumiM') logMode <- FALSE
	if(logMode) {
		if (max(expr, na.rm=TRUE) > 50) {
			## force the expression value as positive in the logMode
			# if (min(expr, na.rm=TRUE) < 0) expr <- expr - min(expr, na.rm=TRUE) + 1
			# remove the negative values
			if (min(expr, na.rm=TRUE) < 0) {
				rMin <- rowMin(expr)
				expr <- expr[rMin > 0, ,drop=FALSE]
			}
			expr <- log2(expr)
		} 
	} else {
		if (max(expr, na.rm=TRUE) < 50) {
			expr <- 2^expr
		}
	}
	if (!is.null(subset)) {
		if (!is.numeric(subset)) stop('subset should be numeric.')
		if (length(subset) == 1) {
			index <- sample(1:nrow(expr), min(subset, nrow(expr)))
		} else {
			index <- subset
			index <- index[index > 0 & index <= nrow(expr)]
		}
	} else {
		index <- 1:nrow(expr)
	}
	if (length(subset) < nrow(expr) && is.null(main)) 
		main <- paste('Pair plot based on', length(subset), 'random sampling.')

	if (smoothScatter) {
		mva.pairs(expr[index, ], log.it=FALSE, plot.method='smoothScatter', main=main, ...)
	} else {
		mva.pairs(expr[index, ], log.it=FALSE, plot.method='normal', main=main, ...)
	}
})


setMethod('plot',
	signature('ExpressionSet', 'missing'),
	function(x, what=c('density', 'boxplot', 'pair', 'MAplot', 'sampleRelation', 'outlier', 'cv'), main, ...)
{
	object <- x
	if (!is(object, 'ExpressionSet')) stop('The object should be class "ExpressionSet".')
	what <- match.arg(what)

	if (what == 'density') {
		if (missing(main)) main <- 'Density plot of intensity'
		density(object, xlab="intensity", ylab="density", main=main, ...)
	} else if (what == 'boxplot') {
		if (missing(main)) main <- 'Boxplot of microarray intensity'
		boxplot(object, xlab='microarrays', ylab='intensity', main=main, ...)
	} else if (what == 'cv') {
		if (missing(main)) main <- 'Density plot of coefficient of variance'
		estimateLumiCV(object, ifPlot=TRUE, main=main, ...)
	} else if (what == 'sampleRelation') {
		plotSampleRelation(object, ...)
	} else if (what == 'pair') {
		if (missing(main)) main <- 'Pairwise plot with sample correlation'
		pairs(object, main=main, ...)
	} else if (what == 'MAplot') {
		if (missing(main)) main <- 'Pairwise MA plots between samples'
		MAplot(object, main=main, ...)
	} else if (what == 'outlier') {
		detectOutlier(object, ifPlot=TRUE, ...)
	} else {
		print('Unsupported .')
	}
	return(invisible(TRUE))	
})


## Example
# library(lumi)
# data(example.lumi)
# test <- asBigMatrix(example.lumi, nCol=6)
setMethod('asBigMatrix',
	signature(object='ExpressionSet'),
	function(object, rowInd=NULL, colInd=NULL, nCol=NULL, dimNames=NULL, saveDir='.', savePrefix=NULL, ...)
{
	## check whether the user just wants a physical copy of the data to a new location
	if (grepl('windows', R.Version()$platform, ignore.case=TRUE)) stop("Package bigmemoryExtras does not support Windows systems.")
	
	if (saveDir == '.') saveDir <- getwd()
	if (is.null(savePrefix)) {
		## use the variable name as the bigmatrix directory prefix
		savePrefix <- match.call(asBigMatrix)[['object']]  
	} 
	if (!file.exists(saveDir)) stop('saveDir does not exist!')
	saveDir <- file.path(saveDir, paste(savePrefix, 'bigmat', sep='_'))
	
	if (is.null(rowInd) && is.null(colInd) && is.null(nCol) && is.null(dimNames) && is(exprs(object), 'BigMatrix')) {
	  fieldnames <- ls(assayData(object)[[assayDataElementNames(object)[1]]])
	  if ('datapath' %in% fieldnames) {
  		oldDir <- dirname(assayData(object)[[assayDataElementNames(object)[1]]]$datapath)
	  } else if ('backingfile' %in% fieldnames) {
  		oldDir <- dirname(assayData(object)[[assayDataElementNames(object)[1]]]$backingfile)
	  }

		if (oldDir != saveDir) {
			if (!file.exists(saveDir)) 	dir.create(saveDir,showWarnings=FALSE)
			sapply(dir(oldDir, full.names=T), file.copy, to=saveDir, overwrite=TRUE, recursive=TRUE)  
		}
		object <- bigmemoryExtras::updateAssayDataElementPaths(object, saveDir)
		return(object)		
	}
	
	if (!is.null(dimNames)) {
		if (!is.list(dimNames) || length(dimNames) != 2) stop("dimNames should be a list with length 2!")
	}
	nRow <- nrow(object)
	if (is.null(nCol)) {
		if (is.null(dimNames)) {
			nCol <- ncol(object)
		} else {
			nCol <- length(dimNames[[2]])
			if (length(nCol) < ncol(object)) stop('The length of dimNames[[2]] should not be less than the columns of the input object!')
		}
	} 
	if (is.null(dimNames)) {
		dimNames <- list(featureNames(object), sampleNames(object))
		## append colnames if nCol is longer than dimNames[[2]]
		if (length(dimNames[[2]]) < nCol) {
			appLen <- nCol - length(dimNames[[2]])
			dimNames[[2]] <- c(dimNames[[2]], paste(rep('unknown', appLen), seq(appLen), sep='.'))
		}
	} else if (length(dimNames[[1]]) < nRow) {
		stop('The length of dimNames[[1]] should be the same as the rows of the input object!')
	}
	subsetMode <- FALSE
	extensionMode <- FALSE 
	if (is.null(rowInd)) {
		rowInd <- 1:nRow
	} else {
		nRow <- length(rowInd)
		dimNames[[1]] <- dimNames[[1]][rowInd]
		subsetMode <- TRUE
	}
	if (is.null(colInd)) {
		colInd <- 1:nCol
	} else {
		nCol <- length(colInd)
		dimNames[[2]] <- dimNames[[2]][colInd]
		subsetMode <- TRUE
	}
	if (nCol != ncol(object)) extensionMode <- TRUE
	
	for (ad.name in assayDataElementNames(object)) {
  	matrix.i <- assayDataElement(object, ad.name)
		if (is.null(matrix.i)) next
		backingfile <- file.path(saveDir, ad.name)
		
		if (!is(assayDataElement(object, ad.name), "BigMatrix") && !extensionMode) {
			x.mat <- bigmemoryExtras::BigMatrix(matrix.i[dimNames[[1]], dimNames[[2]]], backingfile=backingfile, nrow=nRow, ncol=nCol, dimnames=dimNames, ...)
		} else {
			x.mat <- bigmemoryExtras::BigMatrix(backingfile=backingfile, nrow=nRow, ncol=nCol, dimnames=dimNames, ...)
			for (i in 1:ncol(matrix.i)) {
				col.i <- colnames(matrix.i)[i]
				x.mat[dimNames[[1]], col.i] <- matrix.i[dimNames[[1]], col.i]
			}
		}
		assayDataElement(object, ad.name) <- x.mat
  }
	object <- bigmemoryExtras::updateAssayDataElementPaths(object, saveDir)

	if (extensionMode || subsetMode) {
		appLen <- nCol - nrow(pData(object))
		if (length(appLen) > 0) {
			pdata <- rbind(as.matrix(pData(object)), matrix(NA, nrow=appLen, ncol=ncol(pData(object))))
			pdata <- as.data.frame(pdata)
			rownames(pdata) <- dimNames[[2]]
		} else {
			pdata <- pData(object)
			if (length(colInd) < nrow(pdata)) pdata <- pdata[colInd,]
		}
		className <- class(object)
		object.new <- new(className, assayData=assayData(object), phenoData=AnnotatedDataFrame(pdata), 
				featureData=AnnotatedDataFrame(fData(object)[rowInd,,drop=FALSE]))
		object <- object.new
	}
	return(object)
})

`detectOutlier` <-
function(x, metric="euclidean", standardize=TRUE, Th=2, ifPlot=FALSE) {
	
	if (is(x, 'ExpressionSet')) {
	    profile <- exprs(x)		
	} else if (is.numeric(x)) {
		profile <- as.matrix(x)
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}
	
	center <- rowMeans(profile)
	profile1 <- cbind(center, profile)
	colnames(profile1) <- c('Center', colnames(profile))

	if (metric == 'cor') {
		d <- cor(profile1)
		mad.d <- median(1 - d[2:nrow(d),1])
	} else {
		if (standardize) {
			profile1 <- scale(profile1)
		}
		d <- as.matrix(dist(t(profile1), method=metric))
		mad.d <- median(d[2:nrow(d),1])
	}
	
	## remove top 10% samples to re-estimate the Center
	len <- ncol(x)
	excludeInd <- which(rank(d[2:nrow(d),1]) > len * 0.9)
	center <- rowMeans(profile[, -excludeInd])
	profile1 <- cbind(center, profile)
	colnames(profile1) <- c('Center', colnames(profile))
	
	## perform the estimation based on the new center again
	if (metric == 'cor') {
		d <- cor(profile1)
		mad.d <- median(1 - d[2:nrow(d),1])
	} else {
		if (standardize) {
			profile1 <- scale(profile1)
		}
		d <- as.matrix(dist(t(profile1), method=metric))
		mad.d <- median(d[2:nrow(d),1])
	}
	
	Th <- Th * mad.d
	outlier <- (d[2:nrow(d),1] >= Th)
	attr(outlier, 'sampleDistance') <- d
	attr(outlier, 'threshold') <- Th
	main <- paste('Outlier detection based on sample distance to "Center"')

	if (ifPlot) {
		hc = hclust(as.dist(d), 'ave')
		plot(hc, xlab='Sample', ylab='Distance', main=main)
		abline(h=Th, col=2, lty=2)
		return(invisible(TRUE))	
	} else {
		return(outlier)
	}
}

`addNuID2lumi` <-
function(x.lumi, annotationFile=NULL, sep=NULL, lib.mapping=NULL, annotationColName=c(sequence='Probe_Sequence', target='ILMN_Gene', probe='Probe_Id'), verbose=TRUE) {

    history.submitted <- as.character(Sys.time())

	## check whether the object is nuID annotated.
	exprs <- exprs(x.lumi)
	id <- rownames(exprs)
	if(all(sapply(id[1:50], is.nuID))) {
		if (verbose) cat('The lumiBatch object is already nuID annotated!\n')
		return(x.lumi)
	}
	if (!is.null(lib.mapping)) {
		if (length(grep('lumi.*\\.db', lib.mapping)) == 0 && length(grep('lumi.*IDMapping', lib.mapping)) == 0) {
			warning(paste(lib.mapping, 'does not include nuID conversion information!\n'))
			lib.mapping <- NULL
		}
	}

	newId <- id

	## ---------------------------------------
	## First check whether probe sequence information is available 
	annotation <- pData(featureData(x.lumi))
	names(annotation) <- toupper(names(annotation))
	if (toupper(annotationColName['sequence']) %in% names(annotation)) {
		sequence <- annotation[, toupper(annotationColName['sequence'])]
		cat('Directly converting probe sequence to nuIDs ...\n')
		newId <- sapply(sequence, seq2id)
		names(newId) <- id				
	} else if (!is.null(annotationFile)) {
		## identify the Metadata lines 
		info <- readLines(annotationFile, n=10)    # take the first 10 lines to have a taste

		## Use annotationColName[1] as an indicator of Where the metaData stops
		##   intelligently find nMetaDataLines  
		nMetaDataLines <- grep(annotationColName[1], info, ignore.case=TRUE) - 1

		if (is.null(sep)) {
		    ## Find out the separator (sep) by taking the first two line of data, and comparing them.
		    ##  we assume it is either "," or "\t".
	    	titleLine <- info[nMetaDataLines + 1]
			sepNum <- regexpr('\t', titleLine)
			if (sepNum >= 2) {
				sep <- '\t'
			} else {
				sepNum <- regexpr(',', titleLine)
				if (sepNum >= 2) {
					sep <- ','
				} else {
					stop('The seperator is not Tab or comma!\n Please sepecify the seperator used in the file!\n')
				}
			}
		}

		dataLine1 <- strsplit(info[nMetaDataLines + 2], sep)[[1]]
		quoteCount1 <- gregexpr('"', dataLine1[1])[[1]]
		quoteCount2 <- gregexpr('\'', dataLine1[1])[[1]]
		quote <- ''
		if (sep == ',') quote <- '"'
		if (length(quoteCount1) == 2) {
			quote <- '"'
		} else if (length(quoteCount2) == 2) {
			quote <- '\''
		}

		## Read in annotation data
		annotation <- read.table(annotationFile, sep=sep, colClasses="character", header=TRUE, skip=nMetaDataLines,
		 	blank.lines.skip=TRUE, row.names=NULL, check.names=FALSE, quote=quote, comment.char="", strip.white=TRUE, fill=TRUE)
		
		allColName <- toupper(colnames(annotation))
		colnames(annotation) <- allColName
		## Create unique Id based on 50mer sequence
		if (toupper(annotationColName['sequence']) %in% allColName) {
			nuID <- sapply(annotation[, toupper(annotationColName['sequence'])], seq2id)			
		} else {
			stop('The "sequence" column cannot be found!\nPlease check the "annotationColName" of "sequence"!\n')
		}
		## check the TargetID first
		comm_target <- NULL
		if (toupper(annotationColName['target']) %in% allColName) {
			ann_target <- annotation[, toupper(annotationColName['target'])]
			comm_target <- id[id %in% ann_target]
		}
		if (length(comm_target) == 0) {
			## check the ProbeID if id does not match the TargetID
			if (toupper(annotationColName['probe']) %in% allColName) {
				ann_target <- annotation[, toupper(annotationColName['probe'])]
				comm_target <- id[id %in% ann_target]
				if (length(comm_target) == 0) {
					width <- nchar(ann_target[1])
					id <- formatC(as.numeric(id), width=width, flag='0', format='d')
					comm_target <- id[id %in% ann_target]
					if (length(comm_target) == 0) stop('The annotation file does not match the data!\n')
				}
			}
		} 
		if (length(comm_target) < length(id)) {
			diffId.len <- length(id) - length(comm_target)
			warning(paste('The annotation file does not match the data.',  diffId.len, 'ids cannot be replaced!\n'))
		}
		names(nuID) <- ann_target

		newId <- id
		newId[id %in% ann_target] <- nuID[comm_target]
	} else if (!is.null(lib.mapping)) {
		if (!require(lib.mapping, character.only=TRUE)) stop(paste('Annotation library', lib.mapping, 'is not installed!\n'))
		controlId <- c('lysA','pheA','thrB','trpF', 'bla1','bla2','cat1','cat2','cre1','cre2','e1a1',
		'e1a2','gfp1','gfp2','gst1','gst2','gus1','gus2','lux1','lux2','lysA','neo1',
		'neo2','pheA','thrB','trpF')
		if (length(grep('IDMapping', lib.mapping)) == 0) {
			## check the TargetID first
			newId <- mget(id, get(paste(lib.mapping, 'TARGETID2NUID', sep=''), mode='environment'), ifnotfound=NA)
			newId <- unlist(newId)
			if (length(which(!is.na(newId))) == 0) {
				usingTargetID <- FALSE
				## check the ProbeID if id does not match the TargetID
				newId <- mget(id, get(paste(lib.mapping, 'PROBEID2NUID', sep=''), mode='environment'), ifnotfound=NA)
				if (length(which(!is.na(newId))) == 0) {
					width <- nchar(ls(envir=get(paste(lib.mapping, 'PROBEID2NUID', sep=''), mode='environment'))[1])
					id <- formatC(as.numeric(id), width=width, flag='0', format='d')
					newId <- mget(id, get(paste(lib.mapping, 'PROBEID2NUID', sep=''), mode='environment'), ifnotfound=NA)
					if (length(which(!is.na(newId))) == 0) {
						targetID <- pData(featureData(x.lumi))$TargetID
						newId <- mget(targetID, get(paste(lib.mapping, 'TARGETID2NUID', sep=''), mode='environment'), ifnotfound=NA)
						if (length(which(!is.na(newId))) == 0) stop('The library does not match the data!\n')
					}
				} 
			} else {
				usingTargetID <- TRUE
			}
			## Check for the targetIDs cannot be found in the lib.mapping.
			## Some known control genes will not be checked.
			naInd <- which(is.na(newId))
			if (!usingTargetID) {
				TargetID <- featureData(x.lumi)$TargetID
				if (is.null(TargetID)) {
					if (!all(TargetID[naInd] %in% controlId)) {
						if (length(naInd) < 10) {
							warning(paste('Identifiers:', paste(TargetID[naInd], collapse=','), ' cannot be found in the ', lib.mapping, '!\n', sep=''))
						} else {
							warning(paste('Some identifiers cannot be found in the ', lib.mapping, '!\n', sep=''))
						}
					}
				}
			} else if (!all(id[naInd] %in% controlId)) {
				if (length(naInd) < 10) {
					warning(paste('Identifiers:', paste(id[naInd], collapse=','), ' cannot be found in the ', lib.mapping, '!\n', sep=''))
				} else {
					warning(paste('Some identifiers cannot be found in the ', lib.mapping, '!\n', sep=''))
				}
			}
			newId[naInd] <- id[naInd]
		} else {
			chipInfo <- getChipInfo(featureNames(x.lumi), lib.mapping=lib.mapping, idMapping=TRUE, verbose=FALSE)
			newId <- chipInfo$idMapping[,'nuID']
			naInd <- which(is.na(newId))
			if (length(naInd) > 0) {
				newId[naInd] <- id[naInd]
				if (!all(id[naInd] %in% controlId))	{
					if (length(naInd) > 500) {
						warning(paste('More than 500 identifiers cannot be found in the library:', lib.mapping, 
								'!\n The provided library might be wrong or outdated!\n', sep=''))
					} else if (length(naInd) < 10) {
						warning(paste('Identifiers:', paste(id[naInd], collapse=','), ' cannot be found in the ', lib.mapping, '!\n', sep=''))
					} else {
						warning(paste('Some identifiers cannot be found in the ', lib.mapping, '!\n', sep=''))
					}
				}
			}
			selInfoName <- names(chipInfo)
			selInfoName <- selInfoName[selInfoName != 'idMapping']
			chipInfo.print <- paste(selInfoName, unlist(chipInfo[selInfoName]), sep=': ')
			notes(x.lumi) <- c(notes(x.lumi), list('Chip Information'=chipInfo.print))

			annotation(x.lumi) <- switch(chipInfo$species,
					'Human'='lumiHumanAll.db',
					'Mouse'='lumiMouseAll.db',
					'Rat'='lumiRatAll.db')
		}
	} else {
		cat('Please provide Illumina ID Mapping library!\n')
	}
	if (all(newId == id)) {
		conversion <- FALSE
	} else {
		conversion <- TRUE
	}

	if (any(duplicated(newId)))  {
		if (verbose) cat('Duplicated IDs found and were merged!\n')
		dupId <- unique(newId[duplicated(newId)])
		## determine whether the detection p-value close to 0 or 1 is significant
		if (!is.null(detection(x.lumi))) {
			detect.low <- exprs[which.max(detection(x.lumi)[,1]), 1]
			detect.high <- exprs[which.min(detection(x.lumi)[,1]), 1]
		}
		
		rmIndex <- NULL
		for (dupId.i in dupId) {
			dupIndex <- which(newId == dupId.i)
			ave.exp <- colMeans(exprs(x.lumi)[dupIndex,, drop=FALSE])
			exprs(x.lumi)[dupIndex[1],] <- ave.exp
			if (is(x.lumi, 'LumiBatch') && !is.null(beadNum(x.lumi)) && !is.null(detection(x.lumi))) {
				totalBeadNum <- colSums(beadNum(x.lumi)[dupIndex, , drop=FALSE])
				if (detect.low < detect.high) {
					maxDetection <- apply(detection(x.lumi), 2, min)
				} else {
					maxDetection <- apply(detection(x.lumi), 2, max)
				}

				temp <- colSums(se.exprs(x.lumi)[dupIndex, , drop=FALSE]^2 * (beadNum(x.lumi)[dupIndex,, drop=FALSE] - 1))
				temp <- temp / (totalBeadNum - length(dupIndex))
				se.exprs(x.lumi)[dupIndex[1],] <- sqrt(temp * (colSums(1/beadNum(x.lumi)[dupIndex,, drop=FALSE])))
				detection(x.lumi)[dupIndex[1],] <- maxDetection
				beadNum(x.lumi)[dupIndex[1],] <- totalBeadNum
			}
			rmIndex <- c(rmIndex, dupIndex[-1])
		}
		## remove duplicated
		x.lumi <- x.lumi[-rmIndex, ]
		newId <- newId[-rmIndex]
	}

	## update the feature names (probe ids)
	if (conversion) {
		names(newId) <- NULL
		featureNames(x.lumi) <- newId
		## update the feautre data
		featureData <- featureData(x.lumi)
		rownames(pData(featureData)) <- newId
		if (!is.null(pData(featureData)$'PROBE_SEQUENCE')) pData(featureData)$'PROBE_SEQUENCE' <- NULL
		featureData(x.lumi) <- featureData

		## Add history tracking
		if (is(x.lumi, 'LumiBatch')) {
			history.finished <- as.character(Sys.time())
			# history.command <- capture.output(print(match.call(addNuID2lumi)))
			history.command <- paste(deparse(match.call(addNuID2lumi)), collapse='') 
			if (is.null(x.lumi@history$lumiVersion)) x.lumi@history$lumiVersion <- rep(NA, nrow(x.lumi@history))
			lumiVersion <- packageDescription('lumi')$Version
			x.lumi@history<- rbind(x.lumi@history, data.frame(submitted=history.submitted, 
					finished=history.finished, command=history.command, lumiVersion=lumiVersion))
		}
	}

	return(x.lumi)
}

`addNuId2lumi` <-
function(x.lumi, annotationFile=NULL, sep=NULL, lib.mapping=NULL, annotationColName=c(sequence='Probe_Sequence', target='Target', probe='Probe_Id'), verbose=TRUE) {
	cat('Function addNuId2lumi is deprecated!\n Please use addNuID2lumi instead!\n')
	addNuID2lumi(x.lumi, annotationFile, sep, lib.mapping, annotationColName, verbose)
}

`is.nuID` <-
function(id) {

	if (length(id) > 1) {
		return(sapply(id, is.nuID))
	} else {
		if (nchar(id) < 2) return(FALSE)
		code <- c(LETTERS, letters, as.character(0:9), '_', '.')
		ind <- 1:length(code)
		names(ind) <- code
		if(length(grep('[^a-zA-Z0-9_.]', id)) > 0)  return(FALSE)

		id <- substring(id, 1:nchar(id), 1:nchar(id))
		num <- as.numeric(ind[id]) - 1
		checkCode <- num[1]
		num <- num[-1]
		if (checkCode == 63) return(FALSE)

		cutLen <- checkCode %% 3
		res <- floor(checkCode/3)

		codon <- rbind(floor(num / 4^2), floor((num %% 4^2) / 4), num %% 4)

		## Check whether the checkCode matches the sequence
		checkSum <- sum(num)
		if (res != checkSum %% 21) {
			return(FALSE)
		} else {
			return(TRUE)
		}
	}
}

nuID2probeID <- function(nuID, lib.mapping="lumiHumanIDMapping", ...) {

	if (length(nuID) == 0) return(NULL)

	if (!is.null(lib.mapping)) {
		if (length(grep('lumi.*\\.db', lib.mapping)) == 0 && length(grep('lumi.*IDMapping', lib.mapping)) == 0) {
			warning(paste(lib.mapping, 'does not include nuID conversion information!'))
			return(nuID)
		}
	}
	if (require(lib.mapping, character.only=TRUE)) {
		if (length(grep('IDMapping', lib.mapping)) > 0) {
			probe <- nuID2IlluminaID(nuID, lib.mapping=lib.mapping, idType='Probe', ...)
		} else {
			env <- get(paste(lib.mapping, 'PROBEID2NUID', sep = ""), mode = "environment")
			probe2nuID <- unlist(as.list(env))	
			allProbe <- names(probe2nuID)
			keepInd <- !is.na(probe2nuID)
			allProbe <- allProbe[keepInd]
			names(probe2nuID) <- allProbe
			probe <- lapply(nuID, function(x) allProbe[probe2nuID == x])
			names(probe) <- nuID			
		}
		return(probe)
	} else {
		cat(paste(lib.mapping, ' ID mapping library is required!\n', sep=''))
	}
}

nuID2targetID <- function(nuID, lib.mapping="lumiHumanIDMapping", ...) {

	if (length(nuID) == 0) return(NULL)
	if (!is.null(lib.mapping)) {
		if (length(grep('lumi.*\\.db', lib.mapping)) == 0 && length(grep('lumi.*IDMapping', lib.mapping)) == 0) {
			warning(paste(lib.mapping, 'does not include nuID conversion information!'))
			return(nuID)
		}
	}
	if (require(lib.mapping, character.only=TRUE)) {
		if (length(grep('IDMapping', lib.mapping)) > 0) {
			target <- nuID2IlluminaID(nuID, lib.mapping=lib.mapping, idType='Gene', ...)
		} else {
			env <- get(paste(lib.mapping, 'TARGETID2NUID', sep = ""), mode = "environment")
			target2nuID <- unlist(as.list(env))	
			allTarget <- names(target2nuID)
			keepInd <- !is.na(target2nuID)
			allTarget <- allTarget[keepInd]
			names(target2nuID) <- allTarget
			target <- lapply(nuID, function(x) allTarget[target2nuID == x])
			names(target) <- nuID
		}
		return(target)
	} else {
		cat(paste(lib.mapping, ' ID mapping library is required!\n', sep=''))
	}
}

probeID2nuID <- function(probeID, lib.mapping="lumiHumanIDMapping", ...) {

	if (length(probeID) == 0) return(NULL)
	if (!is.null(lib.mapping)) {
		if (length(grep('lumi.*\\.db', lib.mapping)) == 0 && length(grep('lumi.*IDMapping', lib.mapping)) == 0) {
			warning(paste(lib.mapping, 'does not include nuID conversion information!'))
			return(probeID)
		}
	}
	# if (!require(annotate)) cat('Please install "annotate" library!\n')
	if (require(lib.mapping, character.only=TRUE)) {
		if (length(grep('IDMapping', lib.mapping)) > 0) {
			nuID <- IlluminaID2nuID(probeID, lib.mapping=lib.mapping, ...)
		} else {
			nuID <- unlist(lookUp(probeID, lib.mapping, 'PROBEID2NUID'))
		}
		return(nuID)
	} else {
		cat(paste(lib.mapping, ' ID mapping library is required!\n', sep=''))
	}
}

targetID2nuID <- function(targetID, lib.mapping="lumiHumanIDMapping", ...) {

	if (length(targetID) == 0) return(NULL)
	if (!is.null(lib.mapping)) {
		if (length(grep('lumi.*\\.db', lib.mapping)) == 0 && length(grep('lumi.*IDMapping', lib.mapping)) == 0) {
			warning(paste(lib.mapping, 'does not include nuID conversion information!'))
			return(targetID)
		}
	}
	# if (!require(annotate)) cat('Please install "annotate" library!\n')
	if (require(lib.mapping, character.only=TRUE)) {
		if (length(grep('IDMapping', lib.mapping)) > 0) {
			nuID <- IlluminaID2nuID(targetID, lib.mapping=lib.mapping, ...)
		} else {
			nuID <- unlist(lookUp(targetID, lib.mapping, 'TARGETID2NUID'))
		}
		return(nuID)
	} else {
		cat(paste(lib.mapping, ' ID mapping library is required!\n', sep=''))
	}
}

`getControlType` <-
function(controlData) 
{
	if (is(controlData, 'LumiBatch')) {
		controlData <- controlData(controlData)
	} 
	if (is(controlData, 'data.frame')) {
		if (nrow(controlData) == 0) stop('controlData is empty!')
		return(unique(as.character(controlData$controlType)))
	} else if (is(controlData, 'MethyLumiQC')) {
		return(controlTypes(controlData))
	} else {
		return(NA)
	}
}

`vst` <-
function(u, std, nSupport=min(length(u), 500), backgroundStd=NULL, fitMethod=c('linear', 'quadratic'), lowCutoff=1/3, ifPlot=FALSE) {
	# u is the mean of probe beads
	# std is the standard deviation of the probe beads
	# 
	fitMethod <- match.arg(fitMethod)
	## Estimate the background variance c3
	c3 <- ifelse (is.null(backgroundStd), 0, backgroundStd)
		
	ord <- order(u); u.bak <- u
	u <- u[ord]; std <- std[ord]

	## remove NAs if exists
	na.ind <- which(is.na(u) | is.na(std))
	if (length(na.ind) > 0) {
		u <- u[-na.ind]
		std <- std[-na.ind]
	}
	
	if (any(std < 0)) {
		stop('Negative expression standard deviation is not allowed!')
	}
	

	if (fitMethod == 'quadratic') {
		downSampledU <- u; offset=0
		dd <- data.frame(y=std, x2=u^2, x1=u)
		lm2 <- lm(y ~ x2 + x1, dd)
		smoothStd <- predict(lm2, data.frame(x2=downSampledU^2, x1=downSampledU))
	} else {
		## downsampling to speed up 
    # if (min(u) < 1) {
    #   offset <- 1 - min(u)
    # } else {
    #   offset <- 0
    # }
		offset <- 1 - min(u)
		downSampledU <- 2^seq(from=log2(min(u + offset)), to=log2(max(u + offset)), length=nSupport) - offset

		minU <- log2(max(100 + offset, min(u)))
		maxU <- log2(max(u))
		# uCutoff <- 2^((maxU + minU)/2)
		uCutoffLow <- 2^(minU + (maxU - minU) * lowCutoff)
		uCutoffHigh <- 2^(minU + (maxU - minU) * 4/5)
		selInd <- (u > uCutoffLow & u < uCutoffHigh)
		selLowInd <- (u < uCutoffLow)
		if (c3 != 0) {
			selInd <- selInd & (std^2 > c3)
			dd <- data.frame(y=sqrt(std[selInd]^2 - c3), x1=u[selInd])
 			# if (nrow(dd) > 5000) dd <- dd[sample(1:nrow(dd), 5000),]
			lmm <- lm(y ~ x1, dd)
			c1 <- lmm$coef[2]
			c2 <- lmm$coef[1]
		} else {
			## use iteraction to estimate the parameters when background level is not known
			iterNum <- 0
			c3.i <- 0
			while(iterNum <= 20) {
				selInd.i <- selInd & (std^2 > c3.i)
				dd <- data.frame(y=sqrt(std[selInd.i]^2 - c3.i), x1=u[selInd.i])
				# if (nrow(dd) > 5000) dd <- dd[sample(1:nrow(dd), 5000),]
				lm.i <- lm(y ~ x1, dd)
				c1.i <- lm.i$coef[2]
				c2.i <- lm.i$coef[1]
				y <- std[selLowInd]
				x <- u[selLowInd]
				cc <- y^2 - (c1.i * x + c2.i)^2
				c3.i.new <- mean(cc, trim=0.05)
				if (c3.i.new < 0) {
					break
				} else {
					if (abs(c3.i.new - c3.i) < 1e-5) break
					c3.i <- c3.i.new
				}
				iterNum <- iterNum + 1
			}
			c1 <- c1.i; c2 <- c2.i; c3 <- c3.i
			if (c3 < 0) c3 <- 0
		}
		smoothStd <- ((c1 * downSampledU + c2)^2 + c3)^(1/2)
	}

	if (ifPlot) {
		par(mfrow=c(1,2))
		x <- u[u > 0]
		y <- std[u > 0]
		len <- length(x)
		ind <- sample(1:len, min(5000, len))
		plot(x[ind], y[ind], pch='.', log='xy', xlab="Mean", ylab="Standard Deviation", main='(A) Relations of probe Mean and SD')
		lines(downSampledU, smoothStd, col=3, lwd=1.5)
	}
	
	## calculate the integration (h function is the integral)
	if (fitMethod == 'linear') {
		if (c3 == 0) {
			## Transform function h(x) = g * log(a + b * x)
			g <- 1/c1
			a <- c2
			b <- c1
			tmp <- a + b * u.bak
			if (any(tmp < 0)) {
				transformedU <- log(u.bak)
				g <- 1; a <- 0; b <- 1
			} else {
				transformedU <- g * log(a + b * u.bak)
			}
			if (ifPlot) hy <- g * log(a + b * downSampledU) 
			transFun <- 'log'
		} else {
			## Transform function h(x) = g * asinh(a + b * x)
			g <- 1/c1
			a <- c2/sqrt(c3)
			b <- c1/sqrt(c3)
			transformedU <- g * asinh(a + b * u.bak)
			if (ifPlot) hy <- g * asinh(a + b * downSampledU) 
			transFun <- 'asinh'
		}
		transform.parameter <- c(a, b, g, 0)
		names(transform.parameter) <- c('a', 'b', 'g', 'Intercept')
	} else {
		dd <- diff(downSampledU)	## the interval between samples
		dd <- c(dd[1], dd)
		hy <- cumsum(1/smoothStd * dd)		
		# get a smoothed version and interpolation of original data order
	    transformedU <- monoSpline(x=downSampledU, y=hy, newX=u.bak, nKnots=20, ifPlot=FALSE)
		transFun <- 'quadratic'
		transform.parameter <- NULL
	}	

	## rescale to the similar range with log2
	if (fitMethod == 'quadratic') {
		minU <- log2(max(100 + offset, min(u)))
		maxU <- log2(max(u))
		uCutoffLow <- 2^(minU + (maxU - minU) * lowCutoff)
		uCutoffHigh <- 2^(minU + (maxU - minU) * 4/5)		
	}
	cutInd <- which.min(abs(u.bak - uCutoffLow))
	maxInd <- which.max(u.bak)
	y <- c(u.bak[cutInd], u.bak[maxInd])
	x <- c(transformedU[cutInd], transformedU[maxInd])
	m <- lm(log2(y) ~ x)
	if (fitMethod == 'linear') {
		transform.parameter <- c(a, b, g * m$coef[2], m$coef[1])
		names(transform.parameter) <- c('a', 'b', 'g', 'Intercept')
		## The transform parameter is in the transFun below
		## transFun <- g * asinh(a + b * x) * m$coef[2] + m$coef[1]
	} else {
		transform.parameter <- NULL
		transFun <- NULL
	}
	transformedU <- predict(m, data.frame(x=transformedU))
	attr(transformedU, 'parameter') <- transform.parameter
	attr(transformedU, 'transformFun') <- transFun

    if (ifPlot) {
		x <- log2(u.bak[u.bak > 0])
		y <- transformedU[u.bak > 0]
        ii <- order(x)
        plot(x[ii], y[ii], lwd=1.5, type='l', xlab="Log2 transformed value", ylab="VST transformed value", main='(B) Log2 vs. VST')
		abline(a=0, b=1, col=3, lwd=1.5, lty=2)
		par(mfrow=c(1,1))
    }
	
	return(transformedU)
}

`plotSampleRelation` <-
function(x, subset=NULL, cv.Th=0.1, standardize=TRUE, method=c('cluster', 'mds'), dimension=c(1,2), color=NULL, main=NULL, pch=NULL, addLegend=TRUE, ...) {
	if (is(x, 'ExpressionSet')) {
		dataMatrix <- exprs(x)
	} else if (is.matrix(x)) {
		dataMatrix <- x
	} else {
		stop('The class of "x" should be matrix or LumiBatch!')
	}
	
	## Standardize each sample
	if (standardize) dataMatrix <- scale(dataMatrix)

	if (is.null(subset)) {
		## Filter the genes with most of the experiments "Absent"
		probeList <- rownames(dataMatrix)
		if (is.null(probeList)) probeList <- 1:nrow(dataMatrix)
		if (cv.Th > 0) {
			cv.gene <- apply(dataMatrix, 1, function(x) sd(x)/mean(x))
			subset <- probeList[abs(cv.gene) > cv.Th]
			if (is.null(main)) main <- paste('Sample relations based on', length(subset), 'genes with sd/mean >', cv.Th)
		} else {
			subset <- probeList
			if (is.null(main)) main <- paste('Sample relations based on', length(subset), 'genes')
		}
	} else {
		if (length(subset) == 1 && is.numeric(subset)) {
			subset <- sample(1:nrow(dataMatrix), min(subset, nrow(dataMatrix)))
		}
		if (is.null(main)) main <- paste('Sample relations based on', length(subset), 'selected genes')
	}

	dd <- dist(t(dataMatrix[subset,]))
	method <- match.arg(method)
	if (method == 'cluster') {
		hc = hclust(dd, 'ave')
		plot(hc, xlab='Sample', main=main, ...)
		attr(hc, 'geneNum') <- length(subset)
		attr(hc, 'threshold') <- cv.Th
		return(invisible(hc))	
	} else {
		## Multi-Dimension Scaling
		mds.result <- cmdscale(dd, k=max(dimension), eig=TRUE)
		ppoints <- mds.result$points
		eig <- mds.result$eig
		percent <- round(eig/sum(eig) * 100, 1)
		
		colorLegend <- NULL
		if (is.null(color)) {
			color <- 1
		} else {
			if (!is.numeric(color)) {
				allColor <- colors()
				if (!all(is.element(color, allColor))) {
					colorLegend <- unique(color)
					color <- as.numeric(factor(color, levels=colorLegend))
				} 
			}
		}
		if (missing(pch)) {
			plot(ppoints[,dimension[1]], ppoints[,dimension[2]], type='n', 
				xlab=paste('Principal Component ', dimension[1], " (", percent[dimension[1]], "%)", sep=""),
				ylab=paste('Principal Component ', dimension[2], " (", percent[dimension[2]], "%)", sep=""), 
				main=main, ...)
			text(ppoints[,dimension[1]], ppoints[,dimension[2]], col=color, labels=colnames(dataMatrix), cex=1)
		} else {
			plot(ppoints[,dimension[1]], ppoints[,dimension[2]],  
				xlab=paste('Principal Component ', dimension[1], " (", percent[dimension[1]], "%)", sep=""),
				ylab=paste('Principal Component ', dimension[2], " (", percent[dimension[2]], "%)", sep=""), 
				main=main, col=color, pch=pch, ...)
		}
		attr(ppoints, 'geneNum') <- length(subset)
		attr(ppoints, 'threshold') <- cv.Th
		
		## add legend if color is a factor
		if (!is.null(colorLegend) && addLegend) {
			if (!missing(pch)) {
				legend('topleft', legend=colorLegend, col=unique(color), pch=unique(pch))
			} else {
				legend('topleft', legend=colorLegend, text.col=1:length(colorLegend))
			}
		}
		
		return(invisible(ppoints))	
	}
}

`produceGEOSampleInfoTemplate` <-
function(lumiNormalized, lib.mapping=NULL, fileName='GEOsampleInfo.txt') {	
	
	link <- "http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html"
	templateTitle <- c("Sample_title", "Sample_channel_count","Sample_source_name_ch1","Sample_organism_ch1", "Sample_characteristics_ch1", "Sample_molecule_ch1","Sample_extract_protocol_ch1","Sample_label_ch1", "Sample_label_protocol_ch1", "Sample_hyb_protocol","Sample_scan_protocol","Sample_description","Sample_data_processing","Sample_platform_id", "Sample_supplementary_file")
	if (is(lumiNormalized, 'LumiBatch')) {
		labels <- sampleNames(lumiNormalized)
		chipInfo <- getChipInfo(lumiNormalized, lib.mapping=lib.mapping)
		chipVersion <- chipInfo$chipVersion[1]
		chipVersion <- sub("(_V[0-9.]*)_.*$", "\\1", chipVersion)
		organism <- switch(chipInfo$species,
			'Rat'='Rattus norvegicus',
			'Human'="Homo sapiens",
			'Mouse'='Mus musculus')
		templateContent <- c("","1","",organism,"","total RNA","standard as recommended by illumina","Cy3","standard as recommended by illumina","standard as recommended by illumina","standard as recommended by illumina","","",chipVersion, "none")
	} else if (is(lumiNormalized, 'MethyLumiM')) {
		labels <- sampleNames(lumiNormalized)
		chipVersion <- 'unknown'
		organism <- 'unknown'
		templateContent <- c("","1","",organism,"","genomic DNA","standard as recommended by illumina","Cy3","standard as recommended by illumina","standard as recommended by illumina","standard as recommended by illumina","","",chipVersion,"none")
	} else if (is(lumiNormalized, 'matrix') || is(lumiNormalized, 'ExpressionSet')) {
		if (is(lumiNormalized, 'ExpressionSet'))  lumiNormalized <- exprs(lumiNormalized)
		labels <- colnames(lumiNormalized)
		chipVersion <- 'unknown'
		organism <- 'unknown'
		templateContent <- c("","1","",organism,"","","","Cy3","","standard as recommended by manufacturer","standard as recommended by manufacturer","","", "","none")
	} else {
		cat("The input object should be an object of LumiBatch, MethyLumiM, matrix or other ExpressionSet inherited class!\n")
	}
	
	## add code of parsing processing history 
	preprocessMethod <- ''
	if (is(lumiNormalized, 'LumiBatch')) {
		hh <- getHistory(lumiNormalized)
		comm <- hh$command
	}
	if (is(lumiNormalized, 'MethyLumiM')) {
		hh <- methylumi::getHistory(lumiNormalized)
		comm <- hh$command
	}
	if (is(lumiNormalized, 'LumiBatch')) {
		# grep lumiT method
		lumiT.loc <- grep('lumiT', comm)
		if (length(lumiT.loc) > 0) {
			lumiT.comm <- comm[lumiT.loc]
			method.match <- regexpr('method *= *\"([0-9a-zA-Z]*)\"', lumiT.comm)
			if (method.match == -1) {
				lumiT.method <- 'vst'
			} else {
				end.loc <- method.match + attr(method.match, "match.length") - 2
				method.match <- regexpr('method *= *\"', lumiT.comm)
				start.loc <- method.match + attr(method.match, "match.length")
				lumiT.method <- substring(lumiT.comm, start.loc, end.loc)		
			}
			lumiVersion <- hh$lumiVersion[grep('lumiT', comm)]
			# grep lumiN method
			lumiN.comm <- comm[grep('lumiN', comm)]
			method.match <- regexpr('method *= *\"([0-9a-zA-Z]*)\"', lumiN.comm)
			if (method.match == -1) {
				lumiN.method <- 'quantile'
			} else {
				end.loc <- method.match + attr(method.match, "match.length") - 2
				method.match <- regexpr('method *= *\"', lumiN.comm)
				start.loc <- method.match + attr(method.match, "match.length")
				lumiN.method <- substring(lumiN.comm, start.loc, end.loc)		
			}
			preprocessMethod <- paste('The data was preprocessed by Bioconductor lumi package (version ',lumiVersion, '). It was ', lumiT.method, ' transformed and ', lumiN.method, ' normalized.', sep='')		
		} 		
	} 

	templateContent[templateTitle == "Sample_data_processing"] <- preprocessMethod
	template <- templateTitle
	for (i in seq(labels)) {
		template <- rbind(template, templateContent)
	}
	template <- cbind(c('sampleID', labels), template)
	if (!is.null(fileName)) {
		cat('# For the detailed definition of the column names, please refer to ', link, '\n', sep='', file=fileName)
		write.table(template, sep='\t', quote=FALSE, file=fileName, append=TRUE, col.names=FALSE, row.names=FALSE)		
	} else {
		return(template)
	}
}

`produceMethylationGEOSubmissionFile` <-
function(methyLumiM, methyLumiM.raw=NULL, lib.mapping=NULL, idType='Probe', sampleInfo=NULL, fileName='GEOSubmissionFile.txt', supplementaryRdata=FALSE, ...) {
	if (missing(methyLumiM)) stop('Please provide all required input parameters!\n')
	options(stringsAsFactors=FALSE)
	if (is(methyLumiM, "MethyLumiM")) expr.norm <- signif(estimateBeta(methyLumiM, returnType='matrix'),5)
	if (is.null(methyLumiM.raw)) {
		detect <- signif(detection(methyLumiM),5)
		methyData <- signif(methylated(methyLumiM),5)
		unmethyData <- signif(unmethylated(methyLumiM),5)
		expr <- NULL
	} else {
		detect <- signif(detection(methyLumiM.raw),5)
		methyData <- signif(methylated(methyLumiM.raw),5)
		unmethyData <- signif(unmethylated(methyLumiM.raw),5)
		expr <- signif(estimateBeta(methyLumiM.raw, returnType='matrix'),5)
	}
		
	if (is.null(sampleInfo)) {
		sampleInfo <- produceGEOSampleInfoTemplate(methyLumiM, lib.mapping=lib.mapping, fileName=NULL)
	} else if (length(sampleInfo) == 1 && is.character(sampleInfo)) {
		sampleInfo <- read.table(sampleInfo, sep='\t', colClasses='character', skip=1, header=TRUE, strip.white=TRUE, quote='')
	} else if (is.null(nrow(sampleInfo))) {
		stop('Please provide correct sample information (a data.frame, matrix, or sampleInfo file)!\n')
	}
	sampleInfoTitle <- colnames(sampleInfo)
	if (any(sapply(sampleInfo[,-1, drop=F], nchar) == 0)) stop('No blank fields are allowed in the sampleInfo table!\nYou can check some example submissions, like GSM296418, at the GEO website.\n')
	if (supplementaryRdata) sampleInfo[, "Sample_supplementary_file"] <- 'supplementaryData.Rdata'
	nuID <- featureNames(methyLumiM)
	if (!supplementaryRdata) {
	  rm(methyLumiM.raw, methyLumiM)
	  gc()
	}
	
	probeId <- nuID
	if (length(which(is.nuID(sample(nuID, 100)))) < 20) {
		nuID <- NULL
	} else {
		if (!is.null(lib.mapping)) {
			probeId <- nuID2IlluminaID(nuID, lib.mapping=lib.mapping, idType=idType, ...)
		} else {
			nuID <- NULL
		}
	}
	
	sampleID <- sampleInfo[, "sampleID"]
	sampleTitle <- sampleInfo[,'Sample_title']
	# outputFile <- file(fileName, "w")
	outputFile <- fileName
	for (i in seq(sampleID)) {
	    cat('Processing sample', i, '\n')
		if (i == 1) {
			cat('^SAMPLE =', sampleTitle[i], '\n', sep='', file=outputFile, append=FALSE)
		} else {
			cat('^SAMPLE =', sampleTitle[i], '\n', sep='', file=outputFile, append=TRUE)			
		}
		sampleInfo.i <- paste('!', sampleInfoTitle[-1], ' = ', sampleInfo[i,-1], '\n', sep='', collapse='')
		sampleInfo.i <- gsub("'", "\\'", sampleInfo.i)
		cat(sampleInfo.i, file=outputFile, sep='', append=TRUE)
		tableHead <- "ID_REF"
		cat("#ID_REF = Illumina ID\n", file=outputFile, append=TRUE)
		if (!is.null(nuID)) {
			cat("#nuID = nucleotide universal IDentifier (nuID), convertible to and from probe sequence. See Bioconductor lumi package for more details.\n", file=outputFile, append=TRUE)
			tableHead <- c(tableHead, "nuID")
		}
		cat("#VALUE = Beta-value\n", file=outputFile, append=TRUE)
		if (!is.null(expr)) cat("#RAW_VALUE = raw Beta-value\n", file=outputFile, append=TRUE)
		tableHead <- c(tableHead, "VALUE")
		if (!is.null(expr)) tableHead <- c(tableHead, "RAW_VALUE")
		if (!is.null(methyData)) {
			cat("#METHYLATED = the intensities measured by methylated probes\n", file=outputFile, append=TRUE)
			tableHead <- c(tableHead, "METHYLATED")
		}
		if (!is.null(unmethyData)) {
			cat("#UNMETHYLATED = the intensities measured by unmethylated probes\n", file=outputFile, append=TRUE)
			tableHead <- c(tableHead, "UNMETHYLATED")
		}
		if (!is.null(detect)) {
			cat("#Detection_Pval = the detection p-value of the probe\n", file=outputFile, append=TRUE)
			tableHead <- c(tableHead, "Detection_Pval")
		}
		sampleTable.i <- probeId
		if (!is.null(nuID)) sampleTable.i <- cbind(sampleTable.i, nuID)
		sampleTable.i <- cbind(sampleTable.i, expr.norm[,sampleID[i], drop=FALSE])
	
		if (!is.null(expr)) sampleTable.i <- cbind(sampleTable.i, expr[,sampleID[i], drop=FALSE])
		if (!is.null(methyData)) sampleTable.i <- cbind(sampleTable.i, methyData[,sampleID[i], drop=FALSE])
		if (!is.null(unmethyData)) sampleTable.i <- cbind(sampleTable.i, unmethyData[,sampleID[i], drop=FALSE])
		if (!is.null(detect)) sampleTable.i <- cbind(sampleTable.i, detect[,sampleID[i], drop=FALSE])
		sampleTable.i <- rbind(tableHead, sampleTable.i)
		cat("!sample_table_begin\n", file=outputFile, append=TRUE)
		write.table(sampleTable.i, sep='\t', quote=FALSE, file=outputFile, append=TRUE, col.names=FALSE, row.names=FALSE)
		cat("!sample_table_end\n", file=outputFile, append=TRUE)
	}
	#close(outputFile)
	
	if (supplementaryRdata) {
		methyLumiM <- methyLumiM[,sampleID]
		if (!is.null(methyLumiM.raw)) {
			methyLumiM.raw <- methyLumiM.raw[,sampleID]
			save(methyLumiM, methyLumiM.raw, sampleInfo, file='supplementaryData.Rdata')
		} else {
			save(methyLumiM, sampleInfo, file='supplementaryData.Rdata')
		}
	}
}

`lumiN` <-
function(x.lumi, method=c('quantile', 'rsn', 'ssn', 'loess', 'vsn', 'rankinvariant'), verbose=TRUE, ...) {

	if (is(x.lumi, 'ExpressionSet')) {
	    # x.lumi is a lumi object
	    x.matrix <- exprs(x.lumi)		
	} else if (is.numeric(x.lumi)) {
		x.matrix <- as.matrix(x.lumi)
		if (method == 'rsn') x.lumi <- x.matrix
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}

	method <- match.arg(method)
    if (method == 'vsn') {
        loadNamespace("vsn")
		if (max(x.matrix, na.rm=TRUE) < 50) {
			warning('The data seems log2 transformed. VSN should be directly applied to the raw data!')
		}
	} 
	
	if (is(x.lumi, 'LumiBatch')) {
		history.submitted <- as.character(Sys.time())
	}
	if (verbose) cat(paste('Perform', method, 'normalization ...\n'))

	norm.matrix <- switch(method,
		rsn = rsn(x.lumi, ...),
		ssn = ssn(x.lumi, ...),
		loess = normalize.loess(x.matrix, ...),
		quantile = normalize.quantiles(x.matrix, ...),
		vsn = exprs(vsn::vsn2(x.matrix, ...)),
		rankinvariant = rankinvariant(x.lumi, ...) )  # add by Arno Velds
	
	## take the average as the parameters as the transformation parameters if there is no targetArray
	if (is.null(attr(norm.matrix, 'targetArray')) && !is.null(attr(x.lumi, 'vstParameter'))) {
		parameter <- attr(x.lumi, 'vstParameter')
		fun <- attr(x.lumi, 'transformFun')
		tt <- table(fun)
		selFun <- names(tt)[which.max(tt)]		# the most frequent function as the transformation function
		selInd <- which(fun == selFun)
		parameter <- colMeans(parameter)
		fun <- selFun
		attr(norm.matrix, 'vstParameter') <- parameter
		attr(norm.matrix, 'transformFun') <- fun
	} 

	if (is.matrix(norm.matrix)) {
		colnames(norm.matrix) <- colnames(x.matrix)
		rownames(norm.matrix) <- rownames(x.matrix)
	} 
	if (!is.null(attr(norm.matrix, 'vstParameter'))) {
		attr(x.lumi, 'vstParameter') <- attr(norm.matrix, 'vstParameter')
		attr(x.lumi, 'transformFun') <- attr(norm.matrix, 'transformFun')
		if (!is.null(attr(norm.matrix, 'targetArray')))
			attr(x.lumi, 'targetArray') <- attr(norm.matrix, 'targetArray')
	}

	if (is(x.lumi, 'ExpressionSet')) {
		new.lumi <- x.lumi
		exprs(new.lumi) <- if (is(norm.matrix, 'ExpressionSet')) exprs(norm.matrix) else norm.matrix
		
	    # history tracking
		if (is(x.lumi, 'LumiBatch')) {
		  history.finished <- as.character(Sys.time())
			#history.command <- capture.output(print(match.call(lumiN)))
      history.command <- paste(deparse(match.call(lumiN)), collapse='') 
			
			if (is.null(new.lumi@history$lumiVersion)) new.lumi@history$lumiVersion <- rep(NA, nrow(new.lumi@history))
			lumiVersion <- packageDescription('lumi')$Version
			new.lumi@history<- rbind(new.lumi@history, data.frame(submitted=history.submitted, 
					finished=history.finished, command=history.command, lumiVersion=lumiVersion))
		}
	    return(new.lumi)
	} else {
		return(norm.matrix)
	}
}

`lumiR` <-
function(fileName, sep = NULL, detectionTh = 0.01, na.rm = TRUE, convertNuID = TRUE, lib.mapping = NULL, dec='.', parseColumnName=FALSE, checkDupId=TRUE, 
	QC=TRUE, columnNameGrepPattern=list(exprs='AVG_SIGNAL', se.exprs='BEAD_STD', detection='DETECTION', beadNum='Avg_NBEADS'),
	inputAnnotation=TRUE, annotationColumn=c('ACCESSION', 'SYMBOL', 'PROBE_SEQUENCE', 'PROBE_START', 'CHROMOSOME', 'PROBE_CHR_ORIENTATION', 'PROBE_COORDINATES', 'DEFINITION'), verbose=TRUE, ...) 
{
	## the patterns used to grep columns in the BeadStudio output text file 
	## 'exprs' and 'se.exprs' related columns are required
	if (is.null(columnNameGrepPattern$exprs)) columnNameGrepPattern$exprs <- 'AVG_SIGNAL'
	if (is.null(columnNameGrepPattern$se.exprs)) columnNameGrepPattern$se.exprs <- 'BEAD_STD'
	if (is.null(columnNameGrepPattern$detection)) columnNameGrepPattern$detection <- 'DETECTION'
	if (is.null(columnNameGrepPattern$beadNum)) columnNameGrepPattern$beadNum <- 'Avg_NBEADS'

	if (is.na(columnNameGrepPattern$exprs)) {
		columnNameGrepPattern$exprs <- 'AVG_SIGNAL'
		warning('exprs slot is required and default pattern will be used!\n')
	}
	if (is.na(columnNameGrepPattern$se.exprs) && checkDupId) {
		# columnNameGrepPattern$beadNum <- columnNameGrepPattern$detection <- NA
		warning('se.exprs slot is required for the VST transformation!\n We strongly suggest to include BEAD_STD columns!\n')
		# columnNameGrepPattern$se.exprs <- 'BEAD_STD'
		# warning('se.exprs slot is required and default pattern will be used!')
	}
	if (is.na(columnNameGrepPattern$beadNum)) {
		# columnNameGrepPattern$beadNum <- 'Avg_NBEADS'
	}

	# columnNameGrepPattern <- c(exprs='AVG_SIGNAL', se.exprs='BEAD_STD', detection='Detection', beadNum='Avg_NBEADS')
	history.submitted <- as.character(Sys.time())
	## set "stringsAsFactors" as FALSE
	oldSetting <- options("stringsAsFactors")[[1]]
	options(stringsAsFactors = FALSE)
	version <- 2

	if (!file.exists(fileName)) stop('The file does not exist! Please check your file path!\n')
	## ---------------------------------------
	## identify the Metadata lines 
	info <- readLines(fileName, n=20)    # take the first 20 lines to have a taste

	## Use "AVG_SIGNAL" as an indicator of Where the metaData stops
	##   intelligently find nMetaDataLines  
	nMetaDataLines <- grep(columnNameGrepPattern$exprs, info, ignore.case=TRUE) - 1
    
	if (is.null(sep)) {
	    ## Find out the separator (sep) by taking the first two line of data, and comparing them.
	    ##  we assume it is either "," or "\t".
    	titleLine <- info[nMetaDataLines + 1]
	    dataLine1 <- info[nMetaDataLines + 2]
		dataLine2 <- info[nMetaDataLines + 3]
		sepNum1 <- gregexpr('\t', dataLine1)[[1]]
		sepNum2 <- gregexpr('\t', dataLine2)[[1]]
		if (sepNum1[1] > 0 && length(sepNum1) == length(sepNum2)) {
			sep <- '\t'
		} else if (dec != ',') {
			sepNum1 <- gregexpr(',', dataLine1)[[1]]
			sepNum2 <- gregexpr(',', dataLine2)[[1]]
			if (sepNum1[1] > 0 && length(sepNum1) == length(sepNum2)) {
				sep <- ','
			} else {
				stop('The seperator is not Tab or comma!\n Please specify the seperator used in the file!\n')
			}
		} else {
			stop('Please specify the seperator used in the file!\n')
		}
	}
	## determine whether the quote is used or not
	dataLine1 <- strsplit(info[nMetaDataLines + 2], sep)[[1]]
	quoteCount1 <- gregexpr('"', dataLine1[1])[[1]]
	quoteCount2 <- gregexpr('\'', dataLine1[1])[[1]]
	if (length(quoteCount1) == 2) {
		quote <- '"'
	} else if (length(quoteCount2) == 2) {
		quote <- '\''
	} else {
		quote <- ''
	}

	header <- strsplit(info[nMetaDataLines + 1], sep)[[1]]
	probeId.pos <- grep('Probe.?ID', header, ignore.case=TRUE)
	if (length(probeId.pos) > 0) {
		colClasses <- rep(NA, length(header))
		colClasses[probeId.pos] <- 'character'
	} else {
		colClasses <- NA
	}

	## ---------------------------------------
	# get meta data info
	if (nMetaDataLines > 0) {
		info <- readLines(fileName, n=nMetaDataLines)
		## check the version of the beadStudio output
		markerInd <- grep('^\\[.*\\]', info, ignore.case=TRUE)
		if (length(markerInd) > 0) {
			if (length(grep('^\\[Header\\]', info[markerInd[1]], ignore.case=TRUE)) == 0) 
				warning('The data file may not be in the Illumina BeadStudio or GenomeStudio output format!\n')
			if (length(markerInd) > 1) {
				if (length(grep('^\\[.*\\Profile]', info[markerInd[2]], ignore.case=TRUE)) == 0) 
					warning('The data file may not be in the Illumina BeadStudio or GenomeStudio output format!\n')
			}
			version <- 3  # version 3 also includes the GenomeStudio output format
			info <- info[-markerInd]
		}
		if (length(info) > 0) {
			## remove the blanks
			info <- sub("[[:blank:]]+$", "", info)
			info <- sub(paste(sep,"+$", sep=''), "", info)

			## check the meta info of the file
# 			if (version == 2) {
# 				ind <- grep("BeadStudio version", info, ignore.case=TRUE)
# 			} else {
# 				ind <- grep("SGX Version", info, ignore.case=TRUE)
# 			}
# 			if (length(ind) == 0) 	ind <- grep("GenomeStudio version", info, ignore.case=TRUE)
# 			if (length(ind) == 0)   warning("The data file may not be in the Illumina BeadStudio or GenomeStudio output format.\n")

			## should not be normalized in BeadStudio
			ind <- grep("Normalization", info, ignore.case=TRUE)  # find where is the row index
			if (length(ind) > 0) {
				if (version == 2) {
					normalization <- strsplit(info, split='=')[[ind]][2]
					normalization <- gsub(pattern=" |,", replacement="", normalization) # remove space or ","
				} else {
					normalization <- strsplit(info, split=sep)[[ind]][2]
				}
				if (length(grep("none", normalization, ignore.case=TRUE)) == 0) {
				    warning("We recommend the raw data not to be normalized in BeadStudio or GenomeStudio.\n")
				}
			}
		} else {
			info <- NULL
		}
	} else {
		info <- NULL
	}
    
	allData <- read.table(file=fileName, header=TRUE, sep=sep, dec = dec, skip=nMetaDataLines, row.names=NULL, colClasses=colClasses,
		quote=quote, as.is=TRUE, check.names=FALSE, strip.white=TRUE, comment.char="", fill=TRUE, ...)

	## retrieve the possible section line index
	sectionInd <- grep('^\\[.*\\]', allData[,1], ignore.case=TRUE)
    
	if (length(sectionInd) > 0) {
		if (is.na(version)) version <- 3
		otherData <- allData[sectionInd[1]:nrow(allData), ]
		## we assume the first section is the expression data section
		allData <- allData[1:(sectionInd[1]-1),, drop=FALSE]
		## remove the all NA columns, which can be produced when saved in Excel
		naCol <- apply(allData, 2, function(x) all(is.na(x) | x == ''))
		allData <- allData[,!naCol]
    
		## process otherData
		sectionInd <- sectionInd - sectionInd[1] + 1
		sectionName <- otherData[sectionInd, 1]
    
		## retrieve the control data
		controlInd <- grep('^\\[Control.*\\]', sectionName, ignore.case=TRUE)
		if (length(controlInd) > 1) {
			ind <- grep('^\\[Control probe.*\\]', sectionName[controlInd], ignore.case=TRUE)
			if (length(ind) > 0) {
				controlInd <- controlInd[ind[1]]
			} else {
				controlInd <- controlInd[1]
			}
		}
		if (length(controlInd) > 0) {
			startRow <- sectionInd[controlInd] + 1
			if (length(sectionInd) > controlInd)
				endRow <- sectionInd[controlInd + 1] - 1
			else 
				endRow <- nrow(otherData)
			controlData <- otherData[startRow:endRow,]
			## remove the all NA columns, which can be produced when save in Excel
			naCol <- apply(controlData, 2, function(x) all(is.na(x) | x == ''))
			controlData <- controlData[,!naCol, drop=FALSE]
			
			## write the control data as a temporary file and parse it using lumiR
			tmpFile <- tempfile(pattern = "file", tmpdir = tempdir())
			write.table(controlData, tmpFile, sep='\t', col.names=FALSE, row.names=FALSE)			
			controlData <- getControlData(tmpFile, type='data.frame')
			unlink(tmpFile)
		} else {
			controlData <- data.frame()
		}
    
		## retrieve the Sample Table
		summaryInd <- grep('^\\[Sample.*Table\\]', sectionName, ignore.case=TRUE)
		if (length(summaryInd) > 0) {
			startRow <- sectionInd[summaryInd] + 1
			if (length(sectionInd) > summaryInd)
				endRow <- sectionInd[summaryInd + 1] - 1
			else 
				endRow <- nrow(otherData)
			sampleSummary <- otherData[startRow:endRow,]
			## remove the all NA columns, which can be produced when save in Excel
			naCol <- apply(sampleSummary, 2, function(x) all(is.na(x) | x == ''))
			sampleSummary <- sampleSummary[,!naCol]
			colnames(sampleSummary) <- sampleSummary[1,]
			sampleSummary <- sampleSummary[-1,]
		} else {
			sampleSummary <- data.frame()
		}
	} else {
		controlData <- sampleSummary <- data.frame()
	}
	header <- names(allData)
    
	## Get Id. The ProbeID (by default it is the second column) is preferred if provided, 
	# otherwise the TargetID (by default it is the first column) is used.
	targetID <- as.character(as.vector(allData[,1]))
	id <- targetID
	idName <- header[1]
	if (length(grep('Probe.?ID', header[2], ignore.case=TRUE)) > 0) {
		id <- as.character(as.vector(allData[,2]))
		idName <- header[2]
	} else if (!is.null(lib.mapping)) {
		probeId.pos <- grep('Probe.?ID', header, ignore.case=TRUE)
		if (length(probeId.pos) > 0) {
			id <- as.character(as.vector(allData[,probeId.pos[1]]))
			idName <- header[probeId.pos[1]]
		}
	}
    
	## identify where the signal column exists
	ind <- grep(columnNameGrepPattern$exprs, header, ignore.case=TRUE)
	if (length(ind) == 0) {
		stop('Input data format unrecognizable!\nThere is no column name contains "AVG_SIGNAL"!\n')
	} else {
		ind2 <- grep(paste("\\(", columnNameGrepPattern$exprs, "\\)", sep=""), header, ignore.case=TRUE)
		if (length(ind2) == length(ind)/2) ind <- ind[!(ind %in% ind2)]
	}
	exprs <- as.matrix(allData[,ind])
	if (!is.double(exprs[1])) {
		exprs <- matrix(as.double(exprs), nrow=nrow(allData))
	} 
	colnames(exprs) <- header[ind]
	
	## identify where the signal standard deviation column exists 
	if (is.na(columnNameGrepPattern$'se.exprs')) {
		ind <- NULL
	} else {
		ind <- grep(columnNameGrepPattern$'se.exprs', header, ignore.case=TRUE)
	}
	if (length(ind) == 0) {
		se.exprs <- NULL
		 # stop('Input data format unrecognizable!\nThere is no column name contains "BEAD_STDEV"!\n')
	} else {
		se.exprs <- as.matrix(allData[,ind])
		if (!is.double(se.exprs[1])) {
			se.exprs <- matrix(as.double(se.exprs), nrow=nrow(allData))
		}
		colnames(se.exprs) <- header[ind]
	}
	## identify the detection columns
	if (is.na(columnNameGrepPattern$detection)) {
		ind <- NULL
	} else {
		ind <- grep(columnNameGrepPattern$detection, header, ignore.case=TRUE)
	}
	if (length(ind) == 0) {
		detection <- NULL
	} else {
		detection <- as.matrix(allData[,ind])
		if (!is.double(detection[1])) {
			detection <- matrix(as.double(detection), nrow=nrow(allData))
		}
		colnames(detection) <- header[ind]
    
		if (length(grep("Detection Pval", header, ignore.case=TRUE)) == 0) {
			detection <- 1 - detection
		}
	}
	## identify the bead number columns
	if (is.na(columnNameGrepPattern$beadNum)) {
		ind <- NULL
	} else {
		ind <- grep(columnNameGrepPattern$beadNum, header, ignore.case=TRUE)
	}
	if (length(ind) == 0) {
		beadNum <- NULL
	} else {
	    beadNum <- as.matrix(allData[,ind])
		if (!is.double(beadNum[1])) {
			beadNum <- matrix(as.double(beadNum), nrow=nrow(allData))
		} 
		colnames(beadNum) <- header[ind]
	}

    ## check the STD column is STDERR or STDEV. If it is STDERR, then convert it as STDEV.
    if (!is.null(beadNum) && length(grep("BEAD_STDERR", colnames(se.exprs), ignore.case=T)) == ncol(se.exprs)) {
    	se.exprs <- se.exprs * sqrt(beadNum)    
    }
    
	## identify the annotation columns
	annotationInfo <- NULL
	if (inputAnnotation) {
		# It is based on annotationColumn
		annotationColumn <- header[toupper(header) %in% toupper(annotationColumn)]
		if (length(annotationColumn) == 0) {
			#cat('Annotation columns are not available in the data.\n')
		} else {
			annotationInfo <- allData[,annotationColumn, drop=FALSE]
		}
	}

	## check for possible duplicated ids
	if (checkDupId) {
		dupId <- unique(id[duplicated(id)])
		if (length(dupId) > 0) {
			cat('Duplicated IDs found and were merged!\n')
			rmInd <- NULL
			for (dupId.i in dupId) {
				selInd.i <- which(id == dupId.i)
				exprs[selInd.i[1],] <- colMeans(exprs[selInd.i,,drop=FALSE])
				if (is.null(beadNum)) {
					if (!is.null(se.exprs))
						se.exprs[selInd.i[1],] <- colMeans(se.exprs[selInd.i,,drop=FALSE])
				} else {
					totalBead.i <- colSums(beadNum[selInd.i,,drop=FALSE])
					beadNum[selInd.i[1],] <- totalBead.i
					if (!is.null(se.exprs)) {
						temp <- colSums(se.exprs[selInd.i,,drop=FALSE]^2 * (beadNum[selInd.i,,drop=FALSE] - 1))
						temp <- temp / (totalBead.i - length(selInd.i))
						se.exprs[selInd.i[1],] <- sqrt(temp * (colSums(1/beadNum[selInd.i,,drop=FALSE])))
					}				
				}
				if (!is.null(detection)) {
					detection[selInd.i[1],] <- apply(detection[selInd.i,,drop=FALSE], 2, max)
				}
				rmInd <- c(rmInd, selInd.i[-1])
			}
			## remove duplicated
			exprs <- exprs[-rmInd,,drop=FALSE]
			if (!is.null(se.exprs)) se.exprs <- se.exprs[-rmInd,,drop=FALSE]
			id <- id[-rmInd]; targetID <- targetID[-rmInd]
			if (!is.null(detection)) detection <- detection[-rmInd,,drop=FALSE]
			if (!is.null(beadNum)) beadNum <- beadNum[-rmInd,,drop=FALSE]
			if (!is.null(annotationInfo)) annotationInfo <- annotationInfo[-rmInd,,drop=FALSE]
		}
	}
    
	if (na.rm) {
		## remove the probes with all of them as NA
		keepInd <- apply(is.na(exprs), 1, sum) == 0
		exprs <- exprs[keepInd,,drop=FALSE]
		se.exprs <- se.exprs[keepInd,,drop=FALSE]
		if (!is.null(beadNum))  beadNum <- beadNum[keepInd,,drop=FALSE]
		if (!is.null(detection))  detection <- detection[keepInd,,drop=FALSE]
		if (!is.null(annotationInfo)) annotationInfo <- annotationInfo[keepInd,,drop=FALSE]
		id <- id[keepInd]
		targetID <- targetID[keepInd]
	}
	if (!any(duplicated(id))) {
		rownames(exprs) <- id
		if (!is.null(se.exprs)) rownames(se.exprs) <- id
		if (!is.null(beadNum)) rownames(beadNum) <- id
		if (!is.null(detection)) rownames(detection) <- id
	}
    
	# get sample information
	pattern <- paste('[\\.\\:][^[:alnum:]]*', columnNameGrepPattern$exprs, '[^[:alnum:]]*', sep='')
	if (length(grep(pattern, colnames(exprs), ignore.case=TRUE)) == 0) 
		pattern <- paste('[^[:alnum:]]*', columnNameGrepPattern$exprs, '[^[:alnum:]]*', sep='')
	sampleID <-  sub(pattern, '', colnames(exprs), ignore.case=TRUE) 
	if (any(duplicated(sampleID))) {
		warning('Duplicated column names found in the raw data! \n A suffix number is added to the duplicated column names.\n')
		dupId <- which(duplicated(sampleID))
		dupName <- unique(sampleID[dupId])
		for (dupName.i in dupName) {
			dupInd.i <- which(sampleID == dupName.i)
			sampleID[dupInd.i] <- paste(sampleID[dupInd.i], 1:length(dupInd.i), sep='.')
		}
	}
	if (parseColumnName) {
		sampleIDInfo <- strsplit(sampleID, split="_")
		label <- NULL
		newID <- NULL
		temp <- lapply(sampleIDInfo, function(x) {
			label <<- c(label, x[length(x)])
			newID <<- c(newID, paste(x[1:2], collapse="_"))
			})
		if (!any(duplicated(newID))) sampleID <- newID
		if (length(unique(label)) != length(label) || length(label) == 0 || any(is.na(label)))
			label <- sampleID
	} else {
		label <- sampleID
	}
    
	## reportInfo save the id information
	reporterInfo <- data.frame(id)
	names(reporterInfo) <- idName
	varMetadata <- data.frame(labelDescription='The Illumina microarray identifier')
	varName <- idName

	## if ProbeID is used as id, then also keep the TargetID information in the featureData
	if (idName != header[1]) {
		reporterInfo <- data.frame(reporterInfo, TargetID=targetID)
		varMetadata <- data.frame(labelDescription=c(varMetadata$labelDescription, 'The Illumina TargetID'))
		varName <- c(varName, 'TargetID')
	}
	## add annotationInfo to the featureData
	if (!is.null(annotationInfo)) {
		reporterInfo <- data.frame(reporterInfo, annotationInfo)
		varMetadata <- data.frame(labelDescription=c(varMetadata$labelDescription, names(annotationInfo)))
		varName <- c(varName, names(annotationInfo))
	}
	rownames(varMetadata) <- make.names(varName, unique=T)
	if (!any(duplicated(id)))	rownames(reporterInfo) <- id
	featureData <- new("AnnotatedDataFrame", data=reporterInfo, varMetadata=varMetadata)
    
	## check the dimensions of the input data
	colnames(exprs) <- label
	if (!is.null(se.exprs)) {
		if (ncol(exprs) != ncol(se.exprs)) 
			stop('Different column numbers of exprs and se.exprs! Please check the input data format.\n')
		colnames(se.exprs) <- label
	}
	if (!is.null(beadNum)) {
		if (ncol(beadNum) == length(label)) {
			colnames(beadNum) <- label
		} else {
			warning('The number of beadNum columns does not match! Please check the input data format.\n')
			if (ncol(beadNum) > length(label)) beadNum <- beadNum[,1:length(label)]
			if (ncol(beadNum) < length(label)) {
				for (i in 1:(length(label) - ncol(beadNum)))
					beadNum <- cbind(beadNum, rep(NA, nrow(beadNum)))
			}
		}
	}
	if (!is.null(detection)) {
		if (ncol(detection) == length(label)) {
			colnames(detection) <- label
		} else {
			warning('The number of detection columns does not match! Please check the input data format.\n')
			if (ncol(detection) > length(label)) beadNum <- beadNum[,1:length(label)]
			if (ncol(detection) < length(label)) {
				for (i in 1:(length(label) - ncol(detection)))
					detection <- cbind(detection, rep(NA, nrow(detection)))
			}
		}
	}
	## If no se.exprs imported, it will create a ExpressionSet class, instead of LumiBatch class.
	if (is.null(se.exprs)) {
		cmd <- 'x.lumi <- new("ExpressionSet", exprs=exprs'
	} else {
		cmd <- 'x.lumi <- new("LumiBatch", exprs=exprs, se.exprs=se.exprs'
	}
	if (!is.null(detection)) cmd <- paste(cmd, ', detection=detection')
	if (!is.null(beadNum)) cmd <- paste(cmd, ', beadNum=beadNum')
	cmd <- paste(cmd, ', featureData=featureData')		
	
	## produce the phenoData object
	if (!all(sampleID == label)) {
		pData <- data.frame(sampleID=sampleID, label=label)
		rownames(pData) <- label
		varMetadata <- data.frame(labelDescription=c('The unique Illumina microarray Id', 
			'The label of the sample'))
		rownames(varMetadata) <- c('sampleID', 'label')
	}  else {
		pData <- data.frame(sampleID=sampleID)
		rownames(pData) <- sampleID
		varMetadata <- data.frame(labelDescription=c('The unique Illumina microarray Id'))
		rownames(varMetadata) <- c('sampleID')
	}
	pdata <- new("AnnotatedDataFrame", data=pData, varMetadata=varMetadata)
	cmd <- paste(cmd, ', phenoData=pdata')
	cmd <- paste(cmd, ')')
	eval(parse(text=cmd))
	if (is.null(se.exprs)) {
		if (checkDupId && convertNuID) {
			## Add nuID if the ID Mapping library is provided
			x.lumi <- addNuID2lumi(x.lumi, lib.mapping=lib.mapping)			
		}

		## resume the old settings
		options(stringsAsFactors = oldSetting)
		return(x.lumi)
	}
	controlData(x.lumi) <- controlData
	x.lumi@QC <- list(BeadStudioSummary=sampleSummary)
	sampleNames(x.lumi) <- label
	
	if (!is.null(info)) {
		info <- gsub('\t+', '\t', info)
	}
	notes(x.lumi) <- list('Data File Information'=info)

    # history tracking
    history.finished <- as.character(Sys.time())
    # history.command <- capture.output(print(match.call(lumiR))) 
		history.command <- paste(deparse(match.call(lumiR)), collapse='')  
	
	## replace with the real file name
	if (length(grep(',', history.command)) > 0) {
		history.command <- sub('\\(.+,', paste('("', fileName, '",', sep=''), history.command)
	} else {
		history.command <- sub('\\(.+\\)', paste('("', fileName, '")', sep=''), history.command)
	}

	lumiVersion <- packageDescription('lumi')$Version
	x.lumi@history<- data.frame(submitted=history.submitted, 
			finished=history.finished, command=history.command, lumiVersion=lumiVersion)

	## Add the species information if exists
	if (any(toupper(header) == 'SPECIES')) {
		species <- as.character(allData[1,header[toupper(header) == 'SPECIES']])
		annotation(x.lumi) <- switch(tolower(species),
				'homo sapiens'='lumiHumanAll.db',
				'mus musculus'='lumiMouseAll.db',
				'rattus norvegicus'='lumiRatAll.db',
				'human'='lumiHumanAll.db',
				'mouse'='lumiMouseAll.db',
				'rat'='lumiRatAll.db',
				'NA')
	}
	
	## initialize the QC slot in the LumiBatch object
	if (QC)	x.lumi <- lumiQ(x.lumi, detectionTh=detectionTh, verbose=verbose)

	## Add nuID if the annotation library is provided
	if (!convertNuID) lib.mapping <- NULL
	if (convertNuID && !is.null(lib.mapping)) {
		x.lumi <- addNuID2lumi(x.lumi, lib.mapping=lib.mapping, verbose=verbose)
	} 
	## resume the old settings
	options(stringsAsFactors = oldSetting)
    
    return(x.lumi)
}

lumiB <- function(x.lumi, method = c('none', 'bgAdjust', 'forcePositive', 'bgAdjust.affy'), verbose=TRUE, ...) 
{
	# method <- match.arg(method)
	if (is(x.lumi, 'ExpressionSet')) {
	    # x.lumi is a LumiBatch object
	    x.matrix <- exprs(x.lumi)		
	} else if (is.numeric(x.lumi)) {
		x.matrix <- as.matrix(x.lumi)
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}
	
	if (!(is.function(method))) {
		method <- method[1]
		if (!(method %in% c('bgAdjust', 'none', 'forcePositive', 'bgAdjust.affy'))) {
			cat('This method is not supported!\n')
			return(x.lumi)
		} else if (method == 'none') {
			return(x.lumi)
		} else if (method == 'forcePositive') {
			if (min(x.matrix, na.rm=TRUE) > 0)  return(x.lumi)
		}		
		if (verbose) cat(paste('Perform', method, 'background correction ...\n'))
	} else {
		if (verbose) cat('Perform user provided background correction ...\n')		
	}
	
	history.submitted <- as.character(Sys.time())
	if (is.function(method)) {
		x.matrix <- method(x.lumi, ...)
		if (is(x.matrix, 'ExpressionSet')) x.matrix <- exprs(x.matrix)
	} else {
		if (method == 'bgAdjust') {
			x.matrix <- exprs(bgAdjust(x.lumi, ...))
		} else if (method == 'bgAdjust.affy') {
			x.matrix <- apply(x.matrix, 2, bg.adjust, ...) 
		} else if (method == 'forcePositive') {
			offset <- apply(x.matrix, 2, min, na.rm=TRUE)
			offset[offset <= 0] <- offset[offset <= 0] - 1.01 	# to avoid higher fold-change when one value is less than 1
			offset[offset > 0] <- 0
			offset <- rep(1, nrow(x.matrix)) %*% t(offset)
			x.matrix <- x.matrix - offset
		} else {
			cat('The method is not supported!\n')
			return(x.lumi)
		}
	}

	if (is(x.lumi, 'ExpressionSet')) {
		if (!is.function(method)) exprs(x.lumi) <- x.matrix
		if (is(x.lumi, 'LumiBatch')) {
			# history tracking
			history.finished <- as.character(Sys.time())
			# history.command <- capture.output(print(match.call(lumiB)))
      history.command <- paste(deparse(match.call(lumiB)), collapse='') 
			  	
			if (is.null(x.lumi@history$lumiVersion)) x.lumi@history$lumiVersion <- rep(NA, nrow(x.lumi@history))
			lumiVersion <- packageDescription('lumi')$Version
			x.lumi@history<- rbind(x.lumi@history, data.frame(submitted=history.submitted, 
					finished=history.finished, command=history.command, lumiVersion=lumiVersion))
		}
	} else {
		x.lumi <- x.matrix
	}
	return(x.lumi)
}

lumiMethyR <- function(filename, lib=NULL, controlData=NULL, qcfile=NULL, sampleDescriptions=NULL, 
    sep = NULL) {
  methyLumiSet <- methylumiR(filename, qcfile=qcfile, sampleDescriptions=sampleDescriptions, sep=sep)
  methyLumiM <- as(methyLumiSet, "MethyLumiM")
  if (!is.null(lib)) {
    # methyLumiM <- addColorChannelInfo(methyLumiM, lib=lib)
    methyLumiM <- addAnnotationInfo(methyLumiM, lib=lib)
  }
  if (!is.null(controlData)) {
    if (is.character(controlData)) {
      controlData <- methylumiR(controlData)
    }
    if (is(controlData, "MethyLumiQC")) {
      controlData(methyLumiM) <- controlData
    } else {
      cat("Provided controlData is not supported!\n")
    }
  }
  return(methyLumiM)
}


## import Illumina Infinium methylation IDAT files based on barcodes.
## This is an extension of the methylumi::lumIDAT function 
importMethyIDAT <- function(sampleInfo, dataPath=getwd(), lib=NULL, bigMatrix=FALSE, dir.bigMatrix='.', savePrefix.bigMatrix, ...) {
  
  if (missing(sampleInfo)) {
    stop('Please provide "sampleInfo"!')
  }
	if (is.character(sampleInfo)) {
		barcodes <- sampleInfo
		sampleInfo <- NULL
    barcodeInfo <- strsplit(barcodes, split='_')
    if (!all(sapply(barcodeInfo, length) == 2)) {
      warning('Some barcodes not in the correct format!')
      barcodeInfo <- lapply(barcodeInfo, function(x) {
        if (length(x) == 1) {
          x <- c(x, '')
        } else {
          x <- x[1:2]
        }
        return(x)
      })
    }
    barcodeInfo <- matrix(unlist(barcodeInfo), ncol=2)
    colnames(barcodeInfo) <- c('SENTRIX_BARCODE', 'SENTRIX_POSITION')
	} else {
		colnames(sampleInfo) <- toupper(colnames(sampleInfo))
		
		if (any(colnames(sampleInfo) == 'SENTRIX_ID') && !('SENTRIX_BARCODE' %in% colnames(sampleInfo))) {
			colnames(sampleInfo)[colnames(sampleInfo) == 'SENTRIX_ID'] <- 'SENTRIX_BARCODE'
		}
    if (!all(c('SENTRIX_BARCODE', 'SENTRIX_POSITION') %in% colnames(sampleInfo))) {
      stop("'SENTRIX_POSITION' and 'SENTRIX_BARCODE' or 'SENTRIX_ID' and are required in 'sampleInfo'!")
    }
    barcodeInfo <- sampleInfo[,c('SENTRIX_BARCODE', 'SENTRIX_POSITION')]
    barcodes <- paste(barcodeInfo[, 'SENTRIX_BARCODE'], barcodeInfo[, 'SENTRIX_POSITION'], sep='_')
  }
  
  ## read IDAT files
  ## check the folder location
  ## read data path could be either dataPath or dataPath/SENTRIX_BARCODE
  realDataPath <- dataPath
  for (i in 1:length(barcodes)) {
    file.pattern.i <- paste(barcodes[i], '.*\\.idat$', sep='')
    data.path.i <- file.path(dataPath, barcodeInfo[i, 'SENTRIX_BARCODE'])
    if (length(dir(data.path.i, pattern=file.pattern.i, ignore.case=T)) == 2) {
      realDataPath[i] <- data.path.i
    } else if (length(dir(dataPath, pattern=file.pattern.i, ignore.case=T)) != 2) {
      realDataPath[i] <- NA
    }
  }
  if (any(is.na(realDataPath))) {
    stop(paste('IDAT files cannot be located for barcodes:', paste(barcodes[is.na(realDataPath)], collapse=',')))
  }
  
  ## read IDAT files by individual dataPath
  barcodesList <- split(barcodes, factor(realDataPath, levels=unique(realDataPath)))
	if (bigMatrix) {
		## read first batch of samples and create a BigMatrix with all sample information
		## Then fill in the rest sample information
		if (missing(savePrefix.bigMatrix)) {
			warnings('savePrefix.bigMatrix is missing! bigMatrixFiles is used!')
			savePrefix.bigMatrix <- 'bigMatrixFiles'
		}
		
		for (i in 1:length(barcodesList)) {
			lumi450k.i <- lumIDAT(barcodesList[[i]], idatPath=names(barcodesList)[i], ...)
			if (i == 1) {
				dimNames <- list(featureNames(lumi450k.i), barcodes)
				## predefine a big matrix
				lumi450k <- lumi::asBigMatrix(lumi450k.i, nCol=length(barcodes), dimNames=dimNames, saveDir=dir.bigMatrix, savePrefix=savePrefix.bigMatrix)
			} else {
				## fill in data information of new samples
				for (ad.name in assayDataElementNames(lumi450k.i)) {
	        matrixAll.i <- assayDataElement(lumi450k, ad.name)
					if (is.null(matrixAll.i)) next
	        matrix.i <- assayDataElement(lumi450k.i, ad.name)
					matrixAll.i[, colnames(matrix.i)] <- matrix.i
			  }
			}
		}
	} else {
	  lumi450k <- lapply(1:length(barcodesList), function(i) lumIDAT(barcodesList[[i]], idatPath=names(barcodesList)[i], ...)) # return MethyLumiM object
	  suppressWarnings(lumi450k <- do.call('combine', lumi450k))
	}
  
  ## add sample info 
  if (!is.null(sampleInfo)) {
    ## update the barcodes, the order may change after split barcodes to barcodesList
    barcodes <- colnames(lumi450k)
    rownames(sampleInfo) <- paste(barcodeInfo[, 'SENTRIX_BARCODE'], barcodeInfo[, 'SENTRIX_POSITION'], sep='_')
    pData(lumi450k) <- sampleInfo[barcodes,]
    ## rename the samples if SAMPLE_NAME is provided in sampleInfo
    if (!is.null(sampleInfo$SAMPLE_NAME)) {
      samplename <- make.unique(sampleInfo[barcodes,'SAMPLE_NAME'])
      sampleNames(lumi450k) <- as.character(samplename)
    }
  }
  
	if (!is.null(lib)) {
		## add annotation information: 'COLOR_CHANNEL', 'CHROMOSOME', 'POSITION'
		lumi450k <- addAnnotationInfo(lumi450k, lib=lib)
	}

  return(lumi450k)
}


addAnnotationInfo <- function(methyLumiM, lib='FDb.InfiniumMethylation.hg19', annotationColumn=c('COLOR_CHANNEL', 'CHROMOSOME', 'POSITION')) {
  
 	if (is(methyLumiM, 'MethyLumiM')) {
	  # retrieve feature data
	  ff <- fData(methyLumiM)
    probeList <- featureNames(methyLumiM)
	} else if (is.character(methyLumiM)) {
		probeList <- methyLumiM
		ff <- data.frame()
	} else {
		stop('methyLumiM should be either a MethyLumiM object of a character vectors of probe names!')
	}
	
	if (!is.null(lib) && require(lib, character.only=TRUE)) {
		## For FeatureDb annotation libraries
		ff <- NULL
		if (exists(lib)) {
			lib <- get(lib)
			if (is(lib, 'FeatureDb')) {
				allAnnotation <- features(lib)
			}
			ff <- data.frame(ProbeID=probeList, CHROMOSOME=NA, POSITION=NA, COLOR_CHANNEL=NA)
			rownames(ff) <- probeList
			if (any(!(probeList %in% names(allAnnotation)))) {
			  missingProbe <- probeList[!(probeList %in% names(allAnnotation))]
				probeList <- probeList[probeList %in% names(allAnnotation)]
				warnings(paste(paste(missingProbe, collapse=','), 'probes do not exist in the annotation library!'))
			}
			allAnnotation <- allAnnotation[probeList]
			ff[probeList, 'CHROMOSOME'] <- as.character(seqnames(allAnnotation))
			ff[probeList, 'POSITION'] <- as.numeric(IRanges::start(allAnnotation))
			ff[probeList, 'COLOR_CHANNEL'] <- as.character(allAnnotation$channel450)
		} 

		if (is.null(ff)) {
			## For old annotation libraries: IlluminaHumanMethylation450k.db
			colorInfo <- chr <- loc <- rep(NA, length(probeList))
			names(colorInfo) <- names(chr) <- names(loc) <- probeList

	    obj <- get(paste(sub("\\.db$", "", lib), "COLORCHANNEL", sep=""))
			pp <- probeList[probeList %in% keys(obj)]
	    colorInfo[pp] <- sapply(AnnotationDbi::mget(pp, obj), function(x) x[1])
	    ff$COLOR_CHANNEL <- colorInfo

			## Check whether multiple versions of chromosome information is available,
			## If so, only use the latest version.
			chrPattern <- paste(sub("\\.db$", "", lib), "CHR", sep="")
			chrObjs <- ls(paste("package:", lib, sep=""), pattern=paste(chrPattern, "[0-9]+$", sep=""))
			if (length(chrObjs) > 1) {
				chrVersion <- sub(".*[^0-9]([0-9]+)$", "\\1", chrObjs)
				obj <- get(chrObjs[which.max(as.numeric(chrVersion))])
			} else {
				obj <- get(chrPattern)
			}
			pp <- probeList[probeList %in% keys(obj)]
	    chr[pp] <- sapply(AnnotationDbi::mget(pp, obj), function(x) x[1])
	    ff$CHROMOSOME <- as.character(chr)

	    obj <- get(paste(sub("\\.db$", "", lib), "CPGCOORDINATE", sep=""))
	 		pp <- probeList[probeList %in% keys(obj)]
	    loc[pp] <- sapply(AnnotationDbi::mget(pp, obj), function(x) x[1])
	    ff$POSITION <- as.numeric(as.character(loc))
		}

		
	} else {
		if (all(c('CHR', 'MAPINFO') %in% names(ff))) {
	    ff$CHROMOSOME <- ff$CHR
	    ff$POSITION <- as.numeric(ff$MAPINFO)
	  }
		if (!all(c('CHROMOSOME', 'POSITION', 'COLOR_CHANNEL') %in% names(ff))) {
			stop('Probe annotation information is not available. Please provide annotation library!')
		}
	}
	if (!is.data.frame(ff)) ff <- as.data.frame(ff, stringsAsFactors=FALSE)
	
	rownames(ff) <- ff$ProbeID
	if (is(methyLumiM, 'MethyLumiM')) {
	  fData(methyLumiM) <- ff
		return(methyLumiM)
	} else {
		return(ff)
	}
}


addControlData2methyLumiM <- function(controlData, methyLumiM, checkConsistency = TRUE, ...) 
{
  if (missing(methyLumiM) || missing(controlData)) stop('Both controlData and methyLumiM are required!')
  if (is.character(controlData)) {
    controlData <- methylumiR(controlData, ...)
  }
  if (is(controlData, "MethyLumiQC")) {
    ## Match the column names of controlData and LumiBatch object 
    ## Only keep the samples matching methyLumiM
    if (checkConsistency) {
      sampleID <- sampleNames(methyLumiM)
      controlSampleID <- sampleNames(controlData)
      if (all(sampleID %in% controlSampleID)) {
        controlData(methyLumiM) <- controlData[, sampleID]
      } else {
        stop('SampleNames do not match up between controlData and methyLumiM!')
      }
    } else {
      controlData(methyLumiM) <- controlData[, sampleID]
    }
  } else {
    cat("Provided controlData is not supported!\n")
  }

  return(methyLumiM)
}



# normalization
lumiMethyN <- function(methyLumiM, method = c('quantile', 'ssn', 'none'), separateColor=FALSE, verbose=TRUE, overwriteBigMatrix=FALSE, ...) 
{
  if (!is.function(method)) method <- match.arg(method)

  if (!is(methyLumiM, 'MethyLumiM') && !is(methyLumiM, 'MethyGenoSet')) {
    stop('The object should be class "MethyLumiM" or "MethyGenoSet"!')
  }
 
	if (is(assayDataElement(methyLumiM, 'exprs'), "BigMatrix") && !overwriteBigMatrix) {
		stop('BigMatrix was identified in the AssayData, please set overwriteBigMatrix parameter as TRUE if you want to overwrite the existing data.')
	} 
 
  history.submitted <- as.character(Sys.time())
  if (!(is.function(method))) {
    if (!(method %in% c('ssn', 'rssn', 'quantile', 'none'))) {
      cat('This method is not supported!\n')
      return(methyLumiM)
    } else if (method == 'none') {
      return(methyLumiM)
    }     
    if (verbose) cat(paste('Perform', method, 'normalization ...\n'))
  } else {
    if (verbose) cat('Perform user provided normalization ...\n')    
  }
      
  if (is.function(method)) {
    unmethy <- assayDataElement(methyLumiM, 'unmethylated')
    methy <- assayDataElement(methyLumiM, 'methylated')
    if (separateColor) {
      annotation <- pData(featureData(methyLumiM))
      if (is.null(annotation$COLOR_CHANNEL)) {
        cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
        combData <- rbind(methy, unmethy)
        processed.comb <- method(combData, ...)
        if (!is(processed.comb, 'matrix')) stop("The return of user defined method should be a matrix!\n")
        methy <- processed.comb[1:(nrow(processed.comb)/2), ]
        unmethy <- processed.comb[(nrow(processed.comb)/2+1):nrow(processed.comb), ]
      } else {
        allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
        allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')
        allRed <- rbind(methy[allRedInd,], unmethy[allRedInd,])
        allGrn <- rbind(methy[allGrnInd,], unmethy[allGrnInd,])
        processed.red <- method(allRed, ...)
        if (!is(processed.red, 'matrix')) stop("The return of user defined method should be a matrix!\n")
        processed.grn <- method(allGrn, ...)
        methy[allRedInd,] <- processed.red[1:(nrow(processed.red)/2), ]
        unmethy[allRedInd,] <- processed.red[(nrow(processed.red)/2+1):nrow(processed.red), ]
        methy[allGrnInd,] <- processed.grn[1:(nrow(processed.grn)/2), ]
        unmethy[allGrnInd,] <- processed.grn[(nrow(processed.grn)/2+1):nrow(processed.grn), ]
      }
    } else {
      combData <- rbind(methy, unmethy)
      processed.comb <- method(combData, ...)
      if (!is(processed.comb, 'matrix')) stop("The return of user defined method should be a matrix!\n")
      methy <- processed.comb[1:(nrow(processed.comb)/2), ]
      unmethy <- processed.comb[(nrow(processed.comb)/2+1):nrow(processed.comb), ]
    }
    methylated(methyLumiM) <- methy
    unmethylated(methyLumiM) <- unmethy
    methyLumiM <- estimateM(methyLumiM)
  } else {
    if (method == 'quantile') {
      methyLumiM <- normalizeMethylation.quantile(methyLumiM, separateColor=separateColor, ...)
    } else if (method == 'ssn') {
      methyLumiM <- normalizeMethylation.ssn(methyLumiM, separateColor=separateColor, ...)
    #} else if (method == 'rssn') {
    #  methyLumiM <- normalizeMethylation.robust.ssn(methyLumiM, separateColor=separateColor, ...)
    }
  }
  history.finished <- as.character(Sys.time())
  # history.command <- capture.output(print(match.call(lumiMethyN)))
  history.command <- paste(deparse(match.call(lumiMethyN)), collapse='') 
  lumiVersion <- packageDescription('lumi')$Version
  methyLumiM@history<- rbind(methyLumiM@history, data.frame(submitted=history.submitted, 
      finished=history.finished, command=history.command, lumiVersion=lumiVersion))
      
  return(methyLumiM)
}


# color balance adjustmentdim
lumiMethyC <- function(methyLumiM, method = c('quantile', 'ssn', 'none'), verbose=TRUE, overwriteBigMatrix=FALSE, ...) 
{
  if (!is.function(method)) method <- match.arg(method)

  if (!is(methyLumiM, 'MethyLumiM') && !is(methyLumiM, 'MethyGenoSet')) {
    stop('The object should be class "MethyLumiM" or "MethyGenoSet"!')
  }

	if (is(assayDataElement(methyLumiM, 'exprs'), "BigMatrix") && !overwriteBigMatrix) {
		stop('BigMatrix was identified in the AssayData, please set overwriteBigMatrix parameter as TRUE if you want to overwrite the existing data.')
	} 

  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL))
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
  
  history.submitted <- as.character(Sys.time())
  if (!(is.function(method))) {
    if (!(method %in% c('quantile', 'ssn', 'none'))) {
      cat('This method is not supported!\n')
      return(methyLumiM)
    } else if (method == 'none') {
      return(methyLumiM)
    }     
    if (verbose) cat(paste('Perform', method, 'color balance adjustment ...\n'))
  } else {
    if (verbose) cat('Perform user provided color balance adjustment ...\n')    
  }
      
  if (is.function(method)) {
    unmethy <- assayDataElement(methyLumiM, 'unmethylated')
    methy <- assayDataElement(methyLumiM, 'methylated')
    allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
    allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')
    allRed <- rbind(methy[allRedInd,], unmethy[allRedInd,])
    allGrn <- rbind(methy[allGrnInd,], unmethy[allGrnInd,])

    processedData <- method(allRed, allGrn, ...)
    processed.red <- processedData$red
    if (is.null(processed.red)) stop("The return of user defined method should be a list including 'red' and 'green' matrix!\n")
    processed.grn <- processedData$green
    methy[allRedInd,] <- processed.red[1:(nrow(processed.red)/2), ]
    unmethy[allRedInd,] <- processed.red[(nrow(processed.red)/2+1):nrow(processed.red), ]
    methy[allGrnInd,] <- processed.grn[1:(nrow(processed.grn)/2), ]
    unmethy[allGrnInd,] <- processed.grn[(nrow(processed.grn)/2+1):nrow(processed.grn), ]
    methylated(methyLumiM) <- methy
    unmethylated(methyLumiM) <- unmethy
    methyLumiM <- estimateM(methyLumiM)
  } else {
    if (method == 'quantile') {
      methyLumiM <- adjColorBias.quantile(methyLumiM, verbose=verbose, ...)
    } else if (method == 'ssn') {
      methyLumiM <- adjColorBias.ssn(methyLumiM, ...)
    } 
  }
  history.finished <- as.character(Sys.time())
  # history.command <- capture.output(print(match.call(lumiMethyC)))
  history.command <- paste(deparse(match.call(lumiMethyC)), collapse='') 
  
  lumiVersion <- packageDescription('lumi')$Version
  methyLumiM@history<- rbind(methyLumiM@history, data.frame(submitted=history.submitted, 
      finished=history.finished, command=history.command, lumiVersion=lumiVersion))
      
  return(methyLumiM)
}


lumiMethyB <- function(methyLumiM, method = c('bgAdjust2C', 'forcePositive', 'none'), separateColor=FALSE, verbose=TRUE, overwriteBigMatrix=FALSE, ...) 
{
  if (!is.function(method)) method <- match.arg(method)

  if (is(methyLumiM, 'MethyLumiM') || is(methyLumiM, 'MethyGenoSet')) {
    unmethy <- assayDataElement(methyLumiM, 'unmethylated')
    methy <- assayDataElement(methyLumiM, 'methylated')
  } else {
    stop('The object should be class "MethyLumiM" or "MethyGenoSet" inherited!')
  }
	
	if (is(assayDataElement(methyLumiM, 'exprs'), "BigMatrix") && !overwriteBigMatrix) {
		stop('BigMatrix was identified in the AssayData, please set overwriteBigMatrix parameter as TRUE if you want to overwrite the existing data.')
	} 
  
  history.submitted <- as.character(Sys.time())
  if (!(is.function(method))) {
    if (!(method %in% c('bgAdjust2C', 'none', 'forcePositive'))) {
      cat('This method is not supported!\n')
      return(methyLumiM)
    } else if (method == 'none') {
      return(methyLumiM)
    } else if (method == 'forcePositive') {
      mm <- min(c(min(methy, na.rm=TRUE), min(unmethy, na.rm=TRUE)))
      if (mm > 0)  return(methyLumiM)
    }    
    if (verbose) cat(paste('Perform', method, 'background correction ...\n'))
  } else {
    if (verbose) cat('Perform user provided background correction ...\n')    
  }
      
  if (is.function(method)) {
    if (separateColor) {
      annotation <- pData(featureData(methyLumiM))
      if (is.null(annotation$COLOR_CHANNEL)) {
        cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
        combData <- rbind(methy, unmethy)
        bgAdj.comb <- method(combData, ...)
        if (!is(bgAdj.comb, 'matrix')) stop("The return of user defined method should be a matrix!\n")
        methy <- bgAdj.comb[1:(nrow(bgAdj.comb)/2), ]
        unmethy <- bgAdj.comb[(nrow(bgAdj.comb)/2+1):nrow(bgAdj.comb), ]
      } else {
        allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
        allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')
        allRed <- rbind(methy[allRedInd,], unmethy[allRedInd,])
        allGrn <- rbind(methy[allGrnInd,], unmethy[allGrnInd,])
        bgAdj.red <- method(allRed, ...)
        if (!is(bgAdj.red, 'matrix')) stop("The return of user defined method should be a matrix!\n")
        bgAdj.grn <- method(allGrn, ...)
        methy[allRedInd,] <- bgAdj.red[1:(nrow(bgAdj.red)/2), ]
        unmethy[allRedInd,] <- bgAdj.red[(nrow(bgAdj.red)/2+1):nrow(bgAdj.red), ]
        methy[allGrnInd,] <- bgAdj.grn[1:(nrow(bgAdj.grn)/2), ]
        unmethy[allGrnInd,] <- bgAdj.grn[(nrow(bgAdj.grn)/2+1):nrow(bgAdj.grn), ]
      }
    } else {
      combData <- rbind(methy, unmethy)
      bgAdj.comb <- method(combData, ...)
      if (!is(bgAdj.comb, 'matrix')) stop("The return of user defined method should be a matrix!\n")
      methy <- bgAdj.comb[1:(nrow(bgAdj.comb)/2), ]
      unmethy <- bgAdj.comb[(nrow(bgAdj.comb)/2+1):nrow(bgAdj.comb), ]
    }
    methylated(methyLumiM) <- methy
    unmethylated(methyLumiM) <- unmethy
  } else {
    if (method == 'bgAdjust2C') {
      methyLumiM <- bgAdjustMethylation(methyLumiM, separateColor=separateColor, ...)
    } else if (method == 'forcePositive') {
      x.matrix <- rbind(methy, unmethy)
      offset <- apply(x.matrix, 2, min, na.rm=TRUE)
      offset[offset <= 0] <- offset[offset <= 0] - 0.01
      offset[offset > 0] <- 0
      offset <- rep(1, nrow(x.matrix)) %*% t(offset)
      x.matrix <- x.matrix - offset
      methylated(methyLumiM) <- x.matrix[1:nrow(methy),]
      unmethylated(methyLumiM) <- x.matrix[(nrow(unmethy)+1):nrow(x.matrix),]
    } 
  }
  methyLumiM <- estimateM(methyLumiM)
  history.finished <- as.character(Sys.time())
  # history.command <- capture.output(print(match.call(lumiMethyB)))
  history.command <- paste(deparse(match.call(lumiMethyB)), collapse='') 
  
  lumiVersion <- packageDescription('lumi')$Version
  methyLumiM@history<- rbind(methyLumiM@history, data.frame(submitted=history.submitted, 
      finished=history.finished, command=history.command, lumiVersion=lumiVersion))
      
  return(methyLumiM)
}


bgAdjustMethylation <- function(methyLumiM, separateColor=FALSE, targetBGLevel=300, negPercTh=0.25) {
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  methy <- methylated(methyLumiM)
  unmethy <- unmethylated(methyLumiM)
  
  ## check whether control data is available in the MethyLumiM object
  bglevel <- estimateMethylationBG(methyLumiM, separateColor=separateColor)
  if (is.null(colnames(bglevel))) separateColor <- FALSE
  if (separateColor) {
    bg.red <- bglevel[,'red']
    bg.grn <- bglevel[,'green']
    
    annotation <- pData(featureData(methyLumiM))
    allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
    allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')

    ## For 450K chip, the Infinium II type only has one type of probe for each CpG site. 
    ## The color channel will depend on the methylation status
    allBothInd <- which(annotation$COLOR_CHANNEL == 'Both' | annotation$COLOR_CHANNEL == '')
    
    methy[allRedInd, ] <- methy[allRedInd, ] - rep(1, length(allRedInd)) %*% t(bg.red)
    methy[allGrnInd, ] <- methy[allGrnInd, ] - rep(1, length(allGrnInd)) %*% t(bg.grn)
    unmethy[allRedInd, ] <- unmethy[allRedInd, ] - rep(1, length(allRedInd)) %*% t(bg.red)
    unmethy[allGrnInd, ] <- unmethy[allGrnInd, ] - rep(1, length(allGrnInd)) %*% t(bg.grn)
    
    if (length(allBothInd) > 0) {
      ## the methylated probe has 'Grn' color, while the unmethylated probe has 'Red' color
      unmethy[allBothInd, ] <- unmethy[allBothInd,] - rep(1, length(allBothInd)) %*% t(bg.red) 
      methy[allBothInd, ] <- methy[allBothInd,] - rep(1, length(allBothInd)) %*% t(bg.grn) 
    }
  } else {
    if (is.matrix(bglevel)) bglevel <- rowMeans(bglevel)
		if (is(methy, 'BigMatrix')) {
			for (i in 1:ncol(methy)) {
		    methy[,i] <- methy[,i] - bglevel[i]
		    unmethy[,i] <- unmethy[,i] - bglevel[i]
			}
		} else {
	    methy <- methy - rep(1, nrow(methy)) %*% t(bglevel)
	    unmethy <- unmethy - rep(1, nrow(unmethy)) %*% t(bglevel)
		}
  }

  ## check possible error of BG estimation model
	if (is(methy, 'BigMatrix')) {
	  negPerc.methy <- bigmemoryExtras::apply(methy, 2, function(x) length(which(x < 0))/nrow(methy))
	  negPerc.unmethy <- bigmemoryExtras::apply(unmethy, 2, function(x) length(which(x < 0))/nrow(unmethy))
	} else {
	  negPerc.methy <- apply(methy, 2, function(x) length(which(x < 0))/nrow(methy))
	  negPerc.unmethy <- apply(unmethy, 2, function(x) length(which(x < 0))/nrow(unmethy))
	}
  negPerc <- pmax(negPerc.methy, negPerc.unmethy)

  if (any(negPerc > negPercTh)) {
    warning("Possible bad quality samples or possible error of Background estimation model: \n")
    cat("\t", paste(colnames(methy)[negPerc > negPercTh], collapse=", "), "\n")
  }
	
	if (is(methy, 'BigMatrix')) {
		for (i in 1:ncol(methy)) {
	    methy[,i] <- methy[,i] + targetBGLevel
	    unmethy[,i] <- unmethy[,i] + targetBGLevel
		}
	} else {
		methy <- methy + targetBGLevel
	  unmethy <- unmethy + targetBGLevel
	}
	methylated(methyLumiM) <- methy 
  unmethylated(methyLumiM) <- unmethy

  if (is(methyLumiM, "MethyLumiM") || is(methyLumiM, 'MethyGenoSet')) methyLumiM <- estimateM(methyLumiM)
  
  attr(methyLumiM, "EstimatedBG") <- list(rawBG=bglevel, adjBG=rep(targetBGLevel, ncol(methy)))  
  return(methyLumiM)
}



## ---------------------------------------------------------------
# methy.raw1.adj <- adjColorBias(methy.raw1)
# methy.raw2.adj <- adjColorBias(methy.raw2)
# methy.adj <- cbind(methylated(methy.raw1.adj), methylated(methy.raw2.adj))
# unmethy.adj <- cbind(unmethylated(methy.raw1.adj), unmethylated(methy.raw2.adj))
# M.adj <- log2(methy.adj/unmethy.adj)
adjColorBias.ssn <- function(methyLumiM, refChannel=c("green", "red", "mean")) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  if (storageMode(assayData(methyLumiM)) == "environment") storageMode(assayData(methyLumiM)) <- "lockedEnvironment"
  refChannel <- match.arg(refChannel)
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
  if (is.null(unmethy) || is.null(methy)) stop("methylated or unmethylated data is not included in the dataset!\n")
  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL)) {
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
    return(methyLumiM)
  }
  
  allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
  allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')

  ## For 450K chip, the Infinium II type only has one type of probe for each CpG site. 
  ## The color channel will depend on the methylation status
  allBothInd <- which(annotation$COLOR_CHANNEL == 'Both' | annotation$COLOR_CHANNEL == '')
  
  bg <- estimateMethylationBG(methyLumiM, separateColor=TRUE)
  bg.red <- bg[,"red"]
  bg.grn <- bg[,"green"]
  
	## To be compatible with BigMatrix data
	for (i in 1:ncol(methy)) {
	  intensity.grn <- unmethy[allGrnInd,i] + methy[allGrnInd,i]
	  intensity.red <- unmethy[allRedInd,i] + methy[allRedInd,i]
	  m.int.grn <- mean(intensity.grn) - bg.grn[i]
	  m.int.red <- mean(intensity.red) - bg.red[i]
	  if (refChannel == 'green') {
	    m.ref <- m.int.grn
	    bg.ref <- bg.grn[i]
	  } else if (refChannel == 'red') {
	    m.ref <- m.int.red
	    bg.ref <- bg.red[i]
	  } else {
	    m.ref <- (m.int.grn + m.int.red)/2
	    bg.ref <- (bg.grn[i] + bg.red[i])/2
	  }
	  unmethy[allRedInd, i] <- unmethy[allRedInd,i] -  bg.red[i] * m.ref / m.int.red + bg.ref
	  unmethy[allGrnInd, i] <- unmethy[allGrnInd,i] - bg.grn[i] * m.ref / m.int.grn + bg.ref
	  methy[allRedInd, i] <- methy[allRedInd,i] - bg.red[i] * m.ref / m.int.red + bg.ref
	  methy[allGrnInd, i] <- methy[allGrnInd,i] - bg.grn[i] * m.ref / m.int.grn + bg.ref

	  if (length(allBothInd) > 0) {
	    ## the methylated probe has 'Grn' color, while the unmethylated probe has 'Red' color
	    unmethy[allBothInd, i] <- unmethy[allBothInd,i] - bg.red[i] * m.ref / m.int.red + bg.ref
	    methy[allBothInd, i] <- methy[allBothInd,i] - bg.grn[i] * m.ref / m.int.grn + bg.ref
	  }
	}
    
  methy.adj <- methyLumiM
  assayDataElement(methy.adj, 'unmethylated') <- unmethy
  assayDataElement(methy.adj, 'methylated') <- methy
  if (is(methy.adj, "MethyLumiM") || is(methy.adj, 'MethyGenoSet')) methy.adj <- estimateM(methy.adj)
  return(methy.adj)
}


adjColorBias.quantile <- function(methyLumiM, refChannel=c("green", "red"), logMode=TRUE, verbose=TRUE, ...) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  if (storageMode(assayData(methyLumiM)) == "environment") storageMode(assayData(methyLumiM)) <- "lockedEnvironment"
  refChannel <- match.arg(refChannel)
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL)) {
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
    return(methyLumiM)
  }
  redInd <- annotation$COLOR_CHANNEL == 'Red'
  grnInd <- annotation$COLOR_CHANNEL == 'Grn'

  ## For 450K chip, the Infinium II type only has one type of probe for each CpG site. 
  ## The color channel will depend on the methylation status
  bothInd <- which(annotation$COLOR_CHANNEL == 'Both' | annotation$COLOR_CHANNEL == '')
  bandwidth <- ifelse(logMode, 0.3, 200)
  for (i in 1:ncol(unmethy)) {
		if (verbose) cat('Processing sample', colnames(unmethy)[i], '...\n')
    red.a.i <- unmethy[redInd, i]
    red.b.i <- methy[redInd, i]
    red.i <- c(red.a.i, red.b.i)
    grn.a.i <- unmethy[grnInd, i]
    grn.b.i <- methy[grnInd, i]
    grn.i <- c(grn.a.i, grn.b.i)

    if (refChannel == 'green') {
      ## For 450K data
      if (length(bothInd) > 0) {
        ## the methylated probe has 'Grn' color, while the unmethylated probe has 'Red' color
        y.out.i <- smoothQuantileNormalization(red.i, grn.i, adjData=c(red.i, unmethy[bothInd,i]), logMode=logMode, bandwidth=bandwidth, verbose=verbose, ...)
        unmethy[bothInd,i] <- y.out.i[(length(red.i)+1):length(y.out.i)]
        # y.out.i <- smoothQuantileNormalization(red.i, grn.i, adjData=c(red.i, methy[bothInd,i]), logMode=logMode, bandwidth=bandwidth, verbose=verbose, ...)
        # methy[bothInd,i] <- y.out.i[(length(red.i)+1):length(y.out.i)]
      } else {
        y.out.i <- smoothQuantileNormalization(red.i, grn.i, logMode=logMode, bandwidth=bandwidth, verbose=verbose, ...)
      }
      unmethy[redInd, i] <- y.out.i[1:length(red.a.i)]
      methy[redInd, i] <- y.out.i[(length(red.a.i)+1):length(red.i)]
    } else {
      if (length(bothInd) > 0) {
        ## the methylated probe has 'Grn' color, while the unmethylated probe has 'Red' color
        y.out.i <- smoothQuantileNormalization(grn.i, red.i, adjData=c(grn.i, methy[bothInd,i]), logMode=logMode, bandwidth=bandwidth, verbose=verbose, ...)
        methy[bothInd,i] <- y.out.i[(length(grn.i)+1):length(y.out.i)]
      } else {
        y.out.i <- smoothQuantileNormalization(grn.i, red.i, logMode=logMode, bandwidth=bandwidth, verbose=verbose, ...)
      }
      unmethy[grnInd, i] <- y.out.i[1:length(grn.a.i)]
      methy[grnInd, i] <- y.out.i[(length(grn.a.i)+1):length(grn.i)]
    }
  }

  methy.adj <- methyLumiM
  assayDataElement(methy.adj, 'unmethylated') <- unmethy
  assayDataElement(methy.adj, 'methylated') <- methy
  if (is(methy.adj, "MethyLumiM") || is(methy.adj, 'MethyGenoSet')) methy.adj <- estimateM(methy.adj)
  return(methy.adj)
}


smoothQuantileNormalization <- function(dataMatrix, ref=NULL, adjData=NULL, logMode=TRUE, bandwidth=NULL, degree=1, verbose=FALSE, ...) {
  
  # require(KernSmooth)
  bandwidth <- ifelse(logMode, 0.3, 200)
  if (!is.matrix(dataMatrix)) dataMatrix <- matrix(dataMatrix, ncol=1)
  if (is.null(ref)) {
    # normData <- normalize.quantiles.robust(dataMatrix + 0.0)
    normData <- limma::normalizeQuantiles(dataMatrix)
    ref <- normData[,1]
  } 
	
  if (!is.null(adjData)) {
    if (!is.matrix(adjData)) adjData <- matrix(adjData, ncol=1)
    if (ncol(dataMatrix) != ncol(adjData))
      stop("The number of columns of adjData should be consistent with dataMatrix!") 
  }
  interpolationMode <- ifelse(length(ref) != nrow(dataMatrix), TRUE, FALSE) 
  
  if (logMode) {
    ## remove those equal or less than 0, which are unreliable values
    if (interpolationMode) ref <- ref[ref > 0]
    if (min(ref, na.rm=TRUE) < 1) ref <- ref - min(ref, na.rm=TRUE) + 1
    ref <- log2(ref)
  }
	## remove NA in ref
	ref <- ref[!is.na(ref)]
  len <- nrow(dataMatrix)
  refLen <- length(ref)
  gridsize <- min(min(len, refLen)/2, 1000)

  # In the case of different lengthes between reference and data, interpolation will be performed.
  if (interpolationMode) {
    x <- (1:refLen)/refLen
    y <- sort(ref, decreasing=FALSE)
  } else {
    y <- sort(ref, decreasing=FALSE)
  }
  
  # smoothing the quantile normalization results
  if (is.null(adjData)) {
    normData <- dataMatrix
  } else {
    normData <- adjData
  }
  for (i in 1:ncol(dataMatrix)) {
	  # if (verbose)  cat('Processing sample', colnames(dataMatrix)[i], '...\n')
    profile.i <- dataMatrix[,i]
		profile.naInd.i <- which(is.na(profile.i))
		if (length(profile.naInd.i) > 0)
			profile.i <- profile.i[-profile.naInd.i]
    if (!is.null(adjData)) {
      adjData.i <- adjData[,i]
    } else {
      adjData.i <- profile.i
    }
    ## remove possible NAs
    naInd.i <- is.na(adjData.i)
    adjData.i <- adjData.i[!naInd.i]

    if (logMode) {
      mm.i <- min(c(profile.i, adjData.i), na.rm=TRUE)
      if (mm.i < 1) {
        adjData.i <- log2(adjData.i - mm.i + 1)
        profile.i <- log2(profile.i - mm.i + 1)
      } else {
        adjData.i <- log2(adjData.i)
        profile.i <- log2(profile.i)
      }
    }

    if (interpolationMode) {
      ## remove those equal or less than 0, which are unreliable values
      selProfile.i <- profile.i[profile.i > 0]
      len <- length(selProfile.i)
      
      # perform linear interpolation when the length of two profiles different
      x.out.i <- rank(selProfile.i)/len 
      y.out.i <- approx(x=x, y=y, xout=x.out.i, method="linear", rule=2)$y    
    } else {
      selProfile.i <- profile.i
      y.out.i <- y[rank(selProfile.i)]
    }
    
    ## remove outliers points at two ends
		rank.i <- rank(selProfile.i, ties.method='min')
    lowInd.i <- which(rank.i <= 500)
    highInd.i <- which(rank.i > max(rank.i) - 500)
    #boundary.i <- quantile(selProfile.i, c(0.3, 0.7))
    #lowInd.i <- selProfile.i < boundary.i[1]
    #highInd.i <- selProfile.i > boundary.i[2]
    ## fit the low segment
    suppressWarnings(rlm.i <- rlm(selProfile.i[lowInd.i], y.out.i[lowInd.i]))
    dd.i <- y.out.i[lowInd.i] - rlm.i$fitted.values
    outlier.ind.low.i <- lowInd.i[which(abs(dd.i) > 3 * sd(dd.i))]
    ## fit the high segment
    suppressWarnings(rlm.i <- rlm(selProfile.i[highInd.i], y.out.i[highInd.i]))
    dd.i <- y.out.i[highInd.i] - rlm.i$fitted.values
    outlier.ind.high.i <- highInd.i[which(abs(dd.i) > 3 * sd(dd.i))]
    outlier.ind.i <- c(outlier.ind.low.i, outlier.ind.high.i)
    if (length(outlier.ind.i) > 0) {
      tt <- locpoly(selProfile.i[-outlier.ind.i], y.out.i[-outlier.ind.i], degree=degree,  gridsize=gridsize, bandwidth=bandwidth, ...)
      # test <- monoSpline(x=selProfile.i[-outlier.ind.i], y=y.out.i[-outlier.ind.i], newX=adjData.i, nKnots=50, ifPlot=FALSE)
    } else {
      tt <- locpoly(selProfile.i, y.out.i, degree=degree,  gridsize=gridsize, bandwidth=bandwidth, ...)    
    }

    norm.i <- approx(x=tt$x, y=tt$y, xout=adjData.i, rule=2)$y
    ## check the rank difference before and after fitting
    rank.old.i <- rank(adjData.i, ties.method='min')
    rank.new.i <- rank(norm.i, ties.method='min')
    suppressWarnings(rank.fit.i <- rlm(new~old, data.frame(new=rank.new.i, old=rank.old.i)))
    rank.diff.ind.i <- which(abs(rank.fit.i$residuals) > 1)
    ## Add na and infinite indexes
    rank.diff.ind.i <- unique(c(rank.diff.ind.i, which(is.na(norm.i)), which(is.infinite(norm.i))))
    if (length(rank.diff.ind.i) > 0) {
      rank.na.i <- rank.old.i[rank.diff.ind.i]
      replaceVal.i <- NULL
      for (rank.na.ij in unique(rank.na.i)) {
        down.ij <- sort(norm.i[rank.old.i < rank.na.ij & !(rank.old.i %in% rank.na.i)], decreasing=TRUE)[1]
        up.ij <- sort(norm.i[rank.old.i > rank.na.ij & !(rank.old.i %in% rank.na.i)], decreasing=FALSE)[1]
				norm.i[rank.diff.ind.i[rank.na.i == rank.na.ij]] <- mean(c(up.ij, down.ij), na.rm=TRUE)
      }
    }
		if (length(profile.naInd.i) > 0) {
			tmp.i <- norm.i
			norm.i <- rep(NA, nrow(normData))
			norm.i[-profile.naInd.i] <- tmp.i
		}
    
    # plot(selProfile.i, y.out.i, pch='.')
    # lines(tt, col=2)
    # if (max(norm.i) > 30) browser()

    if (logMode) {
      normData[!naInd.i,i] <- 2^norm.i
    } else {
      normData[!naInd.i,i] <- norm.i
    }
  }

  return(normData)
}


estimateMethylationBG <- function(methyLumiM, separateColor=FALSE, nbin=1000) {
    
  estimateBG <- function(dataMatrix, nbin=1000) {
    if (is(dataMatrix, 'BigMatrix')) {
      bg <- bigmemoryExtras::apply(dataMatrix, 2, function(x) {
         hh.x <- hist(x, nbin, plot=FALSE)
         Th <- hh.x$breaks[which.max(hh.x$counts) + 1] * 2
         dd.x <- density(x[x < Th], na.rm=TRUE)
         bg.x <- dd.x$x[which.max(dd.x$y)]    
       })
    } else {
      bg <- apply(dataMatrix, 2, function(x) {
        hh.x <- hist(x, nbin, plot=FALSE)
        Th <- hh.x$breaks[which.max(hh.x$counts) + 1] * 2
        dd.x <- density(x[x < Th], na.rm=TRUE)
        bg.x <- dd.x$x[which.max(dd.x$y)]    
      })
    }

    return(bg)
  }
  
  ## check whether the control data is available
  controlData <- NULL
  if (is(methyLumiM, "MethyLumiM")) {
    controlData <- controlData(methyLumiM)
  }
  if (is(methyLumiM, "MethyLumiSet")) {
    controlData <- QCdata(methyLumiM)
  }
  if (is(controlData, "MethyLumiQC")) {
    # For control data, methylated data corresponds to green channel, "Signal_Grn"
    # For control data, unmethylated data corresponds to red channel, "Signal_Red"
    grnData <- assayDataElement(controlData, "methylated") 
    redData <- assayDataElement(controlData, "unmethylated")
    allControlType <- sapply(strsplit(featureNames(controlData), "\\."), function(x) x[1])
    allControlType <- toupper(allControlType)
    neg.ind <- which(allControlType == "NEGATIVE") 
    if (length(neg.ind) > 0) {
      bg.grn <- apply(grnData[neg.ind, ,drop=F], 2, median)
      bg.red <- apply(redData[neg.ind, ,drop=F], 2, median)
      if (separateColor) {
        bg <- cbind(red=bg.red, green=bg.grn)
      } else {
        bg <- pmin(bg.red, bg.grn)
      }
      return(bg)
    }
  }
  
  ## In the case the negative control data is not available, the background will be estimated based on the mode positions of unmethylated or methylated distribution (the smaller one)
  if (is(methyLumiM, "eSet")) {
		if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
      stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
    }
    unmethy <- assayDataElement(methyLumiM, 'unmethylated')
    methy <- assayDataElement(methyLumiM, 'methylated')
    if (separateColor) {
      annotation <- pData(featureData(methyLumiM))
      if (is.null(annotation$COLOR_CHANNEL)) {
        cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
        bg.methy <- estimateBG(methy, nbin=nbin)
        bg.unmethy <- estimateBG(unmethy, nbin=nbin)
        bg <- pmin(bg.methy, bg.unmethy)
      } else {
        allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
        allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')
        bg.methy.red <- estimateBG(methy[allRedInd,], nbin=nbin)
        bg.unmethy.red <- estimateBG(unmethy[allRedInd,], nbin=nbin)
        bg.red <- pmin(bg.methy.red, bg.unmethy.red)

        bg.methy.grn <- estimateBG(methy[allGrnInd,], nbin=nbin)
        bg.unmethy.grn <- estimateBG(unmethy[allGrnInd,], nbin=nbin)
        bg.grn <- pmin(bg.methy.grn, bg.unmethy.grn)
        bg <- cbind(red=bg.red, green=bg.grn)
      }
    } else {
      bg.methy <- estimateBG(methy, nbin=nbin)
      bg.unmethy <- estimateBG(unmethy, nbin=nbin)
      bg <- pmin(bg.methy, bg.unmethy)
    }
  } else if (is(methyLumiM, 'matrix')) {
    bg <- estimateBG(methyLumiM, nbin=nbin)
  } else {
    stop("The input data class is not supported!\n")
  }
  
  return(bg)
}



normalizeMethylation.ssn <- function(methyLumiM, separateColor=FALSE) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  if (storageMode(assayData(methyLumiM)) == "environment") storageMode(assayData(methyLumiM)) <- "lockedEnvironment"  
  unmethy <- assayDataElement(methyLumiM, 'unmethylated')
  methy <- assayDataElement(methyLumiM, 'methylated')
  
  # estimate the background based on methy
  bg <- estimateMethylationBG(methyLumiM, separateColor=separateColor)
  
  if (is.null(colnames(bg))) separateColor <- FALSE
  if (separateColor) {
    bg.red <- bg[,'red']
    bg.grn <- bg[,'green']
    meanBg.red <- mean(bg.red)
    meanBg.grn <- mean(bg.grn)

    annotation <- pData(featureData(methyLumiM))
    allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
    allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')

    intensity.red <- unmethy[allRedInd, ] + methy[allRedInd, ]
    totalIntensity.red <- colSums(intensity.red)
    meanTotalIntensity.red <- mean(totalIntensity.red)

    intensity.grn <- unmethy[allGrnInd, ] + methy[allGrnInd, ]
    totalIntensity.grn <- colSums(intensity.grn)
    meanTotalIntensity.grn <- mean(totalIntensity.grn)
    
    methy[allRedInd, ] <- (methy[allRedInd, ] - rep(1, length(allRedInd)) %*% t(bg.red)) * (rep(1, length(allRedInd)) %*% t(meanTotalIntensity.red/totalIntensity.red)) + meanBg.red
    methy[allGrnInd, ] <- (methy[allGrnInd, ] - rep(1, length(allGrnInd)) %*% t(bg.grn)) * (rep(1, length(allGrnInd)) %*% t(meanTotalIntensity.grn/totalIntensity.grn)) + meanBg.grn
    unmethy[allRedInd, ] <- (unmethy[allRedInd, ] - rep(1, length(allRedInd)) %*% t(bg.red)) * (rep(1, length(allRedInd)) %*% t(meanTotalIntensity.red/totalIntensity.red)) + meanBg.red
    unmethy[allGrnInd, ] <- (unmethy[allGrnInd, ] - rep(1, length(allGrnInd)) %*% t(bg.grn)) * (rep(1, length(allGrnInd)) %*% t(meanTotalIntensity.grn/totalIntensity.grn)) + meanBg.grn
  } else {
    meanBg <- mean(bg)
		
		if (is(methy, 'BigMatrix')) {
			totalIntensity <- NULL
			for (i in 1:ncol(methy)) {
		    intensity.i <- unmethy[,i] + methy[,i]
		    totalIntensity <- c(totalIntensity, sum(intensity.i))
			}
		  meanTotalIntensity <- mean(totalIntensity)
			for (i in 1:ncol(methy)) {
		  	unmethy[,i] <- (unmethy[,i] - bg[i]) * (meanTotalIntensity[i]/totalIntensity) + meanBg
		    methy[,i] <- (methy[,i] - bg[i]) * (meanTotalIntensity[i]/totalIntensity) + meanBg
			}
		} else {
	    intensity <- unmethy + methy
	    totalIntensity <- colSums(intensity)
	    meanTotalIntensity <- mean(totalIntensity)
	    unmethy <- (unmethy - rep(1, nrow(intensity)) %*% t(bg)) * (rep(1, nrow(intensity)) %*% t(meanTotalIntensity/totalIntensity)) + meanBg
	    methy <- (methy - rep(1, nrow(intensity)) %*% t(bg)) * (rep(1, nrow(intensity)) %*% t(meanTotalIntensity/totalIntensity)) + meanBg
		}
  } 
  
  assayDataElement(methyLumiM, 'unmethylated') <- unmethy
  assayDataElement(methyLumiM, 'methylated') <- methy
  if (is(methyLumiM, "MethyLumiM") || is(methyLumiM, "MethyGenoSet")) methyLumiM <- estimateM(methyLumiM)
  return(methyLumiM)
}


## this internal function was based on limma::normalizeQuantiles
.estimate.quantile.reference <- function(dataMatrix, ..., batchSize=50, repTime=NULL, ties=TRUE) {
	otherDataMatrix <- list(...)

	## subsampling the data to estimate the reference when the size is bigger than the batchSize
	reference <- NULL
	if (ncol(dataMatrix) > batchSize) {
		repTime <- ncol(dataMatrix) / batchSize * 2
		if (ncol(dataMatrix) < batchSize) {
			batchSize <- ncol(dataMatrix)
			repTime <- 1
		}
		for (i in 1:repTime) {
			ind.i <- sample(1:ncol(dataMatrix), batchSize)
			dataMatrix.i <- dataMatrix[, ind.i]
			if (length(otherDataMatrix) > 0) {
				for (j in 1:length(otherDataMatrix)) {
					dataMatrix.i <- rbind(dataMatrix.i, otherDataMatrix[[j]][, ind.i])
				}
			}
			reference.i <- .estimate.quantile.reference(dataMatrix.i)
			reference <- cbind(reference, reference.i)
		}
		reference <- rowMeans(reference)
		return(reference)
	} else {
		dataMatrix <- dataMatrix[,]
		if (length(otherDataMatrix) > 0) {
			for (j in 1:length(otherDataMatrix)) {
				dataMatrix <- rbind(dataMatrix, otherDataMatrix[[j]][,])
			}
		}
	  dm <- dim(dataMatrix)
	  if (is.null(dm)) 
	      return(dataMatrix)
	  if (dm[2] == 1) 
	      return(dataMatrix)
	  O <- S <- array(, dm)
	  nobs <- rep(dm[1], dm[2])
	  i <- (0:(dm[1] - 1))/(dm[1] - 1)
	  for (j in 1:dm[2]) {
	      Si <- sort(dataMatrix[, j], method = "quick", index.return = TRUE)
	      nobsj <- length(Si$x)
	      if (nobsj < dm[1]) {
	          nobs[j] <- nobsj
	          isna <- is.na(dataMatrix[, j])
	          S[, j] <- approx((0:(nobsj - 1))/(nobsj - 1), Si$x, 
	              i, ties = "ordered")$y
	          O[!isna, j] <- ((1:dm[1])[!isna])[Si$ix]
	      }
	      else {
	          S[, j] <- Si$x
	          O[, j] <- Si$ix
	      }
	  }
	  reference <- rowMeans(S)	
		return(reference)		
	}
}


## this internal function was based on limma::normalizeQuantiles
.quantileNormalization.reference <- function(dataMatrix, reference=NULL, ties = TRUE) {
		
	dm <- dim(dataMatrix)
  if (is.null(dm)) {
		dataMatrix <- matrix(dataMatrix, ncol=1)
		dm <- dim(dataMatrix)
	}

  if (dm[2] == 1 && is.null(reference)) 
      return(dataMatrix)
	if (!is.null(reference)) {
		if (length(reference) != dm[1]) stop('The length of reference does not match the dimension of dataMatrix!')
	}
  O <- S <- array(, dm)
  nobs <- rep(dm[1], dm[2])
  i <- (0:(dm[1] - 1))/(dm[1] - 1)
  for (j in 1:dm[2]) {
      Si <- sort(dataMatrix[, j], method = "quick", index.return = TRUE)
      nobsj <- length(Si$x)
      if (nobsj < dm[1]) {
          nobs[j] <- nobsj
          isna <- is.na(dataMatrix[, j])
          S[, j] <- approx((0:(nobsj - 1))/(nobsj - 1), Si$x, 
              i, ties = "ordered")$y
          O[!isna, j] <- ((1:dm[1])[!isna])[Si$ix]
      }
      else {
          S[, j] <- Si$x
          O[, j] <- Si$ix
      }
  }
	if (is.null(reference)) {
		reference <- rowMeans(S)
	} else {
		reference <- sort(reference, method='quick')
	}

	nobs <- rep(dm[1], dm[2])
	index <- (0:(dm[1] - 1))/(dm[1] - 1)

 for (j in 1:dm[2]) {
      if (ties) 
          rk <- rank(dataMatrix[, j])
      if (nobs[j] < dm[1]) {
          isna <- is.na(dataMatrix[, j])
          if (ties) 
              dataMatrix[!isna, j] <- approx(index, reference, (rk[!isna] - 1)/(nobs[j] - 
                1), ties = "ordered")$y
          else dataMatrix[O[!isna, j], j] <- approx(index, reference, (0:(nobs[j] - 
              1))/(nobs[j] - 1), ties = "ordered")$y
      }
      else {
          if (ties) 
              dataMatrix[, j] <- approx(index, reference, (rk - 1)/(dm[1] - 1), ties = "ordered")$y
          else dataMatrix[O[, j], j] <- reference
      }
  }
	return(dataMatrix)
}


## 
normalizeMethylation.quantile <- function(methyLumiM, separateColor=FALSE, reference=NULL, ...) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  if (storageMode(assayData(methyLumiM)) == "environment") storageMode(assayData(methyLumiM)) <- "lockedEnvironment"
  unmethy <- assayDataElement(methyLumiM, 'unmethylated')
  methy <- assayDataElement(methyLumiM, 'methylated')

  if (separateColor) {
    annotation <- pData(featureData(methyLumiM))
    if (is.null(annotation$COLOR_CHANNEL)) {
      cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
      separateColor <- FALSE
    } else {
      allRedInd <- which(annotation$COLOR_CHANNEL == 'Red')
      allGrnInd <- which(annotation$COLOR_CHANNEL == 'Grn')
    }
  }
  
	## It will perform normalization sample by sample when the data is a BigMatrix
	if (is(assayDataElement(methyLumiM, 'exprs'), "BigMatrix")) {
		## estimate the reference by randomly pick columns
		if (is.null(reference)) {
			reference <- .estimate.quantile.reference(dataMatrix=assayDataElement(methyLumiM, 'methylated'), dataMatrix2=assayDataElement(methyLumiM, 'unmethylated'), batchSize=50)
		}
		for (i in 1:ncol(methy)) {
		  x.matrix.i <- rbind(methy[,i, drop=FALSE], unmethy[,i, drop=FALSE])
	    # Normalize the intensity using quantile normalization
	    x.matrix.i <- .quantileNormalization.reference(x.matrix.i, reference=reference, ...)
			methy[,i] <- x.matrix.i[1:nrow(methy),1]
		  unmethy[,i] <- x.matrix.i[(nrow(methy)+1):nrow(x.matrix.i),1]    
			# estimate the M-value
			if (is(methyLumiM, "MethyLumiM")) assayDataElement(methyLumiM, 'exprs')[,i] <- lumi::estimateM(methyLumiM[,i], returnType='matrix')	
		}			
		
	} else {
	  if (separateColor) {
	    methy.n <- methy; unmethy.n <- unmethy
	    methy.red <- methy[allRedInd, ]
	    methy.grn <- methy[allGrnInd, ]
	    unmethy.red <- unmethy[allRedInd, ]
	    unmethy.grn <- unmethy[allGrnInd, ]
	    x.matrix.red <- rbind(methy.red, unmethy.red)
	    x.matrix.grn <- rbind(methy.grn, unmethy.grn)
	    # Normalize the intensity using quantile normalization
			if (!is.null(reference)) {
		    x.matrix.red <- .quantileNormalization.reference(x.matrix.red, reference=reference, ...)
		    x.matrix.grn <- .quantileNormalization.reference(x.matrix.red, reference=reference, ...)
			} else {
		    x.matrix.red <- limma::normalizeQuantiles(x.matrix.red, ...)
		    x.matrix.grn <- limma::normalizeQuantiles(x.matrix.grn, ...)
		    # x.matrix.red <- normalize.quantiles.robust(x.matrix.red + 0.0, ...)
		    # x.matrix.grn <- normalize.quantiles.robust(x.matrix.grn + 0.0, ...)
			}
	    len.red <- length(allRedInd)
	    len.grn <- length(allGrnInd)
	    methy.n[allRedInd,] <- x.matrix.red[1:len.red,]
	    unmethy.n[allRedInd,] <- x.matrix.red[(len.red+1):nrow(x.matrix.red),]
	    methy.n[allGrnInd,] <- x.matrix.grn[1:len.grn,]
	    unmethy.n[allGrnInd,] <- x.matrix.grn[(len.grn+1):nrow(x.matrix.grn),]
	  } else {
	    x.matrix <- rbind(methy, unmethy)
	    # Normalize the intensity using quantile normalization
			if (!is.null(reference)) {
		    x.matrix <- .quantileNormalization.reference(x.matrix, reference=reference, ...)
	    } else {
				x.matrix <- limma::normalizeQuantiles(x.matrix, ...)
	    	#x.matrix <- normalize.quantiles.robust(x.matrix + 0.0, ...)
	    }
			methy.n <- x.matrix[1:nrow(methy),]
	    unmethy.n <- x.matrix[(nrow(unmethy)+1):nrow(x.matrix),]    
	  }
	  colnames(methy.n) <- colnames(unmethy.n) <- colnames(methy)
	  rownames(methy.n) <- rownames(unmethy.n) <- rownames(methy)
	  assayDataElement(methyLumiM, 'unmethylated') <- unmethy.n
	  assayDataElement(methyLumiM, 'methylated') <- methy.n
	  if (is(methyLumiM, "MethyLumiM") || is(methyLumiM, "MethyGenoSet")) methyLumiM <- estimateM(methyLumiM)
	}

  return(methyLumiM)
}



# estimate the intensity measured by Illumina Infinium methylation probes
# which basically is the sum of methylated and unmethylated probe intensity
estimateIntensity <- function(methyLumiM, returnType=c("ExpressionSet", "matrix")) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  returnType <- match.arg(returnType)
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 

  if (returnType == "matrix") {
    intensity <- unmethy[,] + methy[,]
    return(intensity)
  } else {
    # methyLumiM <- as(methyLumiM, "ExpressionSet")
    # exprs(methyLumiM) <- intensity
		if ('dataType' %in% slotNames(methyLumiM)) {
			dataType(methyLumiM) <- 'Intensity'
		}
    # if (is(assayDataElement(methyLumiM, 'exprs'), 'BigMatrix')) {
    #   for (i in 1:ncol(methy)) {
    #     assayDataElement(methyLumiM, 'exprs')[,i] <- unmethy[,i] + methy[,i]
    #   }
    # } else {
			assayDataElement(methyLumiM, 'exprs') <- unmethy[,] + methy[,]
    # }

    return(methyLumiM)
  }
}

# convert beta-value to m-value
beta2m <- function(beta) {
  m <- log2(beta/(1-beta))
  return(m)
}

# convert m-value to beta-value 
m2beta <- function(m) {
  beta <- 2^m/(2^m + 1)
  return(beta)
}

# estimate the M-value based on methylated and unmethylated probe intensities
estimateM <- function(methyLumiM, returnType=c("ExpressionSet", "matrix"), offset=100) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  returnType <- match.arg(returnType)
  
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
	## in case of BigMatrix
	if (is(methy, 'BigMatrix')) {
	  mm <- min(c(bigmemoryExtras::apply(unmethy, 2, min, na.rm=TRUE), bigmemoryExtras::apply(methy, 2, min, na.rm=TRUE)), na.rm=TRUE)
	} else {
		mm <- min(c(apply(unmethy, 2, min, na.rm=TRUE), apply(methy, 2, min, na.rm=TRUE)), na.rm=TRUE)
	}
  if (mm < 0.01) {
		if (is(methy, 'BigMatrix')) {
			for (i in 1:ncol(methy)) {
				if (any(unmethy[,i] < 0.01)) unmethy[which(unmethy[,i] < 0.01), i] <- 0.01 
		    if (any(methy[,i] < 0.01)) methy[which(methy[,i] < 0.01), i] <- 0.01 
			}
		} else {
	    unmethy[unmethy < 0.01] <- 0.01 
	    methy[methy < 0.01] <- 0.01 
		}
  }
	if (returnType == 'matrix') {
	  M <- log2((methy[,] + offset) / (unmethy[,] + offset))
    return(M)
	} else {
		if ('dataType' %in% slotNames(methyLumiM)) {
			dataType(methyLumiM) <- 'M'
		}
		if (is(assayDataElement(methyLumiM, 'exprs'), 'BigMatrix')) {
			for (i in 1:ncol(methy)) {
				assayDataElement(methyLumiM, 'exprs')[,i] <- log2((methy[,i] + offset) / (unmethy[,i] + offset))
			}
		} else {
			assayDataElement(methyLumiM, 'exprs') <- log2((methy[,] + offset) / (unmethy[,] + offset))
		}
    return(methyLumiM)
	}
}

# estimate the Beta-value based on methylated and unmethylated probe intensities
estimateBeta <- function(methyLumiM, returnType=c("ExpressionSet", "matrix"), offset=100) {
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  returnType <- match.arg(returnType)
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
	## in case of BigMatrix
	if (is(methy, 'BigMatrix')) {
	  mm <- min(c(bigmemoryExtras::apply(unmethy, 2, min, na.rm=TRUE), bigmemoryExtras::apply(methy, 2, min, na.rm=TRUE)), na.rm=TRUE)
	} else {
		mm <- min(c(apply(unmethy, 2, min, na.rm=TRUE), apply(methy, 2, min, na.rm=TRUE)), na.rm=TRUE)
	}
  if (mm < 0.01) {
		if (is(methy, 'BigMatrix')) {
			for (i in 1:ncol(methy)) {
				if (any(unmethy[,i] < 0.01)) unmethy[which(unmethy[,i] < 0.01), i] <- 0.01 
		    if (any(methy[,i] < 0.01)) methy[which(methy[,i] < 0.01), i] <- 0.01 
			}
		} else {
	    unmethy[unmethy < 0.01] <- 0.01 
	    methy[methy < 0.01] <- 0.01 
		}
  }
  if (returnType == "matrix") {
	  beta <- methy[,] / (unmethy[,] + methy[,] + offset)
    return(beta)
  } else {
    # methyLumiBeta <- as(methyLumiM, "ExpressionSet")
		if ('dataType' %in% slotNames(methyLumiM)) {
			dataType(methyLumiM) <- 'Beta'
		}
		
		if (is(assayDataElement(methyLumiM, 'exprs'), 'BigMatrix')) {
			for (i in 1:ncol(methy)) {
				assayDataElement(methyLumiM, 'exprs')[,i] <- methy[,i] / (unmethy[,i] + methy[,i] + offset)
			}
		} else {
			assayDataElement(methyLumiM, 'exprs') <- methy[,] / (unmethy[,] + methy[,] + offset)
		}
  	return(methyLumiM)
  }
}

# boxplot unique for MethyLumiM-class
# setMethod("boxplot",signature(x="MethyLumiM"),
#   function(x, main, prob=c(seq(10,90, by=10), 95), col=gray(rev(seq(prob)/length(prob))), logMode=TRUE, ...) {
setMethod("boxplot",signature(x="MethyLumiM"), function(x, main, logMode=TRUE, ...) {
    
  tmp <- description(x)
  if (missing(main) && (is(tmp, "MIAME")))
     main <- tmp@title

  dataMatrix <- exprs(x)
  labels <- colnames(dataMatrix)
  if (is.null(labels)) labels <- as.character(1:ncol(dataMatrix))
  ## set the margin of the plot
  mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
  old.mar <- par(mar=mar)
  on.exit(par(old.mar))

	if (.hasSlot(x, 'dataType')) {
		datatype <- dataType(x)
		if (datatype == '' || length(datatype) == 0) datatype <- 'M'
		ylab <- switch(datatype,
			M='M-value',
			Beta='Beta-value',
			datatype)
		if (datatype == 'Intensity' && logMode) {
			dataMatrix[dataMatrix <= 0] <- NA
			dataMatrix <- log2(dataMatrix)
			ylab <- 'Log2-Intensity of CpG-sites'
		}	
	} else {
		ylab <- 'M-value'
		datatype <- 'M'
	}

 	if (datatype == 'Intensity') {
		boxplot(as(x, 'ExpressionSet'), main=main, xlab='', ylab=ylab, xaxt='n', ...)
	  axis(1, at=1:ncol(dataMatrix), labels=labels, tick=TRUE, las=2)
	} else {
	  # tmp <- lapply(1:ncol(dataMatrix), function(i) dataMatrix[,i])
	  tmp <- data.frame(M=as.vector(dataMatrix), sample=factor(col(dataMatrix), labels=colnames(dataMatrix)))
	   
	  print(bwplot(M ~ sample, data=tmp, ylab=ylab, main=main, scales=list(rot=90),
		       panel = function(..., box.ratio) {
		           panel.violin(..., col = "grey", varwidth = FALSE, box.ratio = box.ratio)
		       }))
	  # hdr.boxplot(tmp, main=main, xlab='', ylab=ylab, prob=prob, col=col, ...)
	}

})


## boxplotColorBias
# boxplotColorBias(methyLumiM)
boxplotColorBias <- function(methyLumiM, logMode=TRUE, channel=c('both', 'unmethy', 'methy', 'sum'), grid=TRUE, main=NULL, mar=NULL, verbose=F, subset=NULL, ...) {
  
  channel <- match.arg(channel)
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  if (!is.null(subset)) {
    if (!is.numeric(subset)) stop('subset should be numeric.')
    if (length(subset) == 1) {
      index <- sample(1:nrow(methyLumiM), min(subset, nrow(methyLumiM)))
    } else {
      index <- subset
      index <- index[index > 0 & index <= nrow(methyLumiM)]
    }
    methyLumiM <- methyLumiM[index,]
  } 

  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL)) {
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
    return(invisible(FALSE))
  }
  redInd <- annotation$COLOR_CHANNEL == 'Red'
  grnInd <- annotation$COLOR_CHANNEL == 'Grn'
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
  
  if (logMode) {
    unmethy[unmethy < 1] <- 1
    methy[methy < 1] <- 1
    unmethy <- log2(unmethy)
    methy <- log2(methy)
  }
  
  nSample <- ncol(unmethy)
  allRed <- allGrn <- NULL
  tmp <- lapply(1:nSample, function(i) {
    red.a.i <- unmethy[redInd, i]
    red.b.i <- methy[redInd, i]
    grn.a.i <- unmethy[grnInd, i]
    grn.b.i <- methy[grnInd, i]
    if (channel == 'both') {
      red <- c(red.a.i, red.b.i)
      grn <- c(grn.a.i, grn.b.i)
    } else if (channel == 'unmethy') {
      red <- red.a.i
      grn <- grn.a.i
    } else if (channel == 'methy') {
      red <- red.b.i
      grn <- grn.b.i
    } else {
      red <- red.a.i + red.b.i
      grn <- grn.a.i + grn.b.i
    }
    allRed <<- cbind(allRed, red)
    allGrn <<- cbind(allGrn, grn)
    return(NULL)
  })
  labels <- colnames(unmethy)
  if (is.null(main)) {
    main <- "Boxplots of Red and Green color channels"
  }
  if (is.null(mar)) {
    mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
    mar[mar > 15] <- 15
    if (verbose) cat("mar:", mar, "\n")
  }
  old.par <- par(mar=mar, xaxt='n')
  info <- switch(channel, 
    "both"="intensity of both methylated and unmethylated probes",
    "methy"="intensity of methylated probes only",
    "unmethy"="intensity of unmethylated probes only",
    "sum"="CpG-site Intensity")
  if (logMode) {
    ylab <- paste("Log2", info)
  } else {
    substr(info, 1, 1) <- toupper(substr(info, 1, 1))
    ylab <- info
  }
  boxplot(allRed ~ col(allRed), col = "red", boxwex = 0.25, at = 1:nSample - 0.175,  ylab=ylab, xlab="", main=main, ...)
  boxplot(allGrn ~ col(allGrn), col = "green", boxwex = 0.25, at = 1:nSample + 0.175, axis=F, add=TRUE, ylab="", xlab="")
  par(xaxt='s')
  axis(1, at=1:ncol(allRed), labels=labels, tick=TRUE, las=2)
  par(old.par)
  if (grid) abline(v=1:(nSample-1) + 0.5, col = "lightgray", lty = "dotted")
  
  return(invisible(TRUE))
}


# plotColorBias1D(methyLumiM)
## plot either density or scatter plot of two color channels
plotColorBias1D <- function(methyLumiM, channel=c('both', 'unmethy', 'methy', 'sum'), colorMode=TRUE, removeGenderProbes=FALSE, logMode=TRUE, subset=NULL, ...) {

  channel <- match.arg(channel)
  otherPar <- list(...)
  densityPar <- otherPar[names(otherPar) %in% names(formals(density.default))]
  otherPar[names(otherPar) %in% names(formals(density.default))] <- NULL
  
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }

  if (!is.null(subset)) {
    if (!is.numeric(subset)) stop('subset should be numeric.')
    if (length(subset) == 1) {
      index <- sample(1:nrow(methyLumiM), min(subset, nrow(methyLumiM)))
    } else {
      index <- subset
      index <- index[index > 0 & index <= nrow(methyLumiM)]
    }
    methyLumiM <- methyLumiM[index,]
  } 
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL)) {
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
    return(methyLumiM)
  }
  if (removeGenderProbes && !is.null(annotation$CHR)) {
    redInd <- annotation$COLOR_CHANNEL == 'Red' & !(annotation$CHR %in% c('X', 'Y'))
    grnInd <- annotation$COLOR_CHANNEL == 'Grn' & !(annotation$CHR %in% c('X', 'Y'))
  } else {
    redInd <- annotation$COLOR_CHANNEL == 'Red'
    grnInd <- annotation$COLOR_CHANNEL == 'Grn'
  }
  
  nSample <- ncol(unmethy)
  density.list <- lapply(1:nSample, function(i) {
    red.a.i <- unmethy[redInd, i]
    red.b.i <- methy[redInd, i]
    grn.a.i <- unmethy[grnInd, i]
    grn.b.i <- methy[grnInd, i]
    if (channel == 'both') {
      red <- c(red.a.i, red.b.i)
      grn <- c(grn.a.i, grn.b.i)
    } else if (channel == 'unmethy') {
      red <- red.a.i
      grn <- grn.a.i
    } else if (channel == 'methy') {
      red <- red.b.i
      grn <- grn.b.i
    } else {
      red <- red.a.i + red.b.i
      grn <- grn.a.i + grn.b.i
    }
    if (colorMode) {
      if (logMode) {
        red[red < 1] <- 1
        grn[grn < 1] <- 1
        if (length(densityPar) > 0) {
          dd.red <- do.call('density', c(list(log2(red)), densityPar))
          dd.grn <- do.call('density', c(list(log2(grn)), densityPar))
        } else {
          dd.red <- density(log2(red))
          dd.grn <- density(log2(grn))
        }
      } else {
        if (length(densityPar) > 0) {
          dd.red <- do.call('density', c(list(red), densityPar))
          dd.grn <- do.call('density', c(list(grn), densityPar))
        } else {
          dd.red <- density(red)
          dd.grn <- density(grn)
        }
      }
    } else {
      pool <- c(red, grn)
      if (logMode) {
        pool[pool < 1] <- 1
        if (length(densityPar) > 0) {
          dd.pool <- do.call('density', c(list(log2(pool)), densityPar))
        } else {
          dd.pool <- density(log2(pool))
        }
      } else {
        if (length(densityPar) > 0) {
          dd.pool <- do.call('density', c(list(pool), densityPar))
        } else {
          dd.pool <- density(pool)
        }
      }
      dd.red <- dd.grn <- dd.pool
    }
    return(list(red=dd.red, green=dd.grn))
  })
  
  mm.density <- max(sapply(density.list, function(x) max(c(x$red$y, x$green$y))))
  xrange <- range(sapply(density.list, function(x) range(c(x$red$x, x$green$x))))

  info <- switch(channel, 
    "both"="intensity of both methylated and unmethylated probes",
    "methy"="intensity of methylated probes only",
    "unmethy"="intensity of unmethylated probes only",
    "sum"="CpG-site Intensity")
  if (logMode) {
    xlab <- paste("Log2", info)
  } else {
    substr(info, 1, 1) <- toupper(substr(info, 1, 1))
    xlab <- info
  }
  
  if (is.null(otherPar$xlab)) otherPar$xlab <- xlab
  if (is.null(otherPar$ylab)) otherPar$ylab <- "Density"
  if (is.null(otherPar$xlim)) otherPar$xlim <- xrange
  if (is.null(otherPar$ylim)) otherPar$ylim <- c(0,mm.density)
  if (is.null(otherPar$main)) otherPar$main <- ifelse(colorMode, "Compare density distribution of two color channels", "Compare density distribution")
  if (is.null(otherPar$type)) otherPar$type <- 'l'
  if (colorMode) {
    if (is.null(otherPar$col)) otherPar$col <- 2
  } else {
    if (is.null(otherPar$col)) {
      otherPar$col <- 1:nSample
    } else if (length(otherPar$col) < nSample)
      otherPar$col <- rep(otherPar$col[1], nSample)
  }
  
  
  for (i in 1:nSample) {
    dd.red.i <- density.list[[i]]$red
    dd.grn.i <- density.list[[i]]$green
    if (i == 1) {
      do.call('plot', c(list(dd.red.i), otherPar))
      if (colorMode) lines(dd.grn.i, col=3)
    } else {
      if (colorMode) {
        lines(dd.red.i, col=otherPar$col, lty=i)
        lines(dd.grn.i, col=3, lty=i)
      } else {
        lines(dd.red.i, col=otherPar$col[i], lty=i)
      }
    }
  }
  return(invisible(TRUE))
}


## plotColorBias2D(methyLumiM, selSample=1)
plotColorBias2D <- function(methyLumiM, selSample=1, combineMode=F, layoutRatioWidth=c(0.75,0.25), layoutRatioHeight=c(0.25, 0.75), 
      margins = c(5, 5, 2, 2), cex=1.25, logMode=TRUE, subset=NULL, ...) {

	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  otherPar <- list(...)
  
  if (!is.null(subset)) {
    if (!is.numeric(subset)) stop('subset should be numeric.')
    if (length(subset) == 1) {
      index <- sample(1:nrow(methyLumiM), min(subset, nrow(methyLumiM)))
    } else {
      index <- subset
      index <- index[index > 0 & index <= nrow(methyLumiM)]
    }
    methyLumiM <- methyLumiM[index,]
  } 

  ff <- pData(featureData(methyLumiM))
  color.channel <- ff[,"COLOR_CHANNEL"]
  if (is.null(color.channel) && !combineMode) stop("No color channel information included in the data!\n Please add it using addAnnotationInfo function.\n")
  
  unmethy <- assayDataElement(methyLumiM, 'unmethylated')[, selSample[1]]
  methy <- assayDataElement(methyLumiM, 'methylated')[, selSample[1]]

  if (logMode) {
    unmethy[unmethy < 1] <- 1
    methy[methy < 1] <- 1
    unmethy <- log2(unmethy)
    methy <- log2(methy)
  }
  
  if (!combineMode) {
    grn.a <- unmethy[color.channel == "Grn"]
    red.a <- unmethy[color.channel == "Red"]
    grn.b <- methy[color.channel == "Grn"]
    red.b <- methy[color.channel == "Red"]
  }

  oldpar <- par(no.readonly = TRUE)
    layout(matrix(c(2,1,0,3), nrow=2), widths = layoutRatioWidth, heights = layoutRatioHeight, respect = FALSE)
  # layout.show(3)
  ## plot the scatter plot 
  par(mar = c(margins[1], margins[2], 0, 0))
  plottype <- ifelse(combineMode, "p", "n")

  if (is.null(otherPar$pch)) otherPar$pch <- '.'
  if (is.null(otherPar$type)) otherPar$type <- plottype
  if (is.null(otherPar$xlim)) otherPar$xlim <- range(unmethy)
  if (is.null(otherPar$ylim)) otherPar$ylim <- range(methy)
  if (is.null(otherPar$xlab)) otherPar$xlab <- 'Unmethylated Probe Intensity'
  if (is.null(otherPar$ylab)) otherPar$ylab <- 'Methylated Probe Intensity'
  if (is.null(otherPar$main)) otherPar$main <- ''

  if (logMode) {
    if (is.null(otherPar$xlab)) otherPar$xlab <- 'Unmethylated Probe Intensity (log2)'
    if (is.null(otherPar$ylab)) otherPar$ylab <- 'Methylated Probe Intensity (log2)'
  } 
  do.call('plot', c(list(unmethy), list(methy), otherPar))

  if (combineMode) {
    dd.a <- density(unmethy)
    dd.b <- density(methy)
    par(mar = c(1, margins[2], margins[3], 0))
    plot(dd.a, xlab='', ylab='Density', xlim=otherPar$xlim, xaxt='n', col='black', type='l', main='')
    ## plot the density plot of methylated probes
    par(mar = c(margins[1], 1, 0, margins[4]))
    plot(dd.b$y, dd.b$x, xlab='Density', ylab='', ylim=otherPar$ylim, yaxt='n', col='black', type='l', main='')
  } else {
    points(red.a, red.b, pch='.', cex=cex, col='red')
    points(grn.a, grn.b, pch='.', cex=cex, col='green')    

    dd.grn.a <- density(grn.a)
    dd.grn.b <- density(grn.b)
    dd.red.a <- density(red.a)
    dd.red.b <- density(red.b)
    ## plot the density plot of unmethylated probes
    par(mar = c(1, margins[2], margins[3], 0))
    plot(dd.red.a, xlab='', ylab='Density', xlim=otherPar$xlim, ylim=range(c(dd.grn.a$y, dd.red.a$y)), xaxt='n', col='red', type='l', main='')
    lines(dd.grn.a, col='green')
    ## plot the density plot of methylated probes
    par(mar = c(margins[1], 1, 0, margins[4]))
    plot(dd.red.b$y, dd.red.b$x, xlab='Density', ylab='', xlim=range(c(dd.grn.b$y, dd.red.b$y)), ylim=otherPar$ylim, yaxt='n', col='red', type='l', main='')
    lines(dd.grn.b$y, dd.grn.b$x, col='green')
  }

  par(oldpar)

  return(invisible(TRUE))
}


# scatterPlotWithDensity <- function(x, y, channel=NULL, col=NULL, layoutRatioWidth=c(0.75,0.25), layoutRatioHeight=c(0.25, 0.75), 
#       margins = c(5, 5, 2, 2), cex=1.25, ...) {
#   
#   if (!is.null(channel)) {
#     grn.a <- unmethy[color.channel == "Grn"]
#     red.a <- unmethy[color.channel == "Red"]
#     grn.b <- methy[color.channel == "Grn"]
#     red.b <- methy[color.channel == "Red"]
#   } else {
#     channel <- rep(1, length(x))
#   }
# 
#   oldpar <- par(no.readonly = TRUE)
#   layout(matrix(c(2,1,0,3), nrow=2), widths = layoutRatioWidth, heights = layoutRatioHeight, respect = FALSE)
#   # layout.show(3)
#   ## plot the scatter plot 
#   par(mar = c(margins[1], margins[2], 0, 0))
# 
#   plot(x, y, type='n', ...)
#   
#   uniChannels <- unique(channel)
#   if (is.null(col)) col <- 1:length(uniChannels)
#   density.all <- NULL
#   for (i in seq(uniChannels)) {
#     x.i <- x[channel == uniChannels[i]]
#     y.i <- y[channel == uniChannels[i]]
#     dd.x.i <- density(x.i)
#     dd.y.i <- density(y.i)
#     density.all <- c(density.all, list(x=dd.x.i, y=dd.y.i))
#   }
#   
#   ## plot densitis
#   for (i in seq(uniChannels)) {
# 
#     x.i <- x[channel == uniChannels[i]]
#     y.i <- y[channel == uniChannels[i]]
#     points(x.i, y.i, pch='.', cex=cex, col=col[i])
# 
#     ## plot the density plot of unmethylated probes
#     par(mar = c(1, margins[2], margins[3], 0))
#     plot(dd.x.i, xlab='', ylab='Density', xlim=otherPar$xlim, ylim=range(c(dd.grn.a$y, dd.red.a$y)), xaxt='n', col='red', type='l', main='')
#     lines(dd.grn.a, col='green')
#     ## plot the density plot of methylated probes
#     par(mar = c(margins[1], 1, 0, margins[4]))
#     plot(dd.red.b$y, dd.red.b$x, xlab='Density', ylab='', xlim=range(c(dd.grn.b$y, dd.red.b$y)), ylim=otherPar$ylim, yaxt='n', col='red', type='l', main='')
#     lines(dd.grn.b$y, dd.grn.b$x, col='green')
#     
#   }
# 
#   par(oldpar)
# }

colorBiasSummary <- function(methyLumiM, logMode=TRUE, channel=c('both', 'unmethy', 'methy', 'sum')) {

  channel <- match.arg(channel)
	if (!all(c("unmethylated", "methylated") %in% assayDataElementNames(methyLumiM))) {
    stop("The input should include 'methylated' and 'unmethylated' elements in the assayData slot!\n")
  }
  unmethy <- assayDataElement(methyLumiM, 'unmethylated') 
  methy <- assayDataElement(methyLumiM, 'methylated') 
  annotation <- pData(featureData(methyLumiM))
  if (is.null(annotation$COLOR_CHANNEL)) {
    cat("No color balance adjustment because lack of COLOR_CHANNEL information!\n Please add it using addAnnotationInfo function.\n")
    return(methyLumiM)
  }
  redInd <- annotation$COLOR_CHANNEL == 'Red'
  grnInd <- annotation$COLOR_CHANNEL == 'Grn'
  
  nSample <- ncol(unmethy)
  summary.list <- lapply(1:nSample, function(i) {
    red.a.i <- unmethy[redInd, i]
    red.b.i <- methy[redInd, i]
    grn.a.i <- unmethy[grnInd, i]
    grn.b.i <- methy[grnInd, i]
    if (channel == 'both') {
      red <- c(red.a.i, red.b.i)
      grn <- c(grn.a.i, grn.b.i)
    } else if (channel == 'unmethy') {
      red <- red.a.i
      grn <- grn.a.i
    } else if (channel == 'methy') {
      red <- red.b.i
      grn <- grn.b.i
    } else {
      red <- red.a.i + red.b.i
      grn <- grn.a.i + grn.b.i
    }
    if (logMode) {
      ss.red <- summary(log2(red))
      ss.grn <- summary(log2(grn))
    } else {
      ss.red <- summary(red)
      ss.grn <- summary(grn)
    }
    return(list(red=ss.red, green=ss.grn))
  })
  red.summary.matrix <- sapply(summary.list, function(x) x$red)
  grn.summary.matrix <- sapply(summary.list, function(x) x$green)
  colnames(red.summary.matrix) <- colnames(grn.summary.matrix) <- sampleNames(methyLumiM)
  
  return(list(red=red.summary.matrix, green=grn.summary.matrix))
}


plotDensity <- function(dataMatrix, logMode=TRUE, addLegend=TRUE, legendPos="topright", subset=NULL, ...) {
  otherPar <- list(...)
  if (is(dataMatrix, 'MethyLumiM')) {
    if (dataType(dataMatrix) == 'M')  logMode <- FALSE
  } 

  if (is(dataMatrix, 'ExpressionSet')) {
      dataMatrix <- exprs(dataMatrix)
  } else if (is.numeric(dataMatrix)) {
    dataMatrix <- as.matrix(dataMatrix)
  } else {
    stop('Un-supported class of dataMatrix.')
  }

  if (!is.null(subset)) {
    if (!is.numeric(subset)) stop('subset should be numeric.')
    if (length(subset) == 1) {
      index <- sample(1:nrow(dataMatrix), min(subset, nrow(dataMatrix)))
    } else {
      index <- subset
      index <- index[index > 0 & index <= nrow(dataMatrix)]
    }
    dataMatrix <- dataMatrix[index,,drop=FALSE]
  } 
  
  if (logMode) {
    if (any(dataMatrix <= 0)) dataMatrix[dataMatrix <= 0.1] <- 0.1 
    tmp <- apply(log2(dataMatrix), 2, density)
  } else {
    tmp <- apply(dataMatrix, 2, density)
  }
  xx <- sapply(tmp, function(x) x$x)
  yy <- sapply(tmp, function(x) x$y)

  if (is.null(otherPar$type)) otherPar$type <- 'l'
  if (is.null(otherPar$xlab)) otherPar$xlab <- "Intensity"
  if (is.null(otherPar$ylab)) otherPar$ylab <- 'Density'
  if (is.null(otherPar$lty)) otherPar$lty <-  1:ncol(dataMatrix)
  if (is.null(otherPar$col)) otherPar$col <-  1:ncol(dataMatrix)
  if (is.null(otherPar$lwd)) otherPar$lwd <-  1
  if (is.null(otherPar$main)) otherPar$main <- "Density plot"
  if (logMode)  otherPar$xlab <- paste(otherPar$xlab, " (log2)", sep="")

  do.call('matplot', c(list(xx), list(yy), otherPar))

  ## add legend
  if (addLegend) {
    labels <- colnames(dataMatrix)
    if (is.null(labels)) labels <- as.character(1:ncol(dataMatrix))

    if (length(legendPos) > 1) {
      x.pos <- legendPos[1]
      y.pos <- legendPos[2]
    } else {
      x.pos <- legendPos[1]
      y.pos <- NULL
    }
    legend(x.pos, y.pos, legend=labels, box.lwd=0, col=otherPar$col, lty=otherPar$lty, lwd=otherPar$lwd)
  }
  return(invisible(TRUE))
}



produceMethylationGEOSubmissionFile <- function(methyLumiM, methyLumiM.raw=NULL, lib.mapping=NULL, idType='Probe', sampleInfo=NULL, fileName='GEOSubmissionFile.txt', supplementaryRdata=TRUE, ...) {
  
  if (missing(methyLumiM)) stop('Please provide all required input parameters!\n')
  if (is(methyLumiM, "MethyLumiM")) expr.norm <- estimateBeta(methyLumiM, returnType='matrix')
  if (is.null(methyLumiM.raw)) {
    detect <- detection(methyLumiM)
    methyData <- methylated(methyLumiM)
    unmethyData <- unmethylated(methyLumiM)
    expr <- NULL
  } else {
    detect <- detection(methyLumiM.raw)
    methyData <- methylated(methyLumiM.raw)
    unmethyData <- unmethylated(methyLumiM.raw)
    expr <- estimateBeta(methyLumiM.raw)
  }
  
  if (is.null(sampleInfo)) {
    sampleInfo <- produceGEOSampleInfoTemplate(methyLumiM, lib.mapping=lib.mapping, fileName=NULL)
  } else if (length(sampleInfo) == 1 && is.character(sampleInfo)) {
    sampleInfo <- read.table(sampleInfo, sep='\t', colClasses='character', skip=1, header=TRUE, strip.white=TRUE, quote='')
  } else if (is.null(nrow(sampleInfo))) {
    stop('Please provide correct sample information (a data.frame, matrix, or sampleInfo file)!\n')
  }
  sampleInfoTitle <- colnames(sampleInfo)
  if (any(sapply(sampleInfo[,-1, drop=F], nchar) == 0)) stop('No blank fields are allowed in the sampleInfo table!\nYou can check some example submissions, like GSM296418, at the GEO website.\n')
  if (supplementaryRdata) sampleInfo[, "Sample_supplementary_file"] <- 'supplementaryData.Rdata'
  nuID <- featureNames(methyLumiM)
  probeId <- nuID
  if (length(which(is.nuID(sample(nuID, 100)))) < 20) {
    nuID <- NULL
  } else {
    if (!is.null(lib.mapping)) {
      probeId <- nuID2IlluminaID(nuID, lib=lib.mapping, idType=idType, ...)
    } else {
      nuID <- NULL
    }
  }
  
  sampleID <- sampleInfo[, "sampleID"]
  sampleTitle <- sampleInfo[,'Sample_title']
  for (i in seq(sampleID)) {
    if (i == 1) {
      cat('^SAMPLE =', sampleTitle[i], '\n', sep='', file=fileName, append=FALSE)
    } else {
      cat('^SAMPLE =', sampleTitle[i], '\n', sep='', file=fileName, append=TRUE)      
    }
    sampleInfo.i <- paste('!', sampleInfoTitle[-1], ' = ', sampleInfo[i,-1], '\n', sep='', collapse='')
    sampleInfo.i <- gsub("'", "\\'", sampleInfo.i)
    cat(sampleInfo.i, file=fileName, append=TRUE, sep='')
    tableHead <- "ID_REF"
    cat("#ID_REF = Illumina ID\n", file=fileName, append=TRUE)
    if (!is.null(nuID)) {
      cat("#nuID = nucleotide universal IDentifier (nuID), convertible to and from probe sequence. See Bioconductor lumi package for more details.\n", file=fileName, append=TRUE)
      tableHead <- c(tableHead, "nuID")
    }
    cat("#VALUE = Beta-value\n", file=fileName, append=TRUE)
    if (!is.null(expr)) cat("#RAW_VALUE = raw Beta-value\n", file=fileName, append=TRUE)
    tableHead <- c(tableHead, "VALUE")
    if (!is.null(expr)) tableHead <- c(tableHead, "RAW_VALUE")
    if (!is.null(methyData)) {
      cat("#METHYLATED = the intensities measured by methylated probes\n", file=fileName, append=TRUE)
      tableHead <- c(tableHead, "METHYLATED")
    }
    if (!is.null(unmethyData)) {
      cat("#UNMETHYLATED = the intensities measured by unmethylated probes\n", file=fileName, append=TRUE)
      tableHead <- c(tableHead, "UNMETHYLATED")
    }
    if (!is.null(detect)) {
      cat("#Detection_Pval = the detection p-value of the probe\n", file=fileName, append=TRUE)
      tableHead <- c(tableHead, "Detection_Pval")
    }
    sampleTable.i <- probeId
    if (!is.null(nuID)) sampleTable.i <- cbind(sampleTable.i, nuID)
    sampleTable.i <- cbind(sampleTable.i, expr.norm[,sampleID[i]])
    if (!is.null(expr)) sampleTable.i <- cbind(sampleTable.i, expr[,sampleID[i]])
    if (!is.null(methyData)) sampleTable.i <- cbind(sampleTable.i, methyData[,sampleID[i]])
    if (!is.null(unmethyData)) sampleTable.i <- cbind(sampleTable.i, unmethyData[,sampleID[i]])
    if (!is.null(detect)) sampleTable.i <- cbind(sampleTable.i, detect[,sampleID[i]])
    sampleTable.i <- rbind(tableHead, sampleTable.i)
    cat("!sample_table_begin\n", file=fileName, append=TRUE)
    write.table(sampleTable.i, sep='\t', quote=FALSE, file=fileName, append=TRUE, col.names=FALSE, row.names=FALSE)
    cat("!sample_table_end\n", file=fileName, append=TRUE)
  }
  
  if (supplementaryRdata) {
    methyLumiM <- methyLumiM[,sampleID]
    if (!is.null(methyLumiM.raw)) {
      methyLumiM.raw <- methyLumiM.raw[,sampleID]
      save(methyLumiM, methyLumiM.raw, sampleInfo, file='supplementaryData.Rdata')
    } else {
      save(methyLumiM, sampleInfo, file='supplementaryData.Rdata')
    }
  }
}


## ---------------------------------------------------------------------------------
# Functions related with estimating methylation status
## EM estimation of the parameters s1, s2 and theta
## E-step: determine the class of each probe
## M-step: 
## 1. estimation the proportion of two classes
## 2. Estimate of optimized s1 and s2 using optimization method
## 3. Estimate theta based on equation

# fittedGamma <- gammaFitEM(M[,1], initialFit=NULL, maxIteration=50, tol=0.0001, plotMode=T, verbose=T)
gammaFitEM <- function(M, initialFit=NULL, fix.k=NULL, weighted=TRUE, maxIteration=50, tol=0.0001, plotMode=FALSE, truncate=FALSE, verbose=FALSE) {
  
  fix.theta=NULL
  eps <- 10^-5
  # if (!require(nleqslv))   stop("Please install nleqslv package!\n")
  if (!is.null(fix.k)) {
    if (length(fix.k) == 1) fix.k <- c(fix.k, fix.k)
  }
  if (!is.null(fix.theta)) {
    if (length(fix.theta) == 1) fix.theta <- c(fix.theta, fix.theta)
  }
    
  fs1 <- function(s1, mi, zi, theta, k, n) {
    nonNegativeInd <- which(mi > s1) # this setting will allow to shift the profile without considering the small part of the trailing points
    sum(zi[nonNegativeInd]/(mi[nonNegativeInd]-s1)) - n /((k-1)*theta)  
    # mi[mi < s1] <- s1 + eps  
    # sum(zi/(mi-s1)) - n /((k-1)*theta)  
    #sum(zi/abs(mi-s1)) - n /((k-1)*theta)    
  }

  fs2 <- function(s2, mi,  zi, theta, k, n) {
    nonNegativeInd <- which(mi < s2)  # this setting will allow to shift the profile without considering the small part of the trailing points
    sum(zi[nonNegativeInd]/(s2-mi[nonNegativeInd])) - n /((k-1)*theta)  
    #mi[mi > s2] <- s2 - eps
    #sum(zi/(s2-mi)) - n /((k-1)*theta)  
    # sum(zi/abs(s2-mi)) - n /((k-1)*theta)  
  }

  fk1 <- function(k1, mi, zi, s, n) {
    nonNegativeInd <- which(mi > s) # this setting will allow to shift the profile without considering the small part of the trailing points
    log(k1) - digamma(k1) - log(sum(zi[nonNegativeInd]*(mi[nonNegativeInd]-s)/n)) + sum(zi[nonNegativeInd]*log(mi[nonNegativeInd]-s)/n)
    # mi[mi < s] <- s + eps
    # log(k1) - digamma(k1) - log(sum(zi*(mi-s)/n)) + sum(zi*log(mi-s)/n)
  }

  fk2 <- function(k2, mi, zi, s, n) {
    nonNegativeInd <- which(mi < s) # this setting will allow to shift the profile without considering the small part of the trailing points
    log(k2) - digamma(k2) - log(sum(zi[nonNegativeInd]*(s-mi[nonNegativeInd])/n)) + sum(zi[nonNegativeInd]*log(s-mi[nonNegativeInd])/n)
    # mi[mi > s] <- s - eps
    # log(k2) - digamma(k2) - log(sum(zi*(s-mi)/n)) + sum(zi*log(s-mi)/n)
  }
      
  if (is.matrix(M)) {
    if (ncol(M) > 1) cat("Only the first column of the matrix was processed!\n")
    x <- M[,1]
  } else {
    x <- M
  }

  # Initial value estimation got from grid search
  initialFit <- .initialGammaEstimation(x, initialFit=initialFit)
  
  k <- initialFit$k
  theta <- initialFit$theta
  s <- initialFit$shift
  p <- initialFit$proportion
  Mode <- initialFit$mode
  k[k < 2] <- 2
  theta[theta < 0.2] <- 0.2

  if (!is.null(fix.k)) k[!is.na(fix.k)] <- fix.k[!is.na(fix.k)]
  if (!is.null(fix.theta)) theta[!is.na(fix.theta)] <- fix.theta[!is.na(fix.theta)]
  if (!is.null(fix.k) || !is.null(fix.theta)) s <- c(Mode[1] - (k[1]-1)*theta[1], Mode[2] + (k[2]-1)*theta[2])

  if (verbose) {
    if (plotMode) plotGammaFit(x, k=k, theta=theta, shift=s, proportion=p)
    cat("\nInitial estimation:\n")
    cat("k:", k, "\n")
    cat("s:", s, "\n")
    cat("theta:", theta, "\n")
    cat("p:", p, "\n")
    f1 <- dgamma(x-s[1], shape=k[1], scale=theta[1])
    f2 <- dgamma(s[2]-x, shape=k[2], scale=theta[2])
    logLikelihood <- sum(log(p[1] * f1 + p[2]*f2))
    cat("logLikelihood:", logLikelihood, "\n")
  }
  
  iter <- 1
  N <- length(x)
  while (iter < maxIteration) {
    if (any(theta < eps) || any(k < 2)) {
      cat('It does not converge based on the current initial values!\n')
      logLikelihood <- -Inf
      return(list(logLikelihood=logLikelihood, k=k, theta=theta, shift=s, proportion=p, mode=Mode, probability=NULL))
    }
    
    Mode <- c(s[1] + (k[1]-1)*theta[1], s[2] - (k[2]-1)*theta[2])

    ## E-step
    f1 <- dgamma(x-s[1], shape=k[1], scale=theta[1])
    f2 <- dgamma(s[2]-x, shape=k[2], scale=theta[2])
    ## TO DO: add weighted function instead of truncate 10/25/2010
    
    if (weighted) {
      # down weight the long tails of two component densities beyond their modes
      w1 <- rep(1, length(f1)); w2 <- rep(1, length(f2))
      w1[x > Mode[2]] <- f2[x > Mode[2]] / max(f2)
      w2[x < Mode[1]] <- f1[x < Mode[1]] / max(f1)
      f1 <- f1 * w1
      f2 <- f2 * w2
      
      # adjust the density values because of the trunction
      # f1 <- f1 / (pgamma(Mode[2]-s[1], shape=k[1], scale=theta[1], lower.tail = TRUE))
      # f2 <- f2 / (pgamma(s[2]-Mode[1], shape=k[2], scale=theta[2], lower.tail = TRUE))
      # z1 <- p[1] * f1 / (p[1] * f1 + p[2] * f2) # + eps * 10)
      # z1[x > Mode[2]] <- 0
      # z1[x < Mode[1]] <- 1
      # f1[x > Mode[2]] <- 0
      # f2[x < Mode[1]] <- 0
    } 
    z1 <- p[1] * f1 / (p[1] * f1 + p[2] * f2 + eps * 10)  # posterior probability of unmethylated
    z2 <- 1 - z1

    if (verbose && iter > 1) {
      # logLikelihood <- sum((z1*log(p[1] * f1))[z1 > 0]) + sum((z2* log(p[2]*f2))[z2 > 0])
      ll <- p[1] * f1 + p[2]*f2
      ll[ll < eps] <- eps
      logLikelihood <- sum(log(ll))    
      cat("logLikelihood:", logLikelihood, "\n")      
    }
    
    ## M-step (estimate parameters: s, theta and k)
    # update the proportion of each class
    n1 <- sum(z1, na.rm=TRUE); n2 <- N - n1
    p.new <- n1 / N
    p.new <- c(p.new, 1 - p.new)
      if (abs(p.new[1]-p[1]) < tol) break
    p <- p.new

    ## Estimation of s, theta and k
    iter.inter <- 1
    while (iter.inter <= 10) {
      # set the range of s parameter    fss(s1, mi, zi, theta, k, n)
      s1.new <- nleqslv(s[1], fs1, mi=x, zi=z1, theta=theta[1], k=k[1], n=n1)$x
      s2.new <- nleqslv(s[2], fs2, mi=x, zi=z2, theta=theta[2], k=k[2], n=n2)$x
      s.new <- c(s1.new, s2.new)

      ## Estimation of k # fk1(k1, mi, zi, s, n)
      if (is.null(fix.k)) {
        k1.new <- nleqslv(k[1], fk1, mi=x, zi=z1, s=s.new[1], n=n1)$x
        k2.new <- nleqslv(k[2], fk2, mi=x, zi=z2, s=s.new[2], n=n2)$x
        k.new <- c(k1.new, k2.new)
      } else {
        k.new <- fix.k
        if (any(is.na(k.new))) {
          if (is.na(fix.k[1])) k.new[1] <- nleqslv(k[1], fk1, mi=x, zi=z1, s=s.new[1], n=n1)$x
          if (is.na(fix.k[2])) k.new[2] <- nleqslv(k[2], fk2, mi=x, zi=z2, s=s.new[2], n=n2)$x
        }
      }

      ## Estimation of theta
      if (is.null(fix.theta)) {
        theta.new <- c(sum(z1*(x - s.new[1]))/(k.new[1]*n1), sum(z2*(s.new[2] - x))/(k.new[2]*n2))
      } else {
        theta.new <- fix.theta
        if (any(is.na(theta.new))) {
          if (is.na(fix.theta[1])) theta.new[1] <- sum(z1*(x - s.new[1]))/(k.new[1]*n1)
          if (is.na(fix.theta[2])) theta.new[2] <- sum(z2*(s.new[2] - x))/(k.new[2]*n2)
        }
      }

      if (min(abs(s.new - s), na.rm=TRUE) < tol && min(abs(k.new - k), na.rm=TRUE) < tol && min(abs(theta.new - theta), na.rm=TRUE) < tol)  break
      # if (verbose) cat("s:", s, " k:", k, " theta:", theta, "\n")
      s <- s.new; k <- k.new; theta <- theta.new
      # if (any(theta < 0.2) || any(k < 2)) {
      if (any(theta < eps) || any(k < 2)) {
        cat('It does not converge based on the current initial values!\n')
        logLikelihood <- -Inf
        return(list(logLikelihood=logLikelihood, k=k, theta=theta, shift=s, proportion=p, mode=Mode, probability=NULL))
      }
      iter.inter <- iter.inter + 1
    }
 
    if (verbose) {
      cat("\nIteration ", iter, "\n")
      cat("k:", k, "\n")
      cat("s:", s, "\n")
      cat("theta:", theta, "\n")
      cat("p:", p, "\n")
      cat("Mode:", Mode, "\n")
      if (plotMode) plotGammaFit(x, k=k, theta=theta, shift=s, proportion=p)
    }
    iter <- iter + 1
  }
  if (iter == maxIteration) {
    Mode <- c(s[1] + (k[1]-1)*theta[1], s[2] - (k[2]-1)*theta[2])
    f1 <- dgamma(x-s[1], shape=k[1], scale=theta[1])
    f2 <- dgamma(s[2]-x, shape=k[2], scale=theta[2])
    if (truncate) {
      f1 <- f1 / (pgamma(Mode[2]-s[1], shape=k[1], scale=theta[1], lower.tail = TRUE))
      f2 <- f2 / (pgamma(s[2]-Mode[1], shape=k[2], scale=theta[2], lower.tail = TRUE))
      z1 <- p[1] * f1 / (p[1] * f1 + p[2] * f2) # + eps)
      z1[x > Mode[2]] <- 0
      z1[x < Mode[1]] <- 1
      f1[x > Mode[2]] <- 0
      f2[x < Mode[1]] <- 0
    } else {
      z1 <- p[1] * f1 / (p[1] * f1 + p[2] * f2) # + eps)  # posterior probability of unmethylated
    }
    z2 <- 1 - z1
  }

  # estimate log-likelihood 
  ll <- p[1] * f1 + p[2]*f2
  ll[ll < eps] <- eps
  logLikelihood <- sum(log(ll))    
  if (iter == maxIteration && verbose) cat("logLikelihood:", logLikelihood, "\n")

  # return the methylation/unmethylation estimation of each probe
  ## check class index to make sure those extreme probe belonging to the corresponding group
  probability <- cbind(z1, z2)
  colnames(probability) <- c('unmethylated', 'methylated')
  Mode <- c(s[1] + (k[1]-1)*theta[1], s[2] - (k[2]-1)*theta[2])
  if (plotMode) plotGammaFit(x, k=k, theta=theta, shift=s, proportion=p)

  fitResult <- list(logLikelihood=logLikelihood, k=k, theta=theta, shift=s, proportion=p, mode=Mode, probability=probability)
  class(fitResult) <- 'gammaFit'

  return(fitResult)
}


# initial gamma parameters estimation
.initialGammaEstimation <- function(x, initialFit=NULL) {

  k <- theta <- s <- p <- Mode <- NULL
  if (!is.null(initialFit)) {
    k <- initialFit$k
    theta <- initialFit$theta
    s <- initialFit$shift
    p <- initialFit$proportion
    Mode <- initialFit$mode
  } 
  
  if (!is.null(k) && !is.null(theta) &&!is.null(s) && !is.null(p) && !is.null(Mode))   return(initialFit)
  
  # mode positions
  if (is.null(Mode)) {
    dd <- density(x)
    density.m <- dd$y
    density.x <- dd$x  # x-axis of density plot (methylation ratio)
    midpoint <- mean(quantile(x, c(0.01, 0.99)))
    unmethy.ind <- density.x <= midpoint
    P1 <- density.x[unmethy.ind][which.max(density.m[unmethy.ind])]
    if (abs(P1 - midpoint) < 0.1) P1 <- (midpoint + quantile(x, 0.01))/2
    methy.ind <- density.x >= midpoint
    P2 <- density.x[methy.ind][which.max(density.m[methy.ind])]    
    if (abs(P2 - midpoint) < 0.1) P2 <- (midpoint + quantile(x, 0.99))/2
    Mode <- c(P1, P2)
  }

  # update the class index
  unmethy.ind <- which(x < (Mode[1] + Mode[2])/2)
  methy.ind <- which(x > (Mode[1] + Mode[2])/2)

  # percentage of two classes
  if (is.null(p)) {
    p <- c(length(unmethy.ind)/length(x), length(methy.ind)/length(x))
  }

  mean.unmethy <- mean(x[unmethy.ind])
  mean.methy <- mean(x[methy.ind])
  var.unmethy <- var(x[unmethy.ind])
  var.methy <- var(x[methy.ind])
  if (is.null(theta)) {
    theta <- c(mean.unmethy - Mode[1], Mode[2] - mean.methy)
    theta[theta < 0.2] <- 0.2
  }
  if (is.null(k))  {
    k <- c(round(var.unmethy/theta[1]^2), round(var.methy/theta[2]^2))
    k[k < 2] <- 2
  }
  if (is.null(s))  s <- c(mean.unmethy - k[1]*theta[1], mean.methy + k[2]*theta[2])

  fitResult <- list(k=k, theta=theta, shift=s, proportion=p, mode=Mode)
  class(fitResult) <- 'gammaFit'
  return(fitResult)
}


# plot gammFit results
plotGammaFit <- function(x, gammaFit=NULL, k=NULL, theta=NULL, shift=NULL, proportion=NULL, plotType=c('histogram', 'density'), ...) {
  plotType <- match.arg(plotType)
  
  if (!is.null(gammaFit)) {
    if (class(gammaFit) != 'gammaFit') stop("gammaFit should be an object of 'gammaFit' class!")
    k <- gammaFit$k
    theta <- gammaFit$theta
    shift <- gammaFit$shift
    proportion <- gammaFit$proportion
  }
  x <- sort(x)

  if (is.null(k) || is.null(theta) || is.null(shift) || is.null(proportion)) stop("Information of parameters k, theta, shift and proportion is required!\n")
  y1 = dgamma(x-shift[1], shape=k[1], scale=theta[1]) 
  y2 = dgamma(shift[2]-x, shape=k[2], scale=theta[2])
  if (plotType == 'histogram') {
    hist(x, 50, probability=TRUE, main='Compare data histogram and fitted distribution', xlab='M value', ...)
  } else {
    plot(density(x), type='l', lwd=1.5, main='Compare data density and fitted distribution', xlab="M value", ...)
  }
  lines(x, y1*proportion[1] + y2 * proportion[2], col=2, lty=3, lwd=1.5)
  lines(x, y1*proportion[1], col=3, lty=2)
  lines(x, y2*proportion[2], col=4, lty=2)
  return(invisible(TRUE))
}


# estimate methylation call probability based on gamma fit parameters
methylationCall <- function(x, threshold=0.95, ...) {
    
  if (length(threshold) == 1) threshold <- rep(threshold, 2)
  probability <- NULL
  if (class(x) != 'gammaFit') {
    fit <- gammaFitEM(x, ...) 
  } else {
    fit <- x
  }
  probability <- fit$probability
  
  # methyCall <- probability > threshold
  methyCall <- apply(probability, 1, function(x) {
    cc <- "Margin"
    if (x[1] > threshold[1]) {
      cc <- "Unmethy"
    } else if (x[2] > threshold[2]) {
      cc <- "Methy"
    }
    return(cc)
  })

  attr(methyCall, "probability") <- probability[,"methylated"]
  return(methyCall)  
}


# methylation status
lumiMethyStatus <- function(methyLumiM, ...) 
{

  if (!is(methyLumiM, 'MethyLumiM')) {
    stop('The object should be class "MethyLumiM" inherited!')
  }
  
  history.submitted <- as.character(Sys.time())
  
  M <- exprs(methyLumiM)
  M.status <- M.prob <- NULL
  for (i in 1:ncol(M)) {
    status.i <- methylationCall(M[,i], ...)
    prob.i <- attr(status.i, "probability")
    M.status <- cbind(M.status, status.i)
    M.prob <- cbind(M.prob, prob.i)
  }
  rownames(M.status) <- rownames(M.prob) <- rownames(M)
  colnames(M.status) <- colnames(M.prob) <- colnames(M)

  history.finished <- as.character(Sys.time())
  # history.command <- capture.output(print(match.call(lumiMethyStatus)))
  history.command <- paste(deparse(match.call(lumiMethyStatus)), collapse='') 
  lumiVersion <- packageDescription('lumi')$Version
  attr(M.status, "history") <- rbind(methyLumiM@history, data.frame(submitted=history.submitted, 
      finished=history.finished, command=history.command, lumiVersion=lumiVersion))
  
  attr(M.status, "probability") <- M.prob  
  return(M.status)
}


## Get chromosome information of a MethyLumiM or MethyGenoSet object
# chrInfo <- getChrInfo(methyLumiM, lib=lib)
getChrInfo <- function(methyData, lib=NULL, ...) {
  
 if (is(methyData, 'MethyLumiM')) {
    methyData <- addAnnotationInfo(methyData, lib=lib, ...)
    ff <- fData(methyData)
    probeList <- featureNames(methyData)
  } else if (is.character(methyData)) {
    ff <- addAnnotationInfo(methyData, lib=lib, ...)
    probeList <- rownames(methyData)
	} else {
    stop('methyData should be a MethyLumiM object or a character vector')
  }

  chrInfo <- data.frame(PROBEID=probeList, ff[,c('CHROMOSOME', 'POSITION')])  
  if (!is.null(ff$END))
    chrInfo <- data.frame(chrInfo, END=ff$END)

  return(chrInfo)
}

`IlluminaID2nuID` <-
function(IlluminaID, lib.mapping=NULL, species=c('Human', 'Mouse', 'Rat', 'Unknown'), chipVersion=NULL, ...) {
	species <- match.arg(species)

	chipInfo <- getChipInfo(IlluminaID, lib.mapping=lib.mapping, species=species, idMapping=TRUE, chipVersion=chipVersion, ...)
	
	if (is.null(chipInfo$IDType)) {
		nuID <- rep(NA, length(IlluminaID))
		names(nuID) <- IlluminaID
		return(nuID)
	} else if (!is.na(chipInfo$IDType)) {
		if (chipInfo$IDType[1] == 'nuID') {
			cat('The input ID is nuID. So no ID converstion will be made!\n')
			return(IlluminaID)
		}
	}		
	idMapping <- chipInfo$idMapping
	return(idMapping)
}

lumiR.batch <- function(fileList, convertNuID = TRUE, lib.mapping = NULL, detectionTh = 0.01, QC = TRUE, transform = c('none', 'vst', 'log2', 'cubicRoot'), sampleInfoFile = NULL, verbose = TRUE, ...) {

	oldDir <- getwd()
	dirMode <- FALSE
	transform <- match.arg(transform)
	if (file.exists(fileList[1])) {
		if (file.info(fileList[1])[1,'isdir']) {
			dirMode <- TRUE
			setwd(fileList)
		}
	}
	if (dirMode && length(fileList) == 1) {
		fileList <- dir(fileList, pattern='.csv')
		if (length(fileList) == 0) stop('No data files were found!\n')
	}
	
	history.submitted <- as.character(Sys.time())

	if (verbose) {
		cat('Inputting the data ...\n')
		if (transform != 'none') cat(paste('Transformation', transform, 'will be performed for each data file ...\n'))
	}

	for (i in 1:length(fileList)) {
		file.i <- fileList[i]
		if (transform != 'none') {
			x.lumi.i <- lumiR(file.i, parseColumnName=FALSE, convertNuID = FALSE, verbose=FALSE, ...)
			x.lumi.i <- lumiT(x.lumi.i, method=transform, simpleOutput=TRUE, verbose=FALSE)
		} else {
			x.lumi.i <- lumiR(file.i, parseColumnName=FALSE, convertNuID = FALSE, QC = FALSE, verbose=FALSE, ...)
		}
		if (i == 1) {
			x.lumi <- x.lumi.i
		} else {
			x.lumi <- combine(x.lumi, x.lumi.i)
		}
	}
	if (!convertNuID) lib.mapping <- NULL	
	if (!is.null(lib.mapping) || convertNuID) {
		if (verbose) cat('\nAdding nuID to the data ...\n')
		x.lumi <- addNuID2lumi(x.lumi, lib.mapping=lib.mapping)
	}

	if (!is.null(sampleInfoFile)) {

		if (is.character(sampleInfoFile) || class(sampleInfoFile)[1] == 'file') {
			if (file.exists(sampleInfoFile)) {
				sampleInfo <- read.table(sampleInfoFile, header=TRUE, sep='\t', colClasses='character', comment.char='', check.names=FALSE)
			} else {
				warning('The provided sampleInfoFile does not exist\n!')
				setwd(oldDir)
				return(x.lumi)
			}
		} else if (is.data.frame(sampleInfoFile)) {
			sampleInfo <- sampleInfoFile
		}
		## force the names to be capitalized
		colName <- toupper(names(sampleInfo))
		ind <- grep('ID$', colName, ignore.case=TRUE)
		if (length(ind) == 0) {
			ID <- sampleInfo[,1]
			if (any(duplicated(ID))) {
				warning('In sampleInfoFile, the ID column is required or the first column should be unique!\n')
				setwd(oldDir)
				return(x.lumi)
			}
			ind <- 1
		} else {
			ID <- sampleInfo[, ind[1]]			
		}
		rownames(sampleInfo) <- ID
		colnames(sampleInfo)[ind[1]] <- 'sampleID'

		sampleName <- sampleNames(x.lumi)
		ID <- ID[ID %in% sampleName]
		if (nrow(sampleInfo) != length(ID)) {
			warning('Some IDs provided in the sampleInfoFile do not exist the data file!\n')
			if (length(ID) == 0) {
				stop('The IDs provided in the sampleInfoFile do not match the data file!\n')
			} 
			
		} 
		x.lumi <- x.lumi[, ID]
		
		if (is.null(pData(phenoData(x.lumi)))) {
			pData <- sampleInfo[ID,]			
		} else {
			pData <- data.frame(pData(phenoData(x.lumi))[!(toupper(names(pData(phenoData(x.lumi)))) %in% c(toupper(names(sampleInfo)), 'ID', 'SAMPLEID'))], sampleInfo[ID,])
		}
		label <- sampleInfo[ID, colName == 'LABEL']
		if (length(label) == length(ID)) {
			rownames(pData) <- label
			sampleNames(x.lumi) <- label
		}

		pdata <- new("AnnotatedDataFrame", data=pData)
		phenoData(x.lumi) <- pdata
	} 
	
	## ----------------------------------------
	# Add history track
	if (is(x.lumi, 'LumiBatch')) {
	    history.finished <- as.character(Sys.time())
	    # history.command <- capture.output(print(match.call(lumiR.batch)))  
			history.command <- paste(deparse(match.call(lumiR.batch)), collapse='')  
			
		## replace with the real file name
		if (length(fileList) > 1) {
			fileList <- paste('c("', paste(fileList, collapse='","', sep=''), '")', sep='')
		} else {
			fileList <- paste('"', fileList, '"', sep='')
		}
		if (length(grep(',', history.command)) > 0) {
			history.command <- sub('\\(.+,', paste('(', fileList, ',', sep=''), history.command)
		} else {
			history.command <- sub('\\(.+\\)', paste('(', fileList, ')', sep=''), history.command)
		}

		lumiVersion <- packageDescription('lumi')$Version
		x.lumi@history<- data.frame(submitted=history.submitted, 
				finished=history.finished, command=history.command, lumiVersion=lumiVersion)		
	}
    
	## initialize the QC slot in the LumiBatch object
	if (QC) x.lumi <- lumiQ(x.lumi, detectionTh=detectionTh, verbose=verbose)

	setwd(oldDir)
	return(x.lumi)
}

### R code from vignette source 'lumi_VST_evaluation.Rnw'

###################################################
### code chunk number 1: libraries
###################################################
library("lumi")
library("vsn")
library("genefilter")
library("RColorBrewer")
library("limma")
library("lumiBarnes")
set.seed(0xbadbeef)
## Load the Barnes data set
data("lumiBarnes")


###################################################
### code chunk number 2: figFittingLinear
###################################################
temp <- lumiT(lumiBarnes[,1], fitMethod='linear', ifPlot=TRUE)


###################################################
### code chunk number 3: figFittingQuad
###################################################
temp <- lumiT(lumiBarnes[,1], fitMethod='quadratic', ifPlot=TRUE)


###################################################
### code chunk number 4: load
###################################################
## Select the blood and placenta samples
selChip = !is.na(lumiBarnes$pctBlood)
x.lumi <- lumiBarnes[, selChip]
presentCount <- detectionCall(x.lumi)

## Since the Barnes data was not background removed, we will do background adjustment first.
## The background estimation will be based on the control probe information. 
##  As the old version lumiBarnes library does not include controlData slot, we will check it first.
if (nrow(controlData(x.lumi)) == 0) {
	## We will use the control probe information in the example.lumi in the updated lumi package
	data(example.lumi)
	controlData(x.lumi) <- controlData(example.lumi)
}

x.lumi <- lumiB(x.lumi, method='bgAdjust')

repl1 <- which(x.lumi$replicate=="A")
repl2 <- which(x.lumi$replicate=="B")
stopifnot(sum(selChip)==12L, length(repl1)==6L, length(repl2)==6L)


###################################################
### code chunk number 5: preprocess
###################################################
## VST transform and Quantile normalization
x.lumi.vst <- lumiT(x.lumi)
x.lumi.vst.quantile <- lumiN(x.lumi.vst, method='quantile')


## log2 transform and Quantile normalization
x.lumi.log <- lumiT(x.lumi, method='log2')
x.lumi.log.quantile <- lumiN(x.lumi.log, method='quantile')


## VSN normalization: use lts.quantile=0.5 since in the blood/placenta
##  comparison more genes are differentially expressed than what is
##   expected by the default of 0.9.
x.lumi.vsn <- lumiN(x.lumi, method='vsn', lts.quantile=0.5)


## Add the vsn based on technical replicates
vsn.pair <- exprs(x.lumi)
cor.i <- NULL
for(i in 1:length(repl1)) {
	vsn.pair[, c(i, i+length(repl1))] <- exprs(vsn2(vsn.pair[, c(repl1[i], repl2[i])], verbose=FALSE))
}
# vsn.quantile <- normalize.quantiles(vsn.pair)
# rownames(vsn.quantile) <- rownames(vsn.pair)
# colnames(vsn.quantile) <- colnames(vsn.pair)


normDataList <- list('VST-Quantile'=exprs(x.lumi.vst.quantile), 
                    'Log2-Quantile'=exprs(x.lumi.log.quantile),
	            'VSN'=exprs(x.lumi.vsn))		# , 'VSN-Quantile'=vsn.quantile)

## scatter plots: 
## pairs(exprs(x.lumi.vsn), panel=function(...){par(new=TRUE);smoothScatter(..., nrpoints=0)})


###################################################
### code chunk number 6: chipCorList
###################################################
## Check the correlation between technique replicates
tempDataList <- c(normDataList, list(vsn.pair))
names(tempDataList) <- c(names(normDataList), 'VSN-techReplicate')
chipCorList <- matrix(as.numeric(NA), nrow=length(repl1), ncol=length(tempDataList))
colnames(chipCorList) <- names(tempDataList)
for (i in seq(along= tempDataList))
  for (j in seq(along=repl1))
    chipCorList[j,i] = cor(tempDataList[[i]][, c(repl1[j], repl2[j])])[1,2]


###################################################
### code chunk number 7: figboxplot
###################################################
labels <- colnames(chipCorList)
## set the margin of the plot
mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
oldpar = par(xaxt='n', mar=mar)
boxplot(chipCorList ~ col(chipCorList),  xlab='', 
        ylab='Correlation between technique replicate chips',
        col='skyblue')
par(xaxt='s')
axis(1, at=1:ncol(chipCorList), labels=labels, tick=TRUE, las=2)
par(oldpar)


###################################################
### code chunk number 8: figmeanSdPlot
###################################################
## select the technique replicates
selChip <-  c(repl1[1],repl2[1]) 
oldpar <- par(mfrow=c(length(normDataList) + 1,1))
for (i in 1:length(normDataList)) {
  meanSdPlot(normDataList[[i]][, selChip], ylab='Standard deviation')
}
meanSdPlot(vsn.pair[, selChip], ylab='Standard deviation')
par(oldpar)


###################################################
### code chunk number 9: fstatistic
###################################################
fac <- factor(paste(x.lumi$pctBlood, x.lumi$pctPlacenta, sep=":"))
rf <- lapply(normDataList, function(x) {
filtered.x = x[presentCount > 0,]
ftest.x = rowFtests(filtered.x, fac=fac)
ftest.x$IDs <- rownames(filtered.x)
return(ftest.x)
})
ef <- sapply( rf, function(x) ecdf(x$p.value))


###################################################
### code chunk number 10: figfstat
###################################################
pcol <- seq(along= normDataList) 
plty <- (1:(1 + length(normDataList))) [-3] 
plwd <- 1.5
x <- seq(0, 0.05, by=0.0001); x = x[-1]
plot(x, ef[[1]](x), type='l', lwd=plwd, lty=plty[1], col=pcol[1],
	main="Cumulative distribution of F-test p-value", xlab="F-test p-value", ylab="Empirical probability", log='x')
for (i in 2:length(ef)) {
	lines(x, ef[[i]](x), lwd=plwd, lty=plty[i], col=pcol[i])
}
legend(0.01, 0.25, names(normDataList), lwd=plwd, lty=plty, col=pcol)


###################################################
### code chunk number 11: Expression and dilution profile correlation
###################################################
modelProfile1 <- c(100, 95, 75, 50, 25, 0, 100, 95, 75, 50, 25, 0)
corrList <- lapply(normDataList, function(x) {
	x <- x[presentCount > 0, ]
	corr1 <- apply(x, 1, cor, y=modelProfile1)
	return(corr1)
	} )


###################################################
### code chunk number 12: histCorrelation
###################################################
freqMatrix <- NULL
breaks <- NULL
for (i in 1:length(corrList)) {
	hist.i <- hist(abs(corrList[[i]]), 30, plot=FALSE)
	breaks <- cbind(breaks, hist.i$breaks)
	freqMatrix <- cbind(freqMatrix, hist.i$counts)
}
freqMatrix <- rbind(freqMatrix, freqMatrix[nrow(freqMatrix),])
matplot(breaks, freqMatrix, type='s', lty=plty, col=pcol, lwd=plwd, ylab='Frequency', xlab='Absolute values of correlation coefficients')
legend(x=0.1, y=2800, legend=names(normDataList), lty=plty, col=pcol, lwd=plwd)


###################################################
### code chunk number 13: fstatistic concordance percentage
###################################################
topNumList <- seq(50, 3000, by=100)
corTh <- 0.8
highCorrNumMatrix <- NULL
for (i in 1:length(rf)) {
	probeList <- rf[[i]]$IDs
	ordProbe.i <- probeList[order(abs(rf[[i]]$p.value), decreasing=FALSE)]
	corr1 <- corrList[[i]]
	matchNum.j <- NULL
	for (topNum.j in topNumList) {
		topProbe.j <- ordProbe.i[1:topNum.j]
		matchNum.j <- c(matchNum.j, length(which(abs(corr1[topProbe.j]) > corTh)))
	}
	highCorrNumMatrix <- cbind(highCorrNumMatrix, matchNum.j)
}
rownames(highCorrNumMatrix) <- topNumList
colnames(highCorrNumMatrix) <- names(rf)


###################################################
### code chunk number 14: figfstatCor
###################################################
matplot(topNumList, (100 * highCorrNumMatrix/(topNumList %*% t(rep(1,ncol(highCorrNumMatrix))))),
 	type='l', xlab='Number of most significant probes by ranking their p-values (F-test)', 
	ylab='Percentage of concordant probes (%)', 
        lty=plty, col=pcol, lwd=plwd, ylim=c(50,100))
legend(x=2000, y=70, legend=colnames(highCorrNumMatrix), lty=plty, col=pcol, lwd=plwd)


###################################################
### code chunk number 15: fitList.limma
###################################################
## Select the comparing chip index
sampleInfo <- pData(phenoData(x.lumi))
sampleType <- paste(sampleInfo[,'pctBlood'], sampleInfo[,'pctPlacenta'], sep=':')
sampleType <- paste('c', sampleType, sep='')
## Comparing index
## used in the paper (the most challenging comparison):
compareInd <- c(repl1[1:2], repl2[1:2])  	
compareType <- sampleType[compareInd]
fitList.limma <- NULL
for (i in 1:length(normDataList)) {
	selDataMatrix <- normDataList[[i]]
	selDataMatrix <- selDataMatrix[presentCount > 0, ]
	selProbe <- rownames(selDataMatrix)
	compareMatrix <- selDataMatrix[, compareInd]
	
	design <- model.matrix(~ 0 + as.factor(compareType))
	colnames(design) <- c('A', 'B')
	fit1 <- lmFit(compareMatrix, design)
	contMatrix <- makeContrasts('A-B'=A - B, levels=design)
	fit2 <- contrasts.fit(fit1, contMatrix)
	fit <- eBayes(fit2)
	fitList.limma <- c(fitList.limma, list(fit))
}
names(fitList.limma) <- names(normDataList)


###################################################
### code chunk number 16: highCorrNumMatrix
###################################################
## Check the correlation of the top differentiated probes based on the limma results 
## rank the probes based on the p-values of limma result
fitList <- fitList.limma
topNumList <- c(30, seq(35, 1000, by=30))
corTh <- 0.8
highCorrNumMatrix <- NULL
for (i in 1:length(fitList)) {
	probeList <- rownames(fitList[[i]]$p.value)
	ordProbe.i <- probeList[order(abs(fitList[[i]]$p.value[,1]), decreasing=FALSE)]
	profileMatrix <- normDataList[[i]][ordProbe.i, ]

	modelProfile1 <- c(100, 95, 75, 50, 25, 0, 100, 95, 75, 50, 25, 0)
	corr1 <- apply(profileMatrix, 1, cor, y=modelProfile1)
	names(corr1) <- ordProbe.i
	matchNum.j <- NULL
	for (topNum.j in topNumList) {
		topProbe.j <- ordProbe.i[1:topNum.j]
		matchNum.j <- c(matchNum.j, length(which(abs(corr1[topProbe.j]) > corTh)))
	}
	highCorrNumMatrix <- cbind(highCorrNumMatrix, matchNum.j)
}
rownames(highCorrNumMatrix) <- topNumList
colnames(highCorrNumMatrix) <- names(fitList)


###################################################
### code chunk number 17: figLimmaConcordance
###################################################
matplot(topNumList, (100 * highCorrNumMatrix/(topNumList %*% t(rep(1,ncol(highCorrNumMatrix))))),
 	type='l', xlab='Number of most significant probes by ranking their p-values', 
	ylab='Percentage of concordant probes (%)', 
        lty=plty, col=pcol, lwd=plwd, ylim=c(0,100))
legend(x=700, y=50, legend=colnames(highCorrNumMatrix), lty=plty, col=pcol, lwd=plwd)


###################################################
### code chunk number 18: sessionInfo
###################################################
toLatex(sessionInfo())

bgAdjust <- function(lumiBatch, probs=0.5, ...) {
	if (!is(lumiBatch, 'LumiBatch')) stop('The object should be class "LumiBatch"!')
	## by subtract an offset, which is estimated based on the quantile of the control probes
	if (min(exprs(lumiBatch), na.rm=TRUE) <= 1) {
		cat('The data has already been background adjusted!\n')
		return(lumiBatch)
	}
	control <- controlData(lumiBatch)
	if (is.null(control) || nrow(control) == 0) {
		cat('There is no control probe information in the LumiBatch object!\n No background adjustment will be performed.\n')
		return(lumiBatch)
	}
	colName <- colnames(control)
	sampleName <- sampleNames(lumiBatch)
	if (!all(sampleName %in% colName)) {
		sampleID <- pData(phenoData(lumiBatch))$sampleID
		if (!all(sampleID %in% colName)) {
			cat('Column names of controlData does not match with the LumiBatch object!\n No background adjustment will be performed.\n')
			return(lumiBatch)			
		} else {
			control <- control[, sampleID]
		}
	} else {
		control <- control[, sampleName]		
	}
	probeType <- rownames(control)
	if ('negative' %in% probeType) control <- control[probeType == 'negative',]
	quantile.ctrl <- apply(control, 2, quantile, probs=probs, ...)
	exprs(lumiBatch) <- exprs(lumiBatch) - matrix(rep(1, nrow(lumiBatch)), ncol=1) %*% quantile.ctrl
	return(lumiBatch)
}

`nuID2IlluminaID` <-
function(nuID, lib.mapping=NULL, species=c('Human', 'Mouse', 'Rat', 'Unknown'), idType=c('All', 'Probe', 'Gene', 'Accession', 'Search_key', 'Symbol'), chipVersion=NULL, ...) {
	
	## retrieve Illumina IDs from Id Mapping 
	retrieveIlluminaID <- function(idMapping, idType) {		
		mapName <- colnames(idMapping)
		illuminaID <- switch(idType,
			'All'=idMapping,
			'Probe'= {
				ind <- grep("probe_id", mapName, ignore.case=TRUE)
				if (length(ind) == 0) ind <- grep("probe", mapName, ignore.case=TRUE)
				idMapping[,ind]
				},
			'Gene'= idMapping[,grep('target|gene', mapName, ignore.case=TRUE)],
			'Accession'= idMapping[,grep('accession', mapName, ignore.case=TRUE)],
			'Search_key'= idMapping[,grep('search_key', mapName, ignore.case=TRUE)],
			'Symbol'= idMapping[,grep('symbol', mapName, ignore.case=TRUE)])
		names(illuminaID) <- rownames(idMapping)
		return(illuminaID)
	}
		
	species <- match.arg(species)
	idType <- match.arg(idType)
	chipInfo <- getChipInfo(nuID, lib.mapping=lib.mapping, species=species, idMapping=TRUE, chipVersion=chipVersion, ...)
	idMapping <- chipInfo$idMapping
	if (length(nuID) > 0) {
		if (chipInfo$IDType[1] != 'nuID') {
			cat('The input ID is not nuID. So no ID conversion will be made!\n')
			return(nuID)
		}
	} 
	if (is(idMapping, 'list')) {
		illuminaID <- lapply(idMapping, function(x) retrieveIlluminaID(x, idType))
	} else {
		illuminaID <- retrieveIlluminaID(idMapping, idType)
	}
	return(illuminaID)		
}

`lumiQ` <-
function(x.lumi, logMode=TRUE, detectionTh=0.01, verbose=TRUE) {
	if (!is(x.lumi, 'LumiBatch')) {
		if (verbose) cat('No Quality Control assessment of the object because it is not a "LumiBatch" object.\n')
		return(x.lumi)
	}
	if (verbose) cat('Perform Quality Control assessment of the LumiBatch object ...\n')
	history.submitted <- as.character(Sys.time())

	expr <- exprs(x.lumi)
	if (any(is.na(expr))) {
		naInd <- apply(expr, 1, function(x) any(is.na(x)))
		expr <- expr[!naInd,, drop=FALSE]
	}

	if (logMode && (max(expr, na.rm=TRUE) > 50)) {
		# remove the negative values
		if (min(expr) < 0) {
			rMin <- rowMin(expr)
			expr <- expr[rMin > 0, , drop=FALSE]
		}
		expr <- log2(expr + 1)
	} 
	sampleName <- colnames(expr)

	## mean, variance, 'cv', 'AP', 'density', 'correlation', 'sample relation'
	mm <- colMeans(expr)
	std <- apply(expr, 2, sd)
	
	## AP calls
	if (!is.null(x.lumi@QC$sampleSummary)) {
		detectionRate <- x.lumi@QC$sampleSummary[3,]
		detectionName <- rownames(x.lumi@QC$sampleSummary)[3]
	} else {
		detectionName <- paste('detection rate(', detectionTh, ')', sep='')
		if (!is.null(detection(x.lumi))) {
			detectionRate <- detectionCall(x.lumi, Th=detectionTh, type='sample') / nrow(x.lumi)
		} else {
			detectionRate <- rep(NA, ncol(x.lumi))
		}
	}

	## detect outlier
	center <- rowMeans(expr)
	profile <- cbind(center, expr)
	colnames(profile) <- c('Center', colnames(expr))
	distCenter <- as.matrix(dist(t(profile), method="euclidean"))

	sampleSummary <- rbind(mm, std, detectionRate, distCenter[2:nrow(distCenter),1])
	rownames(sampleSummary) <- c('mean', 'standard deviation', detectionName, 'distance to sample mean')
	sampleSummary <- signif(sampleSummary, 4)

	## record history
	history.finished <- as.character(Sys.time())
	# history.command <- capture.output(print(match.call(lumiQ)))
	history.command <- paste(deparse(match.call(lumiQ)), collapse='') 
	
	if (is.null(x.lumi@history$lumiVersion)) x.lumi@history$lumiVersion <- rep(NA, nrow(x.lumi@history))
	lumiVersion <- packageDescription('lumi')$Version
	x.lumi@history <- rbind(x.lumi@history, data.frame(submitted=history.submitted, 
			finished=history.finished, command=history.command, lumiVersion=lumiVersion))

	## create a QC slot
	QC <- x.lumi@QC
	QC$sampleSummary <- sampleSummary
	QC$history <- x.lumi@history
	
	x.lumi@QC <- QC
	return(x.lumi)
}

`rankinvariant` <-
function(x.lumi, targetArray=NULL, rrc=.05, lowRank = seq(0.5, 0.25, -0.05), highRank=.9, minSize=.02, maxit=200) {
	if (is(x.lumi, 'ExpressionSet')) {
		expr  <- exprs(x.lumi)
	} else if (is.numeric(x.lumi)) {
		expr <- as.matrix(x.lumi)
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}

	stopifnot(min(lowRank)>0)
	stopifnot(max(lowRank)<1)
	stopifnot(length(highRank) == 1)
	stopifnot(highRank < 1)
	stopifnot(max(lowRank) < highRank)

	externalTarget <- FALSE
	if (!is.null(targetArray)) {
		## check the format of the targetArray
		if (is(targetArray, 'ExpressionSet')) {
			targetArray <- exprs(targetArray)[,1]
		} 
		if (length(targetArray) > 1) {
			if (length(targetArray) != nrow(expr)) stop('targetArray should be an index or a vector has the same length as other samples.')
			expr <- cbind(targetArray, expr)
			targetArray <- 1
			externalTarget <- TRUE
		}
		if (is.numeric(targetArray)) {
			if (targetArray < 1 || targetArray > nrow(expr)) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else if (is.character(targetArray)) {
			if (!(targetArray %in% colnames(expr))) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else {
			warning('The provided targetArray is invalid and will be set as NULL!')
			targetArray <- NULL
		}
	}

	## check whether the data was variance stabilized.
	if (max(expr, na.rm=TRUE) > 100) {
		log2Trans <- FALSE
	} else {
		log2Trans <- TRUE
		expr <- 2^(expr)
	}

	#add virtual sample if target is NULL
	if(is.null(targetArray)) {
		expr <- cbind(rowMeans(expr),expr)
		targetArray <- 1
		externalTarget <- TRUE
	}
	nrows <- nrow(expr)
	nArray <- ncol(expr)

	#prepare normalized data obj
	normalized <- expr

	#rank all data and virtual
	er <- apply(expr, 2, rank)

	## determine the rank invariant geneset
	ri <- rep(F, nrow(expr))
	rankscores <- apply(er, 2, function(r, v) abs( (r - v )/length(v) ), er[,targetArray])
	for(i in 1:nArray) {
		if(i == targetArray) next;
		for(lowrank in lowRank) {	
			if(getOption("verbose")) cat("Sample", i - ifelse(externalTarget,1,0), "using lowrank ", lowrank,"\n")
			ri <- rankscores[,i] < rrc & er[,i] > nrows*lowrank & er[,i] < nrows*highRank
			if(getOption("verbose")) cat("Nr rankinvariant probes=", sum(ri),"\n")
			if(sum(ri) >= minSize * nrows) {
				#normalize this sample
				coef <- rlm(expr[ri, targetArray] ~ expr[ri, i], psi=psi.bisquare, maxit=maxit)$coef
				normalized[,i] <- (expr[,i]*coef[2]) + coef[1]
				break;
			}
		}
		if(lowrank == tail(lowRank,1)) stop(paste("No rankinvariant set for sample", i - ifelse(externalTarget,1,0),"use broader lowRank/highRank or use another normalization method"))
	}

	#remove virtual from normalized data if target is external of rowMeans
	if(externalTarget) normalized <- normalized[,-1]

	## transformed as original scale in not log2transformed
	if (log2Trans) {
		if (min(normalized) <= 0) {
			normalized <- lumiB(normalized, method='forcePositive')
		}
		normalized <- log2(normalized)	
	} 
	
	if (is(x.lumi, 'ExpressionSet')) {
		exprs(x.lumi) <- normalized
	} else {
		x.lumi <- normalized
	}
	
	return(x.lumi)
}

`getControlData` <-
function(x, type=c('data.frame', 'LumiBatch'), ...) 
{
	type <- match.arg(type)
	if (is.character(x)) {
		if (type == 'data.frame') {
			if ('columnNameGrepPattern' %in% names(list(...))) {
				allControlInfo <- lumiR.batch(x, lib.mapping=NULL, checkDupId=FALSE, convertNuID=FALSE, QC=FALSE, annotationColumn=NULL, ...)
			} else {
				allControlInfo <- lumiR.batch(x, lib.mapping=NULL, checkDupId=FALSE, convertNuID=FALSE, 
					columnNameGrepPattern = list(exprs='AVG_SIGNAL', se.exprs=NA, detection=NA, beadNum=NA), QC=FALSE, annotationColumn=NULL, ...)				
			}
		} else {
			allControlInfo <- lumiR.batch(x, lib.mapping=NULL, checkDupId=FALSE, convertNuID=FALSE, QC=FALSE, annotationColumn=NULL, ...)
			return(allControlInfo)
		}
		x <- allControlInfo
	} 

	if (is(x, 'ExpressionSet')) {
		if (type == 'LumiBatch') {
			return(x)
		} else {
			if (.hasSlot(x, 'controlData')) {
				return(controlData(x))
			} 
			controlData <- as.data.frame(exprs(x))
			controlType <- pData(featureData(x))[,'TargetID']
			ProbeID <- pData(featureData(x))$ProbeID
			if (length(which(toupper(controlType) == 'NEGATIVE')) > 10) {
				controlNames <- names(controlData)
				controlData <- data.frame(controlType=as.character(controlType), ProbeID=as.character(ProbeID), controlData, stringsAsFactors=FALSE)
				names(controlData) <- c('controlType', 'ProbeID', controlNames)
			} else {
				stop("Can't find controlData slot, nor do the TargetID looks like control probes")
			}
		}		
	} else {
		stop('Input data should be a control data file or a LumiBatch object!')
	}
	return(controlData)
}

`produceGEOPlatformFile` <-
function(x.lumi, lib.mapping=NULL, nuIDMode=TRUE, includeAllChipProbe=FALSE, fileName='GEOPlatformFile.txt') {

	if (includeAllChipProbe) {
		chipInfo <- getChipInfo(x.lumi, lib.mapping=lib.mapping)
		chipVersion <- chipInfo$chipVersion[1]
		chipInfo <- getChipInfo(NULL, lib.mapping=lib.mapping, chipVersion=chipVersion, idMapping=TRUE)
	} else {
		chipInfo <- getChipInfo(x.lumi, lib.mapping=lib.mapping, idMapping=TRUE)
		chipVersion <- chipInfo$chipVersion[1]
	}
	idMapping <- chipInfo$idMapping
	chipVersion <- sub("(_V[0-9.]*)_.*$", "\\1", chipVersion)
	organism <- switch(chipInfo$species,
		'Rat'='Rattus norvegicus',
		'Human'="Homo sapiens",
		'Mouse'='Mus musculus')

	platformInfoTitle <- c("Platform_title", "Platform_technology", "Platform_distribution", "Platform_organism", "Platform_manufacturer", "Platform_manufacture_protocol", "Platform_description")
	platformInfo <- c(paste("Illumina expression beadchip", chipVersion), "oligonucleotide beads", "commercial", organism, "Illumina Inc.", "see manufacturer's website", "see manufacturer's website")
	if (nuIDMode) {
		platformInfo[platformInfoTitle == "Platform_title"] <- paste("All Illumina", chipInfo$species,  "expression beadchips based on nuID")
		platformInfo[platformInfoTitle == "Platform_description"] <- paste("This platform pools probe IDs of all Illumina", chipInfo$species,  "expression beadchips based on nuID. nuIDs are defnined based on probe sequence, they are universally unique. Using nuIDs will make data integration and maintenance very easier. See Bioconductor lumi package for more details of nuID annotation.")
	} 
	
	headerTitle <- c("Illumina_ProbeID", "nuID", "Illumina_Gene", "Search_Key", "Accession", "Symbol", "ProbeSequence")
	headerDef <- c("Illumina Probe ID", "nucleotide universal NUcleotide (convertible to and from probe sequence)", "Illumina Gene ID", "Internal id useful for custom design array", "Genbank accession number", "Gene Symbol", "Probe Sequence")
	

	cat('^PLATFORM =', platformInfo[1], '\n', sep='', file=fileName, append=FALSE)
	platformInfoPrint <- paste('!', platformInfoTitle, ' = ', platformInfo, '\n', sep='')
	cat(platformInfoPrint, file=fileName, sep='', append=TRUE)
	mapName <- colnames(idMapping)
	nuID <- idMapping[, 'nuID']
	probeSequence <- id2seq(nuID)
	probe <- idMapping[,grep('probe', mapName, ignore.case=TRUE)]
	gene <- idMapping[,grep('target|gene', mapName, ignore.case=TRUE)]
	accession <- idMapping[,grep('accession', mapName, ignore.case=TRUE)]
	searchKey <- idMapping[,grep('search_key', mapName, ignore.case=TRUE)]
	symbol <- idMapping[,grep('symbol', mapName, ignore.case=TRUE)]
	mappingInfo <- cbind(probe, nuID, gene, searchKey, accession, symbol, probeSequence)

	if (nuIDMode) {
		headerTitle[c(1,2)] <- headerTitle[c(2,1)]		
		headerDef[c(1,2)] <- headerDef[c(2,1)]
		mappingInfo[,c(1,2)] <- mappingInfo[,c(2,1)]
	}
	headerTitle[1] <- 'ID'
	mappingInfo <- rbind(headerTitle, mappingInfo)
	
	platformInfoPrint <- paste('#', headerTitle, ' = ', headerDef, '\n', sep='')
	cat(platformInfoPrint, file=fileName, sep='', append=TRUE)
	cat("!platform_table_begin\n", file=fileName, sep='', append=TRUE)
	write.table(mappingInfo, sep='\t', quote=FALSE, file=fileName, append=TRUE, col.names=FALSE, row.names=FALSE)
	cat("!platform_table_end\n", file=fileName, sep='', append=TRUE)
}

`getNuIDMappingInfo` <-
function(nuID=NULL, lib.mapping) {

	if (missing(lib.mapping)) stop('Please specify lib.mapping library!')
	if(!require(lib.mapping, character.only=TRUE)) stop(paste(lib.mapping, 'is required!'))
	# dbconn <- sub("\\.db", "_dbconn", lib.mapping)
	dbconn <- paste(lib.mapping, "_dbconn", sep='')
	conn <- do.call(dbconn, list())
    
	allTableNames <- dbListTables(conn)	
	fieldName <- dbListFields(conn, 'nuID_MappingInfo')
	nuIDMappingInfo <- dbReadTable(conn, 'nuID_MappingInfo')
	allNuID <- nuIDMappingInfo[,'nuID']
	rownames(nuIDMappingInfo) <- allNuID
	nuIDMappingInfo <- nuIDMappingInfo[,-1]
	if (!is.null(nuID)) {
		# check unID
		if (!all(sapply(nuID, is.nuID))) stop('Some inputted nuIDs are not real nuIDs!\n')
		mappingInfo <- matrix(rep(NA, ncol(nuIDMappingInfo)*length(nuID)), ncol=ncol(nuIDMappingInfo))
		rownames(mappingInfo) <- nuID
		colnames(mappingInfo) <- colnames(nuIDMappingInfo)
		selNuID <- nuID[nuID %in% allNuID]
		if (length(selNuID) == 0) {
			warning('No matches were found!\n')
		} else {
			if (length(selNuID) < length(nuID)) warning('Some input IDs can not be matched!\n')
			mappingInfo[selNuID, ] <- as.matrix(nuIDMappingInfo[selNuID,])
		}
	} else {
		mappingInfo <- nuIDMappingInfo
	}
	return(mappingInfo)
}

`detectionCall` <-
function(x.lumi, Th = 0.01, type=c('probe', 'sample', 'matrix')) {
	type <- match.arg(type)
	if (Th > 0.5)  Th <- 1 - Th
	if (is(x.lumi, 'matrix')) {
		detect <- x.lumi
	} else {
		if (!assayDataValidMembers(assayData(x.lumi), "detection")) 
			stop('The object should be class "eSet" inherited classes and include "detection" element in the assayData!')
		detect <- assayDataElement(x.lumi, 'detection')
		if (is.null(detect)) {
			warning('No detection slot found!')
			return(NULL)
		}
		
		## check the detection is p-values or not
		if (class(x.lumi) == "MethyLumiM") {
			expr <- estimateIntensity(x.lumi, returnType='matrix')
		} else {
			expr <- exprs(x.lumi)
		}
		# low <- mean(expr[detect[,1] > 0.9,1])
		# high <- mean(expr[detect[,1] < 0.1,1])
		tmp <- expr[,1]
		low <- tmp[!is.na(tmp)][which.max(detect[!is.na(tmp),1])]
		high <- tmp[!is.na(tmp)][which.min(detect[!is.na(tmp),1])]
		if (low > high) detect <- 1 - detect
	}
	
	if (!is.null(detect)) {
		if (type == 'sample') AP <- colSums(detect<= Th)
		if (type == 'probe') AP <- rowSums(detect<= Th)
		if (type == 'matrix') {
			AP <- detect
			AP[detect <= Th] <- 'P'
			AP[detect > Th] <- 'A'
		}
		attr(AP, 'threshold') <- Th
	} else {
		AP <- rep(NA, ncol(x.lumi))
	}
	return(AP)
}

`rsn` <-
function(x.lumi, targetArray=NULL, excludeFold=2, span=0.03, ifPlot=FALSE,...) {
	if (is(x.lumi, 'ExpressionSet')) {
		# x.lumi is a lumi object
		exprs <- exprs(x.lumi)
	} else if (is.numeric(x.lumi)) {
		exprs <- as.matrix(x.lumi)
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}

	externalTarget <- FALSE
	if (!is.null(targetArray)) {
		## check the format of the targetArray
		if (is(targetArray, 'ExpressionSet')) {
			targetArray <- exprs(targetArray)[,1]
		} 
		if (length(targetArray) > 1) {
			if (length(targetArray) != nrow(exprs)) stop('targetArray should be an index or a vector has the same length as other samples.')
			exprs <- cbind(targetArray, exprs)
			targetArray <- 1
			externalTarget <- TRUE
		}
		if (is.numeric(targetArray)) {
			if (targetArray < 1 || targetArray > nrow(exprs)) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else if (is.character(targetArray)) {
			if (!(targetArray %in% colnames(exprs))) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else {
			warning('The provided targetArray is invalid and will be set as NULL!')
			targetArray <- NULL
		}
	}

	## check whether the data was variance stabilized.
	if (max(exprs, na.rm=TRUE) > 100) {
		log2Trans <- FALSE
		
		offset <- ifelse(min(exprs, na.rm=TRUE) < 0, min(exprs, na.rm=TRUE) - 1, 0)
		exprs <- log2(exprs - offset)
	} else {
		log2Trans <- TRUE
	}

	## Define interal function 
	pairwiseN <- function(ind, exprs, targetArray, exprs0=NULL, ifPlot=FALSE) {
		cat(as.character(Sys.time()), ", processing array ", ind, "\n")

		# normal array ind against targetArray      
		u1 <- exprs[,ind]
		u2 <- exprs[, targetArray]
		u1.original <- u1
		u2.original <- u2

		## define window functions
		win <- function(x, sigma=1, type=c('gaussian', 'tricube', 'bisqure')) {
			type <- match.arg(type)
			ww <- switch(type,
				gaussian = exp(-x^2/(2*sigma^2)) / (sqrt(2*pi)*sigma),
				tricube = (1 - (abs(x)/(3 * sigma))^3)^3,
				bisquare = (1 - (x/(3 * sigma))^2)^2 )
			return(ww)
		}

		if (ind != targetArray[1] || length(targetArray) > 1) {
			## calculate the weights based on the fold change 
			if (!is.null(exprs0)) {
				fd <- exprs0[,ind] - exprs0[, targetArray]
				if (!is.null(excludeFold)) {
					sigma <- log2(excludeFold)/3
				} else {
					sigma <- sd(fd)
				}
 				wt <- win(abs(fd), sigma)
				wt <- wt/max(wt)
			} else {
				wt <- rep(length(u1))
			}

			u1.normalized <- monoSmu(u1, u2, newX=u1.original, span=span, ifPlot=ifPlot, wt=wt, rotate=TRUE,
					xlab=paste("array", ind), ylab=paste("array", targetArray), ...)
		} else {
			u1.normalized <- u1
		}
		return(u1.normalized)
	}

	if (ifPlot) par(mfrow=c(2,2))

	## do quantile normalization for the purpose of estimating fold change
	## Based on the estimated fold change, we can down-weight the differentiated genes.
	if (!is.null(excludeFold)) {
		exprs0 <- normalize.quantiles(exprs)
	} else {
		exprs0 <- NULL
	}

	if (is.null(targetArray)) {
		# find the sample which is the most similar to the mean profile of all samples,
		meanProfile <- apply(exprs, 1, mean, na.rm=TRUE)
		targetArray <- which.min(abs(colSums(exprs - meanProfile)))
	}

	nArray <- ncol(exprs)
	normalized <- lapply(1:nArray, FUN=pairwiseN, exprs=exprs,
					exprs0=exprs0, targetArray=targetArray, ifPlot=ifPlot)
	normalized <- matrix(unlist(normalized), ncol=nArray, byrow=FALSE)	
	colnames(normalized) <- colnames(exprs)
	rownames(normalized) <- rownames(exprs)

	## if the targetArray is an external vector, it will be removed from the normalized data.
	if (externalTarget) normalized <- normalized[,-1]
	## transformed as original scale in not log2transformed
	if (!log2Trans) normalized <- 2^normalized
	
	if (is(x.lumi, 'ExpressionSet')) {
		exprs(x.lumi) <- normalized
		if (is.numeric(targetArray)) {
			if (!is.null(attr(x.lumi, 'vstParameter'))) {
				attr(x.lumi, 'vstParameter') <- attr(x.lumi, 'vstParameter')[targetArray,]
				attr(x.lumi, 'transformFun') <- attr(x.lumi, 'transformFun')[targetArray]
				attr(x.lumi, 'targetArray') <- sampleNames(x.lumi)[targetArray]
			}
		} else {
			if (!is.null(attr(targetArray, 'vstParameter'))) {
				attr(x.lumi, 'vstParameter') <- attr(targetArray, 'vstParameter')
				attr(x.lumi, 'transformFun') <- attr(targetArray, 'transformFun')
			}
			if (is(targetArray, 'ExpressionSet'))
				attr(x.lumi, 'targetArray') <- sampleNames(targetArray)
		}
	} else {
		x.lumi <- normalized
	}
	
	return(x.lumi)
}

`inverseVST` <-
function(x, fun=c('asinh', 'log'), parameter=NULL) {
	if (missing(x)) {
		stop('Please provide the data!')
	}
	if (!is.null(attr(x, 'vstParameter'))) {
		parameter <- attr(x, 'vstParameter')
		fun <- attr(x, 'transformFun')
	} else {
		if (is.null(parameter)) stop('Please provide the transform parameters!')
		fun <- match.arg(fun)
		## parameter is in the format of: "a, b, g, Intercept"
		a <- parameter[1]
		b <- parameter[2]
		g <- parameter[3]
		intercept <- parameter[4]
	}
	if (is(x, 'ExpressionSet')) {
		inv <- x
		attr(inv, 'vstParameter') <- NULL
		attr(inv, 'transformFun') <- NULL
		if (length(fun) == 1) {
			a <- parameter[1]
			b <- parameter[2]
			g <- parameter[3]
			intercept <- parameter[4]
			if (fun == 'asinh') {
				inv.x <- (sinh((exprs(x) - intercept)/g) - a)/b
			} else if (fun == 'log') {
				inv.x <- (exp((exprs(x) - intercept)/g) - a)/b
			}
			rownames(inv.x) <- rownames(exprs(x))
			colnames(inv.x) <- colnames(exprs(x))
			exprs(inv) <- inv.x
		} else {
			if (length(fun) != ncol(inv)) stop('The length of "fun" does not match the data!')
			for (i in 1:length(fun)) {
				fun.i <- fun[i]
				parameter.i <- parameter[i,]
				a <- parameter.i[1]
				b <- parameter.i[2]
				g <- parameter.i[3]
				intercept <- parameter.i[4]
				if (fun.i == 'asinh') {
					exprs(inv)[,i] <- (sinh((exprs(x)[,i] - intercept)/g) - a)/b
				} else if (fun.i == 'log') {
					exprs(inv)[,i] <- (exp((exprs(x)[,i] - intercept)/g) - a)/b
				}
			}
		}
	} else {
		if (fun == 'asinh') {
			## Transform function h(x) = g * asinh(a + b * x)
			## transFun <- g * asinh(a + b * x) * m$coef[2] + m$coef[1]
			inv <- (sinh((x - intercept)/g) - a)/b
		} else if (fun == 'log') {
			## Transform function h(x) = g * log(a + b * x)
			## transFun <- g * log(a + b * x) * m$coef[2] + m$coef[1]
			inv <- (exp((x - intercept)/g) - a)/b
		}
	}
	
	return(inv)
}

`addControlData2lumi` <-
function(controlData, x.lumi) 
{
	if (missing(x.lumi) || missing(controlData)) stop('Both controlData and x.lumi are required!')
	if (is.character(controlData)) {
		controlData <- getControlData(controlData, type='data.frame')
	}
	if (is.matrix(controlData)) controlData <- as.data.frame(controlData)
	if (is(controlData, 'data.frame')) {
		## match the column names of controlData and LumiBatch object
		sampleID <- as.character(pData(phenoData(x.lumi))$sampleID)
		if (length(sampleID) == 0) sampleID <- sampleNames(x.lumi)
		controlSampleID <- names(controlData)
		if ('TargetID' %in% controlSampleID) {
			names(controlData)[controlSampleID == 'TargetID'] <- 'controlType'
		} else {
			names(controlData)[1] <- 'controlType'
		}
		probeId.pos <- grep('Probe.?ID', controlSampleID, ignore.case=TRUE)
		if (length(probeId.pos) > 0) {
			names(controlData)[probeId.pos] <- 'ProbeID'
		} else {
			controlData$ProbeID <- NA
		}
		retrieveColName <- c('controlType', 'ProbeID')
		
		if (all(sampleID %in% controlSampleID)) {
			x.lumi@controlData <- controlData[, c(retrieveColName, sampleID)]
		} else {
			sampleIDInfo <- strsplit(sampleID, split="_")
			newID <- NULL
			temp <- lapply(sampleIDInfo, function(x) {
				newID <<- c(newID, paste(x[1:2], collapse="_"))
			})
			if (all(newID %in% controlSampleID)) {
				controlData(x.lumi) <- controlData[, c(retrieveColName, newID)]
			} else {
				stop('SampleID does not match up between controlData and x.lumi!')				
			}
		}
		names(controlData(x.lumi)) <- c(retrieveColName, sampleNames(x.lumi))		
	} else {
		stop('Input data type is not supported!')
	}
	return(x.lumi)
}

`ssn` <-
function(x.lumi, targetArray=NULL, scaling=TRUE, bgMethod=c('density', 'mean', 'median', 'none'), fgMethod=c('mean', 'density', 'median'), ...) {
	if (is(x.lumi, 'ExpressionSet')) {
		expr  <- exprs(x.lumi)
	} else if (is.numeric(x.lumi)) {
		expr <- as.matrix(x.lumi)
	} else {
		stop('The object should be a matrix or class "ExpressionSet" inherited!')
	}
	bgMethod <- match.arg(bgMethod)
	fgMethod <- match.arg(fgMethod)
	if (bgMethod != 'density' && bgMethod == fgMethod) scaling <- FALSE
	if (bgMethod != 'density' && fgMethod == 'density') {
		warning('fgMethod "density" can only be used together with the bgMethod "density"!\n bgMethod is set as "density".')
		bgMethod <- 'density'
	}
	externalTarget <- FALSE
	if (!is.null(targetArray)) {
		## check the format of the targetArray
		if (is(targetArray, 'ExpressionSet')) {
			targetArray <- exprs(targetArray)[,1]
		} 
		if (length(targetArray) > 1) {
			if (length(targetArray) != nrow(expr)) stop('targetArray should be an index or a vector has the same length as other samples.')
			expr <- cbind(targetArray, expr)
			targetArray <- 1
			externalTarget <- TRUE
		}
		if (is.numeric(targetArray)) {
			if (targetArray < 1 || targetArray > nrow(expr)) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else if (is.character(targetArray)) {
			if (!(targetArray %in% colnames(expr))) {
				warning('The provided targetArray is invalid and will be set as NULL!')
				targetArray <- NULL
			}
		} else {
			warning('The provided targetArray is invalid and will be set as NULL!')
			targetArray <- NULL
		}
	}

	## check whether the data was variance stabilized.
	if (max(expr, na.rm=TRUE) > 100) {
		log2Trans <- FALSE
	} else {
		log2Trans <- TRUE
		expr <- 2^(expr)
	}

	## Estimate the background level and the AUC of density curve (mean) after removing the background level
	twoPoint <- apply(expr, 2, function(xx) {
		hh <- hist(xx, 1000, plot=FALSE)
		Th <- hh$breaks[which.max(hh$counts) + 1] * 2
		dd <- density(xx[xx < Th], na.rm=TRUE, ...)
		# bg <- dd$x[which.max(dd$y)]
		bg <- switch(bgMethod,
			none = 0,
			density = dd$x[which.max(dd$y)],
			mean = mean(xx),
			median = median(xx))
		bgNum <- 2 * length(which(xx <= bg))
		# estimate the foreground level by removing the background probes
		fg <- switch(fgMethod,
			density = (sum(xx) - bgNum * bg) / (length(xx) - bgNum),
			mean = mean(xx),
			median = median(xx))
		twoPoint <- c(bg, fg)
		return(twoPoint)
	})

	if (is.null(targetArray)) {
		target2P <- apply(twoPoint, 1, mean)
	} else {
		target2P <- twoPoint[, targetArray]
	}
	
	## normalization based on twoPoint (background and foreground levels)
	normalized <- expr
	nArray <- ncol(expr)
	for (i in 1:nArray) {
		if (scaling) {
			ff <- abs((target2P[2] - target2P[1]) / (twoPoint[2,i] - twoPoint[1,i]))
		} else {
			ff <- 1
		}
		normalized[,i] <- (expr[,i] - twoPoint[1,i]) * ff + target2P[1]
	}

	## if the targetArray is an external vector, it will be removed from the normalized data.
	if (externalTarget) normalized <- normalized[,-1]
	## transformed as original scale in not log2transformed
	if (log2Trans) {
		if (min(normalized) <= 0) {
			normalized <- lumiB(normalized, method='forcePositive')
		}
		normalized <- log2(normalized)	
	} 
	
	if (is(x.lumi, 'ExpressionSet')) {
		exprs(x.lumi) <- normalized
	} else {
		x.lumi <- normalized
	}
	
	return(x.lumi)
}

## cdf plot in reverse direction (from high value to low value)
plotCDF <- function(x, reverse=TRUE, logMode=TRUE, xlab = NULL, ylab = "Cumulative density", col=1:dim(x)[2], 
	lwd=1, xlim=NULL, index.highlight=NULL, color.highlight=2, addLegend=TRUE,  main='',...) 
{
	if (is(x, 'ExpressionSet')) {
	    expr <- exprs(x)
	} else if (is.numeric(x)) {
		expr <- as.matrix(x)
	} else {
		stop('Un-supported class of x.')
	}
		
    if (logMode && (max(expr, na.rm=TRUE) > 50)) {
		# remove the negative values
		if (min(expr) < 0) {
			rMin <- rowMin(expr)
			expr <- expr[rMin > 0, , drop=FALSE]
		}
		expr <- log2(expr)
		if (is.null(xlab)) 
			xlab <- "log2 intensity"
    } else if (is.null(xlab)) 
        xlab <- "intensity"

	index <- 1:nrow(expr)
	expr <- expr[index,,drop=FALSE]
	
	if (reverse) {
		x.cdf <- apply(expr, 2, function(x) ecdf(-x))
	} else {
		x.cdf <- apply(expr, 2, function(x) ecdf(x))
	}
	usedCol <- usedLwd <- NULL
	for (i in 1:ncol(expr)) {
		col.i <- ifelse(i > length(col), col[1], col[i])
		lwd.i <- ifelse(i > length(lwd), lwd[1], lwd[i])
		if (i %in% index.highlight) {
			col.i <- color.highlight
			lwd.i <- 2
		}
		if (i == 1) {
			if (reverse) {
				x.range <- range(-expr)
			} else {
				x.range <- range(expr)
			}		
			plot(x.cdf[[i]], xlab=xlab, ylab = ylab, verticals = F, col=col.i,  lwd=lwd.i, xlim=x.range, axes=F, main=main, ...)
			tmp <- Axis(x=seq(x.range[1], x.range[2], len=100), side=1, labels=NA)
			if (reverse) {
				axis(side = 1, at = tmp, labels = -tmp)
			} else {
				axis(side = 1, at = tmp, labels = tmp)
			}
			Axis(x=seq(0,1,len=100), side=2)
			box()
		} else {
			plot(x.cdf[[i]], xlab=NULL, ylab = NULL, verticals = F, col=col.i, lwd=lwd.i, add = T, ...)
		}		
		usedCol <- c(usedCol, col.i)
		usedLwd <- c(usedLwd, lwd.i)
	}

	## add legend
	if (addLegend) {
		labels <- colnames(expr)
		if (is.null(labels)) labels <- as.character(1:ncol(expr))
		if (reverse) {
			x.range <- range(-expr)
			x.pos <- (max(x.range) - min(x.range)) * 1/10 + min(x.range)
			y.pos <- 0.95
		} else {
			x.range <- range(expr)
			x.pos <- max(x.range) - (max(x.range) - min(x.range)) * 1/3
			y.pos <- 0.5
		}
		legend(x.pos, y.pos, legend=labels, col=usedCol, lwd=usedLwd)
	}
	
    invisible(TRUE)
}

setGeneric("plot", function(x, y, ...) standardGeneric("plot"))
setGeneric("pairs", function(x, ...) standardGeneric("pairs"))
setGeneric("boxplot", function(x, ...) standardGeneric("boxplot"))
setGeneric("density", function(x, ...) standardGeneric("density"))
setGeneric("hist", function(x, ...) standardGeneric("hist"))
setGeneric("MAplot", function(object, ...) standardGeneric("MAplot"))

setGeneric("beadNum", function(object) standardGeneric("beadNum"))
setGeneric("beadNum<-", function(object, value) standardGeneric("beadNum<-"))

setGeneric("controlData", function(object) standardGeneric("controlData"))
setGeneric("controlData<-", function(object, value) standardGeneric("controlData<-"))

setGeneric("detection", function(object) standardGeneric("detection"))
setGeneric("detection<-", function(object, value) standardGeneric("detection<-"))

setGeneric("getHistory", function(object) standardGeneric("getHistory"))

setGeneric("asBigMatrix", function(object, ...) standardGeneric("asBigMatrix"))


if (is.null(getGeneric("show"))) setGeneric("show", function(object) standardGeneric("show"))
if (is.null(getGeneric("combine"))) setGeneric("combine", function(x, y, ...) standardGeneric("combine"))

`getControlProbe` <-
function(controlData, type=NULL) 
{
	if (is(controlData, 'LumiBatch')) {
		controlData <- controlData(controlData)
	} 
	allControlType <- controlData$controlType
	uniControlType <- getControlType(controlData)
	allProbeID <- controlData$ProbeID
	selProbeID <- allProbeID
	if (!is.null(type)) {
		type <- toupper(type)
		allControlType <- toupper(allControlType)
		if (type %in% uniControlType) {
			selProbeID <- allProbeID[allControlType == type[1]]
		} 
	} 
	return(selProbeID)
}

`produceGEOSubmissionFile` <-
function(lumiNormalized, lumiRaw, lib.mapping=NULL, idType='Probe', sampleInfo=NULL, fileName='GEOSubmissionFile.txt', supplementaryRdata=TRUE, ...) {
	if (missing(lumiNormalized)) stop('Please provide all required input parameters!\n')
	if (is.matrix(lumiNormalized)) {
		expr.norm <- lumiNormalized
		detect <- NULL
		se.expr <- NULL
		expr <- NULL
		beadNum <- NULL
		if (!missing(lumiRaw)) expr <- lumiRaw
	} else {
		expr.norm <- exprs(lumiNormalized)
		detect <- detection(lumiRaw)
		se.expr <- se.exprs(lumiRaw)
		expr <- exprs(lumiRaw)
		beadNum <- beadNum(lumiRaw)
	}
	if (is.null(sampleInfo)) {
		sampleInfo <- produceGEOSampleInfoTemplate(lumiNormalized, lib.mapping=lib.mapping, fileName=NULL)
	} else if (length(sampleInfo) == 1 && is.character(sampleInfo)) {
		sampleInfo <- read.table(sampleInfo, sep='\t', colClasses='character', skip=1, header=TRUE, strip.white=TRUE, quote='')
	} else if (is.null(nrow(sampleInfo))) {
		stop('Please provide correct sample information (a data.frame, matrix, or sampleInfo file)!\n')
	}
	sampleInfoTitle <- colnames(sampleInfo)
	if (any(sapply(sampleInfo[,-1, drop=F], nchar) == 0)) stop('No blank fields are allowed in the sampleInfo table!\nYou can check some example submissions, like GSM296418, at the GEO website.\n')
	if (supplementaryRdata) sampleInfo[, "Sample_supplementary_file"] <- 'supplementaryData.Rdata'
	if (is.matrix(lumiNormalized)) {
		nuID <- rownames(lumiNormalized)
	} else {
		nuID <- featureNames(lumiNormalized)
	}
	probeId <- nuID
	if (length(which(is.nuID(sample(nuID, 100)))) < 20) {
		nuID <- NULL
	} else {
		if (!is.null(lib.mapping)) {
			probeId <- nuID2IlluminaID(nuID, lib.mapping=lib.mapping, idType=idType, ...)
		} else {
			nuID <- NULL
		}
		# else {
		# 	fData <- pData(featureData(lumiRaw))
		# 	if (!is.null(fData[,ID])) {
		# 		probeId <- fData[,ID]
		# 	}
		# }
	}
	
	sampleID <- sampleInfo[, "sampleID"]
	sampleTitle <- sampleInfo[,'Sample_title']
	for (i in seq(sampleID)) {
		if (i == 1) {
			cat('^SAMPLE = ', sampleID[i], '\n', sep='', file=fileName, append=FALSE)
		} else {
			cat('^SAMPLE = ', sampleID[i], '\n', sep='', file=fileName, append=TRUE)			
		}
		sampleInfo.i <- paste('!', sampleInfoTitle[-1], ' = ', sampleInfo[i,-1], '\n', sep='', collapse='')
		sampleInfo.i <- gsub("'", "\\'", sampleInfo.i)
		cat(sampleInfo.i, file=fileName, append=TRUE, sep='')
		tableHead <- "ID_REF"
		cat("#ID_REF = \n", file=fileName, append=TRUE)
		if (!is.null(nuID)) {
			cat("#nuID = nucleotide universal IDentifier (nuID), convertible to and from probe sequence. See Bioconductor lumi package for more details.\n", file=fileName, append=TRUE)
			tableHead <- c(tableHead, "nuID")
		}
		cat("#VALUE = normalized signal intensity\n", file=fileName, append=TRUE)
		tableHead <- c(tableHead, "VALUE")
		if (!is.null(expr)) {
			cat("#RAW_VALUE = raw signal intensity\n", file=fileName, append=TRUE)
			tableHead <- c(tableHead, "RAW_VALUE")			
		}
		if (!is.null(se.expr)) {
			cat("#BEAD_STDERR = the standard error of the probe measurements\n", file=fileName, append=TRUE)
			tableHead <- c(tableHead, "BEAD_STDERR")
		}
		if (!is.null(detect)) {
			cat("#Detection_Pval = the detection p-value of the probe\n", file=fileName, append=TRUE)
			tableHead <- c(tableHead, "Detection_Pval")
		}
		if (!is.null(beadNum)) {
			cat("#Avg_NBEADS = Number of beads for the probe\n", file=fileName, append=TRUE)
			tableHead <- c(tableHead, "Avg_NBEADS")
		}
		sampleTable.i <- probeId
		if (!is.null(nuID)) sampleTable.i <- cbind(sampleTable.i, nuID)
		sampleTable.i <- cbind(sampleTable.i, expr.norm[,sampleID[i]])
		if (!is.null(expr)) sampleTable.i <- cbind(sampleTable.i, expr[,sampleID[i]])
		if (!is.null(se.expr)) sampleTable.i <- cbind(sampleTable.i, se.expr[,sampleID[i]])
		if (!is.null(detect)) sampleTable.i <- cbind(sampleTable.i, detect[,sampleID[i]])
		if (!is.null(beadNum)) sampleTable.i <- cbind(sampleTable.i, beadNum[,sampleID[i]])
		sampleTable.i <- rbind(tableHead, sampleTable.i)
		cat("!sample_table_begin\n", file=fileName, append=TRUE)
		write.table(sampleTable.i, sep='\t', quote=FALSE, file=fileName, append=TRUE, col.names=FALSE, row.names=FALSE)
		cat("!sample_table_end\n", file=fileName, append=TRUE)
	}
	
	if (supplementaryRdata) {
		lumiNormalized <- lumiNormalized[,sampleID]
		if (!missing(lumiRaw)) {
			lumiRaw <- lumiRaw[,sampleID]
			save(lumiNormalized, lumiRaw, sampleInfo, file='supplementaryData.Rdata')
		} else {
			save(lumiNormalized, sampleInfo, file='supplementaryData.Rdata')
		}
	}
}

`plotStringencyGene` <-
function(controlData, lib=NULL, slideIndex=NULL, addLegend=TRUE, logMode=TRUE, ...) 
{
	if (is(controlData, 'LumiBatch')) {
		sampleID <- pData(phenoData(controlData))$sampleID
		controlData <- controlData(controlData)
		if (nrow(controlData) == 0) stop('Slot controlData is empty!')
	} else {
		sampleID <- NULL
	}
	allControlType <- controlData$controlType
	allControlProbe <- controlData$ProbeID
	uniControlType <- getControlType(controlData)
	controlData <- controlData[, -c(1,2)]

	if (is.null(slideIndex)) {
		if (is.null(sampleID)) sampleID <- colnames(controlData)
		sampleIDInfo <- strsplit(sampleID, split="_")
		chipID <- sapply(sampleIDInfo, function(x) x[1])
		ord <- order(chipID)
		chipID <- chipID[ord]
		controlData <- controlData[, ord]
		chip <- as.numeric(as.factor(chipID))
		chipNum <- length(unique(chipID))
	} else {
		ord <- order(slideIndex)
		controlData <- controlData[, ord]
		slideIndex <- slideIndex[ord]
		chip <- as.numeric(as.factor(slideIndex))
		chipNum <- length(unique(slideIndex))
	}
	
	# get HOUSEKEEPING
	ind <- grep('STRINGENCY', allControlType, ignore.case=TRUE)
	if (length(ind) == 0) stop('No STRINGENCY related gene information found!')
	strType <- allControlType[ind]
	ord <- order(strType)
	strType <- strType[ord]
	ind <- ind[ord]
	hkgene <- allControlProbe[ind]
	selControlData <- controlData[ind, ]
	if (!is.null(lib)) {
		# require(annotate)
		require(lib, character.only=TRUE)
		hkSymbol <- getSYMBOL(hkgene, lib)
		hkSymbol[is.na(hkSymbol)] <- hkgene[is.na(hkSymbol)]
		hkgene <- hkSymbol
	}
	
	if (logMode) {
		if (max(selControlData) > 50) {
			selControlData <- selControlData - min(selControlData) + 1
			selControlData <- log2(selControlData)
		}
		ylab <- 'Expression Amplitude (log2)'
	} else {
		if (max(selControlData) < 50) selControlData <- 2^(selControlData)
		ylab <- 'Expression Amplitude'
	}
	labels <- colnames(selControlData)
	if (is.null(labels)) labels <- as.character(1:ncol(selControlData))
	## set the margin of the plot
	mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
	old.mar <- par('mar')
	old.xaxt <- par('xaxt')
	par(xaxt='n')
	par(mar=mar)
	
	col <- lty <- as.numeric(as.factor(strType))
	if (addLegend) {
		xlim <- c(1, ncol(selControlData) + 1)
	} else {
		xlim <- c(1, ncol(selControlData))
	}
	matplot(t(selControlData), pch = 19, cex=1.2, type='o', col=col, lty=lty, xlim=xlim,
	 	xlab='', ylab=ylab)
	title('Expression profile of housekeeping genes')
	# plot the vertical lines
	if (chipNum > 1) abline(v = 0.5 + which(diff(chip) != 0), lty=2)
	if (addLegend) legend('topright', legend=unique(strType), lty=unique(lty), col=unique(col), inset = .02)

	par(xaxt='s')
	axis(1, at=1:ncol(selControlData), labels=labels, tick=TRUE, las=2)
	par(mar=old.mar)
	par(xaxt=old.xaxt)
	
	return(invisible(TRUE))
}

### R code from vignette source 'methylationAnalysis.Rnw'

###################################################
### code chunk number 1: load library
###################################################
library(lumi)


###################################################
### code chunk number 2: load example dataset
###################################################
## load example data (a methyLumiM object)
data(example.lumiMethy)
## summary of the example data
example.lumiMethy
## print sample Names
sampleNames(example.lumiMethy)


###################################################
### code chunk number 3: sampleRelation
###################################################
plotSampleRelation(example.lumiMethy, method='mds', cv.Th=0) 


###################################################
### code chunk number 4: sampleRelationTree
###################################################
plotSampleRelation(example.lumiMethy, method='cluster', cv.Th=0) 


###################################################
### code chunk number 5: load example titration dataset
###################################################
## load the tritration data (a methyLumiM object)
data(example.methyTitration)
## summary of the example data
example.methyTitration
## print sample Names
sampleNames(example.methyTitration)


###################################################
### code chunk number 6: sampleRelationTitration
###################################################
plotSampleRelation(example.methyTitration, method='mds', cv.Th=0) 


###################################################
### code chunk number 7: densityMTitration
###################################################
## plot the density
density(example.methyTitration, xlab="M-value")	


###################################################
### code chunk number 8: densityM
###################################################
## specify the colors of control and treatment samples
sampleColor <- rep(1, ncol(example.lumiMethy))
sampleColor[grep("Treat", sampleNames(example.lumiMethy))] <- 2 

density(example.lumiMethy, col=sampleColor, xlab="M-value")	## plot the density


###################################################
### code chunk number 9: boxplotM
###################################################
## Because the distribution of M-value has two modes, we use a boxplot different from regular ones
boxplot(example.lumiMethy)


###################################################
### code chunk number 10: densityColorBiasBoth
###################################################
plotColorBias1D(example.lumiMethy)


###################################################
### code chunk number 11: densityColorBiasMethy
###################################################
plotColorBias1D(example.lumiMethy, channel='methy')


###################################################
### code chunk number 12: densityColorBiasUnmethy
###################################################
plotColorBias1D(example.lumiMethy, channel='unmethy')


###################################################
### code chunk number 13: boxplotColorBiasMethy
###################################################
boxplotColorBias(example.lumiMethy, channel='methy')


###################################################
### code chunk number 14: boxplotColorBiasUnmethy
###################################################
boxplotColorBias(example.lumiMethy, channel='unmethy')


###################################################
### code chunk number 15: color balance summary
###################################################
## summary of color balance information of individual samples
colorBiasSummary(example.lumiMethy[,1:8], channel='methy')


###################################################
### code chunk number 16: scatterColorBias1
###################################################
plotColorBias2D(example.lumiMethy, selSample=1, cex=2)


###################################################
### code chunk number 17: boxplotColorBiasSum
###################################################
boxplotColorBias(example.lumiMethy, channel='sum')


###################################################
### code chunk number 18: densityColorBiasSum
###################################################
plotColorBias1D(example.lumiMethy, channel='sum')


###################################################
### code chunk number 19: densityIntensity
###################################################
density(estimateIntensity(example.lumiMethy), xlab="log2(CpG-site Intensity)")


###################################################
### code chunk number 20: boxplotIntensity
###################################################
boxplot(estimateIntensity(example.lumiMethy))


###################################################
### code chunk number 21: pairsColor
###################################################
## get the color channel information
colorChannel <- as.character(pData(featureData(example.lumiMethy))[, "COLOR_CHANNEL"])
## replace the "Red" and "Grn" as color names defined in R
colorChannel[colorChannel == 'Red'] <- 'red'
colorChannel[colorChannel == 'Grn'] <- 'green'
## select a subet of sample for pair plot
selSample <- c( "Ctrl1", "Ctrl1.rep", "Treat1", "Treat1.rep")
## plot pair plot with the dots in scatter plot colored based on the color channels
pairs(estimateIntensity(example.lumiMethy[, selSample]), dotColor= colorChannel, main="Pair plot of CpG-site Intensity")


###################################################
### code chunk number 22: color balance adjustment
###################################################
## summary of color balance information of individual samples
lumiMethy.c.adj <- lumiMethyC(example.lumiMethy)


###################################################
### code chunk number 23: densityColorBiasSumAdj
###################################################
plotColorBias1D(lumiMethy.c.adj, channel='sum')


###################################################
### code chunk number 24: boxplotColorBiasSumAdj
###################################################
boxplotColorBias(lumiMethy.c.adj, channel='sum')


###################################################
### code chunk number 25: scatterColorBias1Adj
###################################################
## plot the color balance adjusted scatter plot of two color channels
plotColorBias2D(lumiMethy.c.adj, selSample=1, cex=2)


###################################################
### code chunk number 26: pairsColorAdj
###################################################
## plot pairwise plot after color balance adjustment 
pairs(estimateIntensity(lumiMethy.c.adj[, selSample]), dotColor= colorChannel, main="Pair plot of CpG-site Intensity after color balance adjustment")


###################################################
### code chunk number 27: background adjustment
###################################################
##separately adjust backgrounds of two color channels
lumiMethy.b.adj <- lumiMethyB(example.lumiMethy, method="bgAdjust2C", separateColor=TRUE)

##background adjustment of individual samples
lumiMethy.bc.adj <- lumiMethyB(lumiMethy.c.adj, method="bgAdjust2C")


###################################################
### code chunk number 28: bgDensityMethy
###################################################
## plot the background mode of methylated probe data of first five example samples
plotColorBias1D(example.lumiMethy[,1:5], channel='methy', xlim=c(-1000,5000), logMode=FALSE)


###################################################
### code chunk number 29: bgAdjDensityMethy
###################################################
## plot the background mode of methylated probe data of first five example samples
plotColorBias1D(lumiMethy.b.adj [,1:5], channel='methy', xlim=c(-1000,5000), logMode=FALSE)


###################################################
### code chunk number 30: bcAdjDensityMethy
###################################################
## plot the background mode of methylated probe data of first five example samples
plotColorBias1D(lumiMethy.bc.adj [,1:5], channel='methy', xlim=c(-1000,5000), logMode=FALSE)


###################################################
### code chunk number 31: Normalization
###################################################
## Perform quantile normalization based on color balance adjusted data
lumiMethy.c.quantile <- lumiMethyN(lumiMethy.c.adj, method='quantile')
## Perform SSN normalization based on color balance adjusted data
lumiMethy.c.ssn <- lumiMethyN(lumiMethy.c.adj, method='ssn')


###################################################
### code chunk number 32: sampleRelationTreeNormalized
###################################################
plotSampleRelation(lumiMethy.c.quantile, method='cluster', cv.Th=0) 


###################################################
### code chunk number 33: densityQuantile
###################################################
## plot the density of M-values after quantile normalization
density(lumiMethy.c.quantile, col= sampleColor, main="Density plot after quantile normalization")


###################################################
### code chunk number 34: densityIntensityNormalizedQ
###################################################
density(estimateIntensity(lumiMethy.c.quantile), col= sampleColor,  xlab="log2(CpG-site Intensity)")


###################################################
### code chunk number 35: boxplotColorBiasNormalized
###################################################
boxplotColorBias(lumiMethy.c.quantile, channel='sum')


###################################################
### code chunk number 36: pairMNormalize
###################################################
## select a subet of sample for pair plot
selSample <- c( "Ctrl1", "Ctrl1.rep", "Treat1", "Treat1.rep")
## plot pair plot with the dots in scatter plot colored based on the color channels
pairs(lumiMethy.c.quantile[, selSample], dotColor= colorChannel, main="Pair plot of M-value after normalization")


###################################################
### code chunk number 37: user defined preprocessing functions (eval = FALSE)
###################################################
## ## suppose "userB" is a user defined background adjustment method
##  lumiMethy.b.adj <- lumiMethyB(example.lumiMethy, method=userB, separateColor=TRUE) # not Run
## 
## ## suppose "userC" is a user defined color balance adjustment method
##  lumiMethy.c.adj <- lumiMethyC(example.lumiMethy, method=userC, separateColor=TRUE) # not Run
## 
## ## suppose "userN" is a user defined probe level normalization method
##  lumiMethy.c.n <- lumiMethyN(lumiMethy.c.adj, method= userN, separateColor=TRUE) # not Run


###################################################
### code chunk number 38: Color channel information
###################################################
## retrieve the featureData information
ff <- pData(featureData(example.lumiMethy))

## show the color channel information
head(ff)

## add user provided color channel information if it is not existed in the featureData
# example.lumiMethy <- addAnnotationInfo(example.lumiMethy, lib="IlluminaHumanMethylation27k.db")


###################################################
### code chunk number 39: options to separately process each color channel (eval = FALSE)
###################################################
## ## suppose "userB" is a user defined background adjustment method
##  lumiMethy.b.adj <- lumiMethyB(example.lumiMethy,  separateColor=TRUE) 
## 
## ## suppose "userC" is a user defined color balance adjustment method
##  lumiMethy.c.adj <- lumiMethyC(example.lumiMethy,  separateColor=TRUE) 
## 
## ## suppose "userN" is a user defined probe level normalization method
##  lumiMethy.c.n <- lumiMethyN(lumiMethy.c.adj,  separateColor=TRUE) 


###################################################
### code chunk number 40: Estimate detection call of a CpG site (eval = FALSE)
###################################################
## ## Estimate the detection call of a CpG site
## presentCount <- detectionCall(example.lumiMethy)


###################################################
### code chunk number 41: Preprocessing methylation microarray (eval = FALSE)
###################################################
## 
## library(lumi)
## ## specify the file name
## # fileName <- 'Example_Illumina_Methylation_profile.txt' 
## ## load the data
## # example.lumiMethy <- lumiMethyR(fileName, lib="IlluminaHumanMethylation27k.db")		# Not Run
## 
## ## Quality and color balance assessment
## data(example.lumiMethy)
## ## summary of the example data
## example.lumiMethy
## 
## ## preprocessing and quality control after normalization
## plotColorBias1D(example.lumiMethy, channel='sum')
## boxplotColorBias(example.lumiMethy, channel='sum')
## ## select interested sample to further check color balance in 2D scatter plot
## plotColorBias2D(example.lumiMethy, selSample=1)
## 
## ## Color balance adjustment between two color channels
## lumiMethy.c.adj <- lumiMethyC(example.lumiMethy)
## ## Check color balance after color balance adjustment
## boxplotColorBias(lumiMethy.c.adj, channel='sum')
## 
## ## Background adjustment is skipped because the SSN normalization includes background adjustment
## 
## ## Normalization
## ## Perform SSN normalization based on color balance adjusted data
## lumiMethy.c.ssn <- lumiMethyN(lumiMethy.c.adj, method='ssn')
## 
## ## Or we can perform quantile normalization based on color balance adjusted data
## # lumiMethy.c.q <- lumiMethyN(lumiMethy.c.adj, method='quantile')
## 
## ## plot the density of M-values after SSN normalization
## density(lumiMethy.c.ssn, main="Density plot of M-value after SSN normalization")
## ## comparing with the density of M-values before normalization
## density(example.lumiMethy, main="Density plot of M-value of the raw data")
## 
## ## output the normlized M-value as a Tab-separated text file
## write.exprs(lumiMethy.c.ssn, file='processedMethylationExampleData.txt')


###################################################
### code chunk number 42: sessionInfo
###################################################
toLatex(sessionInfo())

`monoSpline` <-
function(x, y, newX=NULL, nKnots=6, ifPlot=FALSE) {
	# behavior: if extropolation, outside values are constant to the
	#  nearest inside one
	# if(!require(mgcv)) stop('Package "mgcv" should be installed!')
	
	nKnots <- round(nKnots)
	if (is.null(newX)) newX <- x
	
	## make sure there is no other "s" functions before mgcv::s in the search path
	# rmPackage <- find('s')
	# if (rmPackage[1] != "package:mgcv") {
	# 	detach("package:mgcv")
	# 	library(mgcv)
	# }
	
	# Show regular spline fit (and save fitted object)
	f.ug <- gam(y ~ s(x, k=nKnots, bs="cr"))
	
	# Create Design matrix, constraints etc. for monotonic spline....
	dat <- data.frame(x=x, y=y)
	sm <- smoothCon(s(x, k=nKnots, bs="cr"), dat, knots=NULL)[[1]]
	if (length(sm$xp) < 6) warning('Few than 6 nKnots were specified!\n Possible inaccurate fitting!\n')
	F <- mono.con(sm$xp)   # get constraints
	G <- list(X=sm$X, C=matrix(0,0,0), sp=f.ug$sp, p=sm$xp, y=y, w=y*0+1, 
			Ain=F$A, bin=F$b, S=sm$S, off=0)
	
	p <- pcls(G);  # fit spline(using s.p. from unconstrained fit)
	fv <- Predict.matrix(sm, data.frame(x=newX)) %*% p
	fv <- as.vector(fv)
	
	if (ifPlot) {
	    plot(x,y); lines(newX, fv, col=2)
	}
	
	return(fv)
}

`lumiT` <-
function(x.lumi, method=c('vst', 'log2', 'cubicRoot'), ifPlot=FALSE, simpleOutput = TRUE, verbose = TRUE, ...) {
	# if (!is(x.lumi, 'LumiBatch')) stop('The object should be class "LumiBatch"!')
	method <- match.arg(method)
	if (is(x.lumi, 'eSet')) {
		if (method == 'vst') {
			if (is.null(se.exprs(x.lumi)))  stop('Slot se.exprs is required!')
			if (!all(dim(se.exprs(x.lumi)) == dim(exprs(x.lumi))))
				stop('Dimensions of slots exprs and se.exprs do not match!')
		}
	} else {
		stop('The object should be class "eSet" inherited!')
	}
	## the stdCorrection was moved to lumiR function on April 15, 2011.
	# if (!is(x.lumi, 'LumiBatch')) stdCorrection <- FALSE

	## check the negative values
	if (method == 'log2') {
		if (min(exprs(x.lumi), na.rm=TRUE) <= 0) {
			x.lumi <- lumiB(x.lumi, method='forcePositive')
		}
	}
	if (verbose) cat(paste('Perform', method, 'transformation ...\n'))
	history.submitted <- as.character(Sys.time())

    new.lumi <- x.lumi 
	exprs <- exprs(x.lumi) 
	if (max(exprs, na.rm=TRUE) < 50) {
		warning("Looks like the data has been log transformed! \n It should be in the raw scale! \n Please double check!\n")
	}
	if (method == 'log2') {
		exprs(new.lumi) <- log2(exprs)
	} else if (method == 'cubicRoot') {
		exprs(new.lumi) <- sign(exprs) * (abs(exprs))^1/3
	} else {
		se.exprs <- se.exprs(x.lumi)
		## the stdCorrection was moved to lumiR function on April 15, 2011.
#		if (stdCorrection & is(x.lumi, 'LumiBatch')) {
#			bn <- beadNum(x.lumi)
#			if (is.null(bn)) {
#				cat('No Standard Deviation correction was applied because of missing bead number information.\n')
#			} else {
#				se.exprs <- se.exprs * sqrt(bn)
#			}
#		}
		nArray <- ncol(exprs)
		if (is(x.lumi, 'LumiBatch')) {
			detectCall <- detectionCall(x.lumi, Th=0.01, type='matrix')			
		} else {
			detectCall <- NULL
		}
		transExpr <- NULL
		transPara <- NULL
		transFun <- NULL
		for (i in 1:nArray) {
			cat(as.character(Sys.time()), ", processing array ", i, "\n")
			u <- exprs[,i]; std <- se.exprs[,i]
			lowCutoff <- 1/3
			backgroundStd <- NULL
			if (!is.null(detectCall)) {
				ind <- which(detectCall[,i] == 'A')
				if (nrow(se.exprs) - length(ind) < 3000) {
					warning('Too few probes are detectable based on detection p-values!\n 
						Iteration method will be used for VST.')
					lowCutoff <- 1/4
				} else {
					backgroundStd <- mean(se.exprs[ind,i], na.rm=TRUE)
				}
			} else if (is(x.lumi, 'AffyBatch')) {
				lowCutoff <- 1/4
				# afbatch.i <- x.lumi[,i]
				# pmInd <- unlist(indexProbes(afbatch.i, 'pm'))
				# mmInd <- unlist(indexProbes(afbatch.i, 'mm'))
				# pm <- exprs(afbatch.i)[pmInd]
				# mm <- exprs(afbatch.i)[mmInd]
				# bgInd <- pmInd[mm > pm]
				# bgStd <- se.exprs[bgInd,i]
				# hh <- hist(bgStd, 1000, plot=FALSE)
				# Th <- hh$breaks[which.max(hh$counts) + 1] * 2
				# dd <- density(bgStd[bgStd < Th], na.rm=TRUE)
				# backgroundStd <- dd$x[which.max(dd$y)]
				# backgroundStd <- mean(bgStd)
			}
		    x <- vst(u=u, std=std, backgroundStd=backgroundStd, lowCutoff=lowCutoff, ifPlot=ifPlot, ...)
			transExpr <- cbind(transExpr, x)
			
			transPara <- rbind(transPara, attr(x, 'parameter'))
			transFun <- c(transFun, attr(x, 'transformFun'))
		}
         dimnames(transExpr) <- dimnames(exprs)
#        if (!is.null(transExpr)) rownames(transExpr) <- colnames(exprs(x.lumi))
		if (!is.null(transPara))	rownames(transPara) <- colnames(exprs(x.lumi))
		if (!is.null(transFun))	names(transFun) <- colnames(exprs(x.lumi))
		exprs(new.lumi) <- transExpr
	}
	colnames(exprs(new.lumi)) <- colnames(exprs(x.lumi))
	rownames(exprs(new.lumi)) <- rownames(exprs(x.lumi))

	if (simpleOutput) {
		if (is(x.lumi, 'LumiBatch')) {
			se.exprs(x.lumi) <- NULL
			beadNum(x.lumi) <- NULL
			# storage.mode <- storageMode(new.lumi)
			# if ("lockedEnvironment" == storage.mode) {
			# 	aData <- copyEnv(assayData(new.lumi))
			# 	# rm(list=c('se.exprs', 'detection', 'beadNum'), envir=aData)
			# 	rm(list=c('se.exprs', 'beadNum'), envir=aData)
			# 	lockEnvironment(aData, bindings = TRUE)
			# 	assayData(new.lumi) <- aData
			# } else {
			# 	aData <- assayData(new.lumi)
			# 	# rm(list=c('se.exprs', 'detection', 'beadNum'), envir=aData)
			# 	rm(list=c('se.exprs', 'beadNum'), envir=aData)
			# 	assayData(new.lumi) <- aData
			# }
		}
		if (is(x.lumi, 'AffyBatch')) {
			se.exprs(x.lumi) <- new('matrix')
		}
	}

	if (is(x.lumi, 'LumiBatch')) {
		# history tracking
		history.finished <- as.character(Sys.time())
		# history.command <- capture.output(print(match.call(lumiT)))
		history.command <- paste(deparse(match.call(lumiT)), collapse='') 

		if (is.null(new.lumi@history$lumiVersion)) new.lumi@history$lumiVersion <- rep(NA, nrow(new.lumi@history))
		lumiVersion <- packageDescription('lumi')$Version
		new.lumi@history<- rbind(new.lumi@history, data.frame(submitted=history.submitted, 
				finished=history.finished, command=history.command, lumiVersion=lumiVersion))
	}

	if (method == 'vst') {
		attr(new.lumi, 'vstParameter') <- transPara
		attr(new.lumi, 'transformFun') <- transFun
	}

    return(new.lumi)
}

`plotVST` <-
function(x, transFun = NULL, plotRange = NULL, addLegend = TRUE, ...) {
	if (missing(x)) {
		stop('Please provide all input parameters!')
	} else {
		parameter <- NULL
		if (is(x, 'LumiBatch')) {
			parameter <- attr(x, 'vstParameter')
			transFun <- attr(x, 'transformFun')
			if (is.null(plotRange)) {
				plotRange <- range(exprs(x))
				if (max(plotRange) < 100) plotRange <- 2^plotRange
			}
		}
		if (is.null(plotRange)) plotRange <- c(10, 10000)
		if (is(x, 'data.frame')) parameter <- as.matrix(x)
		if (length(x) == 4) {
			parameter <- matrix(x, nrow=1)
			colnames(parameter) <- names(x)
		}
		## parameter is in the format of: "a, b, g, Intercept"
		if (!all(colnames(parameter) %in% c('a', 'b', 'g', 'Intercept')))
			stop('Incorrect format of input x!')
		if (is.null(transFun)) transFun <- rep('asinh', nrow(parameter))
		if (length(transFun) != nrow(parameter)) stop('Length of "transFun" does not match "x"!')
	}
	
	xx <- NULL
	trans <- NULL
	for (i in 1:nrow(parameter)) {
		a <- parameter[i, 'a']
		b <- parameter[i, 'b']
		g <- parameter[i, 'g']
		intercept <- parameter[i, 'Intercept']
		fun <- transFun[i]
		xx.i <- seq(plotRange[1], plotRange[2], length=500)
		if (fun == 'asinh') {
			## Transform function h(x) = g * asinh(a + b * x)
			## transFun <- g * asinh(a + b * x) * m$coef[2] + m$coef[1]
			trans.i <- g * asinh(a + b * xx.i) + intercept
		} else if (fun == 'log') {
			## Transform function h(x) = g * log(a + b * x)
			## transFun <- g * log(a + b * x) * m$coef[2] + m$coef[1]
			trans.i <- g * log(a + b * xx.i) + intercept
		}
		xx <- cbind(xx, xx.i)
		trans <- cbind(trans,trans.i)
	}
	matplot(xx, trans, type='l', lty=1:nrow(parameter), col=1:nrow(parameter), xlab='un-transformed value', ylab='transformed value', ...)
	if (addLegend) {
		r.trans <- range(trans)
		legend(x=max(xx)*2/3, y=min(r.trans) + diff(r.trans)/3, legend=rownames(parameter), lty=1:nrow(parameter), col=1:nrow(parameter))
	}
	return(invisible(list(untransformed=xx, transformed=trans)))
}

`plotControlData` <-
function(controlData, type=NULL, slideIndex=NULL, logMode=FALSE, new=TRUE, ...) 
{
	methylationData <- FALSE
	if (is(controlData, "MethyLumiM")) {
		controlData <- controlData(controlData)
	}
	if (is(controlData, "MethyLumiQC")) {
		methylationData <- TRUE
		# For control data, methylated data corresponds to green channel
		# For control data, unmethylated data corresponds to red channel
		grnData <- assayDataElement(controlData, "methylated") 
		redData <- assayDataElement(controlData, "unmethylated")
		allControlType <- sapply(strsplit(featureNames(controlData), "\\."), function(x) x[1])
		uniControlType <- getControlType(controlData)
	} else {
		if (is(controlData, 'LumiBatch')) {
			sampleID <- pData(phenoData(controlData))$sampleID
			controlData <- controlData(controlData)
			if (nrow(controlData) == 0) stop('Slot controlData is empty!')
		} else {
			sampleID <- NULL
		}
		allControlType <- controlData$controlType
		uniControlType <- getControlType(controlData)
		controlData <- controlData[, -c(1,2)]

		if (is.null(slideIndex)) {
			if (is.null(sampleID)) sampleID <- colnames(controlData)
			sampleIDInfo <- strsplit(sampleID, split="_")
			chipID <- sapply(sampleIDInfo, function(x) x[1])
			ord <- order(chipID)
			chipID <- chipID[ord]
			controlData <- controlData[, ord]
			col <- as.numeric(as.factor(chipID))
			chipNum <- length(unique(chipID))
		} else {
			ord <- order(slideIndex)
			controlData <- controlData[, ord]
			slideIndex <- slideIndex[ord]
			col <- as.numeric(as.factor(slideIndex))
			chipNum <- length(unique(slideIndex))
		}

	}
	
	if (!is.null(type)) {
		type <- toupper(type)
		allControlType <- toupper(allControlType)
		if (!all(type %in% uniControlType)) {
			warning('"type" does not match "controlType". Please use getControlType function to view the available type!')
			type <- ''
		} 
	} else {
		type <- ''
	}
	
	if (methylationData) {
		## dealing with the methylation control data
		if (logMode) {
			if (max(c(redData, grnData)) > 50) {
				redData[redData < 1] <- 1
				grnData[grnData < 1] <- 1

				redData <- log2(redData)
				grnData <- log2(grnData)
			}
			ylab <- 'Intensity (log2)'
		} else {
			if (max(c(redData, grnData)) < 50) {
				redData <- 2^(redData)
				grnData <- 2^(grnData)
			} 
			ylab <- 'Intensity'
		}

		if (length(type) > 1) oldPar <- par(mfrow=c(ceiling(length(type)/2), 2))
		for (type.i in type) {
			if (type.i != '') {
				selRedData <- redData[allControlType == type.i, , drop=FALSE]
				selGrnData <- grnData[allControlType == type.i, , drop=FALSE]
			} else {
				selRedData <- redData
				selGrnData <- grnData
			}

			if (nrow(selRedData) > 1) {
				mm.red <- apply(selRedData, 2, mean)
				std.red <- apply(selRedData, 2, sd)
				mm.grn <- apply(selGrnData, 2, mean)
				std.grn <- apply(selGrnData, 2, sd)
			} else {
				mm.red <- selRedData
				mm.grn <- selGrnData
				std.red <- std.grn <- rep(0, length(mm))
			}

			labels <- colnames(selRedData)
			if (is.null(labels)) labels <- as.character(1:ncol(selRedData))
			## set the margin of the plot
			mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
			old.mar <- par('mar')
			old.xaxt <- par('xaxt')
			par(xaxt='n')
			par(mar=mar)

			ind <- ncol(selRedData)
			if (new) {
				range <- c(min(c(mm.red - std.red * 1.5, mm.grn - std.red * 1.5)), max(c(mm.red + std.red * 1.1, mm.grn + std.grn * 1.1)))
				plot(1:ind, mm.red, pch = 19, cex=1.8, type='o', col='red', xlab='', ylab=ylab, ylim=range, ...)
				points(1:ind, mm.grn, pch = 19, cex=1.8, type='o', col='green', ...)
				title(type.i)
			} else {
				points(1:ind, mm.red, pch = 19, cex=1.8, type='o', col='red', ...)
				points(1:ind, mm.grn, pch = 19, cex=1.8, type='o', col='green', ...)
			}
			if (nrow(selRedData) > 1) {
				arrows(1:ind, mm.red - std.red, 1:ind, mm.red + std.red, code = 3, col='red', angle = 90, length = .1)
				arrows(1:ind, mm.grn - std.grn, 1:ind, mm.grn + std.grn, code = 3, col='green', angle = 90, length = .1)
			}

			par(xaxt='s')
			axis(1, at=1:ncol(selRedData), labels=labels, tick=TRUE, las=2)
			par(mar=old.mar)
			par(xaxt=old.xaxt)
		}
		if (length(type) > 1) par(oldPar)
		
	} else {
		if (logMode) {
			if (max(controlData) > 50) {
				controlData[controlData < 1] <- 1
				controlData <- log2(controlData)
			}
			ylab <- 'Intensity (log2)'
		} else {
			if (max(controlData) < 50) controlData <- 2^(controlData)
			ylab <- 'Intensity'
		}

		if (length(type) > 1) oldPar <- par(mfrow=c(ceiling(length(type)/2), 2))
		for (type.i in type) {
			if (type.i != '') {
				selControlData <- controlData[allControlType == type.i, , drop=FALSE]
			} else {
				selControlData <- controlData
			}

			if (nrow(selControlData) > 1) {
				mm <- apply(selControlData, 2, mean)
				std <- apply(selControlData, 2, sd)
			} else {
				mm <- selControlData
				std <- rep(0, length(mm))
			}

			labels <- colnames(selControlData)
			if (is.null(labels)) labels <- as.character(1:ncol(selControlData))
			## set the margin of the plot
			mar <- c(max(nchar(labels))/2 + 4.5, 5, 5, 3)
			old.mar <- par('mar')
			old.xaxt <- par('xaxt')
			par(xaxt='n')
			par(mar=mar)

			ind <- ncol(selControlData)
			if (new) {
				range <- c(min(mm - std * 1.5), max(mm + std * 1.1))
				plot(1:ind, mm, pch = 19, cex=1.8, type='o', col=col, xlab='', ylab=ylab, ylim=range, ...)
				title(type.i)
			} else {
				points(1:ind, mm, pch = 19, cex=1.8, type='o', col=col, ...)
			}
			if (nrow(selControlData) > 1) {
				arrows(1:ind, mm - std, 1:ind, mm + std, code = 3, col=col, angle = 90, length = .1)
			}
			# plot the vertical lines
			if (chipNum > 1) {
				abline(v=0.5 + which(diff(col) != 0), lty=2)
			}

			par(xaxt='s')
			axis(1, at=1:ncol(selControlData), labels=labels, tick=TRUE, las=2)
			par(mar=old.mar)
			par(xaxt=old.xaxt)
		}
		if (length(type) > 1) par(oldPar)
	}
	
	return(invisible(TRUE))
}

`lumiExpresso` <- 
function (lumiBatch, bg.correct = TRUE, bgcorrect.param = list(method='bgAdjust'), variance.stabilize = TRUE, 
	varianceStabilize.param = list(), normalize=TRUE, normalize.param = list(), 
	QC.evaluation = TRUE, QC.param = list(), verbose = TRUE) 
{
	if (verbose) {
		if (bg.correct) {
			bgMethod <- ifelse(is.null(bgcorrect.param$method), 'bgAdjust', bgcorrect.param$method)
			if (!is(lumiBatch, 'LumiBatch')) {
				bgMethod <- "none"
				bgcorrect.param$method <- bgMethod
				cat("Due to the input is not a LumiBatch object, no background adjustment will be performed.\n")
			} 
			cat("Background Correction:", bgMethod, "\n")
		}
		if (variance.stabilize) {
			vstMethod <- ifelse(is.null(varianceStabilize.param$method), 'vst', varianceStabilize.param$method)
			if (is.null(se.exprs(lumiBatch)) && vstMethod == 'vst') {
				vstMethod <- "log2"
				varianceStabilize.param$method <- vstMethod
				cat("Due to the lack of 'se.exprs' information, 'log2' transformation will be used.\n")
			} 
			cat("Variance Stabilizing Transform method:", vstMethod, "\n")
		}
		if (normalize) {
			normMethod <- ifelse(is.null(normalize.param$method), 'quantile', normalize.param$method)
			normalize.param$method <- normMethod
			cat("Normalization method:", normMethod, "\n")
		}
		cat('\n')
	}
	if (bg.correct) {
		if (verbose) cat("\nBackground correction ...\n")
		lumiBatch <- do.call(lumiB, c(alist(lumiBatch), bgcorrect.param))
		if (verbose) cat("done.\n")
	}
	if (variance.stabilize) {
		if (verbose) cat("\nVariance stabilizing ...\n")
		lumiBatch <- do.call(lumiT, c(alist(lumiBatch), varianceStabilize.param))
		if (verbose) cat("done.\n")
	}
	if (normalize) {
		if (verbose) cat("\nNormalizing ...\n")
		lumiBatch <- do.call(lumiN, c(alist(lumiBatch), normalize.param))
		if (verbose) cat("done.\n")
	}
	if (QC.evaluation) {
		if (verbose) cat("\nQuality control after preprocessing ...\n")
		lumiBatch <- do.call(lumiQ, c(alist(lumiBatch), QC.param))
		if (verbose) cat("done.\n")
	}
	return(lumiBatch)
}

`getChipInfo` <-
function(x, lib.mapping=NULL, species=c('Human', 'Mouse', 'Rat', 'Unknown'), chipVersion=NULL, idMapping=FALSE, returnAllMatches=FALSE, verbose=TRUE) {
	
	## Function to make sure the output mapping matches the inputID.bak
	matchInputID <- function(mapping, inputID.bak) {
		## We assume mapping IDs are a subset of the names of inputID.bak
		nc <- ncol(mapping)
		if (is.null(nc)) {
			len <- length(mapping)
			if (len == length(inputID.bak)) return(mapping)
			mapping.new <- rep(NA, length(inputID.bak))
			names(mapping.new) <- inputID.bak
			mappingID <- names(mapping)
			mapping.new[mappingID] <- mapping
			type <- 'vector'
		} else {
			nr <- nrow(mapping)
			if (nr == length(inputID.bak)) return(mapping)
			mapping.new <- matrix(NA, nrow=length(inputID.bak), ncol=nc)
			rownames(mapping.new) <- inputID.bak
			colnames(mapping.new) <- colnames(mapping)
			mappingID <- rownames(mapping)
			mapping.new[mappingID,] <- mapping[mappingID,]
			type <- 'matrix'
		}
		selID <- inputID.bak[inputID.bak %in% mappingID]
		dupInd <- which(duplicated(selID))
		if (length(dupInd) > 0) {
			dupID <- unique(selID[dupInd])
			for (dupId.i in dupID) {
				ind.i <- which(inputID.bak == dupId.i)
				if (type == 'matrix') {
					mapping.new[ind.i, ] <- mapping[dupId.i, ]
				} else {
					mapping.new[ind.i] <- mapping[dupId.i]					
				}
			}
		}
		return(mapping.new)
	}

	
	if (is(x, 'ExpressionSet')) {
		inputID <- featureNames(x)
	} else if (is(x, 'matrix') || is(x, 'data.frame')) {
		inputID <- rownames(x)		
	} else {
		inputID <- as.character(x)
	}
	inputID.bak <- inputID
	inputID <- unique(inputID)
	species <- match.arg(species)
	if (species == 'Unknown' && is.null(lib.mapping)) {
		human.match <- getChipInfo(inputID, species='Human', idMapping=idMapping, returnAllMatches=returnAllMatches, verbose=FALSE)
		mouse.match <- getChipInfo(inputID, species='Mouse', idMapping=idMapping, returnAllMatches=returnAllMatches, verbose=FALSE)
		rat.match <- getChipInfo(inputID, species='Rat', idMapping=idMapping, returnAllMatches=returnAllMatches, verbose=FALSE)
		bestChip <- which.max(c(length(human.match$matchedProbeNumber), length(mouse.match$matchedProbeNumber), length(rat.match$matchedProbeNumber)))
		best.match <- switch(bestChip, human.match, mouse.match, rat.match)
		return(best.match)
	} else {
		if (is.null(lib.mapping)) {
			lib.mapping <- switch(species,
				'Rat'='lumiRatIDMapping',
				'Human'='lumiHumanIDMapping',
				'Mouse'='lumiMouseIDMapping')			
		}		
		if(!require(lib.mapping, character.only=TRUE)) stop(paste(lib.mapping, 'is required!'))
		# dbconn <- sub("\\.db", "_dbconn", lib.mapping)
		dbconn <- paste(lib.mapping, "_dbconn", sep="")
		conn <- do.call(dbconn, list())
		
		metaInfo <- dbReadTable(conn, 'metadata')
		species <- metaInfo$value[metaInfo[,'name'] == "SPECIES"]
		
		allTableNames <- dbListTables(conn)
		allTableNames <- allTableNames[!(allTableNames %in% c('nuID_MappingInfo', 'metadata'))]
		if (!is.null(chipVersion)) {
			if (chipVersion %in% allTableNames) {
				allTableNames <- chipVersion
			} else {
				warning('The provided chipVersion does not exist in the library!\n')
			}
		}
		lenID <- length(inputID)
		if (lenID == 0 && !is.null(chipVersion)) {
			## return the entire table based on chipVersion
			mapping <- dbReadTable(conn, chipVersion)
			returnList <- list(chipVersion=chipVersion, species=species, IDType=NA, chipProbeNumber=nrow(mapping), 
				inputProbeNumber=0, matchedProbeNumber=NA)
		} else {
			matchLen <- NULL
			matchField <- NULL
			tableLen <- NULL
			cut0.probeId <- FALSE  # determine whether to cut the 0s in front of the ProbeId. E.g. 0004760445 --> 4760445
			for (i in seq(allTableNames)) {
				tableName.i <- allTableNames[i]
				table.i <- dbReadTable(conn, tableName.i)
				fieldNames.i <- names(table.i)
				## Not include the Symbol column during matching the IDs.
				fieldNames.i <- fieldNames.i[!(fieldNames.i %in% c('Symbol'))]
				len.i <- NULL
				for (fieldNames.ij in fieldNames.i) {
					field.ij <- as.character(table.i[,fieldNames.ij])
					len.ij <- length(which(inputID %in% field.ij))
					if (fieldNames.ij %in% c('ProbeId', 'Array_Address_Id')) {
						len.ij.2 <- length(which(inputID %in% as.character(as.integer(field.ij))))
						if (len.ij.2 > len.ij) {
							cut0.probeId <- TRUE
							len.ij <- len.ij.2
						}
					}
					len.i <- c(len.i, len.ij)
				}
				max.ind.i <- which.max(len.i)
				matchLen <- c(matchLen, len.i[max.ind.i])
				matchField <- c(matchField, fieldNames.i[max.ind.i])
				tableLen <- c(tableLen, nrow(table.i))
			}
			bestMatchLen <- max(matchLen)
			if (bestMatchLen == 0) {
				if (verbose) warning('No matches were found!\n')
				return(c(list(chipVersion=NULL, species=species, IDType=NULL, chipProbeNumber=NULL, 
					matchedProbeNumber=bestMatchLen), idMapping=NULL))
			} else if (bestMatchLen < lenID && verbose) {
				warning('Some input IDs can not be matched!\n')
			}			

			if (!returnAllMatches) {
				bestInd <- which(matchLen == bestMatchLen)
				if (length(bestInd) > 1) {
					bestInd <- bestInd[tableLen[bestInd] == min(tableLen[bestInd])]
				}
				matchLen.match <- bestMatchLen
				tableName.match <- allTableNames[bestInd]
				fieldName.match <- matchField[bestInd]
				bestTable <- dbReadTable(conn, tableName.match[1])
				probeNumber <- nrow(bestTable)			
				if (idMapping) {
					## remove duplicated IDs becasue rownames should be unique
					dupInd <- duplicated(bestTable[,fieldName.match[1]])
					bestTable <- bestTable[!dupInd,]
					nuID <- bestTable[,'nuID']
					if (fieldName.match[1] %in% c('ProbeId', 'Array_Address_Id') && cut0.probeId) {
						allID <- as.character(as.integer(bestTable[,fieldName.match[1]]))
					} else {
						allID <- bestTable[,fieldName.match[1]]					
					}
					selInputID <- inputID[inputID %in% allID]
					
					rownames(bestTable) <- allID
					mapping <- as.matrix(bestTable[selInputID, ])
					# if (fieldName.match[1] == 'nuID') {
					# 	bestTable <- bestTable[, names(bestTable) != 'nuID']
					# 	rownames(bestTable) <- nuID
					# 	mapping <- as.matrix(bestTable[selInputID, ])
					# } else {
					# 	names(nuID) <- allID
					# 	mapping <- nuID[selInputID]
					# }
					## match the original input IDs
					mapping <- matchInputID(mapping, inputID.bak)
				}
			} else {
				# if (matchField[which.max(matchLen)] != 'nuID') IDType <- 'nuID'
				matchInd <- which(matchLen > 0)
				tableName.match <- allTableNames[matchInd]
				fieldName.match <- matchField[matchInd]
				matchLen.match <- matchLen[matchInd]
				probeNumber <- NULL
				if (idMapping) mapping <- NULL
				for (i in seq(matchInd)) {
					table.i <- dbReadTable(conn, tableName.match[i])
					if (idMapping) {
						## remove duplicated IDs becasue rownames should be unique
						dupInd.i <- duplicated(table.i[,fieldName.match[i]])
						table.i <- table.i[!dupInd.i,]
						nuID.i <- table.i[,'nuID']
						if (fieldName.match[1] %in% c('ProbeId', 'Array_Address_Id') && cut0.probeId) {
							allID.i <- as.character(as.integer(table.i[,fieldName.match[i]]))
						} else {
							allID.i <- table.i[,fieldName.match[i]]				
						}
						selInputID.i <- inputID[inputID %in% allID.i]
						if (fieldName.match[which.max(matchLen.match)] == 'nuID') {
							table.i <- table.i[, names(table.i) != 'nuID']
							rownames(table.i) <- nuID.i
							mapping.i <- as.matrix(table.i[selInputID.i, ])
							mapping.i <- matchInputID(mapping.i, inputID.bak)
							mapping <- c(mapping, list(mapping.i))
						} else {
							names(nuID.i) <- allID.i
							mapping.i <- nuID[selInputID.i]
							mapping.i <- matchInputID(mapping.i, inputID.bak)
							mapping <- cbind(mapping, mapping.i)
						}
					}
					probeNumber <- c(probeNumber, nrow(table.i))
				}
				rank1 <- rank(matchLen.match)
				rank2 <- rank(-probeNumber)
				ord <- order(rank1 + rank2/length(matchInd), decreasing=TRUE)
				tableName.match <- tableName.match[ord]
				fieldName.match <- fieldName.match[ord]
				probeNumber <- probeNumber[ord]
				matchLen.match <- matchLen.match[ord]
				if (idMapping) {
					if (is(mapping, 'list')) {
						mapping <- mapping[ord]
						names(mapping) <- tableName.match	
					} else if (is(mapping, 'matrix')) {
						rownames(mapping) <- inputID
						mapping <- mapping[,ord]
						colnames(mapping) <- tableName.match		
					}
				}
			}
			returnList <- list(chipVersion=tableName.match, species=species, IDType=fieldName.match, chipProbeNumber=probeNumber, 
				inputProbeNumber=length(inputID), matchedProbeNumber=matchLen.match)
		}
		if (idMapping)  returnList <- c(returnList, list(idMapping=mapping))		

		return(returnList)
	}
}

\name{lumiN}
\alias{lumiN}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Between chip normalization of a LumiBatch object }
\description{
  A main function of between chip normalization of a LumiBatch object. Currently, four methods ("rsn", "ssn", "quantile", "loess", "vsn") are supported.
}
\usage{
lumiN(x.lumi, method = c("quantile", "rsn", "ssn", "loess", "vsn", "rankinvariant"), verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
	\item{x.lumi}{ an ExpressionSet inherited object or a data matrix with columns as samples and rows as genes}
  	\item{method}{ five different between chips normalization methods ("quantile", "rsn", "ssn", "loess", "vsn", "rankinvariant") are supported }
    \item{verbose}{ a boolean to decide whether to print out some messages }
  	\item{\dots}{ other parameters used by corresponding method }
}
\details{
lumiN is an interface for different normalization methods. Currently it supports "RSN" (See \code{\link{rsn}}), "SSN" (See \code{\link{ssn}}), "loess" (See \code{\link[affy]{normalize.loess}}), "quantile" (See \code{\link[affy]{normalize.quantiles}}), "VSN" (See \code{\link[vsn]{vsn}}) and "rankinvariant" (See \code{\link{rankinvariant}}) . See details in individual functions. Note: the "VSN" normalization should be directly applied to the raw data instead of the lumiT processed data.
}

\value{
Return an object with expression values normalized. The class of the return object is the same as the input object x.lumi.
If it is a LumiBatch object, it also includes the VST transform function and its parameters as attributes: "transformFun", "parameter". See \code{\link{inverseVST}} for details.
}
\author{Pan Du, Simon Lin}
\seealso{  \code{\link{rsn}}, \code{\link{ssn}}, \code{\link{rankinvariant}} }
\examples{
## load example data
data(example.lumi)

## Do lumi transform
lumi.T <- lumiT(example.lumi)

## Do lumi between chip normaliazation
lumi.N <- lumiN(lumi.T, method='rsn', ifPlot=TRUE)

}
\keyword{ methods }

\name{plotControlData}
\alias{plotControlData}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Plot the mean expression (with standard deviation bar) of different type of control probes }
\description{
  Plot the mean intensity (with standard deviation bar) of different type of control probes. Multiple control types can be plotted in a single plot. The available control types can be get by running getControlType(controlData).
}
\usage{
plotControlData(controlData, type = NULL, slideIndex = NULL, logMode = FALSE, new = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{ a LumiBatch object including control data, a control data data.frame, a MethyLumiQC object or a MethyLumiM object including MethyLumiQC control data}
  \item{type}{ the control probe type (case insensitive), which can be get by running getControlType(controlData) }
  \item{slideIndex}{ the slide index or ID corresponding to each sample }
  \item{logMode}{ whether show the data in log2 scale }
  \item{new}{ whether refresh the new plot or add it on the old one  }
  \item{\dots}{ other parameters used by default plot function }
}
\details{
  When multiple control types are selected, they will be plotted in a two-column plot. 
For methylation data, the red and green channels will be plotted respectively in red and green colors.
}
\value{
	plot the picture and invisibly return TRUE if everything is OK
}
\author{ Pan Du }
\seealso{ \code{\link{addControlData2lumi}} and \code{\link{addControlData2methyLumiM}}  }
\examples{
	controlFile <- system.file('doc', 'Control_Probe_Profile.txt', package='lumi')
	if (file.exists(controlFile)) {
	  	controlData <- getControlData(controlFile)
	    getControlType(controlData)
	    plotControlData(controlData, type='NEGATIVE')
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{hplot}

\name{lumiR.batch}
\alias{lumiR.batch}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Read BeadStudio output files in batch}
\description{
  Read BeadStudio output files in batch and combine them as a single LumiBatch object
}
\usage{
lumiR.batch(fileList, convertNuID = TRUE, lib.mapping = NULL, detectionTh = 0.01, QC = TRUE, transform = c('none', 'vst', 'log2', 'cubicRoot'), sampleInfoFile = NULL, verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fileList}{ a vector of file names or a directory keeping the data files in the format of .csv }
  \item{convertNuID}{ determine whether convert the probe identifier as nuID }
  \item{lib.mapping}{ same as \code{\link{lumiR}} parameter lib.mapping (optional) }
  \item{detectionTh}{ the p-value threshold of determining detectability of the expression. See more details in \code{\link{lumiQ}}  }
  \item{QC}{ determine whether to do quality control assessment after read in the data.}
  \item{transform}{ determine whether to do transform after input each file }
  \item{sampleInfoFile}{ a Tab-separated text file or a data.frame keeping the sample information (optional) }
  \item{verbose}{ a boolean to decide whether to print out some messages }
  \item{\dots}{ other parameters used by \code{\link{lumiR}} }
}
\details{
The function basically call lumiR for individual files and then combine the returns. The sampleInfoFile parameter is optional. It provides the sample information (for phenoData slot in LumiBatch object), it is a Tab-separated text file. ID column is required. It represents sample ID, which is defined based on the column names of BeadStudio output file. For example, sample ID of column "1881436070\_A\_STA.AVG\_Signal" is "1881436070\_A\_STA". The sample ID column can also be found in the "Samples Table.txt" file output by BeadStudio. Another "Label" column (if provided) will be used as the sampleNames of LumiBatch object. All information of sampleInfoFile will be directly added in the phenoData slot in LumiBatch object.

To save memory space in the case of reading large data set, we can do transformation using lumiT function right after input the data, and the information like se.exprs, beadNum will be removed from the LumiBatch object after transformation.
}
\value{
A LumiBatch object which combines the individual LumiBatch object corresponding to each file
}
\author{ Pan Du }

\seealso{ \code{\link{lumiR}} }
\examples{
## fileList <- c('file1.csv', 'file2.cvs')
## x.lumi <- lumiR.batch(fileList, sampleInfoFile='sampleInfo.txt')

}
\keyword{ methods }

\name{produceGEOPlatformFile}
\alias{produceGEOPlatformFile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Produce GEO Platform Submission File in SOFT format }
\description{
	Produce GEO Sample Submission File in SOFT format based on the provided LumiBatch object and Illumina ID Mapping library
}
\usage{
produceGEOPlatformFile(x.lumi, lib.mapping = NULL, nuIDMode = TRUE, includeAllChipProbe=FALSE, fileName = "GEOPlatformFile.txt")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ The LumiBatch object keeping all probes }
  \item{lib.mapping}{ The Illumina ID Mapping library, e.g., "lumiHumanIDMapping" }
  \item{nuIDMode}{ Determine whether producing the platform indexed by nuID }
  \item{includeAllChipProbe}{ Determine whether including all probes in the Manifest file or just the probes used in the x.lumi object }
  \item{fileName}{ Filename of the GEO Platform File name }
}
\details{
 The function produces the GEO platform submission file based on the chip information kept in the Illumina ID Mapping library (specified by lib.mapping parameter). The determination of chip type will be automatically done by selecting the best matching of the probe IDs with individual chips.
}
\value{
Save the result as a text file in SOFT platform submission format.
}

\references{ http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html }
\author{ Pan Du }

\seealso{ \code{\link{produceGEOSubmissionFile}} }
\examples{
# data(example.lumi)
# produceGEOPlatformFile(example.lumi, lib.mapping='lumiHumanIDMapping')
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{utilities}

\name{rsn}
\alias{rsn}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Robust Spline Normalization between chips  }
\description{
  Robust spline normalization (monotonic curves) between chips
}
\usage{
rsn(x.lumi, targetArray = NULL, excludeFold = 2, span = 0.03, ifPlot = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ an ExpressionSet inherited object or a data matrix with columns as samples and rows as genes }
  \item{targetArray}{ A target chip is the model for other chips to normalize. It can be a column index, a vector or a LumiBatch object with one sample. }
  \item{excludeFold}{ exclude the genes with fold change larger than "excludeFold" during fitting the curve in normalization }
  \item{span}{ the span parameter used by \code{\link{monoSmu}} }
  \item{ifPlot}{ determine whether to plot intermediate results }
  \item{\dots}{ other parameters used by \code{\link{monoSmu}} }
}
\details{
The robust spline normalization (RSN) algorithm combines the features of quantile and loess normalization. It is designed to normalize the variance-stabilized data. The function will check whether the data is variance stabilized (vst or log2 transform), if not, it will automatically run lumiT before run rsn. For details of the algorithm, please see the reference.

The targetArray can be a column index, a vector or a LumiBatch object with one sample, which corresponds to an external sample to be normalized with. This is very useful for handling large data set or normalizing the data set with a common reference (targetArray).
}
\value{
Return an object with expression values normalized. The class of the return object is the same as the input object x.lumi.
If it is a LumiBatch object, it also includes the VST transform function and its parameters as attributes: "transformFun", "parameter". See \code{\link{inverseVST}} for details.
}
\author{Pan Du, Simon Lin}
\seealso{ \code{\link{lumiN}}, \code{\link{monoSmu}} }
\keyword{ methods }

\name{is.nuID}
\alias{is.nuID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ nuID self-identification }
\description{
  Self-identify nuID (nucleotide universal identifier) by verify the check code value and the checksum value
}
\usage{
is.nuID(id)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{id}{ nuId or other string }
}

\value{
  Return TRUE if id is a nuID, or else return FALSE.
}
\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{ Pan Du }

\seealso{  \code{\link{seq2id}}, \code{\link{id2seq}} }
\examples{
## check the function using a random sequence
id <- 'adfasdfafd'
is.nuID(id)		# FALSE

## check the function using a read nuID
seq <- 'ACGTAAATTTCAGTTTAAAACCCCCCG'
id <- seq2id(seq)
is.nuID(id)		# TRUE

}
\keyword{ methods }

\name{lumiMethyB}
\alias{lumiMethyB}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Adjust background level of Illumina Infinium methylation data
}
\description{
Adjust background level of Illumina Infinium methylation data, which is an object in MethyLumiM class.
}
\usage{
lumiMethyB(methyLumiM, method = c("bgAdjust2C", "forcePositive", "none"), separateColor = FALSE, verbose = TRUE, overwriteBigMatrix=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
  \item{method}{ background adjustment methods or user provided function, whose input and output should be a intensity matrix (pool of methylated and unmethylated probe intensities) }
  \item{separateColor}{determine whether to separately process two color channels}
  \item{verbose}{a boolean to decide whether to print out some messages }
  \item{overwriteBigMatrix}{whether to overwrite the result to the BigMatrix data, only valid when the input data is BigMatrix-based}
  \item{\dots}{ other parameters used by corresponding method }
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment.
}
\author{
Pan DU
}

\seealso{
See Also \code{\link{bgAdjustMethylation}} and \code{\link{estimateMethylationBG}}
}
\examples{
data(example.lumiMethy)
lumiMethy.bgAdj = lumiB(example.lumiMethy)
attr(lumiMethy.bgAdj, "EstimatedBG")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{lumiMethyStatus}
\alias{lumiMethyStatus}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate the methylation status of individual methylation sites
}
\description{
Estimate the methylation status of individual methylation sites by fitting a two component Gamma mixture model for each sample
}
\usage{
lumiMethyStatus(methyLumiM, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM class object}
  \item{\dots}{Other parameters used by \code{\link{methylationCall}} }
}
\details{
This function calls \code{\link{methylationCall}} and returns the methylation status of individual methylation sites. The methylation status includes: "Unmethy" (unmethylation probability > unmethylation threshold), "Methy" (methylation probability > methylation threshold), or "Margin". The methylation probability is returned as an attribute of "probability".
}
\value{ return a methylation status matrix with "probability" attribute }

\author{
Pan Du
}

\seealso{
  See Also \code{\link{methylationCall}} and \code{\link{gammaFitEM}}
}
\examples{
data(example.lumiMethy)
methyCall <- lumiMethyStatus(example.lumiMethy)
head(methyCall)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{example.methyTitration}
\alias{example.methyTitration}
\docType{data}
\title{Example Illumina Infinium Methylation titration data in MethyLumiM class}
\description{
An example Illumina Infinium Methylation27k dataset, which includes a titration dataset
}
\usage{data(example.methyTitration)}
\details{
The example dataset is a titration dataset. It includes 8 samples: "A1", "A2", "B1", "B2", "C1", "C2", "D" and "E". They are mixtures of Sample A (a B-lymphocyte sample) and Sample B (is (a colon cancer sample from a female donor) at five different titration ratios: 100:0 (A), 90:10 (C), 75:25 (D), 50:50 (E) and 0:100 (B). 

To save storage space, we randomly subset 10000 CpG sites among about 27000 measured CpG sites.

}

\examples{
data(example.methyTitration)
sampleNames(example.methyTitration)
}
\keyword{datasets}

\name{detectionCall}
\alias{detectionCall}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Estimate the detectable probe ratio }
\description{ 
Estimate the detectable probe ratio of each probe, sample or just return an AP matrix
 }

\usage{
detectionCall(x.lumi, Th = 0.01, type = c('probe', 'sample', 'matrix'))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ a LumiBatch or MethyLumiM object }
  \item{Th}{ the threshold. By default, when the detection p-value is less than 0.01, we suppose it is detectable. For the old version of BeadStudio output (version 2 or earlier), the threshold will automatically transferred as 1 - Th, because in the old format, value close to 1 is suppose to be detectable. }
  \item{type}{ determine to calculate the detection count by probe or by sample }
}

\value{
If the type is 'probe', then returns the presentCount of each probe. If the type is 'sample', then return the detectable probe ratio of each sample. If the type is 'matrix', then return the AP matrix, in which 'A' represents absent (the detect p-value less than threshold) and 'P' represents present.
}

\author{ Pan Du }

\seealso{  \code{\link{lumiQ}} }
\examples{
## load example data
data(example.lumi)
## load example data
data(example.lumi)

## estimate the detect call (percentage of expressed genes) of each sample
temp <- detectionCall(example.lumi, type='sample')
print(temp)

## estimate the present count of each gene (probe)
temp <- detectionCall(example.lumi, type='probe')
hist(temp)

}
\keyword{ methods }

\name{rankinvariant}
\alias{rankinvariant}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Rank Invariant Normalization }
\description{
  This function basically adjusts the samples to the same background level and then optionally scales to the same foreground level.
}
\usage{
rankinvariant(x.lumi, targetArray = NULL, rrc = .05, lowRank = seq(.5, .25, -.05), highRank = .9, minSize = .02, maxit=200)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ an ExpressionSet inherited object or a data matrix with columns as samples and rows as genes }
  \item{targetArray}{ A target chip is the model for other chips to normalize. It can be a column index, a vector or a LumiBatch object with one sample. }
  \item{rrc}{ The relative rank change allowed for a gene to be selected as rank invariant }
  \item{lowRank}{ A vector with, in decreasing order, the minimum ranks where candidate genes can be selected as rank invariant }
  \item{highRank}{ The maximum rank where candidate genes can be selected as rank invariant }
  \item{minSize}{ Fraction of genes required to be selected as rank invariant }
  \item{maxit}{ Maximum number of iterations for \code{\link[MASS]{rlm}} to reach convergence }
}
\details{
Rank invariant normalization uses a set of genes that are rank invariant between a given sample and a target sample. The target sample can be predefined by setting the targetArray argument. If targetArray is NULL the average expression of all samples will be the target. Rank invariant genes are found for each sample seperately by calculation the relative rank change for each gene. Furthermore, only genes with ranks between the lowRank and highRank are considered. If the number of probes is less than minSize multiplies by the number of genes the next lowRank value tried. If no rank invariant set can be found an error is thrown.

The default settings of this function are the same as used Genomstudio (Illumina). The results produced by this method are similar, but not identical to Genomestudio.
}
\value{
Return an object with expression values normalized. The class of the return object is the same as the input object x.lumi.
}
\author{ Arno Velds (contact: a.velds (at) nki.nl) }
\seealso{ \code{\link{lumiN}} }
\keyword{ methods }

\name{normalizeMethylation.ssn}
\alias{normalizeMethylation.ssn}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
 Shift and scaling normalization of Illumina Infinium methylation data at probe level
}
\description{
Shift and scaling normalization of Illumina Infinium methylation data at probe level
}
\usage{
normalizeMethylation.ssn(methyLumiM, separateColor = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
  \item{separateColor}{determine whether separately normalize two color channels}
}

\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment.
}

\author{
Pan DU
}
\seealso{
 See Also \code{\link{lumiMethyN}}, and \code{\link{normalizeMethylation.quantile}}
}
\examples{
data(example.lumiMethy)
lumiMethy.norm = normalizeMethylation.ssn(example.lumiMethy)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{bgAdjustMethylation}
\alias{bgAdjustMethylation}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate and adjust the background levels of Illumina Infinium methylaton microarrays
}
\description{
Estimate and adjust the background levels of Illumina Infinium methylaton microarrays
}
\usage{
bgAdjustMethylation(methyLumiM, separateColor = FALSE, targetBGLevel = 300, negPercTh = 0.25)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object or any eSet object with "methylated" and "unmethylated" data matrix element in the assayData slot }
  \item{separateColor}{determine whether separately process two color channels}
  \item{targetBGLevel}{ adjust background level to a non-zero target background level }
  \item{negPercTh}{the threshold of the percentage of negative values after subtract estimated background levels. A warning will be given if too many probes having intensities lower than background levels.}
}
\details{
The estimation of background level of Infinium methylaton microarray is based on the assumption that the lots of CpG sites are unmethylated, which results in a density mode of the intensities measured by methylated probes. The position of this mode represents the background level.
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment. The estimated background level was kept in the attribute, "EstimatedBG", of the returned methyLumiM object.
}
\author{
Pan DU
}

\seealso{
 See Also \code{\link{lumiMethyB}} and \code{\link{estimateMethylationBG}}
}
\examples{
data(example.lumiMethy)
lumiMethy.bgAdj = bgAdjustMethylation(example.lumiMethy)
attr(lumiMethy.bgAdj, "EstimatedBG")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{plotStringencyGene}
\alias{plotStringencyGene}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ plot the Stringency related control probe profiles }
\description{
  Plot the Stringency related control probe (Low-Stringency, Medium-Stringency and High-Stringency) profiles. Using getControlType function to view available stringency types.
}
\usage{
plotStringencyGene(controlData, lib = NULL, slideIndex = NULL, addLegend = TRUE, logMode = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{ a LumiBatch object including control data or a control data data.frame }
  \item{lib}{ the annotation library (for retrieving the gene name) }
  \item{slideIndex}{ the slide index or ID corresponding to each sample }
  \item{addLegend}{ whether add legend or not }
  \item{logMode}{ whether show the data in log2 scale  }
  \item{\dots}{ other parameters used by default matplot function }
}
\value{
	plot the picture and return TRUE if everything is OK
}
\author{ Pan Du }
\seealso{ \code{\link{addControlData2lumi}}, \code{\link{plotControlData}} }

\examples{
	controlFile <- system.file('doc', 'Control_Probe_Profile.txt', package='lumi')
	if (file.exists(controlFile)) {
  	controlData <- getControlData(controlFile)
  	plotStringencyGene(controlData)
	}
}
\keyword{methods}
\keyword{hplot}

\name{estimateIntensity}
\alias{estimateIntensity}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate intensity of Illumina methylation data
}
\description{
Estimate intensity of Illumina methylation data, which is the sum of Illumina methylated and unmethylated probe intensities
}
\usage{
estimateIntensity(methyLumiM, returnType=c("ExpressionSet", "matrix"))
}

\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{returnType}{determine whether return an ExpressionSet or matrix object}
}
\details{
	The Intensity basically is the sum of Illumina methylated and unmethylated probe intensities. 
}
\value{
An ExpressionSet or matrix object of methylation Intensity-value
}
\author{
Pan DU
}
\seealso{
See Also as \code{\link{estimateBeta}} and \code{\link{estimateM}}

}
\examples{
data(example.lumiMethy)
methyLumiIntensity = estimateIntensity(example.lumiMethy)
boxplot(methyLumiIntensity)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{plotColorBias1D}
\alias{plotColorBias1D}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot the color bias density plot of Illumina Infinium Methylation data 
}
\description{
Plot the color bias density plot of Illumina Infinium Methylation data in one dimension (comparing with 2D scatter plot)
}
\usage{
plotColorBias1D(methyLumiM, channel = c("both", "unmethy", "methy", "sum"), colorMode=TRUE, removeGenderProbes = FALSE, logMode = TRUE, subset = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{channel}{ estimate the intensity in different methods }
  \item{colorMode}{ whether separate two color channels or not }
  \item{removeGenderProbes}{ determine whether exclude probes on X and Y chromosomes if the chromosome information is provided in the methyLumiM object.}
  \item{logMode}{Whether plot the intensities in log-scale }
  \item{subset}{plot subset of randomly selected rows. All data will be plotted if it is NULL. }
  \item{\dots}{ other parameters used by \code{\link[stats]{density}} and  \code{\link[graphics]{plot}}}
}
\details{
Plot the color bias density plot of Illumina Infinium Methylation data. There are four options using "channel" parameter to plot the density plot. "both": estimate the density by pooling together methylated and unmethylated probe intensities. "unmethy" and "methy": plot either unmethylated or methylated probe density. "sum" plot the density of the sum of methylatled and unmethylated probe intensitys.
}
\value{
Invisibly return TRUE if plot successfully.
}
\author{
Pan DU
}

\seealso{
See Also as \code{\link{plotColorBias2D}} and \code{\link{boxplotColorBias}} 
}
\examples{
data(example.lumiMethy)
# before adjustment
plotColorBias1D(example.lumiMethy)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{hplot}

\name{plotCDF}
\docType{methods}
\alias{plotCDF}
\alias{plotCDF.ExpressionSet}
\alias{plotCDF,ExpressionSet-method}

\title{plot the cumulative distribution function  of a ExpressionSet object}
\description{ plot the cumulative distribution function of a ExpressionSet object from high to low value or in reverse }

\usage{
 plotCDF(x, reverse=TRUE, logMode=TRUE, xlab = NULL, ylab = "Cumulative density", 
	col=1:dim(x)[2], lwd=1, xlim = NULL, index.highlight = NULL, color.highlight = 2, 
	addLegend = TRUE, main="", ...)
}

\arguments{
	\item{x}{ a numeric or \code{\link[Biobase]{ExpressionSet}} object }
	\item{reverse}{ determine whether plot the CDF from high to low value or not }
	\item{logMode}{ determine whether the CDF plot is based on a log2 scale }
	\item{xlab}{ xlab of the plotCDF plot }
	\item{ylab}{ ylab of the plotCDF plot }
	\item{col}{ line colors of the plotCDF plot }
	\item{lwd}{ line width of plot function }
	\item{xlim}{ parameter of the plot function}
	\item{index.highlight}{ the column index of the highlighted plotCDF curve }
	\item{color.highlight}{ color of highlighted plotCDF curve }
	\item{addLegend}{ whether add legend to the plot or not }
    \item{main}{ title for the plot}
	\item{\dots}{ additional parameters for \code{\link[stats]{plot.ecdf}} function }
}

\seealso{
  \code{\link{LumiBatch-class}}, \code{\link[stats]{ecdf}}
}
\examples{
## load example data
data(example.lumi)

plotCDF(example.lumi)
}
\keyword{methods}
\keyword{hplot}

\name{pairs-methods}
\docType{methods}
\alias{pairs-methods}
\alias{pairs.ExpressionSet}
\alias{pairs,ExpressionSet-method}

\title{Pair plot of an ExpressionSet object}
\description{ Creating \code{\link[graphics]{pairs}} plot of sample intensities in an ExpressionSet object }

\usage{
\S4method{pairs}{ExpressionSet}(x, ..., smoothScatter = FALSE, logMode = TRUE, subset = 5000, fold=2, dotColor=1, 
	highlight = NULL, highlightColor = 2, main = NULL, checkTransform = TRUE)
}

\arguments{
	\item{x}{ a \code{\link[Biobase]{ExpressionSet}} object}
	\item{\dots}{ optional arguments to \code{\link[graphics]{pairs}}.}
	\item{smoothScatter}{ whether use \code{\link[graphics]{smoothScatter}} function to plot points}	
	\item{logMode}{ whether plot the data in log2 scale }
	\item{subset}{subset of rows used to plot. It can be an index vector, or the length of a random subset}
	\item{fold}{The fold-change threshold used to estimate the number of probes having high fold-changes}
	\item{dotColor}{color of points in the scatter plot}
	\item{highlight}{the subset dots need to be highlighted}
	\item{highlightColor}{the color for those highlighted dots}
	\item{main}{ title of the plot }
	\item{checkTransform}{whether to check the data is log2-transformed or not}
}

\details{
To increase the plot efficiency, by default, we only plot RANDOMLY selected subset of points (based on parameter "subset"). If users want to plot all the points, they can set the parameter "subset = NULL". When smoothScatter is set as TRUE, the subsetting will be suppressed because \code{\link[graphics]{smoothScatter}} function has good plot efficiency for large number of points.
}

\seealso{
  \code{\link{LumiBatch-class}}, \code{\link[graphics]{pairs}}
}
\examples{
## load example data
data(example.lumi)

pairs(example.lumi)

pairs(example.lumi, smoothScatter=TRUE)
}
\keyword{methods}
\keyword{hplot}

\name{example.lumi}
\alias{example.lumi}
\docType{data}
\title{Example Illumina Expression data in LumiBatch class}
\description{
  Example data as a LumiBatch object which is a subset of Barnes data (Barnes, 2005)
}
\usage{data(example.lumi)}
\format{
A 'LumiBatch' object
}

\details{
The data is from (Barnes, 2005). It used Sentrix HumanRef-8 Expression BeadChip. Two samples "100US" and "95US:5P" (each has two technique replicates) were selected. In order to save space, 8000 genes were randomly selected. As a result, the example data includes 8000 genes, each has 4 measurements. The full data set was included in the Bioconductor Experiment data package lumiBarnes.

The entire data set has been built as a lumiBarnes data object and can be downloaded from Bioconductor Experiment Data.
}
\references{
Barnes, M., Freudenberg, J., Thompson, S., Aronow, B. and Pavlidis, P. (2005) Ex-perimental comparison and cross-validation of the Affymetrix and Illumina gene expression analysis platforms, Nucleic Acids Res, 33, 5914-5923.

The detailed data information can be found at: http://www.bioinformatics.ubc.ca/pavlidis/lab/platformCompare/
}

\examples{
## load the data
data(example.lumi)

## summary of the data
example.lumi

}
\keyword{datasets}

\name{addControlData2lumi}
\alias{addControlData2lumi}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Add the control probe data into the controlData slot of LumiBatch object }
\description{
   Add the control probe profile data, outputted by BeadStudio, into the controlData slot of LumiBatch object. 
}
\usage{
addControlData2lumi(controlData, x.lumi)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{ the control data can be a data.frame or the control probe filename outputted by BeadStudio }
  \item{x.lumi}{ a LumiBatch object, to which controlData will be added. }
}
\details{
 The controlData slot in LumiBatch object is a data.frame with first two columns as "controlType" and "ProbeID". The rest columns are the expression amplitudes for individual samples.
}
\value{
Return the LumiBatch object with controlData slot filled.
}
\author{ Pan Du }

\seealso{  \code{\link{getControlData}}, \code{\link{plotControlData}} }
\examples{
	## Not runnable
	# controlFile <- 'Control_Probe_Profile.txt'
	# x.lumi <- addControlData2lumi(controlFile, x.lumi)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }

\name{MAplot-methods}
\docType{methods}
\alias{MAplot-methods}
\alias{MAplot}
\alias{MAplot.ExpressionSet}
\alias{MAplot,ExpressionSet-method}

\title{MAplot of a ExpressionSet object}
\description{ Creating pairwise MAplot of sample intensities in a ExpressionSet object }

\usage{
\S4method{MAplot}{ExpressionSet}(object, ..., smoothScatter = FALSE, logMode = TRUE, subset = 5000, main = NULL)
}

\arguments{
\item{object}{ an \code{\link[Biobase]{ExpressionSet}} object}
\item{\dots}{ optional arguments to \code{\link[affy]{MAplot}}.}
\item{smoothScatter}{ whether use \code{\link[graphics]{smoothScatter}} function to plot points}	
\item{logMode}{ whether plot the data in log2 scale or not}	
\item{subset}{ subset of rows used to plot. It can be an index vector, or the length of a random subset}
\item{main}{ title of the plot }	
}

\details{
To increase the plot efficiency, by default, we only plot RANDOMLY selected subset of points (based on parameter "subset"). If users want to plot all the points, they can set the parameter "subset = NULL". When smoothScatter is set as TRUE, the subsetting will be suppressed because \code{\link[graphics]{smoothScatter}} function has good plot efficiency for large number of points.
}

\seealso{
  \code{\link{LumiBatch-class}}, \code{\link[affy]{MAplot}}
}
\examples{
## load example data
data(example.lumi)

MAplot(example.lumi)

MAplot(example.lumi, smoothScatter=TRUE)
}
\keyword{methods}
\keyword{hplot}

\name{lumiMethyC}
\alias{lumiMethyC}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Color bias adjust of Illumina Infinium methylation data
}
\description{
Color bias adjust of Illumina Infinium methylation data, which is an object in MethyLumiM class.
}
\usage{
lumiMethyC(methyLumiM, method = c("quantile", "ssn", "none"), verbose = TRUE, overwriteBigMatrix=FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
  \item{method}{ color bias adjustment methods or user provided function, see "details" for more information of user defined function. }
  \item{verbose}{a boolean to decide whether to print out some messages }
  \item{overwriteBigMatrix}{whether to overwrite the result to the BigMatrix data, only valid when the input data is BigMatrix-based}
  \item{\dots}{ other parameters used by corresponding method }
}
\details{
	The first two arguments of the user defined function should be two intensity matrix (pool of methylated and unmethylated probe intensities) of red and green channel respectively. The return of the user defined function should be a list including color adjusted matrix of red and green channel. For example: return(list(red=redData, green=grnData)). "redData" and "grnData" are two color adjusted matrix.
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment.
}

\author{
Pan DU
}

\seealso{
See Also \code{\link{adjColorBias.quantile}} and \code{\link{adjColorBias.ssn}}
}
\examples{
data(example.lumiMethy)
# before adjustment
plotColorBias1D(example.lumiMethy)
# plot in 2D plot of one selected sample
plotColorBias2D(example.lumiMethy, selSample = 1)
lumiMethy.adj = lumiMethyC(example.lumiMethy)
# after adjustment
plotColorBias1D(lumiMethy.adj)
# plot in 2D plot of one selected sample
plotColorBias2D(lumiMethy.adj, selSample = 1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{addNuID2lumi}
\alias{addNuID2lumi}
\alias{addNuId2lumi}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Add the nuID information to the LumiBatch object }
\description{ Replace the Illumina Id (Target ID or Probe Id) as nuID (nucleotide universal identifier) for indexing genes in the LumiBatch object }

\usage{
addNuID2lumi(x.lumi, annotationFile=NULL, sep = NULL, lib.mapping = NULL,  annotationColName = c(sequence = "Probe_Sequence", target = "ILMN_Gene", probe='Probe_Id'), verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ a LumiBatch object }
  \item{annotationFile}{ a annotation file, which includes the Illumina ID (target or probe ids) and probe sequence information}
  \item{sep}{ the separation used in the annotation file. Automatically detect the separator if it is "," or "\\t". }
  \item{lib.mapping}{ a Illumina ID mapping package, e.g, lumiHumanIDMapping }
  \item{annotationColName}{ the annotation column name in the annotation file used for the probe sequence and TargetID and ProbeID }
  \item{verbose}{ a boolean to decide whether to print out some messages }
}
\details{
Since the default Illumina IDs (TargetID (ILMN\_Gene ID) and ProbeId (Probe\_Id)) are not consistent between different arrays and batches, we invented a nuID, which is one-to-one matching with the probe sequence. This function is to replace the Illumina ID with the nuID. If the annotation library (the unzipped manifest file (.bgx)) is provided, the function will automatically check whether the Illumina ID is provided for the microarray data. We recommend output the data using ProbeID when using Illumina BeadStudio software, because the TargetID (ILMN\_Gene ID) are not unique. 
}
\value{
a LumiBatch object with Illumina ID replaced by nuID. 
}
\references{ Du, P., Kibbe, W.A., Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", submitted.}
\author{ Pan Du }

\seealso{  \code{\link{IlluminaID2nuID}}, \code{\link{lumiR}} }
\examples{
## load example data
# data(example.lumi)

## specify the annotation file for the Illumina chip
# annotationFile <- 'Human_RefSeq-8.csv'
## Replace the Target ID with nuID
# lumi.nuID <- addNuID2lumi(example.lumi, annotationFile)

## An alternative way is to load the Annotation library and match the targetID (or Probe Id) with nuID
# lumi.nuId <- addNuID2lumi(example.lumi, lib.mapping='lumiHumanIDMapping')

}
\keyword{ methods }

\name{getControlProbe}
\alias{getControlProbe}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Get the control probe Ids}
\description{
  Get the control probe Ids corresponding to the control probe type provided. The control probe ids are kept in the second column of controlData data.frame.
}
\usage{
getControlProbe(controlData, type = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{ a LumiBatch object including control data or a control data data.frame }
  \item{type}{ the type of control probe (case insensitive), which can be get by using \code{\link{getControlType}} function }
}
\value{
returns the corresponding probe Ids for the control type.
}
\author{ Pan Du }
\seealso{  \code{\link{addControlData2lumi}} }
\examples{
	controlFile <- system.file('doc', 'Control_Probe_Profile.txt', package='lumi')
	if (file.exists(controlFile)) {
    ## return a data.frame
    controlData <- getControlData(controlFile)
    getControlType(controlData)
    getControlProbe(controlData, type='housekeeping')
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{importMethyIDAT}
\alias{importMethyIDAT}
\title{
Import Illumina methylation .idat files as an MethyLumiM object
}
\description{
Import Illumina methylation .idat files as an MethyLumiM object. An extension of \code{\link[methylumi]{lumIDAT}} function
}
\usage{
importMethyIDAT(sampleInfo, dataPath = getwd(), lib = NULL, bigMatrix=FALSE, dir.bigMatrix='.', savePrefix.bigMatrix, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{sampleInfo}{A data.frame of sample information or a character vector of barcodes.}
  \item{dataPath}{The path of .idat files}
  \item{lib}{Annotation library}
  \item{bigMatrix}{whether to save the data as BigMatrix (designed for very large dataset)}
  \item{dir.bigMatrix}{the parent directory to save the BigMatrix data files}
  \item{savePrefix.bigMatrix}{the folder name prefix of the directory to save the BigMatrix data files. The fold name will be like this: paste(savePrefix.bigMatrix, '_bigmat', sep='') }
  \item{\dots}{other parameters used by \code{\link[methylumi]{lumIDAT}} function}
}
\details{
This function is an extension of \code{\link[methylumi]{lumIDAT}}. It adds sample information and probe annotation information to the data. As Illumina organizes the output .idat files by barcodes, the function will automatically check the sub-folders in the names of barcodes for .idat files. The "sampleInfo" parameter can be either a barcode vector, e.g., "7310440039_R04C02" "7310440039_R05C02". Or a data.frame with required columns of 'Sentrix_Barcode' and 'Sentrix_Position'. If "sampleInfo" is a data.frame, it will be added as the pData of the output MethyLumiM object.
}

\value{
A MethyLumiM object
}

\author{
Pan Du, Tim Triche
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link[methylumi]{lumIDAT}}, \code{\link{lumiMethyR}}, \code{\link{addAnnotationInfo}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{getChipInfo}
\alias{getChipInfo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Get Illumina Chip Information based on probe identifiers}
\description{
	Retrieve the matched Illumina chip information by searching the provided probe identifiers through the Illumina identifiers in all manifest files.
}
\usage{
getChipInfo(x, lib.mapping = NULL, species = c("Human", "Mouse", "Rat", "Unknown"), chipVersion = NULL, idMapping = FALSE, returnAllMatches = FALSE, verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a vector of probe identifiers, ExpressionSet object or a matrix with probe identifiers as row names }
  \item{lib.mapping}{ the ID mapping library. If it is provided, the parameter "species" will be ignored. }
  \item{species}{ species of the chip designed for. If users do not know it, it can be set as "Unknown". }
  \item{chipVersion}{ chipVersion information returned by function \code{\link{getChipInfo}} }
  \item{idMapping}{ determine whether return the idMapping information (between Illumina ID and nuID) }
  \item{returnAllMatches}{ determine whether return all matches or just the best match }
  \item{verbose}{ determine whether print some warning information }
}
\details{
  The function searches the provided probe Identifiers (Illumina IDs or nuIDs) through all the manifest file ID information kept in the IDMapping libraries (lumiHumanIDMapping, lumiMouseIDMapping, lumiRatIDMapping). The Illumina IDs kept in the library include "Search\_key" ("Search\_Key"), "Target" ("ILMN\_Gene"), "Accession", "Symbol", "ProbeId" ("Probe\_Id"). To determine the best match, the function calculate the number of matched probes. The higher "matchedProbeNumber" is claimed as better. When the "matchedProbeNumber" is the same, the manifest file with fewer probes is claimed as better. If x is NULL and chipVersion is provided, it will return the entire mapping table of the chip.
}
\value{
  The function returns a list with following items:
  \item{chipVersion}{the file name of the manifest file for the corresponding version and release}
  \item{species}{the species of the chip designed for}
  \item{IDType}{the type of probe identifier}
  \item{chipProbeNumber}{the number of probes in the manifest file}
  \item{matchedProbeNumber}{the number of input probes matching the manifest file}
  \item{idMapping}{id mapping information between Illumina ID and nuID}
  When parameter "returnAllMatches" is TRUE, the items of "chipVersion", "IDType", "chipProbeNumber", "inputProbeNumber", "matchedProbeNumber" will be a vector corresponding to the matched manifest files, whose "matchedProbeNumber" is larger than zero, and the "idMapping" will be a matrix with each column corresponding to one matched manifest file. All of the items are sorted from the best match to worst (The higher "matchedProbeNumber" is claimed as better. When the "matchedProbeNumber" is the same, the manifest file with fewer probes is claimed as better.). 
}
\author{Pan Du}
\seealso{ \code{\link{nuID2IlluminaID}}, \code{\link{IlluminaID2nuID}} }
\examples{
	## load example data
	data(example.lumi)
	if (require(lumiHumanIDMapping)) {
		chipInfo <- getChipInfo(example.lumi, species='Human')
		chipInfo
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{utilities}

\name{lumiR}
\alias{lumiR}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Read in Illumina expression data }
\description{
  Read in Illumina expression data. We assume the data was saved in a comma or tab separated text file.  
}
\usage{
lumiR(fileName, sep = NULL, detectionTh = 0.01, na.rm = TRUE, convertNuID = TRUE, lib.mapping = NULL, dec = '.', parseColumnName = FALSE, checkDupId = TRUE,
QC = TRUE, columnNameGrepPattern = list(exprs='AVG_SIGNAL', se.exprs='BEAD_STD', detection='DETECTION', beadNum='Avg_NBEADS'),
inputAnnotation=TRUE, annotationColumn=c('ACCESSION', 'SYMBOL', 'PROBE_SEQUENCE', 'PROBE_START', 'CHROMOSOME', 'PROBE_CHR_ORIENTATION', 'PROBE_COORDINATES', 'DEFINITION'), verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{fileName}{ fileName of the data file }
  \item{sep}{ the separation character used in the text file.  }
  \item{detectionTh}{ the p-value threshold of determining detectability of the expression. See more details in \code{\link{lumiQ}} }
  \item{na.rm}{ determine whether to remove NA }
  \item{convertNuID}{ determine whether convert the probe identifier as nuID }
  \item{lib.mapping}{ a Illumina ID mapping package, e.g, lumiHumanIDMapping, used by \code{\link{addNuID2lumi}} }
  \item{dec}{ the character used in the file for decimal points.}
  \item{parseColumnName}{ determine whether to parse the column names and retrieve the sample information (Assume the sample information is separated by "\_".) }
  \item{checkDupId}{ determine whether to check duplicated TargetIDs or ProbeIds. The duplicated ones will be averaged.}
  \item{QC}{ determine whether to do quality control assessment after read in the data.}
  \item{columnNameGrepPattern}{ the string grep patterns used to determine the slot corresponding columns.}
  \item{inputAnnotation}{ determine whether input the annotation information outputted by BeadStudio if exists.}
  \item{annotationColumn}{ the column names of the annotation information outputted by BeadStudio}
  \item{verbose}{ a boolean to decide whether to print out some messages }
  \item{\dots}{ other parameters used by \code{\link[utils]{read.table}} function }
}
\details{
The function can automatically determine the separation character if it is Tab or comma. Otherwise, the user should specify the separator manually. If the annotation library is provided, the Illumina Id will be replaced with nuID, which is used as the index Id for the lumi annotation packages. If the annotation library is not provided, it will try to directly convert the probe sequence (if provided in the BeadStudio output file) as nuIDs.

The parameter "columnNameGrepPattern" is designed for some advanced users. It defines the string grep patterns used to determine the slot corresponding columns. For example, for the "exprs" slot in LumiBatch object, it is composed of the columns whose name includes "AVG\_SIGNAL". In some cases, the user may not want to read the "detection" and "beadNum" related columns to save memory. The user can set the "detection" and "beadNum" as NA in "columnNameGrepPattern". If the 'se.exprs' is set as NA or the corresponding columns are not available, then lumiR will create a ExpressionSet object instead of LumiBatch object.

The parameter "parseColumnName" is designed to parse the column names and retrieve the sample information. We assume the sample information is separated by "\_" and the last element after "\_" is the sample label (sample names of the LumiBatch object).  If the parsed sample labels are not unique, then the entire string will be used as the sample label. For example: "1881436055\_A\_STA 27aR" is included in one of the column names of BeadStudio output file. Here, the program will first treat "STA 27aR" as the sample label. If it is not unique across the samples, "1881436055\_A\_STA 27aR" will be the sample label. If it is still not unique, the program will report warning messages. All the parsed information is kept in the phenoData slot. By default, "parseColumnName" is FALSE. We suggest the users use it only when they know what they are doing.

Current version of lumiR can adaptively read the output of BeadStudio Verson 1 and 3. The format Version 3 made quite a few changes comparing with previous versions. One change is the detection value. It was called detectable when the detection value is close to one for Version 1 format. However, the detection value became a p-value in the Version 3. As a result, the detectionTh is automatically changed based on the version. The detectionTh 0.01 for the Version 3 will be changed as the detectionTh 0.99 for Version 1. Another big change is that Version 3 separately output the control probe (gene) information and a "Samples Table". As a result, the controlData slot in LumiBatch class was added to keep the control probe (gene) information, and a QC slot to keep the quality control information, including the "Sample Table" output by BeadStudio version 3.

The recent version of BeadStudio can also output the annotation information together with the expression data. In the users also want to input the annotation information, they can set the parameter "inputAnnotation" as TRUE. At the same time, they can also specify which columns to be inputted by setting parameter "annotationColumn". The BeadStudio annotation columns include: SPECIES, TRANSCRIPT, ILMN\_GENE, UNIGENE\_ID, GI, ACCESSION, SYMBOL, PROBE\_ID, ARRAY\_ADDRESS\_ID, PROBE\_TYPE, PROBE\_START, PROBE\_SEQUENCE, CHROMOSOME, PROBE\_CHR\_ORIENTATION, PROBE\_COORDINATES, DEFINITION, ONTOLOGY\_COMPONENT, ONTOLOGY\_PROCESS, ONTOLOGY\_FUNCTION, SYNONYMS, OBSOLETE\_PROBE\_ID. As the annotation data is huge, by default, we only input: ACCESSION, SYMBOL, PROBE\_START, CHROMOSOME, PROBE\_CHR\_ORIENTATION, PROBE\_COORDINATES, DEFINITION. This annotation information is kept in the featureData slot of ExpressionSet, which can be retrieved using pData(featureData(x.lumi)), suppose x.lumi is the LumiBatch object. As some annotation information may be outdated. We recommend using Bioconductor annotation packages to retrieve the annotation information.

The BeadStudio may output either STDEV or STDERR (standard error of the mean) columns. As the variance stabilization (see \code{\link{vst}} function) requires the information of the standard deviation instead of the standard error of the mean, the value correction is required. The lumiR function will automatically check whether the BeadStudio output file includes STDEV or STDERR columns. If it is STDERR columns, it will correct STDERR as STDEV. The corrected value will be x * sqrt(N), where x is the STDERR value (standard error of the mean), N is the number of beads corresponding to the probe. (Thanks Sebastian Balbach and Gordon Smyth kindly provided this information.). This correction was previous implemented in the lumiT function.

}
\value{
return a LumiBatch object
}

\author{ Simon Lin, Pan Du }
\seealso{ \code{\link{LumiBatch}}, \code{\link{addNuID2lumi}} }

\examples{
## specify the file name
# fileName <- 'Barnes_gene_profile.txt' # Not Run
## load the data
# x.lumi <- lumiR(fileName)

## load the data with empty detection and beadNum slots
# x.lumi <- lumiR(fileName, columnNameGrepPattern=list(detection=NA, beadNum=NA))
}
\keyword{ IO }

\name{lumiB}
\alias{lumiB}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Background correction of Illumina Expression data }
\description{
  Background correction of Illumina Expression data
}
\usage{
lumiB(x.lumi, method = c('none', 'bgAdjust', 'forcePositive', 'bgAdjust.affy'), verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ an ExpressionSet inherited object or a data matrix with columns as samples and rows as genes. For 'bgAdjust' method, it should be a LumiBatch Object }
  \item{method}{ the background correction method, it can be any function with a ExpressionSet Object or matrix as the first argument and return an processed object with the same class }
  \item{verbose}{ a boolean to decide whether to print out some messages }
  \item{\dots}{ other parameters used by the user provided background correction method  }
}
\details{
  We assume the BeadStudio output data is background corrected. So by default, it will do nothing. The 'bgAdjust' method will estimate the background based on the control probe information, which is kept in the controlData slot of LumiBatch object. The 'forcePositive' method will force all expression values to be positive by adding an offset (minus minimum value plus one), it does nothing if all expression values are positive. The purpose of this is to avoid NA when do logarithm transformation. 'none' does not but return the LumiBatch object. 'bgAdjust.affy' will call the \code{\link[affy]{bg.adjust}} function in affy package. User can also provide their own function with a LumiBatch Object as the first argument and return a LumiBatch Object with background corrected.

Thanks Kevin Coombes (M.D. Anderson Cancer Center) suggested adding this function.
}
\value{
 Return an object with background corrected. The class of the return object is the same as the input object x.lumi.
}

\author{ Pan Du, Kevin Coombes }
\seealso{ \code{\link{bgAdjust}}, \code{\link{lumiExpresso}} }
\examples{
## load example data
data(example.lumi)

## Do the default background correction method
lumi.B <- lumiB(example.lumi, method='bgAdjust', probs=0)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{detectOutlier}
\alias{detectOutlier}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Detect the outlier sample (or gene)}
\description{ 
Detect the outlier sample (or gene) based on distance to the cluster center
}

\usage{
detectOutlier(x, metric = "euclidean", standardize = TRUE, Th = 2, ifPlot = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a LumiBatch object, ExpressionSet object or a matrix with each column corresponding to a sample or other profile }
  \item{metric}{ the distance matric}
  \item{standardize}{ standardize the profile or not }
  \item{Th}{ the threshold of outlier,  }
  \item{ifPlot}{ to plot the result (as a hierarchical tree) or not }
}
\details{
The current outlier detection is based on the distance from the sample to the center (average of all samples after removing 10 percent samples farthest away from the center).
The assumption of the outlier detection is that there is only one single cluster and the distance from the sample
to the center is Gaussian distributed.

The outlier is detected when its distance to the center is larger than a certain threshold.
The threshold is calculated as Th * median distances to the center.

The profile relations can be visualized as a hierarchical tree.
}
\value{
Plot the results or	return the outlier (a logic vector) with the distance matrix and threshold as attributes.
}

\author{ Pan Du }

\seealso{ \code{\link{lumiQ}} }
\examples{
## load example data
data(example.lumi)

## detect the outlier (Further improvement needed.)
temp <- detectOutlier(example.lumi, ifPlot=TRUE)

}
\keyword{ methods }

\name{estimateLumiCV}
\alias{estimateLumiCV}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Estimate the coefficient of variance matrix of LumiBatch object }
\description{ 
Estimate the coefficient of variance matrix of LumiBatch object for each measurement or probe.
}

\usage{
estimateLumiCV(x.lumi, type = c("measurement", "probe"), ifPlot = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ a LumiBatch object }
  \item{type}{ estimate the coefficient of variance of each measurement or each probe }
  \item{ifPlot}{ determince whether to plot the density plot or not }
  \item{\dots}{ optional arguments to \code{\link[graphics]{plot}}.}
}
\details{
  By default, the coefficient of variance is the ratio of the mean and variance of the bead expression values.
Basically, it is the ration of \code{exprs}  and \code{se.exprs} element  of \code{LumiBatch} object.
If the type is "probe", it is the ratio of the mean and variance of probe expression profile.
}
\value{
A matrix of coefficient of variance
}

\author{Pan Du}

\seealso{ \code{\link{lumiQ}}}
\examples{
## load example data
data(example.lumi)

## estimate the coefficient of variance and plot the density plot of it
cv <- estimateLumiCV(example.lumi, ifPlot = TRUE)

}
\keyword{ methods }

\name{gammaFitEM}
\alias{gammaFitEM}

\title{ Estimate the methylation status by fitting a Gamma mixture model using EM algorithm }

\description{
Estimate the methylation status by fitting a two component Gamma mixture model using EM algorithm based on the all M-values of a particular sample 
}
\usage{
gammaFitEM(M, initialFit = NULL, fix.k = NULL, weighted = TRUE, maxIteration = 50, tol = 1e-04, plotMode = FALSE, truncate=FALSE, verbose = FALSE)
}

\arguments{
  \item{M}{ a vector of M-values covering the whole genome }
  \item{initialFit}{ the initial estimation of the gamma parameters returned by .initialGammaEstimation function }
  \item{fix.k}{ the k parameter of  the gamma function which is fixed during estimation }
  \item{weighted}{ determine whether to down-weight the long tails of two component densities beyond their modes }
  \item{maxIteration}{ maximum iterations allowed before converging }
  \item{tol}{ the difference threshold used to determine convergence }
  \item{plotMode}{ determine whether plot the histogram and density plot estimation }
  \item{truncate}{ determine whether to truncate the tails beyond the modes during parameter estimation}
  \item{verbose}{ determine whether plot intermediate messages during iterations }
}
\details{

The assumption of this function is that the M-value distribution is composed of the mixture of two shifted gamma distributions, which are defined as:
dgamma(x-s[1], shape=k[1], scale=theta[1]) and dgamma(s[2]-x, shape=k[2], scale=theta[2]). Here s represents the shift.


NOTE: the methylation status modeling algorithm was developed based on 27K methylation array. It has not been tested for 450K array. Considering 450K array covers both promoter and gene body, the two component Gamma mixture model assumption may not be valid any more.

}
\value{

The return is a list with "gammaFit" class attribute, which includes the following items:
 \item{logLikelihood}{the log-likelihood of the fitting model}
 \item{k}{parameter k of gamma distribution}
 \item{theta}{parameter theta of gamma distribution}
 \item{shift}{parameter shift of gamma distribution}
 \item{proportion}{the proportion of two components (gamma distributions)}
 \item{mode}{the mode positions of the gamma distributions}
 \item{probability}{the estimated methylation status posterior probability of each CpG site}
}

\author{ Pan Du }

\seealso{
  \code{\link{methylationCall}} and \code{\link{plotGammaFit}}
}

\examples{

data(example.lumiMethy)
M <- exprs(example.lumiMethy)
fittedGamma <- gammaFitEM(M[,1], initialFit=NULL, maxIteration=50, tol=0.0001, plotMode=TRUE, verbose=FALSE)

}

\keyword{methods}

\name{monoSmu}
\alias{monoSmu}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Monotonic smooth method }
\description{
  Fit the monotonic-constraint spline curve
}
\usage{
monoSmu(x, y, newX = NULL, nSupport = min(200, length(x)), nKnots = 6, rotate = FALSE, ifPlot = FALSE, xlab = 'x', ylab = 'y', ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a vector represents x values }
  \item{y}{ a vector represents y values }
  \item{newX}{ the new values to be transformed. If not provided, "x" will be used. }
  \item{nSupport}{ downsampled data points }
  \item{nKnots}{ parameter used by \code{\link{monoSpline}} }
  \item{rotate}{ determine whether to rotate the axis with 45 degrees in clockwise, i.e., fit the curve in the MA-plot. }
  \item{ifPlot}{ determine whether to plot intermediate results }
  \item{xlab}{ the xlab of the plot }
  \item{ylab}{ the ylab of the plot }
  \item{\dots}{ parameters used by \code{\link{supsmu}} and \code{\link{plot}} }
}
\details{
function called by lumiN.rsn. The function first fits a monotonic spline between vector x and y, then transforms the vector newX based on the fitted spline. (After transformation the fitted spline is supposed to be a diagonal line, i.e., x=y)
}
\value{
Return the transformed "newX" based on the smoothed curve
}
\references{ 
Lin, S.M., Du, P., Kibbe, W.A., (2008) 'Model-based  Variance-stabilizing Transformation for Illumina Microarray Data', Nucleic Acids Res. 36, e11
}
\author{ Simon Lin, Pan Du }

\seealso{ \code{\link{monoSpline}}  }

\keyword{ methods }

\name{lumiMethyN}
\alias{lumiMethyN}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Normalize the Illumina Infinium methylation data
}
\description{
Normalize the Illumina Infinium methylation data, which is an object in MethyLumiM class.
}
\usage{
lumiMethyN(methyLumiM, method = c("quantile", "ssn", "none"), separateColor = FALSE, verbose = TRUE, overwriteBigMatrix = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
 \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
 \item{method}{ supported normalization methods or user provided function, whose input and output should be a intensity matrix (pool of methylated and unmethylated probe intensities) }
 \item{separateColor}{determine whether to separately process two color channels}
 \item{verbose}{a boolean to decide whether to print out some messages }
 \item{overwriteBigMatrix}{whether to overwrite the result to the BigMatrix data, only valid when the input data is BigMatrix-based}
 \item{\dots}{ other parameters used by corresponding method }
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment.
}

\author{
Pan DU
}
\seealso{
 See Also \code{\link{normalizeMethylation.ssn}} and \code{\link{normalizeMethylation.quantile}}
}
\examples{
data(example.lumiMethy)
lumiMethy.norm = lumiN(example.lumiMethy)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{estimateMethylationBG}
\alias{estimateMethylationBG}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate the background levels of Illumina Infinium methylaton microarrays
}
\description{
Estimate the background levels of Illumina Infinium methylaton microarrays. It is called by function \code{\link{bgAdjustMethylation}}
}
\usage{
estimateMethylationBG(methyLumiM, separateColor = FALSE, nbin = 1000)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object or any eSet object with "methylated" and "unmethylated" data matrix element in the assayData slot }
  \item{separateColor}{determine whether to separately process two color channels}
  \item{nbin}{ the number of bins in the histogram used to estimate the mode position of the density }
}
\details{
When the controlData includes the negative control probe information, the background estimation will be the median of the negative control probes. Red and Green color channels will be estimated separately. 

In the case the negative control data is not available, the background will be estimated based on the mode positions of unmethylated or methylated distribution (the smaller one). The assumption is that the lots of CpG sites are unmethylated, which results in a density mode of the intensities measured by methylated probes. The position of this mode represents the background level.
}
\value{
a vector of estimated background levels for individual samples.
}
\author{
Pan DU
}

\seealso{
See Also \code{\link{lumiMethyB}} and \code{\link{bgAdjustMethylation}}
}
\examples{
data(example.lumiMethy)
estimatedBG = estimateMethylationBG(example.lumiMethy)
estimatedBG
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{beta2m}
\alias{beta2m}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Convert methylation Beta-value to M-value
}
\description{
Convert methylation Beta-value to M-value through a logistic transformation
}
\usage{
beta2m(beta)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{beta}{
	a matrix or vector of methylation Beta-value
}
}
\details{
Convert methylation Beta-value to M-value through a logistic transformation
}
\value{
return methylation M-value with the same size of input Beta-value
}
\references{
Du, P., Zhang, X, Huang, C.C., Jafari, N., Kibbe, W.A., Hou, L., and Lin, S.M.,  (2010) 'Comparison of Beta-value and M-value methods for quantifying methylation levels by microarray analysis', (under review)

}
\author{
Pan Du
}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
 See Also as \code{\link{m2beta}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{adjColorBias.ssn}
\alias{adjColorBias.ssn}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Color bias adjustment of Illumina Infinium methylaton microarrays using simple shift and scaling normalization
}
\description{
Color bias adjustment of Illumina Infinium methylaton microarrays using simple shift and scaling normalization
}
\usage{
adjColorBias.ssn(methyLumiM, refChannel = c("green", "red", "mean"))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{methyLumiM}{ a MethyLumiM object or any eSet object with "methylated" and "unmethylated" data matrix element in the assayData slot }
\item{refChannel}{ the reference color channel for color bias adjustment }
}
\details{
Perform color bias adjustment of Illumina Infinium methylaton microarrays. It requires the input methyLumiM object includes the color channel information in the featureData. Basically, there should be a "COLOR_CHANNEL" column in the data.frame returned by pData(featureData(methyLumiM)).

The basic idea of color bias adjustment is to treat it as the normalization between two color channels. It uses simple scaling normalization to normalize two color channels. The background levels are estimated using function \code{\link{estimateMethylationBG}}.
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after color bias adjustment.
}
\author{
Pan DU
}

\seealso{
See Also  \code{\link{lumiMethyC}}, \code{\link{estimateMethylationBG}} and \code{\link{adjColorBias.quantile}}
}
\examples{
data(example.lumiMethy)
# before adjustment
plotColorBias1D(example.lumiMethy)
lumiMethy.adj = adjColorBias.ssn(example.lumiMethy)
# after adjustment
plotColorBias1D(lumiMethy.adj)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{method}

\name{probeID2nuID}
\alias{probeID2nuID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Mapping Illumina ProbeID as nuID }
\description{
Mapping Illumina ProbeID as nuID. 
}
\usage{
probeID2nuID(probeID, lib.mapping = "lumiHumanIDMapping", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{probeID}{ a vector of Illumina ProbeID }
  \item{lib.mapping}{ an Illumina ID mapping library }
  \item{\dots}{ other parameters of \code{\link{IlluminaID2nuID}} }
}
\details{
The function will call \code{\link{IlluminaID2nuID}} when ID mapping library were provided.
}
\value{
see function \code{\link{IlluminaID2nuID}}
}
\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{ Pan Du }
\seealso{ \code{\link{nuID2probeID}}, \code{\link{IlluminaID2nuID}} }
\examples{
if (require(lumiHumanIDMapping)) {
    probeID2nuID('0001240020', lib='lumiHumanIDMapping')
}

}
\keyword{ methods }

\name{plotColorBias2D}
\alias{plotColorBias2D}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot the color bias of Illumina Infinium Methylation data in two dimensions
}
\description{
Plot the color bias (red and green channel) of Illumina Infinium Methylation data of one selected sample in two dimensions (methylated and unmethylated probe intensities)
}
\usage{
plotColorBias2D(methyLumiM, selSample = 1, combineMode = F, layoutRatioWidth = c(0.75, 0.25), layoutRatioHeight = c(0.25, 0.75), margins = c(5, 5, 2, 2), cex = 1.25, logMode = TRUE, subset = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{selSample}{ The index of sample name of the selected sample to plot color bias }
  \item{combineMode}{ Whether combine two color channels together and plot as one color}
  \item{layoutRatioWidth}{ the plot figure ratio between scatter plot and density plot}
  \item{layoutRatioHeight}{the plot figure ratio between scatter plot and density plot}
  \item{margins}{margin of the plot}
  \item{cex}{A numerical value giving the amount by which plotting text and symbols should be magnified relative to the default. See \code{\link[graphics]{par}} }
  \item{logMode}{ Whether plot the intensities in log-scale}
  \item{subset}{plot subset of randomly selected rows. All data will be plotted if it is NULL. }
  \item{\dots}{ other parameters used by \code{\link[graphics]{plot}}}
}
\details{
The function basically plots the probe intensities in 2-dimension (methylated vs unmethylated), and colors the dots in Red and Green based on their color channel information. The related density plot will also be plotted at the right and top of the scatter plot.
}
\value{
Invisibly return TRUE if plot successfully.
}
\author{
Pan DU
}

\seealso{
 See Also as \code{\link{plotColorBias1D}}
}
\examples{
data(example.lumiMethy)
# plot in 2D plot of one selected sample
plotColorBias2D(example.lumiMethy, selSample = 1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{hplot}

\name{IlluminaID2nuID}
\alias{IlluminaID2nuID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Matching Illumina IDs to nuID based on Illumina ID mapping library }
\description{
  Matching Illumina IDs to nuID based on Illumina ID mapping libraries.
}
\usage{
IlluminaID2nuID(IlluminaID, lib.mapping=NULL, species = c("Human", "Mouse", "Rat", "Unknown"), chipVersion = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{IlluminaID}{ a vector of Illumina IDs }
  \item{lib.mapping}{ the ID mapping library. If it is provided, the parameter "species" will be ignored. }
  \item{species}{ the species of the chip designed for. If users do not know it, it can be set as "Unknown". }
  \item{chipVersion}{ chipVersion information returned by function \code{\link{getChipInfo}}  }
  \item{\dots}{ other parameters of \code{\link{getChipInfo}} }
}
\details{
When the parameter "chipVersion" is not provided, this function basically returned the "idMapping" item returned by function \code{\link{getChipInfo}}. 
}
\value{
The mapping information from Illumina ID to nuID. It will be a matrix with each column corresponding to one matched manifest file when parameter "returnAllMatches" is TRUE. In this case, the columns are sorted from the best match to worst. If IlluminaID is NULL and chipVersion is provided, it will return all mapping information of the chip. 

}
\author{Pan Du}
\seealso{ \code{\link{getChipInfo}}, \code{\link{nuID2IlluminaID}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{utilities}

\name{seq2id}
\alias{seq2id}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Transfer a nucleotide sequence as a nuID}
\description{
The nuID (nucleotide universal identifier) is uniquely corresponding to probe sequence. The nuID is also self-identification and error checking
}

\usage{
seq2id(seq)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{seq}{ a nucleotide sequence composed of A, C, G, T (U). }
}
\details{
  The nuID is a exact mapping of nucleotide sequence based on Base64 encoding scheme. 
A character set A-Z, a-z, 0-9, "\_" and "." is used to represent to the base-64 numbers of 0-63.
The first character of nuID is a checking code, which provide information of both the number of 
padded "A"s at the nucleotide sequence and error checking.
Please refer to reference for more details.
}
\value{
A string represents nuID
}
\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{ Pan Du }
\seealso{\code{\link{id2seq}}}
\examples{
seq <- 'ACGTAAATTTCAGTTTAAAACCCCCCG'
id <- seq2id(seq)
id
id2seq(id)
}
\keyword{methods}

\name{boxplot-methods}
\docType{methods}
\alias{boxplot-methods}
\alias{boxplot.ExpressionSet}
\alias{boxplot,ExpressionSet-method}

\title{boxplot of a ExpressionSet object}
\description{ Creating \code{\link[graphics]{boxplot}} of sample intensities in a ExpressionSet object }

\usage{
\S4method{boxplot}{ExpressionSet}(x, range = 0, main, logMode = TRUE, subset = NULL, xlab = "", ylab = "Amplitude", ...)
}

\arguments{
\item{x}{ a \code{\link[Biobase]{ExpressionSet}} object}
\item{range}{ parameter of \code{\link[graphics]{boxplot}} }
\item{main}{ title of the boxplot }
\item{logMode}{ whether plot the data in log2 scale or not}
\item{subset}{subset of rows used to plot. It can be an index vector, or the length of a random subset}
\item{xlab}{xlab of the plot}
\item{ylab}{ylab of the plot}
\item{\dots}{ optional arguments to \code{\link[graphics]{boxplot}}.}
}

\details{
The boxplot function has a "subset" parameter. By default, it is set as 5000, i.e., randomly selected 5000 probes to plot the boxplot. The purpose of this is to plot the picture faster, but it will also make the boxplot has slightly different each time. If the user wants to make sure the boxplot is the same each time, you can set the "subset" parameter as NULL. 
}
\seealso{
\code{\link{LumiBatch-class}}, \code{\link[graphics]{boxplot}} and \code{\link{boxplot,MethyLumiM-method}}
}
\examples{
## load example data
data(example.lumi)

boxplot(example.lumi)
}
\keyword{methods}
\keyword{hplot}

\name{inverseVST}
\alias{inverseVST}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Inverse VST transform }
\description{
 Inverse transform of VST (variance stabilizing transform), see \code{\link{vst}}.
}
\usage{
inverseVST(x, fun = c('asinh', 'log'), parameter)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a VST transformed LumiBatch object or a numeric matrix or vector}
  \item{fun}{ function used in VST transform }
  \item{parameter}{ parameter of VST function }
}
\details{
Recover the raw data from VST transformed data returned by \code{\link{vst}}.
This function can be directly applied to the VST transformed or VST + RSN normalized LumiBatch object to reverse transform the data to the original scale. 
}
\value{
Return the raw data before VST transform
}
\references{
 Lin, S.M., Du, P., Kibbe, W.A.,  "Model-based Variance-stabilizing Transformation for Illumina Mi-croarray Data", submitted
 }
\author{ Pan Du }

\seealso{ \code{\link{vst}} }
\examples{

## load example data
data(example.lumi)

## get the gene expression mean for one chip
u <- exprs(example.lumi)[,1]
## get the gene standard deviation for one chip
std <- se.exprs(example.lumi)[,1]

## do variance stabilizing transform
transformedU <- vst(u, std)

## do inverse transform and recover the raw data
parameter <- attr(transformedU, 'parameter')
transformFun <- attr(transformedU, 'transformFun')
recoveredU <- inverseVST(transformedU, fun=transformFun, parameter=parameter)

## compare with the raw data
print(u[1:5])
print(recoveredU[1:5])

## do inverse transform of the VST + RSN processed data
lumi.N <- lumiExpresso(example.lumi[,1:2])
## Inverse transform. 
## Note: as the normalization is involved, the processed data will be different from the raw data.
lumi.N.raw <- inverseVST(lumi.N)

}
\keyword{ methods }

\name{density-methods}
\docType{methods}
\alias{density-methods}
\alias{density.ExpressionSet}
\alias{density,ExpressionSet-method}

\title{Density plot of a ExpressionSet object}
\description{ Creating density plot of sample intensities in a ExpressionSet object. It is equivalent to \link{hist-methods}. }

\usage{
\S4method{density}{ExpressionSet}(x, logMode=TRUE, xlab = NULL, ylab = "density", type = "l", 
	col=1:dim(x)[2], lty=1:dim(x)[2], lwd=1, xlim = NULL, index.highlight = NULL, color.highlight = 2, 
	symmetry = NULL, addLegend = TRUE, legendPos="topright", subset = NULL, main="", ...)
}

\arguments{
	\item{x}{ a \code{\link[Biobase]{ExpressionSet}} object}
	\item{logMode}{ determine whether the density plot is based on a log2 scale }
	\item{xlab}{ xlab of the density plot }
	\item{ylab}{ ylab of the density plot }
	\item{type}{ parameter of plot function }
	\item{col}{ line colors of the density plot }
	\item{lty}{ line types of the density plot }
	\item{lwd}{ line width of plot function }
	\item{xlim}{ parameter of the plot function}
	\item{index.highlight}{ the column index of the highlighted density curve }
	\item{color.highlight}{ color of highlighted density curve }
	\item{symmetry}{ the boundary position suppose to be symmetric distributed  }
	\item{addLegend}{ whether add legend to the plot or not }
	\item{legendPos}{the legend position. It can be a string specifying the position, or a length two vector specifying the x and y position. Please check \code{\link[graphics]{legend}} for more details. }
	\item{subset}{ subset of rows used to plot. It can be an index vector, or the length of a random subset}
    \item{main}{ title for the plot}
	\item{\dots}{ additional parameters for \code{\link[stats]{density}} function }
}

\seealso{
  \code{\link{LumiBatch-class}}, \link{hist-methods}, \code{\link[stats]{density}}
}
\examples{
## load example data
data(example.lumi)

density(example.lumi)
}
\keyword{methods}
\keyword{hplot}

\name{addControlData2methyLumiM}
\alias{addControlData2methyLumiM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Add methylation control data to a MethyLumiM object}
\description{
Add methylation control data to a MethyLumiM object
}
\usage{
addControlData2methyLumiM(controlData, methyLumiM, checkConsistency = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{a methylation control data file (output by GenomeStudio), or a MethyLumiQC object}
  \item{methyLumiM}{a MethyLumiM object to add control data}
  \item{checkConsistency}{whether to check the sample names consistency between methyLumiM and controlData}
  \item{\dots}{other parameters for reading controlData}
}
\details{
This function aims to add the controlData (MethyLumiQC object) to the controlData slot of a methyLumiM object
For control data, methylated data matrix in assayData slot corresponds to green channel, and unmethylated data matrix in assayData slot corresponds to red channel.

}
\value{
Return the methyLumiM object with the controlData added
}
\author{ Pan DU}
\seealso{
\code{\link{lumiMethyR}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{boxplot,MethyLumiM-method}
\docType{methods}
\alias{boxplot,MethyLumiM-method}

\title{boxplot of a MethyLumiM object}
\description{ Creating a \code{\link[hdrcde]{hdr.boxplot}} of the M-value in a MethyLumiM object }

\usage{
\S4method{boxplot}{MethyLumiM}(x, main, logMode = TRUE, ...)
}

\arguments{
\item{x}{ a \code{\link{MethyLumiM-class}} object}
\item{main}{ title of the boxplot }
\item{logMode}{only works when the dataType of x is "Intensity"}
\item{\dots}{ optional arguments to \code{\link[lattice]{bwplot}}.}
}

\details{
 Because the density plot of M-values usually includes two modes, using the traditional boxplot cannot accurately represent the distribution of the data. Here we use 
violin plot to show the density of M-values by samples
}
\seealso{
\code{\link{MethyLumiM-class}}, \code{\link[lattice]{panel.violin}} and \code{\link{boxplot,ExpressionSet-method}}
}
\examples{
## load example data
data(example.lumiMethy)
boxplot(example.lumiMethy)
}
\keyword{methods}
\keyword{hplot}

\name{adjColorBias.quantile}
\alias{adjColorBias.quantile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Color bias adjustment of Illumina Infinium methylaton microarrays using smooth quantile normalization
}
\description{
Color bias adjustment of Illumina Infinium methylaton microarrays using smooth quantile normalization \code{\link{smoothQuantileNormalization}}
}
\usage{
adjColorBias.quantile(methyLumiM, refChannel = c("green", "red"), logMode = TRUE, verbose = TRUE,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ a MethyLumiM object or any eSet object with "methylated" and "unmethylated" data matrix element in the assayData slot }
  \item{refChannel}{ the reference color channel for color bias adjustment }
  \item{logMode}{ whether perform the adjustment in log scale or not }
  \item{verbose}{ whether print extra information during processing }
  \item{\dots}{ other parameters used by \code{\link{smoothQuantileNormalization}}}
}
\details{
Perform color bias adjustment of Illumina Infinium methylaton microarrays. It requires the input methyLumiM object includes the color channel information in the featureData. Basically, there should be a "COLOR_CHANNEL" column in the data.frame returned by pData(featureData(methyLumiM)).

The basic idea of color bias adjustment is to treat it as the normalization between two color channels. It uses smooth quantile normalization \code{\link{smoothQuantileNormalization}} to normalize two color channels.
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after color bias adjustment.
}
\author{
Pan DU
}

\seealso{
 See Also  \code{\link{lumiMethyC}}, \code{\link{smoothQuantileNormalization}} and \code{\link{adjColorBias.ssn}}
}
\examples{
data(example.lumiMethy)
# before adjustment
plotColorBias1D(example.lumiMethy)
lumiMethy.adj = adjColorBias.quantile(example.lumiMethy)
# after adjustment
plotColorBias1D(lumiMethy.adj)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{produceGEOSubmissionFile}
\alias{produceGEOSubmissionFile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Produce GEO Sample Submission File in SOFT format }
\description{
  Produce GEO Sample Submission File in the SOFT format based on the provided LumiBatch object and sample information
}
\usage{
produceGEOSubmissionFile(lumiNormalized, lumiRaw, lib.mapping = NULL, idType = 'Probe', sampleInfo = NULL, fileName = "GEOSubmissionFile.txt", supplementaryRdata = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lumiNormalized}{ The normalized data (LumiBatch object) }
  \item{lumiRaw}{  The raw data (LumiBatch object), e.g., returned by \code{\link{lumiR}} }
  \item{lib.mapping}{ The Illumina ID Mapping library, e.g., "lumiHumanIDMapping" }
  \item{idType}{ the idType parameter of function \code{\link{nuID2IlluminaID}} }
  \item{sampleInfo}{ The sample information filename or data.frame, which is returned by \code{\link{produceGEOSampleInfoTemplate}} }
  \item{fileName}{ The file name of GEO Submission file  }
  \item{supplementaryRdata}{ determine whether produce the Rdata supplement data, which include both lumiNormalized and lumiRaw R objects. }
  \item{\dots}{ other parameters used by function \code{\link{nuID2IlluminaID}} }
}
\details{
The function produces the GEO sample submission file including both normalized and raw data information  in the SOFT format. The sample information should be provided by the user as a data.frame or Tab separated text file following the format of the template, which can be produced by function \code{\link{produceGEOSampleInfoTemplate}}. Users need to fill in the detailed sample descriptions in the template, especially the Sample\_title, Sample\_description and some protocols. Users are also required to fill in the "Sample\_platform\_id" by checking information of the GEO Illumina platform.

When the parameter "supplementaryRdata" is TRUE, the R objects, lumiNormalized, lumiRaw and sampleInfo, will be saved in a file named 'supplementaryData.Rdata'.
}
\value{
Save the result as a text file in SOFT sample submission format. The supplementary Rdata will be saved in a file 'supplementaryData.Rdata'.
}
\references{ http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html }
\author{ Pan Du }

\seealso{ \code{\link{produceGEOSampleInfoTemplate}}, \code{\link{produceGEOPlatformFile}} }
\examples{
## Not run
## Produce the sample information template
# produceGEOSampleInfoTemplate(lumiNormalized, lib.mapping = NULL, fileName = "GEOsampleInfo.txt")
## After editing the 'GEOsampleInfo.txt' by filling in sample information
# produceGEOSubmissionFile(lumiNormalized, lumiRaw, lib='lumiHumanIDMapping', sampleInfo='GEOsampleInfo.txt')
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{utilities}

\name{example.lumiMethy}
\alias{example.lumiMethy}
\docType{data}
\title{Example Illumina Infinium Methylation data in MethyLumiM class}
\description{
An example Illumina Infinium Methylation27k dataset, which includes a control and treatment dataset with both technique and biological replicates
}
\usage{data(example.lumiMethy)}
\details{
The example dataset includes four control and four treatment samples together with their technique replicates. The original samples and technique replicates were measured in two batches. Here are the names of sixteen samples: Treat1, Treat2, Treat3, Treat4, Ctrl1, Ctrl2, Ctrl3, Ctrl4, Treat1.rep, Treat2.rep, Treat3.rep, Treat4.rep, Ctrl1.rep, Ctrl2.rep, Ctrl3.rep, Ctrl4.rep.

To save storage space, we randomly subset 5000 CpG sites among about 27000 measured CpG sites.

}

\examples{
data(example.lumiMethy)
sampleNames(example.lumiMethy)
}
\keyword{datasets}

\name{getChrInfo}
\alias{getChrInfo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
get the chromosome location information of methylation probes
}
\description{
get the chromosome location information of methylation probes
}
\usage{
getChrInfo(methyData, lib = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyData}{
a MethyLumiM object
}
  \item{lib}{
Methylation annotation library
}
\item{\dots}{ optional arguments to \code{\link{addAnnotationInfo}}.}
}
\value{
a data.frame  
}
\author{
Pan Du
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{lumiT}
\alias{lumiT}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Transfer the Illumina data to stabilize the variance }
\description{
  Transfer the Illumina data to stabilize the variance.
}
\usage{
lumiT(x.lumi, method = c("vst", 'log2', 'cubicRoot'), ifPlot = FALSE, simpleOutput = TRUE, verbose = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ LumiBatch object }
  \item{method}{ four methods are supported: "vst", "log2", "cubicRoot" }
  \item{ifPlot}{ determine whether to plot the intermediate results }
  \item{simpleOutput}{ determine whether to simplify the output LumiBatch object, which will set the se.exprs, detection and beadNum slots as NULL. }
  \item{verbose}{ a boolean to decide whether to print out some messages }
  \item{\dots}{ other parameters used by \code{\link{vst}} }
}
\details{
lumiT is an interface of difference variance stabilizing transformation. See \code{\link{vst}} for details of VST (Variance Stabilizing Transform) of Illumina data.

NOTE: This correction of STDERR as STDEV was moved to the lumiR function.
}

\value{
Return a LumiBatch object with transformed expression values. It also includes the VST transform function and its parameters as attributes: "transformFun", "parameter". See \code{\link{inverseVST}} for details.
	
}
\references{
 Lin, S.M., Du, P., Kibbe, W.A., (2008) 'Model-based  Variance-stabilizing Transformation for Illumina Microarray Data', Nucleic Acids Res. 36, e11
 }
\author{ Pan Du, Simon Lin }

\seealso{ \code{\link{vst}} }
\examples{

## load example data
data(example.lumi)

## Do default VST variance stabilizing transform
lumi.T <- lumiT(example.lumi, ifPlot=TRUE)


}
\keyword{ methods }

\name{nuID2probeID}
\alias{nuID2probeID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Mapping nuID into Illumina ProbeID }
\description{
  Mapping nuID into Illumina ProbeID.
}
\usage{
nuID2probeID(nuID, lib.mapping = "lumiHumanIDMapping", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nuID}{ a vector of nuID }
  \item{lib.mapping}{ an Illumina ID mapping library }
  \item{\dots}{ other parameters of \code{\link{nuID2IlluminaID}} }
}
\details{
The function will call \code{\link{nuID2IlluminaID}} when ID mapping library were provided.
}
\value{
see function \code{\link{nuID2IlluminaID}}
}
\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{ Pan Du }
\seealso{ \code{\link{probeID2nuID}}, \code{\link{nuID2IlluminaID}} }
\examples{
if (require(lumiHumanIDMapping)) {
    nuID2probeID("B2J6WGhV.RevOJYff4", lib.mapping = "lumiHumanIDMapping")
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }

\name{lumi-package}
\alias{lumi-package}
\alias{lumi}
\docType{package}
\title{
A package for preprocessing Illumina microarray data
}
\description{
lumi R package is designed to preprocess the Illumina microarray (BeadArray) data. It includes functions of Illumina data input, quality control, variance stabilization, normalization and gene annotation. 
}
\details{
	\tabular{ll}{
		Package: \tab lumi\cr
		Type: \tab Package\cr
		Version: \tab 1.1.0\cr
		Date: \tab 2007-03-23\cr
		License: \tab  LGPL version 2 or newer \cr
	}

}
\author{
Pan Du, Simon Lin
Maintainer: Pan Du <dupan@northwestern.edu>
}
\references{
1. Du, P., Kibbe, W.A. and Lin, S.M., (2008) 'lumi: a pipeline for processing Illumina microarray', Bioinformatics 24(13):1547-1548

2. Lin, S.M., Du, P., Kibbe, W.A., (2008) 'Model-based  Variance-stabilizing Transformation for Illumina Microarray Data', Nucleic Acids Res. 36, e11

3. Du, P., Kibbe, W.A. and Lin, S.M., (2007) 'nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays', Biology Direct, 2, 16

}
\keyword{ package }

\name{LumiBatch-class}
\docType{class}
\alias{class:LumiBatch}
\alias{LumiBatch-class}
\alias{LumiBatch}

\alias{se.exprs}
\alias{se.exprs<-}
\alias{se.exprs,ExpressionSet-method}
\alias{se.exprs<-,ExpressionSet-method}
\alias{se.exprs<-,ExpressionSet,ANY-method}

\alias{beadNum}
\alias{beadNum<-}
\alias{beadNum,ExpressionSet-method}
\alias{beadNum<-,ExpressionSet-method}
\alias{beadNum<-,ExpressionSet,ANY-method}

\alias{detection}
\alias{detection<-}
\alias{detection,ExpressionSet-method}
\alias{detection<-,ExpressionSet-method}
\alias{detection<-,ExpressionSet,ANY-method}

\alias{controlData}
\alias{controlData<-}
\alias{controlData,LumiBatch-method}
\alias{controlData<-,LumiBatch-method}
\alias{controlData<-,LumiBatch,ANY-method}

\alias{initialize,LumiBatch-method}
\alias{summary,LumiBatch-method}
\alias{show,LumiBatch-method}
\alias{combine,LumiBatch,LumiBatch-method}
\alias{combine,ExpressionSet,LumiBatch-method}
\alias{combine,LumiBatch,ExpressionSet-method}
\alias{[,LumiBatch-method}
\alias{[,LumiBatch,ANY,ANY,ANY-method}
\alias{sampleNames<-,LumiBatch,ANY-method}
\alias{getHistory,LumiBatch-method}
\alias{getHistory}

\title{Class LumiBatch: contain and describe Illumina microarray data}
\description{This is a class representation for Illumina microarray data. It extends \code{\link[Biobase]{ExpressionSet}}.}
\section{Extends}{
   Directly extends class \code{\link[Biobase]{ExpressionSet}}.
}

\section{Creating Objects}{

  \code{new("LumiBatch",
    exprs = [matrix],
	se.exprs = [matrix],
	beadNum = [matrix],
	detection = [matrix],
    phenoData = [AnnotatedDataFrame],
    history = [data.frame],
    ...)
  }

  \code{LumiBatch} instances are usually created through
  \code{new("LumiBatch", ...)}. The arguments to \code{new}
  should include \code{exprs} and \code{se.exprs}, others can be 
  missing, in which case they are assigned default values.

  Objects can be created using the function \code{\link{lumiR}}.
}
\section{Slots}{
   Slot specific to \code{LumiBatch}:
   \describe{
   		\item{\code{history}:}{ a data.frame recording the operation history of the LumiBatch object.}
   		\item{\code{controlData}:}{ a data.frame with first two columns as "controlType" and "ProbeID". The rest columns are the control probe expression amplitudes for individual samples.}
   		\item{\code{QC}:}{ a the quality control information of the LumiBatch object, returned by lumiQ function.}
   }

   Slots inherited from \code{ExpressionSet}:
   \describe{
      \item{\code{assayData}}{ contains equal dimensional matrices:
 		\code{exprs} (contains gene expression level, which is the mean of its bead replicates.),
 		\code{se.exprs} (contains gene expression standard error, which is the standard error of its bead replicates.), 
 		\code{beadNum} (records the number of beads for the probe.),
 		\code{detection} (records the detection p-value of the probe. The number is from [0,1]. By default, < 0.01 indicates good detection.). 
		For more details of \code{assayData}, please see \code{\link[Biobase]{ExpressionSet}} } 
      \item{\code{phenoData}:}{See \code{\link[Biobase]{eSet}}}
      \item{\code{experimentData}:}{See \code{\link[Biobase]{eSet}}}
      \item{\code{annotation}:}{See \code{\link[Biobase]{eSet}}}
  	}
}

\section{Methods}{

  \bold{Class-specific methods:}
  \describe{
     \item{\code{se.exprs(LumiBatch)}, \code{se.exprs(LumiBatch,matrix)<-}:}{Access and
       set elements named \code{se.exprs} in the \code{AssayData-class} slot.}
     \item{\code{beadNum(LumiBatch)}, \code{beadNum(LumiBatch)<-}:}{Access and
       set elements named \code{beadNum} in the \code{AssayData-class} slot. Use "beadNum(LumiBatch) <- NULL" to remove the beadNum element.}
     \item{\code{detection(LumiBatch)}, \code{detection(LumiBatch)<-}:}{Access and
       set elements named \code{detection} in the \code{AssayData-class} slot. Use "detection(LumiBatch) <- NULL" to remove the detection element.}
     \item{\code{getHistory(LumiBatch)}:}{Access the operation history of \code{LumiBatch} object.}
  }

   \bold{Derived from \code{\link[Biobase]{ExpressionSet}}} (For the directly inherited methods, 
	 please see \code{\link[Biobase]{ExpressionSet}} and \code{\link[Biobase]{eSet}}):
   \describe{
     \item{\code{combine(LumiBatch,missing)}:}{Combine two LumiBatch objects, including \code{history} slot. See \code{\link[Biobase]{eSet}}}
	 \item{\code{exprs(LumiBatch)}, \code{exprs(LumiBatch,matrix)<-}:}{Access and
	    set elements named \code{exprs} in the \code{AssayData-class} slot.}
	 \item{\code{object[(i,j)}:}{Conduct subsetting of the data in a LumiBatch object }
   }

   \bold{Standard generic methods} (For the directly inherited methods, 
	 please see \code{\link[Biobase]{ExpressionSet}} and \code{\link[Biobase]{eSet}}):
  \describe{
    \item{\code{initialize(LumiBatch)}:}{Object instantiation, used
      by \code{new}; not to be called directly by the user.}
    \item{\code{validObject(LumiBatch)}:}{Validity-checking method, ensuring that \code{exprs} and \code{se.exprs} 
	  is a member of \code{assayData}. Other validity check is the same as \code{checkValidity(ExpressionSet)}.}
    \item{\code{show(LumiBatch)}}{A summary of the \code{LumiBatch} object.}
   }
}

\author{ Pan Du, Simon Lin }
\seealso{ \code{\link{lumiR}}, \code{\link{lumiT}}, \code{\link{lumiN}},
	\code{\link{boxplot-methods}}, \code{\link{pairs-methods}}, \code{\link{MAplot-methods}}
}
\examples{
## load example data
data(example.lumi)

## show the summary of the data
# summary(example.lumi)
example.lumi	

## get expression matrix
temp <- exprs(example.lumi)

## get a subset
temp <- example.lumi[,1]	## retrieve the first sample

## get the probe id
featureNames(example.lumi)[1:3]

## combine LumiBatch objects
temp <- combine(example.lumi[,1], example.lumi[,3])
temp

}

\keyword{classes}

\name{estimateM}
\alias{estimateM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate methylation M-value matrix 
}
\description{
Estimate methylation M-value matrix from MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities
}
\usage{
estimateM(methyLumiM, returnType=c("ExpressionSet", "matrix"), offset=100)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{returnType}{determine whether return an ExpressionSet (MethyLumiM in this case) or matrix object}
  \item{offset}{ offset added to the  methylated and unmethylated probe intensities when estimating the M-value }
}
\details{
M-value is the log2 ratio between Illumina methylated and unmethylated probe intensities. As variations of small intensities can cause big changes in the ratio estimation, so an offset is added to methylated and unmethylated probe intensities when estimating the M-value.
}
\value{
 A MethyLumiM or matrix object of methylation M-value
}
\references{
Du, P., Zhang, X, Huang, C.C., Jafari, N., Kibbe, W.A., Hou, L., and Lin, S.M.,  (2010) 'Comparison of Beta-value and M-value methods for quantifying methylation levels by microarray analysis', (under review)
}
\author{
Pan DU
}

\seealso{
  See Also as \code{\link{estimateBeta}}, \code{\link{estimateIntensity}}
}
\examples{
data(example.lumiMethy)
methyLumiM = estimateM(example.lumiMethy)
boxplot(methyLumiM)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{getNuIDMappingInfo}
\alias{getNuIDMappingInfo}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ get the mapping information from nuID to RefSeq ID }
\description{
  Get the mapping information (including mapping quality information) of nuIDs to the most recent RefSeq release. These information was kept in the IDMapping libraries.
}
\usage{
getNuIDMappingInfo(nuID = NULL, lib.mapping)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nuID}{ a vector of nuIDs. If it is NULL, all mappings will be returned. }
  \item{lib.mapping}{ the ID mapping library }
}
\details{
 The function basically return the nuID mapping information kept in the "nuID\_MappingInfo" table of IDMapping libraries (lumiHumanIDMapping, lumiMouseIDMapping, lumiRatIDMapping). For more details of nuID mapping, please refer to the help of corresponding IDMapping library.
}
\value{
	It returns a data.frame with each row corresponding to an input nuID.
}
\author{Warren Kibbe, Pan Du, Simon Lin}
\examples{
	## load example data
	data(example.lumi)
	if (require(lumiHumanIDMapping)) {
		nuIDs <- featureNames(example.lumi)
		mappingInfo <- getNuIDMappingInfo(nuIDs, lib.mapping='lumiHumanIDMapping')
		head(mappingInfo)
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{utilities}

\name{getControlData}
\alias{getControlData}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Get control probe information }
\description{
  Get control probe information from Bead Studio output or a LumiBatch object.
}
\usage{
getControlData(x, type = c('data.frame', 'LumiBatch'), ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ the control data can be a LumiBatch object or the Control Probe Profile file outputted by BeadStudio }
  \item{type}{ determine the return data type }
  \item{\dots}{ other parameters used by \code{\link{lumiR}} function }
}
\value{
   By default, it returns a data.frame with first two columns as "controlType" and "ProbeID". The rest columns are the expression amplitudes for individual samples. When type is 'LumiBatch', it returns a LumiBatch object, which basically is the return of lumiR without combining duplicated TargetIDs. As the return is a LumiBatch object, it includes more information, like probe number, detection p-value and standard error of the measurement.
}
\author{ Pan Du }
\seealso{ \code{\link{addControlData2lumi}} }
\examples{
	controlFile <- system.file('doc', 'Control_Probe_Profile.txt', package='lumi')
	if (file.exists(controlFile)) {
  	## return a data.frame
  	controlData <- getControlData(controlFile)
  	class(controlData)
  	names(controlData)

  	## return a LumiBatch object
  	controlData <- getControlData(controlFile, type='LumiBatch')
  	summary(controlData)
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }

\name{vst}
\alias{vst}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Variance Stabilizing Transformation }
\description{
  Stabilizing the expression variance based on the bead level expression variance and mean relations
}
\usage{
vst(u, std, nSupport = min(length(u), 500), backgroundStd=NULL, fitMethod = c('linear', 'quadratic'), lowCutoff = 1/3, ifPlot = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{u}{ mean expression of the beads with same sequence }
  \item{std}{ expression standard deviation of the beads with same sequence }
  \item{nSupport}{ the number of down-sampling to speed processing }
  \item{backgroundStd}{ pre-estimated background standard deviation level}
  \item{fitMethod}{ methods of fitting the relations between expression variance and mean relations }
  \item{lowCutoff}{ cutoff ratio to determine the low expression range. Do not change this until you now what you are doing. }
  \item{ifPlot}{ plot intermediate results or not }
}
\details{
 The variance-stabilizing transformation (VST) takes the advantage of larger number of technical replicates available on the Illumina microarray. It models the mean-variance relationship of the within-array technical replicates at the bead level of Illumina microarray. An arcsinh transform is then applied to stabilize the variance. See reference for more details.

For the methods of fitting the relations between expression variance and mean relations, the 'linear' method is more robust and provides detailed parameters for inverseVST. 
}
\value{
Return the transformed (variance stabilized) expression values.
}
\references{
 Lin, S.M., Du, P., Kibbe, W.A.,  "Model-based Variance-stabilizing Transformation for Illumina Mi-croarray Data", submitted
 }
\author{ Pan Du, Simon Lin }

\seealso{ \code{\link{lumiT}}, \code{\link{inverseVST}} }
\examples{
## load example data
data(example.lumi)

## get the gene expression mean for one chip
u <- exprs(example.lumi)[,1]
## get the gene standard deviation for one chip
std <- se.exprs(example.lumi)[,1]

## do variance stabilizing transform
transformedU <- vst(u, std)

## do variance stabilizing transform with plotting intermediate result 
transformedU <- vst(u, std, ifPlot=TRUE)


}
\keyword{ methods }

\name{monoSpline}
\alias{monoSpline}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Fitting a curve with monotonic spline }
\description{
Fitting a curve with monotonic spline
}
\usage{
monoSpline(x, y, newX=NULL, nKnots = 6, ifPlot = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
	\item{x}{ a vector represents x values }
	\item{y}{ a vector represents y values }
	\item{newX}{ the new values to be transformed. If not provided, "x" will be used. }
	\item{nKnots}{ parameter used by function smoothCon in package mgcv }
  	\item{ifPlot}{ determine whether to plot intermediate results }
}
\details{
Function internally called by monoSmu
}
\value{
return the transformed "newX" based on the smoothed curve
}
\author{ Simon Lin, Pan Du }

\seealso{ \code{\link{monoSmu}}  }

\keyword{ methods }

\name{methylationCall}
\alias{methylationCall}

\title{ Estimated methylation call }

\description{
Estimated methylation call based on the fitting results of \code{\link{gammaFitEM}}
}

\usage{
methylationCall(x, threshold = 0.95, ...)
}

\arguments{
	\item{x}{ a vector of M-values covering the whole genome or a "gammaFit" class object returned by \code{\link{gammaFitEM}} }
	\item{threshold}{the probability threshold to make a methylation call. The threshold can be a vector of two: unmethylation threshold and methylation threshold}
	\item{\dots}{ other parameters used by \code{\link{gammaFitEM}} }
}
\details{
	Retrieve the probability element returned by \code{\link{gammaFitEM}}, and convert it as three status calls based on probability threshold
	
	NOTE: the methylation status modeling algorithm was developed based on 27K methylation array. It has not been tested for 450K array. Considering 450K array covers both promoter and gene body, the two component Gamma mixture model assumption may not be valid any more.

}
\value{
A vector of three methylation status: "Unmethy" (unmethylation posterior probability > unmethylation threshold), "Methy" (methylation posterior probability > methylation threshold), or "Margin". The sum of unmethylation posterior probability and methylation posterior probability equals one. The methylation probability is returned as an attribute of "probability".
}

\author{ Pan DU }

\seealso{
\code{\link{gammaFitEM}}
}

\examples{
data(example.lumiMethy)
M <- exprs(example.lumiMethy)
fittedGamma <- gammaFitEM(M[,1], initialFit=NULL, maxIteration=50, tol=0.0001, plotMode=TRUE, verbose=FALSE)
methyCall <- methylationCall(fittedGamma)
table(methyCall)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{produceMethylationGEOSubmissionFile}
\alias{produceMethylationGEOSubmissionFile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Produce GEO Sample Submission File of Illumina methylation microarray data in SOFT format}
\description{
Produce GEO Sample Submission File in the SOFT format based on the provided MethyLumiM object and sample information
}
\usage{
produceMethylationGEOSubmissionFile(methyLumiM, methyLumiM.raw = NULL, lib.mapping = NULL, idType = "Probe", sampleInfo = NULL, fileName = "GEOSubmissionFile.txt", supplementaryRdata = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{The normalized data in MethyLumiM class}
  \item{methyLumiM.raw}{The raw data in MethyLumiM class}
  \item{lib.mapping}{Currently not used for Illumina methylation data}
  \item{idType}{Currently no other options for Illumina methylation data}
  \item{sampleInfo}{The sample information filename or data.frame, which is returned by \code{\link{produceGEOSampleInfoTemplate}}}
  \item{fileName}{The file name of GEO Submission file}
  \item{supplementaryRdata}{determine whether produce the Rdata supplement data, which include both methyLumiM and methyLumiM.raw R objects.}
  \item{\dots}{other parameters used by function \code{\link{nuID2IlluminaID}}, but not implemented for methylation data}
}
\details{
The function produces the GEO sample submission file including both normalized and raw data information  in the SOFT format. The sample information should be provided by the user as a data.frame or Tab separated text file following the format of the template, which can be produced by function \code{\link{produceGEOSampleInfoTemplate}}. Users need to fill in the detailed sample descriptions in the template, especially the Sample\_title, Sample\_description and some protocols. Users are also required to fill in the "Sample\_platform\_id" by checking information of the GEO Illumina platform.

When the parameter "supplementaryRdata" is TRUE, the R objects, methyLumiM, methyLumiM.raw and sampleInfo, will be saved in a file named 'supplementaryData.Rdata'.
}
\value{
Save the result as a text file in SOFT sample submission format. The supplementary Rdata will be saved in a file 'supplementaryData.Rdata'.
}
\references{ http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html }
\author{ Pan Du }

\seealso{ \code{\link{produceGEOSampleInfoTemplate}}, \code{\link{produceGEOPlatformFile}} }
\examples{
## Not run
## Produce the sample information template
# produceGEOSampleInfoTemplate(methyLumiM, fileName = "GEOsampleInfo.txt")
## After editing the 'GEOsampleInfo.txt' by filling in sample information
# produceMethylationGEOSubmissionFile(methyLumiM, methyLumiM.raw, sampleInfo='GEOsampleInfo.txt')
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{utilities}

\name{plot-methods}
\docType{methods}
\alias{plot-methods}
\alias{plot.ExpressionSet}
\alias{plot,ExpressionSet-method}
\alias{plot,ExpressionSet,missing-method}

\title{Plot of a ExpressionSet object}
\description{
Creating quality control plots of a ExpressionSet object
}

\usage{
\S4method{plot}{ExpressionSet,missing}(x, what = c("density", "boxplot", "pair", "MAplot", "sampleRelation", "outlier", "cv"), main, ...)
}

\arguments{
\item{x}{ a ExpressionSet object returned by \code{\link{lumiQ}} }
\item{what}{ one of the six kinds of QC plots }
\item{main}{ the title of the QC plot }
\item{\dots}{ additional parameters for the corresponding QC plots }
}

\details{
The parameter "what" of \code{plot} function controls the type of QC plots,
which includes:
\itemize{
\item{\bold{density}:}{ the density plot of the chips, see \code{\link{hist-methods}}}
\item{\bold{boxplot}:}{ box plot of the chip intensities, see \code{\link{boxplot-methods}}}
\item{\bold{pair}:}{ the correlation among chips, plot as a hierarchical tree, see \code{\link{pairs-methods}}}
\item{\bold{MAplot}:}{ the MAplot between chips, see \code{\link{MAplot-methods}}}
\item{\bold{sampleRelation}:}{ plot the sample relations. See \code{\link{plotSampleRelation}}}
\item{\bold{outlier}:}{ detect the outliers based on the sample distance to the center. See 
	\code{\link{detectOutlier}}}
\item{\bold{cv}:}{ the density plot of the coefficients of variance of the chips. See \code{\link{estimateLumiCV}}}
}
}
\seealso{ 
\code{\link{LumiBatch-class}}, \code{\link{hist-methods}}, \code{\link{boxplot-methods}}, \code{\link{MAplot-methods}},
 	\code{\link{pairs-methods}}, \code{\link{plotSampleRelation}}, \code{\link{estimateLumiCV}}, \code{\link{detectOutlier}} 
}

\examples{

## load example data
data(example.lumi)

## Quality control estimation
lumi.Q <- lumiQ(example.lumi)

## summary
summary(lumi.Q)

## plot the density
plot(lumi.Q, what='density')

## plot the pairwise sample correlation
plot(lumi.Q, what='pair')

## plot the pairwise MAplot
plot(lumi.Q, what='MAplot')

## sample relations
plot(lumi.Q, what='sampleRelation', method='mds', color=c('100US', '95US:5P', '100US', '95US:5P'))

## detect outlier based on the distance to the mean profile
plot(lumi.Q, what='outlier')

## Density plot of coefficient of variance
plot(lumi.Q, what='cv')

}
\keyword{methods}
\keyword{hplot}

\name{nuID2RefSeqID}
\alias{nuID2RefSeqID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Map nuID to RefSeq ID }
\description{
  Map nuID to RefSeq ID based on IDMapping libraries.
}
\usage{
nuID2RefSeqID(nuID = NULL, lib.mapping, filterTh = c(Strength1 = 95, Uniqueness = 95), returnAllInfo = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nuID}{ a vector of nuIDs. If it is NULL, all mappings will be returned. }
  \item{lib.mapping}{ the ID mapping library }
  \item{filterTh}{the mapping quality filtering threshold used to filter the ID mapping. Obsolete for lumi ID mapping package > version 1.3!}
  \item{returnAllInfo}{ determine to return the detailed mapping information or just the matched RefSeq IDs }
}
\details{
  This function is based on the return of \code{\link{getNuIDMappingInfo}} function. It uses mapping quality information to filter out the bad mappings. The parameter "filterTh" is obsolete for lumi ID mapping package > version 1.3, which only keeps the perfect mapping. For the old version of ID mapping package (< 1.3), the names of "filterTh" are basically the field names of "nuID\_MappingInfo" table, which include 'Strength1', 'Strength2', 'Uniqueness' and 'Total hits'. For the definition of these metrics, please refer to the IDMapping library or see the reference website.
}
\value{
	returns the matched RefSeq IDs or a data.frame with each row corresponding to an input nuID (when "returnAllInfo" is TRUE).	
}
\references{ 
 https://prod.bioinformatics.northwestern.edu/nuID/
}
\author{Warren Kibbe, Pan Du, Simon Lin}
\seealso{ See Also \code{\link{getNuIDMappingInfo}} }
\examples{
	## load example data
	data(example.lumi)
	if (require(lumiHumanIDMapping)) {
		nuIDs <- featureNames(example.lumi)
		mappingInfo <- nuID2RefSeqID(nuIDs, lib.mapping='lumiHumanIDMapping')
		head(mappingInfo)
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{utilities}

\name{colorBiasSummary}
\alias{colorBiasSummary}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
A summary of colorBias information
}
\description{
A summary of colorBias information, which is a data.frame summarizing the intensities of individual samples
}
\usage{
colorBiasSummary(methyLumiM, logMode = TRUE, channel = c("both", "unmethy", "methy", "sum"))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{logMode}{ Whether plot the intensities in log-scale }
  \item{channel}{estimate the intensity in different methods}
}
\details{
A summary of colorBias information. There are four options using "channel" parameter to plot the density plot. "both": estimate the density by pooling together methylated and unmethylated probe intensities. "unmethy" and "methy": plot either unmethylated or methylated probe density. "sum" plot the density of the sum of methylatled and unmethylated probe intensitys.}
\value{
A data.frame summarizing the intensities of individual samples
}

\author{
Pan DU
}

\keyword{methods}

\name{produceGEOSampleInfoTemplate}
\alias{produceGEOSampleInfoTemplate}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Produce the template of GEO sample information }
\description{
  Produce the template of GEO sample information, which is used for function \code{\link{produceGEOSubmissionFile}}.
}
\usage{
produceGEOSampleInfoTemplate(lumiNormalized, lib.mapping = NULL, fileName = "GEOsampleInfo.txt")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lumiNormalized}{ The normalized data (LumiBatch object) }
  \item{lib.mapping}{ The Illumina ID Mapping library, e.g., "lumiHumanIDMapping" }
  \item{fileName}{ The file name of Tab separated sample information file }
}
\details{
  This function just produces a template of sample information with some default fillings. Users need to fill in the detailed sample descriptions, especially the Sample\_title, Sample\_description and some protocols. No blank fields are allowed. Function \code{\link{produceGEOSubmissionFile}} will produce the file GEO submission file based on this sample information. The users should not use "\#" in the description as it is a reserved character.
}
\value{
	Save the result as a Tab separated text file or return a data.frame if the fileName is NULL.
}
\references{ http://www.ncbi.nlm.nih.gov/projects/geo/info/soft2.html }
\author{ Pan Du }

\seealso{ \code{\link{produceGEOSubmissionFile}} }
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{utilities}

\name{estimateBeta}
\alias{estimateBeta}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimate methylation Beta-value matrix 
}
\description{
Estimate methylation Beta-value matrix from MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities
}
\usage{
estimateBeta(methyLumiM, returnType=c("ExpressionSet", "matrix"), offset = 100)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{returnType}{determine whether return an ExpressionSet or matrix object}
  \item{offset}{ An offset value added to the denominator to avoid close to zero intensities  }
}

\details{
Beta-value is ratio between Illumina methylated probe intensity and total probe intensities (sum of methylated and unmethylated probe intensities, see \code{\link{estimateIntensity}}). An offset value added to the denominator to avoid close to zero intensities in the denominator. Beta-value is in the range of 0 and 1. If we assume the probe intensity follows Gamma distribution, then the Beta-value follows a Beta distribution.

}
\value{
An ExpressionSet or matrix object of methylation Beta-value
}
\author{
Pan DU
}


\seealso{
See Also as \code{\link{estimateIntensity}} and \code{\link{estimateM}}
}
\examples{
data(example.lumiMethy)
methyLumiBeta = estimateBeta(example.lumiMethy)
density(methyLumiBeta)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{nuID2IlluminaID}
\alias{nuID2IlluminaID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Matching nuIDs to Illumina IDs based on Illumina ID mapping library}
\description{
  Matching nuIDs to Illumina IDs based on Illumina ID mapping library
}
\usage{
nuID2IlluminaID(nuID, lib.mapping=NULL, species = c("Human", "Mouse", "Rat", "Unknown"), idType=c('All', 'Probe', 'Gene', 'Accession', 'Search_key', 'Symbol'), chipVersion = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{nuID}{ a vector of nuIDs }
  \item{lib.mapping}{ the ID mapping library. If it is provided, the parameter "species" will be ignored. }
  \item{species}{ the species of the chip designed for. If users do not know it, it can be set as "Unknown".}
  \item{idType}{ the Illumina ID type}
  \item{chipVersion}{ chipVersion information returned by function \code{\link{getChipInfo}} }
  \item{\dots}{ other parameters of \code{\link{getChipInfo}} }
}
\details{
The parameter "idType" represents different types of Illumina IDs. It returns the entire table when idType = "All". When idType = 'Probe', it returns "ProbeId" or "Probe\_Id". When idType = 'Gene', it returns "Target" or "ILMN\_Gene" IDs.
 
This function basically returned the "idMapping" item returned by function \code{\link{getChipInfo}}.
If nuID is NULL and chipVersion is provided, it will return all mapping information of the chip. 
}
\value{
The mapping information from nuID to Illumina ID. It will be a matrix with each column corresponding to one matched manifest file when parameter "returnAllMatches" is TRUE. In this case, the columns are sorted from the best match to worst.
}
\author{Pan Du}
\seealso{ \code{\link{getChipInfo}}, \code{\link{IlluminaID2nuID}} }
\examples{
	## load example data
	data(example.lumi)
	nuIDs <- featureNames(example.lumi)
	if (require(lumiHumanIDMapping)) {
		illuminaID <- nuID2IlluminaID(nuIDs[1:5], lib='lumiHumanIDMapping')
		illuminaID
	}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{utilities}

\name{getControlType}
\alias{getControlType}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Get the types of the control probes }
\description{
  Get the types of the control probes, which is in the first column of the controlData data.frame for LumiBatch objects. For methylation data, it is the return of controlTypes function 
}
\usage{
getControlType(controlData)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{controlData}{ a LumiBatch object including control data, a control data data.frame, or a MethyLumiQC object for methylation data }
}
\value{
	return the unique type of control probe type.
}
\author{ Pan Du }

\seealso{  \code{\link{addControlData2lumi}}, \code{\link[methylumi]{controlTypes}} for methylation data }
\examples{
	controlFile <- system.file('doc', 'Control_Probe_Profile.txt', package='lumi')
	if (file.exists(controlFile)) {
	  ## return a data.frame
	  controlData <- getControlData(controlFile)
	  getControlType(controlData)
	}

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{plotSampleRelation}
\alias{plotSampleRelation}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ visualize the sample relations }
\description{
  plot the sample relations based on MDS or hierarchical clustering
}
\usage{
plotSampleRelation(x, subset = NULL, cv.Th = 0.1, standardize = TRUE, method = c("cluster", "mds"), dimension = c(1, 2), color = NULL, main = NULL, pch=NULL, addLegend=TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{a LumiBatch object, ExpressionSet object or a matrix with each column corresponding to a sample}
  \item{subset}{ the subset probes used to determine the sample relations. If it is one number, then randomly selected "number" of probes will be used. If not provide, all the probes will be used. }
  \item{cv.Th}{ the threshold of the coefficient of variance of probes used to select probes to estimate sample relations }
  \item{standardize}{ standardize the expression profiles or not }
  \item{method}{ "MDS" or "hierarchical clustering" }
  \item{dimension}{ the principle components to visualize the MDS plot }
  \item{color}{ the color for each sample during plot. Only support the "mds" method }
  \item{main}{ the title of the plot }
  \item{pch}{ use symbols instead of text to label the samples }
  \item{addLegend}{ Whether to add legend to MDS (two-dimensional PCA) plot }
  \item{\dots}{ Other parameters used by plot function. }
}
\details{
  Estimate the sample relations based on selected probes (based on large coefficient of variance (mean / standard variance)). Two methods can be used: MDS (Multi-Dimensional Scaling) or hierarchical clustering methods.
}
\value{
Invisibly return the hierarchical clustering results (if 'cluster' method used) or coordinates of the mds plot (if 'mds' method used) .
}

\author{ Pan Du }

\seealso{  \code{\link{lumiQ}}, \code{\link{LumiBatch}}, , \code{\link{plot,ExpressionSet-method}} }
\examples{

## load example data
data(example.lumi)

## plot the sample relations with MDS
## the color of sample is automatically set based on the sample type
plotSampleRelation(example.lumi, col=c('100US', '95US:5P', '100US', '95US:5P'))

## plot the sample relations with hierarchical clustering
plotSampleRelation(example.lumi, method='cluster')

}
\keyword{ hplot }

\name{id2seq}
\alias{id2seq}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Transfer a nuID as a nucleotide sequence }
\description{
The nuID (nucleotide universal identifier) is uniquely corresponding to probe sequence. The nuID is also self-identification and error checking
}

\usage{
id2seq(id)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{id}{ a nuID (nucleotide universal identifier) }
}
\details{
A reverse of \code{\link{seq2id}}.
Please refer to reference for more details.
}
\value{
a string of nucleotide sequence
}
\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{ Pan Du }
\seealso{\code{\link{seq2id}}}
\examples{
seq <- 'ACGTAAATTTCAGTTTAAAACCCCCCG'
id <- seq2id(seq)
id
id2seq(id)
}
\keyword{methods }

\name{normalizeMethylation.quantile}
\alias{normalizeMethylation.quantile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Quantile normalization of Illumina Infinium methylation data at probe level
}
\description{
Quantile normalization of Illumina Infinium methylation data at probe level. Input data is a MethyLumiM object
}
\usage{
normalizeMethylation.quantile(methyLumiM, separateColor = FALSE, reference = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
  \item{separateColor}{determine whether separately normalize two color channels}
  \item{reference}{the reference vector (same length as the number of matrix rows) for quanitle normalization}
  \item{\dots}{ other parameters used by normalize.quantiles.robust method }
}
\value{
Return an object (same class as input methyLumiM) with updated "methylated" and "unmethylated" data matrix after background level adjustment.
}
\author{
Pan DU
}
\seealso{
 See Also \code{\link{lumiMethyN}}, and \code{\link{normalizeMethylation.ssn}}
}
\examples{
data(example.lumiMethy)
lumiMethy.norm = normalizeMethylation.quantile(example.lumiMethy)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{bgAdjust}
\alias{bgAdjust}
\title{ Background adjustment for Illumina data }
\description{
 The method adjusts the data by subtracting an offset, which is estimated based on the quantile of the control probes
}
\usage{
bgAdjust(lumiBatch, probs = 0.5, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lumiBatch}{ A LumiBatch object with controlData slot include control probe information }
  \item{probs}{ The quantile used to estimate the background }
  \item{\dots}{ other parameters used by \code{\link{quantile}} method }
}
\details{
The method adjusts the data by subtracting an offset, which is estimated based on the quantile of the control probes. The control probe information is kept in the controlData slot of the LumiBatch object. If no control data information, the method will do nothing.
}
\value{
  It returns a LumiBatch object with background adjusted.
}

\author{ Pan Du }

\seealso{ \code{\link{lumiB}} }
\examples{
	data(example.lumi)
	## Here will assume the minimum of the control probe as the background,
	##    because there is no negative control (blank beads) information for the Barnes data.
	example.lumi.b <- bgAdjust(example.lumi, probs=0)
}
\keyword{methods}

\name{smoothQuantileNormalization}
\alias{smoothQuantileNormalization}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Smooth quantile normalization
}
\description{
Smooth quantile normalization with a reference sample
}
\usage{
smoothQuantileNormalization(dataMatrix, ref = NULL, adjData=NULL, logMode = TRUE, bandwidth = NULL, degree = 1, verbose = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{dataMatrix}{ a matrix of microarray intensity data}
  \item{ref}{ a vector of reference sample intensity, which matches the dataMatrix }
  \item{adjData}{ data to be adjusted based on the ref and dataMatrix distribution }
  \item{logMode}{ whether perform the analysis in log2 scale}
  \item{bandwidth}{ a parameter used by \code{\link[KernSmooth]{locpoly}} }
  \item{degree}{ a parameter used by \code{\link[KernSmooth]{locpoly}} }
  \item{verbose}{ whether print the processing sample names }
  \item{\dots}{other parameters used by \code{\link[KernSmooth]{locpoly}} }
}
\value{
a data matrix with intensity normalized.
}
\author{
Pan DU
}

\seealso{
 See Also \code{\link{adjColorBias.quantile}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{asBigMatrix-methods}
\docType{methods}
\alias{asBigMatrix-methods}
\alias{asBigMatrix}
\alias{asBigMatrix,ExpressionSet-method}
\title{convert the data matrix in the assayData of a ExpressionSet as BigMatrix}
\description{
 convert the data matrix in the assayData of a ExpressionSet as BigMatrix
}
\usage{
\S4method{asBigMatrix}{ExpressionSet}(object, rowInd=NULL, colInd=NULL, nCol=NULL, dimNames=NULL, saveDir='.', savePrefix=NULL, ...)
}

\arguments{
\item{object}{ an object of \code{\link[Biobase]{ExpressionSet}} or its inherited class }
\item{rowInd}{ the subset of row index }
\item{colInd}{ the subset of column index }
\item{nCol}{ the number of columns of the data, which can be larger than the real data dimension. It is designed for adding future data.}
\item{dimNames}{ the dimension names, which is a list of two character vectors (rownames and colnames)}
\item{saveDir}{the parent directory to save the BigMatrix data files}
\item{savePrefix}{the folder name prefix of the directory to save the BigMatrix data files. The fold name will be like this: paste(savePrefix, '_bigmat', sep='') }
\item{\dots}{ optional arguments to \code{\link[bigmemoryExtras]{BigMatrix}} }
}
\details{
This function does not work in Windows because the dependent package bigmemoryExtras does not support it. 
In order to make lumi package still compilation under Windows, I deliberately remove the dependency of bigmemoryExtras package.
As a result, users need to manually load the bigmemoryExtras function before using this function.

The BigMatrix data files will be save in the directory file.path(saveDir, paste(savePrefix, '_bigmat', sep=''))
}
\seealso{
\code{\link[bigmemoryExtras]{BigMatrix}}
}

\keyword{methods}

\name{hist-methods}
\docType{methods}
\alias{hist-methods}
\alias{hist.ExpressionSet}
\alias{hist,ExpressionSet-method}

\title{Density plot of a ExpressionSet object}
\description{ Creating density plot of sample intensities in a ExpressionSet object. It is equivalent to \link{density-methods}. }

\usage{
\S4method{hist}{ExpressionSet}(x, ...)
}

\arguments{
	\item{x}{ a \code{\link[Biobase]{ExpressionSet}} object}
	\item{\dots}{ other parameters for \link{density-methods} function }
}

\seealso{
  \code{\link{LumiBatch-class}}, \link{density-methods}, \code{\link[graphics]{hist}}
}
\examples{
## load example data
data(example.lumi)

hist(example.lumi)
}
\keyword{methods}
\keyword{hplot}

\name{plotVST}
\alias{plotVST}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ plot the VST (Variance Stabilizing Transform) function }
\description{
plot the VST (Variance Stabilizing Transform) function of VST transformed LumiBatch object or parameters of VST function.
}
\usage{
plotVST(x, transFun = NULL, plotRange = NULL, addLegend = TRUE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a LumiBatch object after lumiT transform, or a matrix or data.frame with VST parameter }
  \item{transFun}{ a character vector of transformation function (asinh or log2) }
  \item{plotRange}{ the plot range of untransformed data }
  \item{addLegend}{ add legend or not }
  \item{\dots}{ other parameter used by \code{\link[graphics]{plot}} function  }
}
\value{
invisibly return the untransformed and transformed values.
}
\author{ Pan Du }
\seealso{  \code{\link{vst}} }
\examples{

## load example data
data(example.lumi)

## Do default VST variance stabilizing transform
lumi.T <- lumiT(example.lumi, ifPlot=TRUE)

## plot the transform function
plotVST(lumi.T)

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{hplot}

\name{ssn}
\alias{ssn}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Simple Scaling Normalization }
\description{
  This function basically adjusts the samples to the same background level and then optionally scales to the same foreground level.
}
\usage{
ssn(x.lumi, targetArray = NULL, scaling = TRUE, bgMethod=c('density', 'mean', 'median', 'none'), fgMethod=c('mean', 'density', 'median'), ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ an ExpressionSet inherited object or a data matrix with columns as samples and rows as genes }
  \item{targetArray}{ A target chip is the model for other chips to normalize. It can be a column index, a vector or a LumiBatch object with one sample. }
  \item{scaling}{ determine whether do scaling or just background shift }
  \item{bgMethod}{ optional methods of determining the background level }
  \item{fgMethod}{ optional methods of determining the foreground level }
  \item{\dots}{ other parameters used by \code{\link[stats]{density}} function }
}
\details{
	This function basically adjusts the samples to the same background level and then optionally scales to the same foreground level. The adjustment is based on the raw scale data (For the transformed data, it still estimates the parameters in the raw scale by inverse transformation.). 
	
	Comparing with other normalization methods, like quantile and curve-fitting methods, SSN is a more conservative method. The only assumption is that each sample has the same background levels and the same scale (if do scaling). There are three methods ('density', 'mean' and 'median') for background estimation. If bgMethod is 'none', then the background level will be set as 0, i.e., no background adjustment. For the 'density' bgMethod, it estimates the background based on the mode of probe intensities based on the assumption that the background level intensity is the most frequent value across all the probes in the chip. For the foreground level estimation, it also provides three methods ('mean', 'density', 'median'). For the 'density' fgMethod, it assumes the background probe levels are symmetrically distributed. Then we estimate the foreground levels by taking the intensity mean of all other probes except from the background probes. For the 'mean' and 'median' methods (for both bgMethod and fgMethod), it basically estimates the level based on the mean or median of all probes of the sample. If the fgMethod is the same as bgMethod (except 'density' method), no scaling will be performed.   
}
\value{
Return an object with expression values normalized. The class of the return object is the same as the input object x.lumi.
}
\author{ Pan Du, Simon Lin }
\seealso{ \code{\link{lumiN}} }
\keyword{ methods }

\name{addAnnotationInfo}
\alias{addAnnotationInfo}
\title{Add probe color channel and basic annotation information based on the annotation library of Illumina methylation microarray }
\description{
Add probe color channel and basic annotation information based on the annotation library of Illumina methylation microarray
}
\usage{
addAnnotationInfo(methyLumiM, lib = 'FDb.InfiniumMethylation.hg19', annotationColumn=c('COLOR_CHANNEL', 'CHROMOSOME', 'POSITION'))
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{a MethyLumiM object includes Illumina Infinium methylation data}
  \item{lib}{Annotation library of Illumina methylation microarray. }
  \item{annotationColumn}{only include 'COLOR_CHANNEL', 'CHROMOSOME' and 'POSITION' information}
}
\value{
return the MethyLumiM object with COLOR_CHANNEL, CHROMOSOME and chromome POSITION information added to the featureData.
}
\details{
The "lib" parameter supports both FeatureDb based annotation libraries and old array-based annotation libraries.
'FDb.InfiniumMethylation.hg19' is the FeatureDb based annotation library, which includes both 450k and 27k data. 
"IlluminaHumanMethylation27k.db" (for 27k array) and "IlluminaHumanMethylation450k.db" (450k infinium array) are old types of annotation libraries.
}

\author{Pan DU}

\seealso{
\code{\link{lumiMethyR}}
}
\examples{
	data(example.lumiMethy)
	head(pData(featureData(example.lumiMethy)))
	## removing color channel information
	# testData = example.lumiMethy
	# pData(featureData(testData))$COLOR_CHANNEL = NULL
	# testData = addAnnotationInfo(testData, lib="IlluminaHumanMethylation27k.db")
	## check whether the color channel information is added
	# head(pData(featureData(testData)))
	
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{lumiQ}
\alias{lumiQ}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Quality control evaluation of the LumiBatch object }
\description{
Quality control evaluation of the LumiBatch object and returns a summary of the data
}

\usage{
lumiQ(x.lumi, logMode = TRUE, detectionTh = 0.01, verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x.lumi}{ a LumiBatch object }
  \item{logMode}{ transform as log2 or not (the function can check whether it is already log transformed.)}
  \item{detectionTh}{ the detection threshold used by \code{\link{detectionCall}} } 
  \item{verbose}{ a boolean to decide whether to print out some messages }
}
\details{
  Quality control of a LumiBatch object includes estimating the mean and standard deviation of the chips, detectable probe ratio of each chip, sample (chip) relations, detecting outliers of samples (chips). The produced QC information is kept in the QC slot of LumiBatch class. The summary function will provide a summary of the QC information (See example).
}
\value{
a LumiBatch object with QC slot keeping the QC information
}

\author{ Pan Du }

\seealso{\code{\link{LumiBatch}}, \code{\link{plot,ExpressionSet-method}} }
\examples{
## load example data
data(example.lumi)

## Do quality control estimation
lumi.Q <- lumiQ(example.lumi)

## A summary of the QC
summary(lumi.Q, 'QC')

## Plot the results
## plot the pairwise sample correlation
plot(lumi.Q, what='pair')

## see more examples in "plot,ExpressionSet-method" help documents

}
\keyword{methods}

\name{lumiExpresso}
\alias{lumiExpresso}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{From raw Illumina probe intensities to expression values}
\description{
  Goes from raw Illumina probe intensities to expression values
}
\usage{
lumiExpresso(lumiBatch, bg.correct = TRUE, bgcorrect.param = list(method='bgAdjust'), variance.stabilize = TRUE, 
	varianceStabilize.param = list(), normalize = TRUE, normalize.param = list(), QC.evaluation = TRUE, 
	QC.param = list(), verbose = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lumiBatch}{ a LumiBatch object, which can be the return of \code{\link{lumiR}} }
  \item{bg.correct}{ a boolean to decide whether to do background correction or not }
  \item{bgcorrect.param}{ a list of parameters of \code{\link{lumiB}} }
  \item{variance.stabilize}{ a boolean to decide whether to do variance stabilization or not }
  \item{varianceStabilize.param}{ a list of parameters of \code{\link{lumiT}} }
  \item{normalize}{ a boolean to decide whether to do normalization or not }
  \item{normalize.param}{ a list of parameters of \code{\link{lumiN}} }
  \item{QC.evaluation}{ a boolean to decide whether to do quality control estimation before and after preprocessing }
  \item{QC.param}{ a list of parameters of \code{\link{lumiQ}} }
  \item{verbose}{ a boolean to decide whether to print out some messages }
}
\details{
  The function is to encapsulate the major functions of Illumina preprocessing. It is organized in a similar way as the \code{\link[affy]{expresso}} function in affy package.
}
\value{
  return a processed LumiBatch object. The operation history can be track in the history slot of the object.
}

\author{ Pan Du }

\seealso{ \code{\link{lumiB}}, \code{\link{lumiT}}, \code{\link{lumiN}} }
\examples{
## load example data
data(example.lumi)

## Do all the default preprocessing in one step
lumi.N <- lumiExpresso(example.lumi)

## Do customized preprocessing. No variance stabilizing or log transform, use Quantile normalization.
lumi.N <- lumiExpresso(example.lumi, variance.stabilize=FALSE, normalize.param = list(method='quantile'))

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }

\name{lumiMethyR}
\alias{lumiMethyR}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Reading Illumina methylation microarray data
}
\description{
This function is a wrap of \code{\link[methylumi]{methylumiR}} function in methylumi package.
}
\usage{
lumiMethyR(filename, lib=NULL, controlData=NULL, qcfile=NULL, sampleDescriptions=NULL, sep = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{filename}{ file name output by GenomeStudio}
  \item{lib}{Annotation library of Illumina methylation microarray}
  \item{controlData}{the controlData file name or a MethyLumiQC object to be added to the "controlData" slot of the MethyLumiM object}
  \item{qcfile}{ parameter of \code{\link[methylumi]{methylumiR}} function}
  \item{sampleDescriptions}{ parameter of \code{\link[methylumi]{methylumiR}} function}
  \item{sep}{ parameter of \code{\link[methylumi]{methylumiR}} function}
}
\details{
This function is a wrap of \code{\link[methylumi]{methylumiR}} function in methylumi package. It will coerce the returned object as MethyLumiM class. The methylated and unmethylated probe intensity information is required for color-bias adjustment and normalization. If users have the Illumina IDAT files, we suggest use  \code{\link{importMethyIDAT}} function to import the data. The  \code{\link{importMethyIDAT}} function will automatically retrieve the required information and return a MethyLumiM object. 
}
\value{ return a MethyLumiM object }

\author{
Pan Du
}

\seealso{
  See Also \code{\link{importMethyIDAT}}, \code{\link[methylumi]{methylumiR}} and \code{\link{addControlData2methyLumiM}}
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}

\name{m2beta}
\alias{m2beta}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Convert methylation M-value to Beta-value
}
\description{
Convert methylation M-value to Beta-value
}
\usage{
m2beta(m)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{m}{
	a matrix or vector of methylation M-value
}
}
\details{
Convert methylation M-value to Beta-value
}
\value{
return methylation Beta-value with the same size of input M-value
}
\references{
Du, P., Zhang, X, Huang, C.C., Jafari, N., Kibbe, W.A., Hou, L., and Lin, S.M.,  (2010) 'Comparison of Beta-value and M-value methods for quantifying methylation levels by microarray analysis', (under review)
}
\author{
Pan Du
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
See Also as \code{\link{beta2m}}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }

\name{targetID2nuID}
\alias{targetID2nuID}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Mapping Illumina TargetID (GeneID) into nuID}
\description{
Mapping Illumina TargetID (GeneID) into nuID.
}
\usage{
targetID2nuID(targetID, lib.mapping = "lumiHumanIDMapping", ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{targetID}{ a vector of Illumina TargetID (GeneID) }
  \item{lib.mapping}{ an Illumina ID mapping library }
  \item{\dots}{ other parameters of \code{\link{IlluminaID2nuID}} }
}
\details{
The function will call \code{\link{IlluminaID2nuID}} when ID mapping library were provided.
}
\value{
see function \code{\link{IlluminaID2nuID}}
}

\references{
Du, P., Kibbe, W.A. and Lin, S.M., "nuID: A universal naming schema of oligonucleotides for Illumina, Affymetrix, and other microarrays", Biology Direct 2007, 2:16 (31May2007).
}
\author{Pan Du}
\seealso{ \code{\link{nuID2targetID}}, \code{\link{IlluminaID2nuID}} }
\examples{
if (require(lumiHumanIDMapping)) {
    targetID2nuID('GI_21389350-S', lib='lumiHumanIDMapping')
}
}
% R documentation directory.
\keyword{ methods }

\name{plotDensity}
\docType{methods}
\alias{plotDensity}
\alias{plotDensity.ExpressionSet}

\title{plot the density distribution}
\description{ plot the density distribution of a dataMatrix or ExpressionSet object }

\usage{
 plotDensity(dataMatrix, logMode=TRUE, addLegend=TRUE, legendPos="topright", subset = NULL, ...)
}

\arguments{
	\item{dataMatrix}{ a data matrix or \code{\link[Biobase]{ExpressionSet}} object }
	\item{logMode}{ determine whether the CDF plot is based on a log2 scale }
	\item{addLegend}{ whether add legend to the plot or not }
	\item{legendPos}{ the position of the legend. If its length is two, then it specifies the x and y location of legend. }
	\item{subset}{plot subset of randomly selected rows. All data will be plotted if it is NULL. }
	\item{\dots}{ additional parameters for \code{\link[graphics]{matplot}} function }
}

\seealso{
  \code{\link{LumiBatch-class}}, \code{\link[stats]{density}}
}
\examples{
## load example data
data(example.lumi)

plotDensity(example.lumi)
}
\keyword{methods}
\keyword{hplot}

\name{boxplotColorBias}
\alias{boxplotColorBias}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot the Illumina Infinium methylation color bias in terms of boxplot
}
\description{
Plot the Illumina Infinium methylation color bias in terms of boxplot. boxplot of red and green color channel will be plotted side by side
}
\usage{
boxplotColorBias(methyLumiM, logMode = TRUE, channel = c("both", "unmethy", "methy", "sum"), grid = TRUE, main = NULL, mar = NULL, verbose = F, subset = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{methyLumiM}{ MethyLumiM-class object or eSet-class object, which include methylated and unmethylated probe intensities }
  \item{logMode}{ whether plot the intensities in log-scale }
  \item{channel}{ estimate the intensity in different methods }
  \item{grid}{ whether to add grid on the plot }
  \item{main}{ title of the plot }
  \item{mar}{ margin of the plot }
  \item{verbose}{ whether print verbose information during plot }
  \item{subset}{plot subset of randomly selected rows. All data will be plotted if it is NULL. }
  \item{\dots}{ other parameters of \code{\link[graphics]{boxplot}} }
}
\details{
Plot the Illumina Infinium methylation color bias in terms of boxplot. boxplot of red and green color channel will be plotted side by side
}
\value{
Invisibly return TRUE if plot successfully.
}
\author{
Pan DU
}

\seealso{
 See Also as \code{\link[graphics]{boxplot}} and \code{\link{plotColorBias1D}} 
}
\examples{
data(example.lumiMethy)
boxplotColorBias(example.lumiMethy)
}

\keyword{methods}
\keyword{hplot}

\name{plotGammaFit}
\alias{plotGammaFit}

\title{ plot the fitting results of \code{\link{gammaFitEM}} }

\description{
plot the fitting results of \code{\link{gammaFitEM}}. It basically plot the histogram of M-values plus the estimated gamma density plots and their mixture.
}

\usage{
plotGammaFit(x, gammaFit = NULL, k = NULL, theta = NULL, shift = NULL, proportion = NULL, plotType = c("histogram", "density"), ...)
}

\arguments{
  \item{x}{ a vector of M-values covering the whole genome }
  \item{gammaFit}{ a "gammaFit" class object returned by \code{\link{gammaFitEM}} }
  \item{k}{parameter k of gamma distribution}
  \item{theta}{parameter theta of gamma distribution}
  \item{shift}{parameter shift of gamma distribution}
  \item{proportion}{the proportion of two components (gamma distributions)}
  \item{plotType}{ determine the way to show the distribution of the input data, either histogram or density plot }
  \item{\dots}{ Other parameters used by \code{\link[graphics]{hist}} or \code{\link[graphics]{plot}} (for "density" plotType) function. }
}

\details{
 This function is to visualize the fitting results, which helps us understand how well the fitting is.
}
\value{
Invisibly return TRUE if the plot is successful.
}
\author{ Pan DU }

\seealso{
\code{\link{gammaFitEM}}
}

\examples{

data(example.lumiMethy)
M <- exprs(example.lumiMethy)
fittedGamma <- gammaFitEM(M[,1], initialFit=NULL, maxIteration=50, tol=0.0001, plotMode=FALSE, verbose=FALSE)
plotGammaFit(M[,1], gammaFit=fittedGamma)

}

\keyword{methods}
